/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECDiagnoseDlg.cpp : implementation file
//

#include "stdafx.h"
#include "XDiagnoser.h"
#include "CECDiagnoseDlg.h"
#include "..\..\HTMLHelp\XDiagnoser\Logging_Levels.inl"
#include "Htmlhelp.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECDiagnoseDlg dialog


CECDiagnoseDlg::CECDiagnoseDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECDiagnoseDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECDiagnoseDlg)
    //}}AFX_DATA_INIT
    m_pCorba = new CORBAServer( 0, NULL );
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

}


void CECDiagnoseDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECDiagnoseDlg)
    DDX_Control(pDX, IDC_FULL_HELP, m_HelpButton);
    DDX_Control(pDX, IDC_SAVE, m_SaveButton);
    DDX_Control(pDX, IDC_GROUP, m_GroupBox);
    DDX_Control(pDX, IDC_CORBA_REFRESH, m_RefreshButton);
    DDX_Control(pDX, IDC_LEVELS, m_lcLevels);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECDiagnoseDlg, CDialog)
//{{AFX_MSG_MAP(CECDiagnoseDlg)
ON_BN_CLICKED(IDC_CORBA_REFRESH, OnCorbaRefresh)
ON_BN_CLICKED(IDC_SAVE, OnSave)
ON_WM_SIZE()
ON_WM_DESTROY()
ON_WM_HELPINFO()
ON_BN_CLICKED(IDC_FULL_HELP, OnFullHelp)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECDiagnoseDlg message handlers

void CECDiagnoseDlg::SetServer(CYRServer *pServer)
{
    m_pServer = pServer;
}

void CECDiagnoseDlg::Init(CString strProject)
{
    m_ActiveProject = strProject;
    // Populate list box headers
    SetCorbaHelper();
    InitListControl();
    PopulateLevels();
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method opens the file of trace settings stored within the
//          OpenSTARep and copies it to the local machine.  The file is
//          then imported into the list control.  The method then searches the
//          network for any active logs and sets the dynamic value of the
//          trace settings.
void CECDiagnoseDlg::PopulateLevels()
{
    // Set cursor to be hourglass
    CWaitCursor cursor;

    // Empty any existing entries
    m_lcLevels.DeleteAllItems();

    // Get log settings from server
    if(m_pServer->CheckOutFile(CYRServer::enRootFolder,"",LOG_SETTINGS_FILE))
    {
        CString strPath = m_pServer->GetFolderPath(CYRServer::enRootFolder)+LOG_SETTINGS_FILE;

        m_lcLevels.ImportFile( strPath );
        m_lcLevels.ImportCorba();
    }

    // Just populated list control so cannot be dirty
    m_lcLevels.SetDirtyFlag( false );
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method initialises the list control by setting up the column
//          information.
void CECDiagnoseDlg::InitListControl()
{
    const int nNumColumns = 3;
    // Get column header information and add columns to list control
    for ( int i = 0; i < nNumColumns; i++ )
        m_lcLevels.CreateColumn( i, IDS_LEVELNAME_COL + i );
}

void CECDiagnoseDlg::OnCorbaRefresh()
{
    if ( m_lcLevels.ImportCorba() == false )
    {
        // Some kind of exception or error occurred so show error message.
        CString strTitle, strError;
        strError.LoadString( IDS_FACTORY_NOT_FOUND );
        strTitle= "OpenSTA" ;
        MessageBox( strError, strTitle, MB_ICONEXCLAMATION | MB_OK );
    }

}

void CECDiagnoseDlg::OnSave()
{
    // Display wait cursor
    CWaitCursor cursor;

    // If there have been no changes to the entries within the list control
    // then there is no need to save and we can exit this function.
    if ( m_lcLevels.GetDirtyFlag() == false )
        return;

    // Get log settings file location from OpenSTARep
    CString strPath = m_pServer->GetFolderPath(CYRServer::enRootFolder) + LOG_SETTINGS_FILE;

    m_lcLevels.ExportFile( strPath );
    m_lcLevels.ExportCorba();

    // Check in new file
    m_pServer->CheckInFile(CYRServer::enRootFolder,"",LOG_SETTINGS_FILE,LOG_SETTINGS_FILE);

    // Details have been saved so entries are now clean
    m_lcLevels.SetDirtyFlag( false );
}

void CECDiagnoseDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    if (m_GroupBox.m_hWnd != NULL)
    {
        m_GroupBox.SetWindowPos(NULL, 5,5,cx-100, cy-15, SWP_NOZORDER);
        m_lcLevels.SetWindowPos(NULL, 20,20,cx-120, cy-40, SWP_NOZORDER);
        m_RefreshButton.SetWindowPos(NULL, cx-80,10,0, 0, SWP_NOSIZE);
        m_SaveButton.SetWindowPos(NULL, cx-80,50,0, 0, SWP_NOSIZE);
        m_HelpButton.SetWindowPos(NULL, cx-80,90,0, 0, SWP_NOSIZE);

        const int nNumColumns = 3;

        CRect rc;
        m_lcLevels.GetClientRect(&rc);
        int nWidth = rc.Width()/nNumColumns;
        for(int i=0;i<nNumColumns;i++)
            m_lcLevels.SetColumnWidth(i,nWidth);
    }
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method simply assigns a CORBA helper class to the list of trace
//          levels.
// @parm    Pointer to the CORBA helper class.
void CECDiagnoseDlg::SetCorbaHelper()
{
    m_lcLevels.SetCorbaHelper( m_pCorba );
}

void CECDiagnoseDlg::OnDestroy()
{
    CDialog::OnDestroy();

    if (m_pCorba != NULL)
    {
        m_pCorba->GetBOA()->destroy();
        m_pCorba->GetORB()->NP_destroy();
        delete m_pCorba;
    }
}

BOOL CECDiagnoseDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
		HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

void CECDiagnoseDlg::OnFullHelp()
{
    HtmlHelp(m_hWnd, m_szHtmlHelpFile, HH_DISPLAY_TOPIC, (DWORD)"xdiagnos.htm");
}
