/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// FilterDlg.cpp : implementation file
//

#include "stdafx.h"
#include "xmanagermon.h"
#include "FilterDlg.h"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFilterDlg dialog
#include "..\..\HTMLHelp\XManagerMon\Test_Manager_Filter_screen.inl"

CFilterDlg::CFilterDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CFilterDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CFilterDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}


void CFilterDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CFilterDlg)
    DDX_Control(pDX, IDC_USERS, m_ctlUsers);
    DDX_Control(pDX, IDC_PROJECTS, m_ctlProjects);
    DDX_Control(pDX, IDC_INJECTORS, m_ctlInjectors);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CFilterDlg, CDialog)
//{{AFX_MSG_MAP(CFilterDlg)
ON_WM_HELPINFO()
ON_BN_CLICKED(IDC_FULL_HELP, OnFullHelp)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFilterDlg message handlers
void CFilterDlg::SetInjectors(const CStringArray& arstrInjectors)
{
    m_arstrInjectors.SetSize(arstrInjectors.GetSize());
    for(int nNdx=0;nNdx<arstrInjectors.GetSize();nNdx++)
        m_arstrInjectors[nNdx] = arstrInjectors[nNdx];
}

void CFilterDlg::SetUsers(const CStringArray& arstrUsers)
{
    m_arstrUsers.SetSize(arstrUsers.GetSize());
    for(int nNdx=0;nNdx<arstrUsers.GetSize();nNdx++)
        m_arstrUsers[nNdx] = arstrUsers[nNdx];
}

void CFilterDlg::SetProjects(const CStringArray& arstrProjects)
{
    m_arstrProjects.SetSize(arstrProjects.GetSize());
    for(int nNdx=0;nNdx<arstrProjects.GetSize();nNdx++)
        m_arstrProjects[nNdx] = arstrProjects[nNdx];
}

BOOL CFilterDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    HBITMAP     hBitMap;
    HINSTANCE   hInstance = AfxGetInstanceHandle ();
    hBitMap = LoadBitmap (hInstance ,MAKEINTRESOURCE (IDB_HELP));
    ASSERT (hBitMap != NULL);
    CButton* pBP = (CButton*)GetDlgItem(IDC_FULL_HELP);
    pBP->SetBitmap(hBitMap);

    m_ctlInjectors.AddString(FILTER_ALL);
    m_ctlUsers.AddString(FILTER_ALL);
    m_ctlProjects.AddString(FILTER_ALL);

    for(int nNdx=0;nNdx<m_arstrInjectors.GetSize();nNdx++)
    {
        if(m_ctlInjectors.FindStringExact(-1,m_arstrInjectors[nNdx])==CB_ERR)
            m_ctlInjectors.AddString(m_arstrInjectors[nNdx]);
    }

    for(nNdx=0;nNdx<m_arstrUsers.GetSize();nNdx++)
    {
        if(m_ctlUsers.FindStringExact(-1,m_arstrUsers[nNdx])==CB_ERR)
            m_ctlUsers.AddString(m_arstrUsers[nNdx]);
    }

    for(nNdx=0;nNdx<m_arstrProjects.GetSize();nNdx++)
    {
        if(m_ctlProjects.FindStringExact(-1,m_arstrProjects[nNdx])==CB_ERR)
            m_ctlProjects.AddString(m_arstrProjects[nNdx]);
    }

    int nSel=m_ctlInjectors.FindStringExact(-1,m_strSelectedInjector);
    m_ctlInjectors.SetCurSel( (nSel == CB_ERR) ? 0 : nSel );

    nSel = m_ctlUsers.FindStringExact(-1,m_strSelectedUser);
    m_ctlUsers.SetCurSel( (nSel == CB_ERR) ? 0 : nSel );

    nSel = m_ctlProjects.FindStringExact(-1,m_strSelectedProject);
    m_ctlProjects.SetCurSel( (nSel== CB_ERR) ? 0 : nSel );

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CFilterDlg::OnOK()
{
    int nSel = m_ctlInjectors.GetCurSel();
    if(nSel != CB_ERR)
        m_ctlInjectors.GetLBText(nSel,m_strSelectedInjector);

    nSel = m_ctlUsers.GetCurSel();
    if(nSel != CB_ERR)
        m_ctlUsers.GetLBText(nSel,m_strSelectedUser);

    nSel = m_ctlProjects.GetCurSel();
    if(nSel != CB_ERR)
        m_ctlProjects.GetLBText(nSel,m_strSelectedProject);

    CDialog::OnOK();
}

BOOL CFilterDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

void CFilterDlg::OnFullHelp()
{
    HtmlHelp(m_hWnd, m_szHtmlHelpFile, HH_DISPLAY_TOPIC, (DWORD)"xmanage2.htm");
}
