/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECProjectSelector.cpp : implementation file
//

#include "stdafx.h"
#include "XProjectViewer.h"
#include "CECProjectSelector.h"
//#include "..\..\Studio\Common\CECXServerError.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECProjectSelector dialog


CECProjectSelector::CECProjectSelector(CWnd* pParent /*=NULL*/)
        : CDialog(CECProjectSelector::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECProjectSelector)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_nOldSel=0;
    m_pServer = NULL;
}


void CECProjectSelector::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECProjectSelector)
    // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECProjectSelector, CDialog)
//{{AFX_MSG_MAP(CECProjectSelector)
ON_CBN_DROPDOWN(IDC_PROJ_COMBO, OnDropdownProjCombo)
ON_CBN_CLOSEUP(IDC_PROJ_COMBO, OnCloseupProjCombo)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECProjectSelector message handlers

BOOL CECProjectSelector::OnInitDialog()
{
    CDialog::OnInitDialog();

    GetListOfProjects();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECProjectSelector::GetListOfProjects()
{
    CComboBox* pCombo = (CComboBox*) GetDlgItem(IDC_PROJ_COMBO);
    pCombo->ResetContent();

    ASSERT(m_pServer != NULL);
    //
    //    Get a list of the projects from the server
    //
    CStringArray arstrProjects;
    m_pServer->GetProjectList(arstrProjects);

    //
    //    ...and add them to the combo box.
    //
    for(int nNdx=0;nNdx<arstrProjects.GetSize();nNdx++)
    {
        pCombo->AddString(arstrProjects[nNdx]);
    }
    pCombo->SetCurSel(m_nOldSel);
}

void CECProjectSelector::OnDropdownProjCombo()
{
    CComboBox* pCombo = (CComboBox*) GetDlgItem(IDC_PROJ_COMBO);
    m_nOldSel = pCombo->GetCurSel();

    GetListOfProjects();
}

void CECProjectSelector::OnCloseupProjCombo()
{
    // Get Project Name from Combo
    CComboBox* pCombo = (CComboBox*) GetDlgItem(IDC_PROJ_COMBO);
    int sel = pCombo->GetCurSel();

    CString newProjName;
    pCombo->GetLBText(sel, newProjName);

    if (!newProjName.IsEmpty() && (newProjName != m_ProjectName))
    {
        m_ProjectName = newProjName;
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_PROJECT_CHANGED,0,0);
    }

}

CString CECProjectSelector::GetProjectName()
{
    CString FullName = "Cyr."+m_strType+"."+m_ProjectName;
    return FullName;
}

void CECProjectSelector::SetProjectName(CString ProjectName)
{
    //
    // Refresh project list
    //
    GetListOfProjects();

    //
    // Strip off the "Cyr." prefix
    //
    if (ProjectName.Left(4) == "Cyr.")
    {
        int nLen = ProjectName.GetLength();

        ProjectName=ProjectName.Right(nLen - 4);

        nLen = ProjectName.GetLength();
        int nPos = ProjectName.Find('.');
        if (nPos != -1)
        {
            CString projType = ProjectName.Left(nPos);
            if (projType != m_strType)
            {
                return;
            }

            ProjectName = ProjectName.Right(nLen - nPos-1);
        }
    }


    //
    // Find the given ProjectName and select it in the combo box
    //
    m_ProjectName = ProjectName;
    CComboBox* pCombo = (CComboBox*) GetDlgItem(IDC_PROJ_COMBO);
    int sel = pCombo->FindStringExact(0,ProjectName);
    if (sel != -1)
    {
        pCombo->SetCurSel(sel);
    }
    else
    {
        pCombo->SetCurSel(0);
    }
}

void CECProjectSelector::SetServer(CYRServer* pServer)
{
    ASSERT (pServer != NULL);
    m_pServer = pServer;
}

void CECProjectSelector::SetProjectType(CString projType)
{
    m_strType = projType;
}
