/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECXFloorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "XProjectViewer.h"
#include "CECXFloorDlg.h"
#include "..\..\Studio\Interfaces\CyrPlugins.h"
#include "Solutions.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "resource.h"

/////////////////////////////////////////////////////////////////////////////
// CECXFloorDlg dialog


CECXFloorDlg::CECXFloorDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECXFloorDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECXFloorDlg)
    //}}AFX_DATA_INIT

    m_strActiveProject = "";
    m_selectedNode = -1;
    m_bShowNodeMenus = TRUE;
}


void CECXFloorDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECXFloorDlg)
    DDX_Control(pDX, IDC_NODELIST, m_nodeList);
    DDX_Control(pDX, IDC_XREPOS_OUTPUT, m_OutputView);
    DDX_Control(pDX, IDC_XREPOS_PROJECT, m_ProjectView);
    DDX_Control(pDX, IDC_XREPOS_SOURCE, m_SourceView);
    DDX_Control(pDX, IDC_XREPOS_TEST, m_TestView);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECXFloorDlg, CDialog)
ON_WM_CONTEXTMENU()
//{{AFX_MSG_MAP(CECXFloorDlg)
ON_WM_SIZE()
ON_NOTIFY(NM_DBLCLK, IDC_NODELIST, OnDblclkNodelist)
ON_COMMAND(ID_POPUP_DELETE, OnNodeDelete)
ON_COMMAND(ID_POPUP_EDIT, OnNodeEdit)
ON_COMMAND(ID_POPUP_NEW, OnNodeNew)
ON_WM_PAINT()
//}}AFX_MSG_MAP
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECXFloorDlg message handlers

void CECXFloorDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    if (m_wndBarOutlook.m_hWnd != NULL)
    {
        m_wndBarOutlook.SetWindowPos(NULL,0,0,cx,cy,SWP_NOZORDER);
    }
}

BOOL CECXFloorDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    AfxEnableControlContainer(NULL);

	if (!m_wndBarOutlook.Create(this, 
		WS_CHILD | WS_VISIBLE | CBRS_LEFT | CBRS_HIDE_INPLACE,
		AFX_IDW_CONTROLBAR_FIRST + 22))
	{
		return FALSE;		// fail to create
	}

	m_wndBarOutlook.SetTransparentColor (RGB (255, 0, 255));

    int nNextPane = 0;
    if (m_strConfig.Find("TEST_TREE", 0) != -1)
    {
        m_TestView.SetServer((long)m_pServer);
        m_TestView.XInitialize("TEST_TREE");

		m_wndBarOutlook.AddPage (nNextPane, _T("Tests"),&m_TestView);

//		m_TestView.ShowWindow(SW_SHOW);
//		m_XFloor.AttachWindow(nNextPane,(long)pWnd);//&m_TestView);
        nNextPane++;
    }
    if (m_strConfig.Find("PROJECT_TREE", 0) != -1)
    {
        m_ProjectView.SetServer((long)m_pServer);
        m_ProjectView.XInitialize("PROJECT_TREE");
		m_wndBarOutlook.AddPage (nNextPane, _T("Project View"),&m_ProjectView);
        nNextPane++;
    }

    if (m_strConfig.Find("SOURCE_TREE", 0) != -1)
    {
        m_SourceView.SetServer((long)m_pServer);
        m_SourceView.XInitialize("SOURCE_TREE");
		m_wndBarOutlook.AddPage (nNextPane, _T("User Scripts"),&m_SourceView);
        nNextPane++;
    }

    if (m_strConfig.Find("OUTPUT_TREE", 0) != -1)
    {
        m_OutputView.SetServer((long)m_pServer);
        m_OutputView.XInitialize("OUTPUT_TREE");
		m_wndBarOutlook.AddPage (nNextPane, _T("Results"),&m_OutputView);
        nNextPane++;
    }
    if (m_strConfig.Find("NODE_LIST", 0) != -1)
    {
        FillNodeList();
        m_ImageList.Create(IDB_NODELIST_BMP,16,0,RGB(0,255,0));
        m_nodeList.SetImageList(&m_ImageList,LVSIL_SMALL);
		m_wndBarOutlook.AddPage (nNextPane, _T("Injectors"),&m_nodeList);
        m_nodePage = nNextPane;
        nNextPane++;
    }

	m_wndBarOutlook.EnableAnimation();
	m_wndBarOutlook.SetActivePage(0,TRUE);
	
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECXFloorDlg::ChangeProject(CString strProjName)
{
    CString initString;

    m_strActiveProject = strProjName;

    if (m_strConfig.Find("PROJECT_TREE", 0) != -1)
    {
        initString.Format("PROJECT_TREE|%s",strProjName);
        m_ProjectView.XInitialize(initString);
    }

    if (m_strConfig.Find("SOURCE_TREE", 0) != -1)
    {
        initString.Format("SOURCE_TREE|%s",strProjName);
        m_SourceView.XInitialize(initString);
    }

    if (m_strConfig.Find("OUTPUT_TREE", 0) != -1)
    {
        initString.Format("OUTPUT_TREE|%s",strProjName);
        m_OutputView.XInitialize(initString);
    }
    if (m_strConfig.Find("TEST_TREE", 0) != -1)
    {
        initString.Format("TEST_TREE|%s",strProjName);
        m_TestView.XInitialize(initString);
    }
    if (m_strConfig.Find("NODE_LIST", 0) != -1)
    {
        FillNodeList();
    }
}

BEGIN_EVENTSINK_MAP(CECXFloorDlg, CDialog)
//{{AFX_EVENTSINK_MAP(CECXFloorDlg)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_PROJECT, 2 /* XInitiateTest */, OnXInitiateTest, VTS_BSTR)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_OUTPUT, 3 /* XOpenTestRun */, OnXOpenTestRun, VTS_BSTR VTS_BSTR)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_PROJECT, 4 /* XEditFile */, OnXEditFile, VTS_BSTR)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_OUTPUT, 6 /* XAddTest */, OnXAddTest, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_OUTPUT, 7 /* XAddScript */, OnXAddScript, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_SOURCE, 8 /* XExportFile */, OnXExportFile, VTS_BSTR)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_SOURCE, 9 /* XOpenFile */, OnXOpenFile, VTS_BSTR)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_OUTPUT, 5 /* XViewResults */, OnXViewResults, VTS_BSTR VTS_BSTR VTS_I2)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_OUTPUT, 11 /* XViewHttpStats */, OnXViewHttpStatsXreposOutput, VTS_BSTR VTS_BSTR)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_SOURCE, 4 /* XEditFile */, OnXEditFile, VTS_BSTR)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_PROJECT, 6 /* XAddTest */, OnXAddTest, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_SOURCE, 6 /* XAddTest */, OnXAddTest, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_PROJECT, 7 /* XAddScript */, OnXAddScript, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_SOURCE, 7 /* XAddScript */, OnXAddScript, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_PROJECT, 8 /* XExportFile */, OnXExportFile, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_OUTPUT, 8 /* XExportFile */, OnXExportFile, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_PROJECT, 9 /* XOpenFile */, OnXOpenFile, VTS_NONE)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_TEST, 4 /* XEditFile */, OnXEditFile, VTS_BSTR)
ON_EVENT(CECXFloorDlg, IDC_XREPOS_TEST, 2 /* XInitiateTest */, OnXInitiateTest, VTS_BSTR)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CECXFloorDlg::OnXInitiateTest(LPCTSTR strTestName)
{
    m_strTestToExecute = strTestName;
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_EXECUTE_TEST,0,0);
}


CString CECXFloorDlg::GetTestName()
{
    return m_strTestToExecute;
}

void CECXFloorDlg::OnXOpenTestRun(LPCTSTR TestName, LPCTSTR TestRunName)
{
    m_OpenRun_Test = TestName;
    m_OpenRun_RunName = TestRunName;
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_OPEN_TESTRUN,0,0);
}

void CECXFloorDlg::GetOpenTestRun(CString &TestName, CString &RunName)
{
    TestName = m_OpenRun_Test;
    RunName = m_OpenRun_RunName;
}

CString CECXFloorDlg::GetScriptName()
{
    return m_strScriptToEdit;
}

void CECXFloorDlg::SetConfig(CString strConfig)
{
    m_strConfig = strConfig;
}

void CECXFloorDlg::OnXEditFile(LPCTSTR strFileName)
{
    m_strScriptToEdit = strFileName;
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_EDIT_FILE,0,0);
}

void CECXFloorDlg::OnXAddTest()
{
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_NEW_TEST,0,0);

}

void CECXFloorDlg::OnXAddScript()
{
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_NEW_SCRIPT,0,0);

}

void CECXFloorDlg::OnXExportFile(LPCTSTR strFileName)
{
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_EXPORT_FILE,0,(LPARAM)strFileName);

}

void CECXFloorDlg::Refresh()
{
    if (m_strConfig.Find("PROJECT_TREE", 0) != -1)
    {
        m_ProjectView.XRefresh();
    }

    if (m_strConfig.Find("OUTPUT_TREE", 0) != -1)
    {
        m_OutputView.XRefresh();
    }

    if (m_strConfig.Find("SOURCE_TREE", 0) != -1)
    {
        m_SourceView.XRefresh();
    }

    if (m_strConfig.Find("TEST_TREE", 0) != -1)
    {
        m_TestView.XRefresh();
    }
    if (m_strConfig.Find("NODE_LIST", 0) != -1)
    {
        FillNodeList();
    }
}


void CECXFloorDlg::OnXOpenFile(LPCTSTR strFileName)
{
    m_strScriptToEdit = strFileName;
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_OPEN_FILE,0,0);
}


void CECXFloorDlg::OnXViewResults(LPCTSTR strTestName, LPCTSTR strTestRunName, short nType)
{
    CString msg;
    m_OpenRun_Test = strTestName;
    m_OpenRun_RunName = strTestRunName;

    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_XVIEW_RESULTS,nType,0);
}

void CECXFloorDlg::FillNodeList()
{
    m_nodeList.DeleteAllItems();

    if (m_strActiveProject == "" || m_strActiveProject == "<No Project Selected>")
    {
        return;
    }

    CStringArray arstrNodes;
    m_pServer->GetFileList(CYRServer::enNodesFolder,"",arstrNodes);

    for(int nNdx=0;nNdx<arstrNodes.GetSize();nNdx++)
    {
        m_nodeList.InsertItem(nNdx,arstrNodes[nNdx], 0);
    }
}

void CECXFloorDlg::SetServer(CYRServer *pServer)
{
    m_pServer = pServer;
}

void CECXFloorDlg::OnDblclkNodelist(NMHDR* pNMHDR, LRESULT* pResult)
{
    int index = RetrieveHitItem();
    CString strItem = m_nodeList.GetItemText(index,0);
    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT, (WPARAM)WMU_EDIT_NODE, (LPARAM)&strItem);
    FillNodeList();


    *pResult = 0;
}

//
// RetrieveHitItem
// ===============
//
// Perform a hit test on the list control using the current cursor position,
// and retrieve the index of the item that was hit, deselecting the old item.
int CECXFloorDlg::RetrieveHitItem()
{
    //
    // Locate the position of the click
    CPoint point;
    GetCursorPos(&point);
    m_nodeList.ScreenToClient(&point);

    //
    // Perform a hit test -*-*-*- which item in the list view was selected
    int nItem;
    UINT    nFlags = LVHT_ONITEM;
    nItem = m_nodeList.HitTest(point ,&nFlags);

    return nItem;
}


void CECXFloorDlg::OnContextMenu(CWnd*, CPoint point)
{
    if (!m_bShowNodeMenus)
        return;

    // CG: This block was added by the Pop-up Menu component
    {
        if (point.x == -1 && point.y == -1)
        {
            //keystroke invocation
            CRect rect;
            GetClientRect(rect);
            ClientToScreen(rect);

            point = rect.TopLeft();
            point.Offset(5, 5);
        }

        if (m_wndBarOutlook.GetActivePageID() != m_nodePage)
        {
            return;
        }

        m_selectedNode = RetrieveHitItem();

        CMenu menu;
        VERIFY(menu.LoadMenu(ID_NODELIST_POPUP));

        CMenu* pPopup = menu.GetSubMenu(0);


        if (m_selectedNode == -1)
        {
            if (m_strActiveProject == "" || m_strActiveProject == "<No Project Selected>")
                return;
            pPopup->EnableMenuItem( ID_POPUP_EDIT , MF_BYCOMMAND | MF_GRAYED);
            pPopup->EnableMenuItem( ID_POPUP_DELETE , MF_BYCOMMAND | MF_GRAYED);
        }


        ASSERT(pPopup != NULL);

        pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,
                               this);
    }
}

void CECXFloorDlg::OnNodeDelete()
{
    CString strItem = m_nodeList.GetItemText(m_selectedNode,0);

    int retVal = MessageBox( "Confirm delete?", "Deleting...", MB_ICONQUESTION | MB_YESNO);
	if (retVal != IDYES)
		return;

    m_pServer->DeleteFile(CYRServer::enNodesFolder,"",strItem);

    FillNodeList();
}

void CECXFloorDlg::OnNodeEdit()
{
    CString strItem = m_nodeList.GetItemText(m_selectedNode,0);
    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT, (WPARAM)WMU_EDIT_NODE, (LPARAM)&strItem);
    FillNodeList();
}

void CECXFloorDlg::OnNodeNew()
{
    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT ,(LPARAM)STUDIO_CUSTOM_LINK_3);
    FillNodeList();
}

void CECXFloorDlg::EnableMenus(LPCTSTR strPage, BOOL bEnabled)
{
    CString page(strPage);

    if (page == "PROJECT_TREE")
    {
        m_ProjectView.XEnableMenus(bEnabled);
    }

    if (page == "OUTPUT_TREE")
    {
        m_OutputView.XEnableMenus(bEnabled);
    }

    if (page == "SOURCE_TREE")
    {
        m_SourceView.XEnableMenus(bEnabled);
    }

    if (page == "TEST_TREE")
    {
        m_TestView.XEnableMenus(bEnabled);
    }
    if (page == "NODE_LIST")
    {
        m_bShowNodeMenus = bEnabled;
    }
}


void CECXFloorDlg::OnPaint()
{
    CPaintDC dc(this); // device context for painting


    // Do not call CDialog::OnPaint() for painting messages
}


BOOL CECXFloorDlg::OnEraseBkgnd(CDC *pDC)
{
    return true;
}


void CECXFloorDlg::OnXViewHttpStatsXreposOutput(LPCTSTR TestName, LPCTSTR TestRunName)
{
    m_OpenRun_Test = TestName;
    m_OpenRun_RunName = TestRunName;
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_HTTP_STATS,0,0);
}
