/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XProjectViewerCtl.cpp : Implementation of the CXProjectViewerCtrl ActiveX Control class.

#include "stdafx.h"
#include "XProjectViewer.h"
#include "XProjectViewerCtl.h"
#include "XProjectViewerPpg.h"
#include "Server.h"
#include "..\..\studio\interfaces\cyrplugins.h"

#include "..\..\Studio\Common\CECXServerError.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CXProjectViewerCtrl, COleControl)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CXProjectViewerCtrl, COleControl)
//{{AFX_MSG_MAP(CXProjectViewerCtrl)
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_ERASEBKGND()
ON_WM_PAINT()
//}}AFX_MSG_MAP
ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
ON_MESSAGE (WMU_PROJECT_CHANGED, OnProjectChanged)
ON_MESSAGE (WMU_EXECUTE_TEST, OnExecuteTest)
ON_MESSAGE (WMU_OPEN_TESTRUN, OnOpenTestRun)
ON_MESSAGE (WMU_EDIT_FILE, OnEditFile)
ON_MESSAGE (WMU_XVIEW_RESULTS, OnViewResults)
ON_MESSAGE (WMU_NEW_TEST, OnNewTest)
ON_MESSAGE (WMU_NEW_SCRIPT, OnNewScript)
ON_MESSAGE (WMU_EXPORT_FILE, OnExportFile)
ON_MESSAGE (WMU_OPEN_FILE, OnOpenFile)
ON_MESSAGE (WMU_HTTP_STATS, OnViewHttpStats)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CXProjectViewerCtrl, COleControl)
//{{AFX_DISPATCH_MAP(CXProjectViewerCtrl)
DISP_PROPERTY_EX(CXProjectViewerCtrl, "Server", GetServer, SetServer, VT_I4)
DISP_FUNCTION(CXProjectViewerCtrl, "SetActiveProject", SetActiveProject, VT_EMPTY, VTS_BSTR)
DISP_FUNCTION(CXProjectViewerCtrl, "XInitialize", XInitialize, VT_EMPTY, VTS_BSTR VTS_BSTR)
DISP_FUNCTION(CXProjectViewerCtrl, "XRefresh", XRefresh, VT_EMPTY, VTS_NONE)
DISP_FUNCTION(CXProjectViewerCtrl, "XEnableMenus", XEnableMenus, VT_EMPTY, VTS_BSTR VTS_BOOL)
//}}AFX_DISPATCH_MAP
DISP_FUNCTION_ID(CXProjectViewerCtrl, "AboutBox", DISPID_ABOUTBOX, AboutBox, VT_EMPTY, VTS_NONE)
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CXProjectViewerCtrl, COleControl)
//{{AFX_EVENT_MAP(CXProjectViewerCtrl)
EVENT_CUSTOM("XChangeActiveProject", FireXChangeActiveProject, VTS_BSTR)
EVENT_CUSTOM("XExecuteTest", FireXExecuteTest, VTS_BSTR)
EVENT_CUSTOM("XOpenTestRun", FireXOpenTestRun, VTS_BSTR  VTS_BSTR)
EVENT_CUSTOM("XEditFile", FireXEditFile, VTS_BSTR)
EVENT_CUSTOM("XViewResults", FireXViewResults, VTS_BSTR  VTS_BSTR  VTS_I2)
EVENT_CUSTOM("XRequestNewTest", FireXRequestNewTest, VTS_NONE)
EVENT_CUSTOM("XRequestNewScript", FireXRequestNewScript, VTS_NONE)
EVENT_CUSTOM("XExportFile", FireXExportFile, VTS_BSTR)
EVENT_CUSTOM("XOpenFile", FireXOpenFile, VTS_BSTR)
EVENT_CUSTOM("XViewHttpStats", FireXViewHttpStats, VTS_BSTR  VTS_BSTR)
//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CXProjectViewerCtrl, 1)
PROPPAGEID(CXProjectViewerPropPage::guid)
END_PROPPAGEIDS(CXProjectViewerCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CXProjectViewerCtrl, "XPROJECTVIEWER.XProjectViewerCtrl.1",
                       0x8e709f66, 0xf8a3, 0x11d2, 0xbd, 0xc5, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CXProjectViewerCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DXProjectViewer =
    {
        0x8e709f64, 0xf8a3, 0x11d2, { 0xbd, 0xc5, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14 }
    };
const IID BASED_CODE IID_DXProjectViewerEvents =
    {
        0x8e709f65, 0xf8a3, 0x11d2, { 0xbd, 0xc5, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14 }
    };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwXProjectViewerOleMisc =
    OLEMISC_ACTIVATEWHENVISIBLE |
    OLEMISC_SETCLIENTSITEFIRST |
    OLEMISC_INSIDEOUT |
    OLEMISC_CANTLINKINSIDE |
    OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CXProjectViewerCtrl, IDS_XPROJECTVIEWER, _dwXProjectViewerOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl::CXProjectViewerCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CXProjectViewerCtrl

BOOL CXProjectViewerCtrl::CXProjectViewerCtrlFactory::UpdateRegistry(BOOL bRegister)
{
    // TODO: Verify that your control follows apartment-model threading rules.
    // Refer to MFC TechNote 64 for more information.
    // If your control does not conform to the apartment-model rules, then
    // you must modify the code below, changing the 6th parameter from
    // afxRegApartmentThreading to 0.

    if (bRegister)
        return AfxOleRegisterControlClass(
                   AfxGetInstanceHandle(),
                   m_clsid,
                   m_lpszProgID,
                   IDS_XPROJECTVIEWER,
                   IDB_XPROJECTVIEWER,
                   afxRegApartmentThreading,
                   _dwXProjectViewerOleMisc,
                   _tlid,
                   _wVerMajor,
                   _wVerMinor);
    else
        return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl::CXProjectViewerCtrl - Constructor

CXProjectViewerCtrl::CXProjectViewerCtrl()
{
    InitializeIIDs(&IID_DXProjectViewer, &IID_DXProjectViewerEvents);

}


/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl::~CXProjectViewerCtrl - Destructor

CXProjectViewerCtrl::~CXProjectViewerCtrl()
{
    if (m_pServer != NULL)
    {
        //  delete m_pServer;
        m_pServer = NULL;
    }
}


/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl::OnDraw - Drawing function

void CXProjectViewerCtrl::OnDraw(
    CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
    if(!m_XFloorDlg.GetSafeHwnd())
    {
        pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(GRAY_BRUSH)));

        CString text = "XProjectViewer Control";
        pdc->TextOut(20,20,text);
    }
}


/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl::DoPropExchange - Persistence support

void CXProjectViewerCtrl::DoPropExchange(CPropExchange* pPX)
{
    ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
    COleControl::DoPropExchange(pPX);

    // TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl::OnResetState - Reset control to default state


void CXProjectViewerCtrl::OnResetState()
{
    COleControl::OnResetState();  // Resets defaults found in DoPropExchange

    // TODO: Reset any other control state here.
}


/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl::AboutBox - Display an "About" box to the user


void CXProjectViewerCtrl::AboutBox()
{
    CDialog dlgAbout(IDD_ABOUTBOX_XPROJECTVIEWER);
    dlgAbout.DoModal();
}


/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl message handlers

int CXProjectViewerCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (COleControl::OnCreate(lpCreateStruct) == -1)
        return -1;

    AfxEnableControlContainer();

    return 0;
}

void CXProjectViewerCtrl::OnSize(UINT nType, int cx, int cy)
{
    COleControl::OnSize(nType, cx, cy);

    if (m_XFloorDlg.m_hWnd != NULL)
    {
        //      m_XFloorDlg.SetWindowPos(NULL,0,25,cx,cy-25,SWP_NOZORDER);
        m_XFloorDlg.SetWindowPos(NULL,0,0,cx,cy,SWP_NOZORDER);
        m_XFloorDlg.Invalidate();
    }

    /*
        if (m_Selector.m_hWnd != NULL)
        {
            CComboBox* pCombo = (CComboBox*) m_Selector.GetDlgItem(IDC_PROJ_COMBO);
     
            CRect r;
            pCombo->GetClientRect(r);
            pCombo->SetWindowPos( NULL, 60, r.top, cx-60, r.bottom, SWP_NOZORDER);
     
            m_Selector.SetWindowPos(NULL,0,0,cx,25,SWP_NOZORDER);
            m_Selector.Invalidate();
        }   
    */
}

LRESULT CXProjectViewerCtrl::OnProjectChanged(WPARAM,LPARAM)
{
    //  CString strProjName = m_Selector.GetProjectName();
    //  m_XFloorDlg.ChangeProject(strProjName);
    //  FireXChangeActiveProject(strProjName);
    return 0;
}

LRESULT CXProjectViewerCtrl::OnExecuteTest(WPARAM,LPARAM)
{
    CString strTestName = m_XFloorDlg.GetTestName();
    FireXExecuteTest(strTestName);
    return 0;
}

void CXProjectViewerCtrl::SetActiveProject(LPCTSTR ProjectName)
{
    //  m_Selector.SetProjectName(ProjectName);
    m_XFloorDlg.ChangeProject(ProjectName);
}

LRESULT CXProjectViewerCtrl::OnOpenTestRun(WPARAM,LPARAM)
{
    CString TestName;
    CString RunName;

    m_XFloorDlg.GetOpenTestRun(TestName, RunName);

    FireXOpenTestRun(TestName, RunName);
    return 0;
}

LRESULT CXProjectViewerCtrl::OnViewHttpStats(WPARAM,LPARAM)
{
    CString TestName;
    CString RunName;

    m_XFloorDlg.GetOpenTestRun(TestName, RunName);

    FireXViewHttpStats(TestName, RunName);
    return 0;
}

LRESULT CXProjectViewerCtrl::OnEditFile(WPARAM,LPARAM)
{
    CString strScriptName = m_XFloorDlg.GetScriptName();

    FireXEditFile(strScriptName);
    return 0;
}

LRESULT CXProjectViewerCtrl::OnOpenFile(WPARAM,LPARAM)
{
    CString strFileName = m_XFloorDlg.GetScriptName();
    FireXOpenFile(strFileName);
    return 0;
}

void CXProjectViewerCtrl::XInitialize(LPCTSTR strProjType, LPCTSTR strConfig)
{
    /*
        m_Selector.SetServer(m_pServer);
        m_Selector.SetProjectType(strProjType);
        m_Selector.Create(IDD_PROJ_SELECT,this);
        m_Selector.ShowWindow(SW_SHOW);
        m_Selector.Invalidate();
    */
    m_XFloorDlg.SetConfig(strConfig);
    m_XFloorDlg.SetServer(m_pServer);
    m_XFloorDlg.Create(IDD_XFLOORDLG,this);
    m_XFloorDlg.ShowWindow(SW_SHOW);
    m_XFloorDlg.Invalidate();

    Invalidate();

}


LRESULT CXProjectViewerCtrl::OnViewResults(WPARAM type, LPARAM)
{
    CString TestName;
    CString RunName;

    m_XFloorDlg.GetOpenTestRun(TestName, RunName);

    FireXViewResults(TestName, RunName, type);
    return 0;
}

LRESULT CXProjectViewerCtrl::OnNewTest(WPARAM,LPARAM)
{
    FireXRequestNewTest();
    return 0;
}

LRESULT CXProjectViewerCtrl::OnNewScript(WPARAM,LPARAM)
{
    FireXRequestNewScript();
    return 0;
}
LRESULT CXProjectViewerCtrl::OnExportFile(WPARAM ID,LPARAM ptrFileName)
{
    FireXExportFile((LPCTSTR)ptrFileName);
    return 0;
}

void CXProjectViewerCtrl::XRefresh()
{
    m_XFloorDlg.Refresh();
}

void CXProjectViewerCtrl::XEnableMenus(LPCTSTR strPage, BOOL bEnabled)
{
    m_XFloorDlg.EnableMenus(strPage, bEnabled);
}

BOOL CXProjectViewerCtrl::OnEraseBkgnd(CDC* pDC)
{
    return true;
    return COleControl::OnEraseBkgnd(pDC);
}


void CXProjectViewerCtrl::OnPaint()
{
    CPaintDC dc(this); // device context for painting

    // TODO: Add your message handler code here

    // Do not call COleControl::OnPaint() for painting messages
}


long CXProjectViewerCtrl::GetServer()
{
    return (long)m_pServer;
}

void CXProjectViewerCtrl::SetServer(long nNewValue)
{
    m_pServer = (CYRServer*)nNewValue;

    SetModifiedFlag();
}
