/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XReposTreeCtl.cpp : Implementation of the CXReposTreeCtrl ActiveX Control class.

#include "stdafx.h"

#include "XReposTree.h"
#include "XReposTreeCtl.h"
#include "XReposTreePpg.h"
#include "StringEx.h"
#include "..\..\Studio\Common\CECXServerError.h"
#include "..\..\Studio\Common\CECClipboard.h"
#include "..\..\Studio\Interfaces\CyrPlugins.h"
#include "Solutions.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CXReposTreeCtrl, COleControl)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CXReposTreeCtrl, COleControl)
//{{AFX_MSG_MAP(CXReposTreeCtrl)
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_ERASEBKGND()
//}}AFX_MSG_MAP
ON_OLEVERB(AFX_IDS_VERB_EDIT, OnEdit)
ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
ON_MESSAGE (WMU_CHANGE_PROJECT, OnChangeProject)
ON_MESSAGE (WMU_INIT_TEST, OnInitiateTest)
ON_MESSAGE (WMU_EDIT_FILE, OnEditFile)
ON_MESSAGE (WMU_OPEN_FILE, OnOpenFile)
ON_MESSAGE (WMU_OPEN_TESTRUN, OnOpenTestRun)
ON_MESSAGE (WMU_XVIEW_RESULTS, OnViewResults)
ON_MESSAGE (WMU_CHECK_IN, OnCheckInFile)
ON_MESSAGE (WMU_CHECK_OUT, OnCheckOutFile)
ON_MESSAGE (WMU_DELETE_FILE, OnDeleteFile)
ON_MESSAGE (WMU_ADD_NODE, OnAddNode)
ON_MESSAGE (WMU_ADD_TEST, OnAddTest)
ON_MESSAGE (WMU_ADD_SCRIPT, OnAddScript)
ON_MESSAGE (WMU_EXPORT_FILE, OnExportFile)
ON_MESSAGE (WMU_XEDIT_NODE, OnEditNode)
ON_MESSAGE (WMU_DELETE_TEST, OnDeleteTest)
ON_MESSAGE (WMU_DELETE_TESTRUN, OnDeleteTestRun)
ON_MESSAGE (WMU_DEL_RESULTS, OnDeleteAllResults)
ON_MESSAGE (WMU_DELETE_NODE, OnDeleteNode)
ON_MESSAGE (WMU_HTTP_STATS, OnViewHttpStats)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CXReposTreeCtrl, COleControl)
//{{AFX_DISPATCH_MAP(CXReposTreeCtrl)
DISP_PROPERTY_EX(CXReposTreeCtrl, "Server", GetServer, SetServer, VT_I4)
DISP_FUNCTION(CXReposTreeCtrl, "XInitialize", XInitialize, VT_EMPTY, VTS_BSTR)
DISP_FUNCTION(CXReposTreeCtrl, "XActivate", XActivate, VT_EMPTY, VTS_NONE)
DISP_FUNCTION(CXReposTreeCtrl, "XRefresh", XRefresh, VT_EMPTY, VTS_NONE)
DISP_FUNCTION(CXReposTreeCtrl, "XEnableMenus", XEnableMenus, VT_EMPTY, VTS_BOOL)
//}}AFX_DISPATCH_MAP
DISP_FUNCTION_ID(CXReposTreeCtrl, "AboutBox", DISPID_ABOUTBOX, AboutBox, VT_EMPTY, VTS_NONE)
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CXReposTreeCtrl, COleControl)
//{{AFX_EVENT_MAP(CXReposTreeCtrl)
EVENT_CUSTOM("XChangeActiveProject", FireXChangeActiveProject, VTS_BSTR)
EVENT_CUSTOM("XInitiateTest", FireXInitiateTest, VTS_BSTR)
EVENT_CUSTOM("XOpenTestRun", FireXOpenTestRun, VTS_BSTR  VTS_BSTR)
EVENT_CUSTOM("XEditFile", FireXEditFile, VTS_BSTR)
EVENT_CUSTOM("XViewResults", FireXViewResults, VTS_BSTR  VTS_BSTR  VTS_I2)
EVENT_CUSTOM("XAddTest", FireXAddTest, VTS_NONE)
EVENT_CUSTOM("XAddScript", FireXAddScript, VTS_NONE)
EVENT_CUSTOM("XExportFile", FireXExportFile, VTS_BSTR)
EVENT_CUSTOM("XOpenFile", FireXOpenFile, VTS_BSTR)
EVENT_CUSTOM("XEditTof", FireXEditTof, VTS_I4  VTS_BSTR)
EVENT_CUSTOM("XViewHttpStats", FireXViewHttpStats, VTS_BSTR  VTS_BSTR)
//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CXReposTreeCtrl, 1)
PROPPAGEID(CXReposTreePropPage::guid)
END_PROPPAGEIDS(CXReposTreeCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CXReposTreeCtrl, "XREPOSTREE.XReposTreeCtrl.1",
                       0x6bcda16, 0xde1d, 0x11d2, 0xbc, 0xfc, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CXReposTreeCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DXReposTree =
    {
        0x6bcda14, 0xde1d, 0x11d2, { 0xbc, 0xfc, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14 }
    };
const IID BASED_CODE IID_DXReposTreeEvents =
    {
        0x6bcda15, 0xde1d, 0x11d2, { 0xbc, 0xfc, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14 }
    };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwXReposTreeOleMisc =
    OLEMISC_ACTIVATEWHENVISIBLE |
    OLEMISC_SETCLIENTSITEFIRST |
    OLEMISC_INSIDEOUT |
    OLEMISC_CANTLINKINSIDE |
    OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CXReposTreeCtrl, IDS_XREPOSTREE, _dwXReposTreeOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl::CXReposTreeCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CXReposTreeCtrl

BOOL CXReposTreeCtrl::CXReposTreeCtrlFactory::UpdateRegistry(BOOL bRegister)
{
    // TODO: Verify that your control follows apartment-model threading rules.
    // Refer to MFC TechNote 64 for more information.
    // If your control does not conform to the apartment-model rules, then
    // you must modify the code below, changing the 6th parameter from
    // afxRegInsertable | afxRegApartmentThreading to afxRegInsertable.

    if (bRegister)
        return AfxOleRegisterControlClass(
                   AfxGetInstanceHandle(),
                   m_clsid,
                   m_lpszProgID,
                   IDS_XREPOSTREE,
                   IDB_XREPOSTREE,
                   afxRegInsertable | afxRegApartmentThreading,
                   _dwXReposTreeOleMisc,
                   _tlid,
                   _wVerMajor,
                   _wVerMinor);
    else
        return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl::CXReposTreeCtrl - Constructor

CXReposTreeCtrl::CXReposTreeCtrl()
{
    InitializeIIDs(&IID_DXReposTree, &IID_DXReposTreeEvents);

    m_pServer = NULL;
    m_XReposTree=NULL;
    m_XProjectTree=NULL;
    m_XOutputTree=NULL;
    m_XSourceTree=NULL;
    m_XTestTree=NULL;

    m_bShowMenus = TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl::~CXReposTreeCtrl - Destructor

CXReposTreeCtrl::~CXReposTreeCtrl()
{
    if(m_pServer!=NULL)
    {
        //      m_pBorrowThread->Dispose(m_pServer);
        //      delete m_pServer;
        m_pServer = NULL;
        //      delete m_pBorrowThread;
        //      m_pBorrowThread = NULL;
    }

    if(m_XReposTree!=NULL)
        delete m_XReposTree;
    m_XReposTree=NULL;

    if(m_XProjectTree!=NULL)
        delete m_XProjectTree;
    m_XProjectTree=NULL;

    if(m_XOutputTree!=NULL)
        delete m_XOutputTree;
    m_XOutputTree=NULL;

    if(m_XSourceTree!=NULL)
        delete m_XSourceTree;
    m_XSourceTree=NULL;

    if(m_XTestTree!=NULL)
        delete m_XTestTree;
    m_XTestTree=NULL;
}


/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl::OnDraw - Drawing function

void CXReposTreeCtrl::OnDraw(
    CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
    if(m_pServer == NULL)
    {
        // TODO: Replace the following code with your own drawing code.
        pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(WHITE_BRUSH)));
        pdc->Ellipse(rcBounds);
    }
}


/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl::DoPropExchange - Persistence support

void CXReposTreeCtrl::DoPropExchange(CPropExchange* pPX)
{
    ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
    COleControl::DoPropExchange(pPX);

    // TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl::OnResetState - Reset control to default state


void CXReposTreeCtrl::OnResetState()
{
    COleControl::OnResetState();  // Resets defaults found in DoPropExchange

    // TODO: Reset any other control state here.
}


/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl::AboutBox - Display an "About" box to the user


void CXReposTreeCtrl::AboutBox()
{
    CDialog dlgAbout(IDD_ABOUTBOX_XREPOSTREE);
    dlgAbout.DoModal();
}


/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl message handlers

int CXReposTreeCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (COleControl::OnCreate(lpCreateStruct) == -1)
        return -1;
    AfxEnableControlContainer(NULL);

    //////////////////////////////////////////////////////


    //////////////////////////////////////////////////////

    return 0;
}

void CXReposTreeCtrl::OnSize(UINT nType, int cx, int cy)
{
    COleControl::OnSize(nType, cx, cy);

    if( m_XReposTree->GetSafeHwnd() != NULL )
    {
        m_XReposTree->SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
    else if( m_XProjectTree->GetSafeHwnd() != NULL )
    {
        m_XProjectTree->SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
    else if( m_XOutputTree->GetSafeHwnd() != NULL )
    {
        m_XOutputTree->SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
    else if( m_XSourceTree->GetSafeHwnd() != NULL )
    {
        m_XSourceTree->SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
    else if( m_XTestTree->GetSafeHwnd() != NULL )
    {
        m_XTestTree->SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
}

void CXReposTreeCtrl::XInitialize(LPCTSTR strConfig)
{

    BeginWaitCursor();

    BOOL bUseSCC = UseSourceControl();
    CStringEx config = strConfig;

    if (config.GetField('|',0) == "REPOS_TREE")
    {
        m_nTreeType = 0;
        DeleteCurrentWindow();
        m_XReposTree = new CXReposTree;
        m_XReposTree->SetServer(m_pServer);
        m_XReposTree->Create(IDD_REPOS_TREE,this);
        m_XReposTree->ShowWindow(SW_SHOW);
    }
    else if (config.GetField('|',0) == "TEST_TREE")
    {
        m_nTreeType = 4;
        DeleteCurrentWindow();
        m_XTestTree = new CXTestTree(bUseSCC);
        m_XTestTree->SetServer(m_pServer);
        m_XTestTree->SetActiveProject(config.GetField('|',1));
        m_XTestTree->Create(IDD_TEST_TREE,this);
        m_XTestTree->ShowWindow(SW_SHOW);
        m_XTestTree->Activate();
    }
    else if (config.GetField('|',0) == "PROJECT_TREE")
    {
        m_nTreeType = 1;
        DeleteCurrentWindow();

        m_XProjectTree = new CXProjectTree(bUseSCC);
        m_XProjectTree->SetServer(m_pServer);
        m_XProjectTree->Create(IDD_PROJECT_TREE,this);
        m_XProjectTree->SetActiveProject(config.GetField('|',1));
        m_XProjectTree->EnableMenus(m_bShowMenus);
        m_XProjectTree->ShowWindow(SW_SHOW);
        m_XProjectTree->Activate();
    }
    else if (config.GetField('|',0) == "OUTPUT_TREE")
    {
        m_nTreeType = 2;
        DeleteCurrentWindow();
        m_XOutputTree = new CXOutputTree;
        m_XOutputTree->SetServer(m_pServer);
        m_XOutputTree->Create(IDD_OUTPUT_TREE,this);
        m_XOutputTree->SetActiveProject(config.GetField('|',1));
        m_XOutputTree->EnableMenus(m_bShowMenus);
        m_XOutputTree->ShowWindow(SW_SHOW);
        m_XOutputTree->Activate();
    }
    if (config.GetField('|',0) == "SOURCE_TREE")
    {
        m_nTreeType = 3;
        DeleteCurrentWindow();
        m_XSourceTree = new CXSourceTree(bUseSCC);
        m_XSourceTree->SetServer(m_pServer);
        m_XSourceTree->Create(IDD_SOURCE_TREE,this);
        m_XSourceTree->SetActiveProject(config.GetField('|',1));
        m_XSourceTree->ShowWindow(SW_SHOW);
        m_XSourceTree->Activate();
    }


    RECT r;
    GetClientRect(&r);
    int cx = r.right - r.left;
    int cy = r.bottom - r.top;
    OnSize(0,cx,cy);

    EndWaitCursor();
}

void CXReposTreeCtrl::XActivate()
{
    switch (m_nTreeType)
    {
    case 0:
        {
            m_XReposTree->Activate();
            break;
        }
    case 1:
        {
            m_XProjectTree->Activate();
            break;
        }
    case 2:
        {
            m_XOutputTree->Activate();
            break;
        }
    case 3:
        {
            m_XSourceTree->Activate();
            break;
        }
    case 4:
        {
            m_XTestTree->Activate();
            break;
        }
    }
}
LRESULT CXReposTreeCtrl::OnChangeProject(WPARAM ID,LPARAM ptrProjectName)
{
    LPCSTR string = (LPCSTR) ptrProjectName;

    CString projName=string;
    if( !projName.IsEmpty())
    {
        FireXChangeActiveProject(projName);
    }
    return 0;
}

void CXReposTreeCtrl::DeleteCurrentWindow()
{
    if (m_XReposTree != NULL)
    {
        delete m_XReposTree;
        m_XReposTree=NULL;
    }
    if (m_XProjectTree != NULL)
    {
        delete m_XProjectTree;
        m_XProjectTree=NULL;
    }
    if (m_XOutputTree != NULL)
    {
        delete m_XOutputTree;
        m_XOutputTree=NULL;
    }
    if (m_XSourceTree != NULL)
    {
        delete m_XSourceTree;
        m_XSourceTree=NULL;
    }
    if (m_XTestTree != NULL)
    {
        delete m_XTestTree;
        m_XTestTree=NULL;
    }
}

LRESULT CXReposTreeCtrl::OnInitiateTest(WPARAM ID,LPARAM ptrTestName)
{
    LPCSTR string = (LPCSTR) ptrTestName;

    CString testName=string;
    if( !testName.IsEmpty())
    {
        FireXInitiateTest(testName);
    }
    return 0;
}

LRESULT CXReposTreeCtrl::OnEditFile(WPARAM ID,LPARAM ptrFileName)
{
    LPCSTR string = (LPCSTR) ptrFileName;
    CString szFileName=string;
    if( !szFileName.IsEmpty())
    {
        FireXEditFile(szFileName);
    }
    return 0;
}

LRESULT CXReposTreeCtrl::OnOpenFile(WPARAM ID,LPARAM ptrFileName)
{
    LPCSTR string = (LPCSTR) ptrFileName;

    CString szFileName=string;
    if( !szFileName.IsEmpty())
    {
        FireXOpenFile(szFileName);
    }
    return 0;
}

LRESULT CXReposTreeCtrl::OnOpenTestRun(WPARAM,LPARAM)
{
    CString TestName;
    CString RunName;

    m_XOutputTree->GetOpenTestRun(TestName, RunName);

    FireXOpenTestRun(TestName, RunName);
    return 0;
}

LRESULT CXReposTreeCtrl::OnViewResults(WPARAM type, LPARAM )
{
    CString TestName;
    CString RunName;

    m_XOutputTree->GetOpenTestRun(TestName, RunName);

    FireXViewResults(TestName, RunName, type);
    return 0;
}


LRESULT  CXReposTreeCtrl::OnCheckInFile(WPARAM FolderID,LPARAM ptrFileName)
{
    LPCSTR folderString = (LPCSTR) FolderID;
    CString szFolderName=folderString;

    LPCSTR string = (LPCSTR) ptrFileName;
    CString szFileName=string;

    CWnd* pMainWnd = AfxGetMainWnd();
    CString checkInInfo;
    checkInInfo.Format("%s|%s|%s",SOLUTIONS_SC_DLL,szFileName, FolderID);
    BOOL retVal = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                         ,(WPARAM )WMU_SC_CHECKIN
                                         ,(LPARAM) &checkInInfo);

    return (LRESULT)retVal;
}
LRESULT  CXReposTreeCtrl::OnCheckOutFile(WPARAM FolderID,LPARAM ptrFileName)
{
    LPCSTR folderString = (LPCSTR) FolderID;
    CString szFolderName=folderString;

    LPCSTR string = (LPCSTR) ptrFileName;
    CString szFileName=string;

    CWnd* pMainWnd = AfxGetMainWnd();
    CString checkOutInfo;
    checkOutInfo.Format("%s|%s|%s",SOLUTIONS_SC_DLL,szFileName, FolderID);
    BOOL retVal = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                         ,(WPARAM )WMU_SC_CHECKOUT
                                         ,(LPARAM) &checkOutInfo);

    return (LRESULT)retVal;

}
LRESULT  CXReposTreeCtrl::OnDeleteFile(WPARAM FolderID,LPARAM ptrFileName)
{
    LPCSTR folderString = (LPCSTR) FolderID;
    CString szFolderName=folderString;

    LPCSTR string = (LPCSTR) ptrFileName;
    CString szFileName=string;

    int retVal = MessageBox( "Confirm delete?", "Deleting...", MB_ICONQUESTION | MB_YESNO);
	if (retVal != IDYES)
		return 0;

    m_pServer->DeleteFile(CYRServer::enSourceFolder,"",szFileName);

    return 0;
}

LRESULT  CXReposTreeCtrl::OnDeleteTest(WPARAM FolderID,LPARAM ptrFolderName)
{
    LPCSTR folderString = (LPCSTR) FolderID;
    CString szFolderName=folderString;

    LPCSTR string = (LPCSTR) ptrFolderName;
    CString szTestFolder=string;

    // Confirm delete
    CString msg;
    msg.Format("Confirm deletion of test %s?\n Warning: This will delete all script\nfiles and test results.\n\nAny group tests that this test is part of will no longer run.", szTestFolder);
    if(MessageBox(msg, "OpenSTA", MB_YESNO) == IDNO)
        return 0;

    m_pServer->DeleteTest(szTestFolder);

    CString szDll(SOLUTIONS_GENERIC_DLL);
    AfxGetMainWnd()->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_REFRESH_TREE_VIEWS,(LPARAM)&szDll);

    return 0;
}

LRESULT  CXReposTreeCtrl::OnAddNode(WPARAM,LPARAM)
{
    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT ,(LPARAM)STUDIO_CUSTOM_LINK_3);
    XRefresh();
    return 0;
}
LRESULT  CXReposTreeCtrl::OnEditNode(WPARAM ID,LPARAM ptrFileName)
{
    CString node = (LPCTSTR) ptrFileName;
    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT, (WPARAM)WMU_EDIT_NODE, (LPARAM)&node);
    return 0;
}

LRESULT  CXReposTreeCtrl::OnAddTest(WPARAM,LPARAM)
{
    FireXAddTest();
    return 0;
}

LRESULT  CXReposTreeCtrl::OnAddScript(WPARAM,LPARAM)
{
    FireXAddScript();
    return 0;
}

LRESULT CXReposTreeCtrl::OnExportFile(WPARAM ID,LPARAM ptrFileName)
{
    FireXExportFile((LPCTSTR) ptrFileName);
    return 0;
}

void CXReposTreeCtrl::XRefresh()
{
    switch (m_nTreeType)
    {
    case 0:
        {
            m_XReposTree->RefreshTree();
            break;
        }
    case 1:
        {
            m_XProjectTree->RefreshTree();
            break;
        }
    case 2:
        {
            m_XOutputTree->RefreshTree();
            break;
        }
    case 3:
        {
            m_XSourceTree->RefreshTree();
            break;
        }
    case 4:
        {
            m_XTestTree->RefreshTree();
            break;
        }
    }
}

BOOL CXReposTreeCtrl::UseSourceControl()
{
    CWnd* pMainWnd = AfxGetMainWnd();

    if (pMainWnd == NULL)
    {
        return FALSE;
    }

    CString SourceDLL = SOLUTIONS_SC_DLL;
    BOOL retVal = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                         ,(WPARAM )WMU_SC_ENABLED
                                         ,(LPARAM) &SourceDLL);

    return retVal;
}

LRESULT CXReposTreeCtrl::OnDeleteTestRun(WPARAM ptrTestName,LPARAM ptrRunName)
{
    CString strTest = (LPCTSTR) ptrTestName;
    CString strRun = (LPCTSTR) ptrRunName;

    m_pServer->DeleteFolder(CYRServer::enTestsFolder,strTest + "\\" + strRun);

    return 0;
}

LRESULT CXReposTreeCtrl::OnDeleteAllResults(WPARAM ID,LPARAM ptrTestName)
{
    CString strTest = (LPCTSTR) ptrTestName;

    CStringArray arstrRuns;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,strTest,arstrRuns);
    for(int nNdx=0;nNdx<arstrRuns.GetSize();nNdx++)
    {
        m_pServer->DeleteFolder(CYRServer::enTestsFolder,strTest + "\\" + arstrRuns[nNdx]);
    }

    return 0;
}

LRESULT CXReposTreeCtrl::OnDeleteNode(WPARAM FolderID,LPARAM ptrNodeName)
{
    LPCSTR folderString = (LPCSTR) FolderID;
    CString szFolderName=folderString;

    LPCSTR nodeString = (LPCSTR) ptrNodeName;
    CString szNodeName = nodeString;

    m_pServer->DeleteFile(CYRServer::enNodesFolder,"",szNodeName);
    return 0;
}

void CXReposTreeCtrl::XEnableMenus(BOOL bEnabaled)
{
    m_bShowMenus = bEnabaled;
    switch (m_nTreeType)
    {
    case 0:
        {
            m_XReposTree->EnableMenus(bEnabaled);
            break;
        }
    case 1:
        {
            m_XProjectTree->EnableMenus(bEnabaled);
            break;
        }
    case 2:
        {
            m_XOutputTree->EnableMenus(bEnabaled);
            break;
        }
    case 3:
        {
            m_XSourceTree->EnableMenus(bEnabaled);
            break;
        }
    case 4:
        {
            m_XTestTree->EnableMenus(bEnabaled);
            break;
        }
    }
}

BOOL CXReposTreeCtrl::OnEraseBkgnd(CDC* pDC)
{
    return true;
    return COleControl::OnEraseBkgnd(pDC);
}


LRESULT CXReposTreeCtrl::OnViewHttpStats(WPARAM,LPARAM)
{
    CString TestName;
    CString RunName;

    m_XOutputTree->GetOpenTestRun(TestName, RunName);

    FireXViewHttpStats(TestName, RunName);
    return 0;
}

long CXReposTreeCtrl::GetServer()
{
    return (long) m_pServer;
}

void CXReposTreeCtrl::SetServer(long nNewValue)
{
    m_pServer = (CYRServer*)nNewValue;

    SetModifiedFlag();
}
