/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CXReposTreeCtrl.h
// -----------------
//
//    This is the include file for the CXReposTreeCtrl class
//
//=========================================================================



#if !defined(AFX_XREPOSTREECTL_H__06BCDA24_DE1D_11D2_BCFC_00608CB45114__INCLUDED_)
#define AFX_XREPOSTREECTL_H__06BCDA24_DE1D_11D2_BCFC_00608CB45114__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// XReposTreeCtl.h : Declaration of the CXReposTreeCtrl ActiveX Control class.
#include "CXReposTree1.h"
#include "CXProjectTree.h"
#include "CXOutputTree.h"
#include "CXSourceTree.h"
#include "XTestTree.h"

class CYRServer;
/////////////////////////////////////////////////////////////////////////////
// CXReposTreeCtrl : See XReposTreeCtl.cpp for implementation.

class CXReposTreeCtrl : public COleControl
{
    DECLARE_DYNCREATE(CXReposTreeCtrl)

    // Constructor
public:
    CXReposTreeCtrl();
    CXReposTree* m_XReposTree;
    CXProjectTree* m_XProjectTree;
    CXOutputTree* m_XOutputTree;
    CXSourceTree* m_XSourceTree;
    CXTestTree* m_XTestTree;

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CXReposTreeCtrl)
public:
    virtual void OnDraw(CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid);
    virtual void DoPropExchange(CPropExchange* pPX);
    virtual void OnResetState();
    //}}AFX_VIRTUAL

    // Implementation
protected:
    ~CXReposTreeCtrl();

    DECLARE_OLECREATE_EX(CXReposTreeCtrl)    // Class factory and guid
    DECLARE_OLETYPELIB(CXReposTreeCtrl)      // GetTypeInfo
    DECLARE_PROPPAGEIDS(CXReposTreeCtrl)     // Property page IDs
    DECLARE_OLECTLTYPE(CXReposTreeCtrl)     // Type name and misc status

    // Message maps
    //{{AFX_MSG(CXReposTreeCtrl)
    afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg BOOL OnEraseBkgnd(CDC* pDC);
    //}}AFX_MSG
    afx_msg LRESULT OnChangeProject(WPARAM,LPARAM);
    afx_msg LRESULT OnInitiateTest(WPARAM,LPARAM);
    afx_msg LRESULT OnEditFile(WPARAM,LPARAM);
    afx_msg LRESULT OnOpenFile(WPARAM,LPARAM);
    afx_msg LRESULT OnOpenTestRun(WPARAM,LPARAM);
    afx_msg LRESULT OnContextMenuClick (WPARAM,LPARAM);
    afx_msg LRESULT OnViewResults(WPARAM,LPARAM);
    afx_msg LRESULT OnCheckInFile(WPARAM,LPARAM);
    afx_msg LRESULT OnCheckOutFile(WPARAM,LPARAM);
    afx_msg LRESULT OnDeleteFile(WPARAM,LPARAM);
    afx_msg LRESULT OnAddNode(WPARAM,LPARAM);
    afx_msg LRESULT OnAddTest(WPARAM,LPARAM);
    afx_msg LRESULT OnAddScript(WPARAM,LPARAM);
    afx_msg LRESULT OnExportFile(WPARAM,LPARAM);
    afx_msg LRESULT OnEditNode(WPARAM,LPARAM);
    afx_msg LRESULT  OnDeleteTest(WPARAM,LPARAM);
    afx_msg LRESULT OnDeleteTestRun(WPARAM,LPARAM);
    afx_msg LRESULT OnDeleteAllResults(WPARAM,LPARAM);
    afx_msg LRESULT OnDeleteNode(WPARAM,LPARAM);
    afx_msg LRESULT OnViewHttpStats(WPARAM,LPARAM);
    DECLARE_MESSAGE_MAP()

    // Dispatch maps
    //{{AFX_DISPATCH(CXReposTreeCtrl)
    afx_msg long GetServer();
    afx_msg void SetServer(long nNewValue);
    afx_msg void XInitialize(LPCTSTR strConfig);
    afx_msg void XActivate();
    afx_msg void XRefresh();
    afx_msg void XEnableMenus(BOOL bEnabaled);
    //}}AFX_DISPATCH
    DECLARE_DISPATCH_MAP()

    afx_msg void AboutBox();

    // Event maps
    //{{AFX_EVENT(CXReposTreeCtrl)
    void FireXChangeActiveProject(LPCTSTR strProjName)
    {
        FireEvent(eventidXChangeActiveProject,EVENT_PARAM(VTS_BSTR), strProjName);
    }
    void FireXInitiateTest(LPCTSTR strTestName)
    {
        FireEvent(eventidXInitiateTest,EVENT_PARAM(VTS_BSTR), strTestName);
    }
    void FireXOpenTestRun(LPCTSTR TestName, LPCTSTR TestRunName)
    {
        FireEvent(eventidXOpenTestRun,EVENT_PARAM(VTS_BSTR  VTS_BSTR), TestName, TestRunName);
    }
    void FireXEditFile(LPCTSTR strFileName)
    {
        FireEvent(eventidXEditFile,EVENT_PARAM(VTS_BSTR), strFileName);
    }
    void FireXViewResults(LPCTSTR strTestName, LPCTSTR strTestRunName, short nType)
    {
        FireEvent(eventidXViewResults,EVENT_PARAM(VTS_BSTR  VTS_BSTR  VTS_I2), strTestName, strTestRunName, nType);
    }
    void FireXAddTest()
    {
        FireEvent(eventidXAddTest,EVENT_PARAM(VTS_NONE));
    }
    void FireXAddScript()
    {
        FireEvent(eventidXAddScript,EVENT_PARAM(VTS_NONE));
    }
    void FireXExportFile(LPCTSTR strFileName)
    {
        FireEvent(eventidXExportFile,EVENT_PARAM(VTS_BSTR), strFileName);
    }
    void FireXOpenFile(LPCTSTR strFileName)
    {
        FireEvent(eventidXOpenFile,EVENT_PARAM(VTS_BSTR), strFileName);
    }
    void FireXEditTof(long nNodeType, LPCTSTR TofName)
    {
        FireEvent(eventidXEditTof,EVENT_PARAM(VTS_I4  VTS_BSTR), nNodeType, TofName);
    }
    void FireXViewHttpStats(LPCTSTR TestName, LPCTSTR TestRunName)
    {
        FireEvent(eventidXViewHttpStats,EVENT_PARAM(VTS_BSTR  VTS_BSTR), TestName, TestRunName);
    }
    //}}AFX_EVENT
    DECLARE_EVENT_MAP()

    // Dispatch and event IDs
public:
    BOOL m_bShowMenus;
    enum {
        //{{AFX_DISP_ID(CXReposTreeCtrl)
        dispidServer = 1L,
        dispidXInitialize = 2L,
        dispidXActivate = 3L,
        dispidXRefresh = 4L,
        dispidXEnableMenus = 5L,
        eventidXChangeActiveProject = 1L,
        eventidXInitiateTest = 2L,
        eventidXOpenTestRun = 3L,
        eventidXEditFile = 4L,
        eventidXViewResults = 5L,
        eventidXAddTest = 6L,
        eventidXAddScript = 7L,
        eventidXExportFile = 8L,
        eventidXOpenFile = 9L,
        eventidXEditTof = 10L,
        eventidXViewHttpStats = 11L,
        //}}AFX_DISP_ID
    };
private:
    BOOL UseSourceControl();
    CString m_Password;
    CString m_Username;

    void DeleteCurrentWindow();
    int m_nTreeType;
    CYRServer*  m_pServer;

};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_XREPOSTREECTL_H__06BCDA24_DE1D_11D2_BCFC_00608CB45114__INCLUDED)
