/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CYRTreeCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "CYRTreeCtrl.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//class CECGlobalCmdTarget;
//#include "CECGlobalCmdTarget.h"

/////////////////////////////////////////////////////////////////////////////
// CYRTreeCtrl

CYRTreeCtrl::CYRTreeCtrl():
	m_hLastItemHit(NULL)
{
    //
    // Initialize the image list
    //
    m_imageList.Create(16,
                       16,
                       TRUE,
                       8, // number of initial images
                       8);
    m_bMenus = TRUE;
}

CYRTreeCtrl::~CYRTreeCtrl()
{}



BEGIN_MESSAGE_MAP(CYRTreeCtrl, CTreeCtrl)
//{{AFX_MSG_MAP(CYRTreeCtrl)
ON_WM_LBUTTONDBLCLK()
ON_NOTIFY_REFLECT(TVN_ITEMEXPANDING, OnItemexpanding)
ON_COMMAND(ID_POPUP_PROPERTIES, OnPopupProperties)
ON_WM_LBUTTONDOWN()
ON_WM_KEYDOWN()
ON_WM_CONTEXTMENU()
//}}AFX_MSG_MAP
ON_WM_LBUTTONUP()
ON_WM_MOUSEMOVE()
ON_WM_CREATE()
ON_COMMAND_RANGE(ID_CONTEXT_USER, ID_CONTEXT_USER+10, OnContextClick)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRTreeCtrl message handlers

int CYRTreeCtrl::AddImageToList(CBitmap* pBmp)
{

    ASSERT(m_imageList.m_hImageList);

    int index = m_imageList.Add( pBmp, RGB(255,255,255));

    SetImageList(&m_imageList, TVSIL_NORMAL);

    return index;
}

HTREEITEM CYRTreeCtrl::AddItem(HTREEITEM hti, int NodeType, CString strLabel, int imageIndex,BOOL bHasChildren)
{
    if (hti == NULL)
    {
        hti=TVI_ROOT;
    }

    TV_INSERTSTRUCT tvis;
    tvis.hParent = hti;
    tvis.hInsertAfter = TVI_LAST;
    tvis.item.mask = TVIF_TEXT | TVIF_PARAM | TVIF_CHILDREN | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
    tvis.item.pszText = strLabel.GetBuffer(0);
    CString strItemName = strLabel;
    int nLength = strLabel.GetLength();
    tvis.item.cchTextMax = nLength;
    tvis.item.cChildren = (int)bHasChildren;

    HTREEITEM treeItem = InsertItem(&tvis); // Set the item to use the correct icon...
    SetItemImage(treeItem,imageIndex,imageIndex);

    TreeItem item;
    item.Type=NodeType;
    item.Path=GetItemPath(treeItem);
    item.ImageIndex = imageIndex;

    m_TreeItems.Add(item);

    return treeItem;
}

void CYRTreeCtrl::DefineType(int strType, CString strMenuItems, BOOL bProperties ,BOOL bDraggable)
{
    MenuEntry menu;
    menu.Type = strType;
    menu.Entries = strMenuItems;
    menu.bHasProperties = bProperties;
    menu.bDraggable = bDraggable;
    m_MenuEntries.Add(menu);
}

CString CYRTreeCtrl::GetItemPath(HTREEITEM hti)
{
    CString path="";

    path=path+GetItemText(hti);
    HTREEITEM parent = GetParentItem(hti);

    if (parent == NULL)
    {
        return path;
    }
    CString parentLabel=GetItemText(parent);
    while (!parentLabel.IsEmpty())
    {
        path=parentLabel+"|"+path;
        parent = GetParentItem(parent);
        if (parent == NULL)
        {
            return path;
        }

        parentLabel=GetItemText(parent);
    }

    return path;
}

void CYRTreeCtrl::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    HTREEITEM   hTreeItem = NULL;

    // Get the item clicked on
    hTreeItem = HitTest (point ,&nFlags);

    if (hTreeItem != NULL && nFlags & TVHT_ONITEM)
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DBLCLICK_ITEM,0,0);
    }

    CTreeCtrl::OnLButtonDblClk(nFlags, point);
    Invalidate();
}

int CYRTreeCtrl::GetItemType(HTREEITEM hti)
{
    CString path = GetItemPath(hti);
    int nImage;
    int nSelImage;
    GetItemImage(hti, nImage, nSelImage);

    int nItems = m_TreeItems.GetSize();

    for (int nCount = 0; nCount<nItems; nCount++)
    {
        TreeItem item = m_TreeItems[nCount];
        if (item.Path == path && nImage == item.ImageIndex)
        {
            return item.Type;
        }
    }
    return -1;

}

void CYRTreeCtrl::GetSelectedTreeItem(CString &Path, int &Type)
{
    HTREEITEM hTreeItem = GetSelectedItem();
    Path = GetItemPath(hTreeItem);
    Type = GetItemType(hTreeItem);
}

void CYRTreeCtrl::AddAdditionalEntries(CMenu *pPopup, CString type)
{
    int nSize = m_MenuEntries.GetSize();

    for (int nCount=0; nCount<nSize; nCount++)
    {
        MenuEntry menu = m_MenuEntries[nCount];
        if (menu.Type == type)
        {
            char* pMenuEntries = menu.Entries.GetBuffer(0);
            char* pEntry;
            int nIndex = 0;
            pEntry = strtok (pMenuEntries ,"|");
            while (pEntry != NULL)
            {
                CString Entry = pEntry;
                int nLen = Entry.GetLength();
                CString Label = Entry.Left(nLen -2);
                BOOL bState = atoi(Entry.Right(1));
                if (bState)
                {
                    pPopup->AppendMenu(MF_STRING| MF_ENABLED, ID_CONTEXT_USER+nIndex, Label);
                }
                else
                {
                    pPopup->AppendMenu(MF_STRING| MF_GRAYED , ID_CONTEXT_USER+nIndex, Label);
                }
                nIndex++;
                pEntry = strtok (NULL ,"|");
            }
        }
    }
}

//
// @func
//
void CYRTreeCtrl::OnContextClick (UINT nID)
{
    m_nMenuIndexClicked = nID - ID_CONTEXT_USER;
    ::SendNotifyMessage(GetParent()->m_hWnd ,WMU_CONTEXT_MENU_CLICK ,nID - ID_CONTEXT_USER ,0);
}

void CYRTreeCtrl::GetMenuItemClicked(CString & Path, int & Type, CString & Entry)
{
    GetSelectedTreeItem(Path,Type);

    int nSize = m_MenuEntries.GetSize();

    for (int nCount=0; nCount<nSize; nCount++)
    {
        MenuEntry menu = m_MenuEntries[nCount];
        if (menu.Type == Type)
        {
            CString entries = menu.Entries;

            for (int nCount=0; nCount<m_nMenuIndexClicked; nCount++)
            {
                int nLen = entries.GetLength();
                int nPos = entries.Find("|",0);
                entries=entries.Right(nLen-nPos-1);
            }

            int Pos=entries.Find("|",0);
            if(Pos==-1)
            {
                Entry=entries;
            }
            else
            {
                Entry=entries.Left(Pos);
            }


            // Strip off the enabled state bit
            int nLen = Entry.GetLength();
            Entry = Entry.Left(nLen-2);

            return;
        }
    }
}



void CYRTreeCtrl::OnItemexpanding(NMHDR* pNMHDR, LRESULT* pResult)
{
    NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

    m_ItemToExpand = pNMTreeView->itemNew.hItem;
    UINT nState = GetItemState(m_ItemToExpand, TVIF_STATE);
    if (nState & TVIS_EXPANDED)
    {}

    else
    {
        if (ItemHasChildren(m_ItemToExpand))
        {
            HTREEITEM child = GetChildItem(m_ItemToExpand);
            while (child != NULL)
            {
                DeleteItem(child);
                child = GetChildItem(m_ItemToExpand);
            }
        }
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_EXPANDING,0,0);
    }

    *pResult = 0;
}

HTREEITEM CYRTreeCtrl::GetItemToExpand()
{
    return m_ItemToExpand;
}

BOOL CYRTreeCtrl::ItemCanBeDragged(HTREEITEM hItem)
{
    //
    //    Recover the type of the entry in question and from that determine whether this type
    //    is draggable by recovering the menu for this type.
    //
    int type = GetItemType (hItem);
    MenuEntry* pMenuEntry = GetMenuEntry (type);
    if ((pMenuEntry != NULL) && (pMenuEntry->bDraggable)) return TRUE;
    return FALSE;
}

void CYRTreeCtrl::CopyItemProperties(HTREEITEM hNewItem, HTREEITEM hDraggedItem)
{
    //
    // To do: Need to add item copy operation
    //

}






void CYRTreeCtrl::OnContextMenu (CPoint point)
{
    if (!m_bMenus)
        return;

    UINT uFlags;
    CPoint ptTree = point;

    ScreenToClient(&ptTree);
    HTREEITEM htItem = HitTest(ptTree, &uFlags);

    if ((htItem != NULL) && (uFlags & TVHT_ONITEM))
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_SHOW_CONTEXT,0,(LPARAM)htItem);

        Select (htItem ,TVGN_CARET);
        ShowPopupMenu (ptTree ,htItem);
    }

    else
    {
        return;
    }
}

void CYRTreeCtrl::ShowPopupMenu( CPoint& point ,HTREEITEM hTreeItem)
{
    CMenu   menu;
    CMenu   *pPopup;

    VERIFY(menu.LoadMenu(IDR_POPUP_TREE));

    pPopup = menu.GetSubMenu(0);    // ...get a pointer to teh submenu to pop up
    ASSERT(pPopup != NULL);         // ...ensure that it is a valid menu

    //
    // Extend the menu by appending the two options to expand/contract all items
    //
    int type = GetItemType (hTreeItem);
    AddAdditionalEntries (pPopup, type);

    /// If type does not support properties remove the properties menu item
    if (!DoesTypeHaveProperties(type))
    {
        pPopup->RemoveMenu(0,MF_BYPOSITION);
    }
    //
    //    ...track the popup menu sending all messages to the repository tree viewer (this)
    //
    ClientToScreen (&point);
    pPopup->TrackPopupMenu (TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this);
}

void CYRTreeCtrl::OnPopupProperties()
{
    CString Path;
    int Type;

    GetSelectedTreeItem(Path, Type);
    LPCTSTR ptrPath = Path;
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_ITEM_PROPERTIES,(WPARAM)Type,(LPARAM)ptrPath);
}

BOOL CYRTreeCtrl::DoesTypeHaveProperties(int type)
{
    MenuEntry* pMyEntry = GetMenuEntry(type);
    if (pMyEntry == NULL) return FALSE;
    return pMyEntry->bHasProperties;
}

MenuEntry*  CYRTreeCtrl::GetMenuEntry (int type)
{
    //
    //    Scan the menu entries looking for the one with the specified type and return it if found
    //
    int nSize = m_MenuEntries.GetSize();
    for (int nCount=0; nCount<nSize; nCount++)
    {
        MenuEntry menu = m_MenuEntries[nCount];
        if (menu.Type == type)
        {
            return &m_MenuEntries[nCount];
        }
    }
    return NULL;
}

void CYRTreeCtrl::EnableMenuItem(long nNodeType, LPCTSTR szLabel, BOOL bEnabled)
{
    int nSize = m_MenuEntries.GetSize();

    for (int nCount=0; nCount<nSize; nCount++)
    {
        MenuEntry menu = m_MenuEntries[nCount];
        if (menu.Type == nNodeType)
        {
            CString Entries = menu.Entries;

            CString Search;
            Search.Format("%s,",szLabel);
            int nPos = Entries.Find(Search);
            if (nPos == -1)
                return;

            int StatePos = nPos+Search.GetLength();

            if(bEnabled)
            {
                Entries.SetAt(StatePos, '1');
            }
            else
            {
                Entries.SetAt(StatePos, '0');
            }

            menu.Entries = Entries;
            m_MenuEntries[nCount] = menu;
        }
    }
}

void CYRTreeCtrl::OnLButtonDown(UINT nFlags, CPoint point)
{
	RelayEvent(WM_LBUTTONDOWN,(WPARAM)nFlags,MAKELPARAM(LOWORD(point.x),HIWORD(point.y)));

    CTreeCtrl::OnLButtonDown (nFlags, point);

    //
    //    Perform a hit test to see what item (if any) we are over and determine whether or not
    //    it is a draggable item.
    //
    UINT uFlags;
    CPoint ptTree = point;

    //    ScreenToClient(&ptTree);
    HTREEITEM htItem = HitTest(ptTree, &uFlags);

    //
    //    To drag we must have been on a draggable item in the tree so check this
    //
    if ((htItem != NULL) && (uFlags & TVHT_ONITEM))
    {
        if (ItemCanBeDragged (htItem))
        {
            COleDataSource* pSource = GetDragItem(htItem);

            //
            //    ...and do the drag/drop noting that this does not return until we cancel or drop
            //
            DROPEFFECT dropEffect = pSource->DoDragDrop (DROPEFFECT_MOVE|DROPEFFECT_COPY
                                    ,CRect(0, 0, 0, 0));
            delete pSource;
        }
    }

    Invalidate();
}

//
//    GetDragItem
//    ============
//
//    Construct a COleDataSource for a drag operation and return it.
//
COleDataSource* CYRTreeCtrl::GetDragItem (HTREEITEM hTreeItem)
{
    COleDataSource* pSource = new COleDataSource();

    CString GlobalID;
    GlobalID.Format("%s|%d",GetItemText(hTreeItem),GetItemType(hTreeItem));

    HGLOBAL hGlobData = ::GlobalAlloc(GMEM_SHARE, GlobalID.GetLength() + 1);
    LPVOID pGlobData = ::GlobalLock(hGlobData);
    memcpy(pGlobData, GlobalID.GetBuffer(0), GlobalID.GetLength() + 1);
    ::GlobalUnlock(hGlobData);
    pSource->CacheGlobalData(CF_TEXT, hGlobData);
    return pSource;
}

void CYRTreeCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    if (nChar == VK_F5)
    {
        RefreshTree();
    }
    if (nChar == VK_DELETE)
    {
        CString Path;
        int Type;

        GetSelectedTreeItem(Path, Type);
        if (!Path.IsEmpty())
        {
            LPCTSTR ptrPath = Path;
            ::SendNotifyMessage(GetParent()->m_hWnd,WMU_ITEM_DELETE,(WPARAM)Type,(LPARAM)ptrPath);
        }
    }
    if (nChar == VK_F1)
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_SHOW_HELP, 0, 0);
    }
    CTreeCtrl::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CYRTreeCtrl::RefreshTree()
{
    HTREEITEM root = GetRootItem();
    RefreshBranch(root);
}

void CYRTreeCtrl::RefreshBranch(HTREEITEM child)
{
    if(child == NULL)
        return;

    while (child !=NULL)
    {
        BOOL bItemHasChildren = ItemHasChildren(child);
        CArray <int,int> childExpandState;
        // Find current expand state of children
        if (bItemHasChildren)
        {
            HTREEITEM nextChild = GetChildItem(child);
            while (nextChild != NULL)
            {
                UINT nState = GetItemState(nextChild, TVIF_STATE);
                if (nState & TVIS_EXPANDED)
                {
                    childExpandState.Add(1);
                }
                else
                {
                    childExpandState.Add(0);
                }
                nextChild = GetNextSiblingItem(nextChild);
            }

        }

        UINT nState = GetItemState(child, TVIF_STATE);
        if ((nState & TVIS_EXPANDED) || !bItemHasChildren)
        {
            int isub=0;

            // Set the item to expand ( this item )
            m_ItemToExpand = child;

            // Delete child items of this item
            HTREEITEM nextChild = GetChildItem(child);
            while (nextChild != NULL)
            {
                DeleteItem(nextChild);
                nextChild = GetChildItem(child);
            }

            // Ask for items to be refreshed and make sure the the item is expanded again
            ::SendNotifyMessage(GetParent()->m_hWnd,WMU_EXPANDING,0,0);
            Expand(child, TVE_EXPAND);

            // Loop through child items refreshing each one restoring it's previous expand state
            nextChild = GetChildItem(child);
            while (nextChild != NULL)
            {
                BOOL bExpandState = 1;

                //iSub will be larger than the array if items have been added to scc
                //so we need to check this
                if(childExpandState.GetSize()>isub)
                {
                    bExpandState = childExpandState[isub];
                }
                RefreshBranch(nextChild);
                if (bExpandState)
                {
                    Expand(nextChild, TVE_EXPAND);
                }
                isub++;

                nextChild = GetNextSiblingItem(nextChild);
            }
        }

        child = GetNextSiblingItem(child);
    }
}

void CYRTreeCtrl::EnableMenus(BOOL bEnabled)
{
    m_bMenus = bEnabled;
}

void CYRTreeCtrl::OnLButtonUp(UINT nFlags,CPoint point)
{
	RelayEvent(WM_LBUTTONUP,(WPARAM)nFlags,MAKELPARAM(LOWORD(point.x),HIWORD(point.y)));

	CTreeCtrl::OnLButtonUp(nFlags,point);
}

void CYRTreeCtrl::OnMouseMove(UINT nFlags,CPoint point)
{
	RelayEvent(WM_MOUSEMOVE,(WPARAM)nFlags,MAKELPARAM(LOWORD(point.x),HIWORD(point.y)));

	CTreeCtrl::OnLButtonUp(nFlags,point);

	UINT uFlags;
	HTREEITEM hItem = HitTest(point,&uFlags);
	if(m_hLastItemHit != hItem)
	{
		m_hLastItemHit = hItem;
		m_ctlToolTips.Activate(FALSE);
		m_strTipText.Empty();
		if(m_hLastItemHit)
			::SendNotifyMessage(GetParent()->m_hWnd,WMU_NEED_TIP_TEXT,0,(long)m_hLastItemHit);
	}

	CTreeCtrl::OnMouseMove(nFlags,point);
}

void CYRTreeCtrl::RelayEvent(UINT message,WPARAM wParam,LPARAM lParam)
{
	if(NULL != m_ctlToolTips.m_hWnd)
	{
		MSG msg;
		msg.hwnd = m_hWnd;
		msg.message = message;
		msg.wParam = wParam;
		msg.lParam = lParam;
		msg.time = 0;
		msg.pt.x = LOWORD(lParam);
		msg.pt.y = HIWORD(lParam);

		m_ctlToolTips.RelayEvent(&msg);
	}
}

int CYRTreeCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if(CTreeCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_ctlToolTips.Create(this,TTS_ALWAYSTIP);
	m_ctlToolTips.AddTool(this);
	m_ctlToolTips.SetMaxTipWidth(SHRT_MAX);
	m_ctlToolTips.SetDelayTime(TTDT_AUTOPOP,SHRT_MAX);
	m_ctlToolTips.SetDelayTime(TTDT_INITIAL,200);
	m_ctlToolTips.SetDelayTime(TTDT_RESHOW,200);
	m_ctlToolTips.Activate(FALSE);
	m_ctlToolTips.SetFont(GetFont());
	EnableToolTips();

	return 0;
}

void CYRTreeCtrl::SetTipText(LPCTSTR szTipText)
{
	m_strTipText = szTipText;
	if(m_strTipText.IsEmpty())
		m_ctlToolTips.Activate(FALSE);
	else
	{
		m_ctlToolTips.Activate(TRUE);
		m_ctlToolTips.UpdateTipText(m_strTipText,this);
	}
}