/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/* +--------------------------------------------------------------------
   | NOM FICHIER     :   process.cpp
   | DESCRIPTION     :   
   | DATE CREATION   :   Mon Jan  4 12:01:14 1999
   +--------------------------------------------------------------------
*/


#include "process.hpp"

CTProcess::~CTProcess()
{}


#ifdef UNIX

void* CTProcess_Start(void * args)
{
    CTProcess* CTP=reinterpret_cast<CTProcess*>(args);
    CTP->waitAndExit();
    return NULL;
}
/* called after the fork
   to avoid zombies... */
void CTProcess::waitAndExit()
{
    int child_sts;

    waitpid(pid_, &child_sts, 0);

    if (autodelete_)
    {
        delete this;
    }
}
#endif



int CTProcess::start()
{
#ifdef WIN32
    STARTUPINFO         si;
    PROCESS_INFORMATION pi;

    memset(&si, 0, sizeof(si));
    si.cb = sizeof(si);


    if (CreateProcess(NULL, (char *)processName_.c_str(), NULL, NULL, FALSE,
                      CREATE_DEFAULT_ERROR_MODE|DETACHED_PROCESS,
                      //              CREATE_DEFAULT_ERROR_MODE|CREATE_NEW_CONSOLE,
                      NULL, NULL, &si, &pi) == FALSE)
    {
        LPVOID lpMsgBuf;
        iErrorCode_ = GetLastError() ;
        FormatMessage(  FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                        NULL,
                        iErrorCode_,
                        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                        (LPTSTR) &lpMsgBuf,
                        0,
                        NULL );
        errorMessage_ = (char *)lpMsgBuf ;
        LocalFree(lpMsgBuf) ;
    }
    else
    {
        pid_ = pi.hProcess ;
        actual_pid_ = pi.dwProcessId;
        CloseHandle(pi.hThread);
        iErrorCode_ = 0 ;
        errorMessage_ = "" ;
    }
#else
    RWCString path ;
    RWCString fileToTest;
    bool fileFound = FALSE;
    RWCString token;
    RWCString stProgramName;
    RWCTokenizer tokProgramName(processName_);

    stProgramName = tokProgramName();

#ifdef DEBUG
    syslog(LOG_DEBUG,"looking for '%s'", stProgramName.c_str());
#endif

    if ((stProgramName.c_str())[0] == '/')
    {
        fileToTest = stProgramName;
    }
    else
    {
        path = getenv("PATH");
        iErrorCode_ = 0;

        {
            RWCTokenizer nextDir(path);
            token=nextDir(":");
            while (!token.isNull() && !fileFound)
            {
                FILE * tmp;

                fileToTest = token + RWCString("/") + stProgramName;
#ifdef DEBUG
                syslog(LOG_DEBUG,"Testing '%s'", fileToTest.c_str());
#endif

                if (!access(fileToTest.c_str(), X_OK | R_OK))
                {
                    fileFound = true;
                }
                token=nextDir(":");
            }
        }

        if (!fileFound)
        {
            iErrorCode_ = -1;
            errorMessage_ = " " + stProgramName + " not found in $PATH";

            syslog(LOG_DEBUG,"ERR : '%s' not found in $PATH", stProgramName.c_str());

            return iErrorCode_;
        }
    }

    syslog(LOG_DEBUG,"launching '%s'", stProgramName.c_str());

#ifdef SOLARIS
    /* To duplicate onlyt the calling thread (specific to Solaris threads). */
    switch(pid_=fork1())
#else
switch(pid_=fork())
#endif
    {
    case -1:
        /* Error ... */
        /* Don't know what to do ? */
        iErrorCode_ = errno;
        break;
    case 0:
        {
            /* child process : will do an execv */
            RWCTokenizer next1(processName_);
            RWCTokenizer next2(processName_);
            unsigned int iCpt=0;
            unsigned int iMax=0;
            char ** pSzCommand;
            int i;


            // Advance until the null string is returned:
            while (!(token=next1()).isNull())
            {
                if (iCpt == 0)
                {
                    iMax = (fileToTest.length()>iMax ? fileToTest.length():iMax);
                }
                else
                {
                    iMax = (token.length()>iMax ? token.length():iMax);
                }
                iCpt++;
            }
            pSzCommand = new char* [iCpt+1]; // +1 for the null terminated pointer
            pSzCommand[iCpt]=NULL;

            for (i=0;i<iCpt;i++)
            {
                pSzCommand[i] = new char [iMax];
            }
            iCpt = 0;
            while (!(token=next2()).isNull())
            {
                if ((token.c_str()[0]=='\"') /* " */ && (token.c_str()[token.length()-1]=='\"' /* " */))
                {
                    strncpy(pSzCommand[iCpt], (char*) (token.c_str()+1), token.length()-2);
                    pSzCommand[iCpt][token.length()-2] = '\0';
                }
                else
                {
                    if (iCpt==0)
                    {
                        /* copying the fullt program name */
                        strcpy(pSzCommand[iCpt],fileToTest.c_str() );
                    }
                    else
                    {
                        strcpy(pSzCommand[iCpt], token.c_str());
                    }
                }
                iCpt++;
            }

            close(0);
            close(1);
            close(2);
#ifdef DEBUG
            RWCString log = stProgramName+".debug.log";
            int outFile = open(log.c_str(),O_WRONLY | O_CREAT,0666 );
#else
    RWCString log = "/dev/null";
    int outFile = open(log.c_str(),O_WRONLY );
#endif

            if (outFile==-1)
            {
                syslog(LOG_DEBUG,"ERR: open(%s,...)",log.c_str());
                exit(-1);
            }
            dup2(outFile,1);
            dup2(outFile,2);
            close(outFile);
            execv(fileToTest.c_str(), pSzCommand);
            /* If we are here : execv did not work ! */
            syslog(LOG_DEBUG,"ERR: execv(%s,...)",fileToTest.c_str());

            exit(-1);
            break;
        }
    default:
        /* parent process */
        /* Will have to wait for it child
        to avoid zombies... */

        threadWait_ = new omni_thread(&CTProcess_Start,this);
        threadWait_->start();

        break;
    }
#endif

    return iErrorCode_ ;
}



bool CTProcess::isRunning()
{
    bool    bRes = FALSE ;
#ifdef WIN32
    DWORD ret ;
    if ((ret = WaitForSingleObject(pid_, 0)) == WAIT_FAILED)
        bRes = FALSE;
    else
    {
        if (ret == WAIT_OBJECT_0)
            bRes = FALSE ;
        else
            bRes = true;
    }
#else
int child_sts;
    if ( waitpid(pid_, &child_sts, WNOHANG)==0)
    {
        bRes = true;
    }
#endif
    return bRes ;
}

int CTProcess::wait()
{
#ifdef WIN32
    DWORD ret ;
    if ((ret = WaitForSingleObject(pid_, INFINITE)) == WAIT_FAILED)
        return -1 ;
    else
    {
        // The process is terminated
        if (ret == WAIT_OBJECT_0)
        {
            // get the termination status
            unsigned long ulExitCode = 0 ;
            if (GetExitCodeProcess(pid_, &ulExitCode))
                return (int)ulExitCode ;
            else
                return -1 ;
        }
    }
    return -1 ;
#else
    int child_sts;
    waitpid(pid_, &child_sts, 0);
    return WEXITSTATUS(child_sts);
#endif
}

bool CTProcess::closePid()
{
#ifdef WIN32
    return CloseHandle(pid_)?true:false ;
#else
    return true ;
#endif

}

int CTProcess::setPidInSharedMemory(string memoryName)
{
#ifdef WIN32
    string cyranoMemoryName = "CYRANO" ;
    cyranoMemoryName += memoryName ;
    HANDLE memoryHandle = CreateFileMapping((HANDLE)0xFFFFFFFF, NULL, PAGE_READWRITE, 0, 512, (LPCTSTR)cyranoMemoryName.c_str() ) ;
    if (memoryHandle == NULL)
    {
        return GetLastError() ;
    }

    int * pPid = (int *)MapViewOfFile(memoryHandle, FILE_MAP_WRITE, 0, 0, 0);
    if (pPid == NULL)
    {
        return GetLastError() ;
    }
    *pPid = _getpid() ;
    if (UnmapViewOfFile((LPVOID)pPid) == FALSE)
    {
        return GetLastError() ;
    }
    //  CloseHandle(memoryHandle) ;
#else
    pid_t myPid = getpid();
    /* shared Memory is a file...*/
    RWCString strFile = RWCString("/tmp/") + memoryName + RWCString(".pid");
    FILE* filePid = fopen(strFile.c_str() ,"w+");

    if (filePid==NULL)
    {
        return -1;
    }

    if (fprintf(filePid,"%d\n",myPid)==-1)
    {
        return -1;
    }
    fclose(filePid);

#endif
    return 0 ;
}

int CTProcess::getPidFromSharedMemory(string memoryName, int & iPid)
{
#ifdef WIN32
    string cyranoMemoryName = "CYRANO" ;
    cyranoMemoryName += memoryName ;
    HANDLE memoryHandle = OpenFileMapping(  FILE_MAP_READ, FALSE, (LPCTSTR)cyranoMemoryName.c_str() ) ;
    if (memoryHandle == NULL)
    {
        return GetLastError() ;
    }
    int * pPid = (int *)MapViewOfFile(memoryHandle, FILE_MAP_READ, 0, 0, 0);
    if (pPid == NULL)
    {
        return -1 ;
    }
    iPid = *pPid ;
    if (UnmapViewOfFile((LPVOID)pPid) == FALSE)
    {
        return GetLastError() ;
    }
    CloseHandle(memoryHandle) ;
#else
    /* shared Memory is a file...*/
    RWCString strFile = RWCString("/tmp/") + memoryName + RWCString(".pid");
    FILE* filePid = fopen(strFile.c_str() ,"r");

    if (filePid==NULL)
    {
        return -1;
    }

    if (fscanf(filePid,"%d\n",&iPid)==EOF)
    {
        return -1;
    }
    fclose(filePid);
#endif
    return 0 ;
}

int CTProcess::stop(int iPid)
{
#ifdef WIN32
    HANDLE processHandle = OpenProcess(PROCESS_ALL_ACCESS, FALSE, iPid) ;
    if (processHandle == NULL)
    {
        return GetLastError() ;
    }

    if (TerminateProcess(processHandle, 1) == FALSE)
    {
        return GetLastError() ;
    }

    CloseHandle(processHandle) ;
#else
    if ( kill(iPid,SIGKILL)==-1)
    {
        return -1;
    }
#endif
    return 0 ;
}

int CTProcess::removeSharedMemory(string memoryName)
{
#ifdef WIN32
    return 0;
#else
    /* shared Memory is a file...*/
    RWCString strFile = RWCString("/tmp/") + memoryName + RWCString(".pid");

    return unlink(strFile.c_str());
#endif
}
