/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrFFApi
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECFFTestRun_h
#define _CECFFTestRun_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrffApi/CECFFFolder.h>
#include <CyrWrapLib/CECTestOutput.h>
#include <CyrffApi/CECFFTestResultsFlusher.h>

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <functional>

///////////////////////////////////////////////////////////////////////////////
// Typedefs
//typedef std::list<string> OutputList;
//typedef std::map<string,OutputList*,std::less<string> > OutputTable;
//typedef OutputList::iterator OutputListIt;
//typedef std::map<string,OutputListIt, std::less<string> > OutputItTable;

///////////////////////////////////////////////////////////////////////////////
// @class   CECFFTestRun |
// @base    public | CECFFFolder
class CYR_EXPORT CECFFTestRun : public CECFFFolder
{
    // @access Public Members
public:
    // @cmember Constructor for retrieving test runs
    CECFFTestRun( CECFFFolder* pParent, const char* name );
    // @cmember Constructor for creating new test runs
    CECFFTestRun( CECFFFolder* pParent, const char* desc, const char* node, const char* executor );
    virtual ~CECFFTestRun();  // @cmember Destructor

    string GetDescription() const;  // @cmember Returns test run description
    void SetDescription( const char* desc );    // @cmember Sets new description of test run
    CTTime GetTimeStarted() const;     // @cmember Returns test run start time
    CTTime GetTimeStopped() const;     // @cmember Returns test run stop time
    string GetTestName() const;     // @cmember Returns name of test being run
    long GetTestVersion() const;       // @cmember Returns version of test being run
    string GetNodeName() const;     // @cmember Returns test run node name
    string GetExecutorName() const; // @cmember Returns test run executor name

    void StopTestRun();     // @cmember Stops test run.

    // @cmember Adds a single item of test output to the database
    void AddTestOutput( CECTestOutput* output );
    // @cmember Adds an output string of the specified type
    void AddTestOutput( const char* outType, const char* entry );

    // @cmember Start iterating through timer values
    CECTimerOutput* GetFirstTimer();
    // @cmember Get next timer in iteration
    CECTimerOutput* GetNextTimer();
    // @cmember Start iterating through test log values
    CECTestLog* GetFirstTestLog();
    // @cmember Get next test log in iteration
    CECTestLog* GetNextTestLog();
    CECOtaSimpleQuery* GetFirstSimpleQuery();
    CECOtaSimpleQuery* GetNextSimpleQuery();
    CECTestReport* GetFirstReport();
    CECTestReport* GetNextReport();
    CECTestHistory* GetFirstHistory();
    CECTestHistory* GetNextHistory();
    CECOtaHeader* GetFirstOTAHeader();
    CECOtaHeader* GetNextOTAHeader();

    // @cmember Retrieves complete list of output for specified type
    OutputList* GetOutputList( const char* outType );
    string   GetFirstOutputString( const char* outType );
    string   GetNextOutputString( const char* outType );
    string  GetOutputString( const char* outType, long Iter);
    long    GetNoOutputEntries( const char* outType);

    // @cmember Retrieves the offset for a specified list
    long GetOutputOffset(const char* outType);

    // @access Protected Members
protected:
    void PutDescription(string desc)
    {
        m_desc=desc;
    };
    string m_testName;       // @cmember Name of test being executed
    string m_desc;           // @cmember Description of test run
    string m_timeStart;      // @cmember Time that test run started
    string m_timeStop;       // @cmember Time that test run stopped
    string m_execName;       // @cmember Name of executor
    string m_nodeName;       // @cmember Name of node
    string m_testVersion;    // @cmember Version of test being run
    CECFFFolder*    m_folder;
    OutputTable   m_output;     // @cmember Table of all output related to run.
    OutputItTable m_iterator;   // @cmember Table of current iterator statuses.
    bool m_readonly;            // @cmember Determines if Test Run is readonly or not
    bool m_new;                 // @cmember Determines if Test Run is new or not
    OutputList m_PurgeList;     // @cmember List of Items to be purged

    std::map<string,int, std::less<string> > m_OffsetMap; // @cmember count of items purged so far (per list)

    // @cmember Constructs a single string from all parameters
    string BuildFullDescription();
    // @cmember Constructs a unique name for the test run
    string BuildTestRunName( CECFFFolder* pParent );
    // @cmember Sets up arrays for test output data
    void PrepareArrays();

    // @cmember Saves all Test Output to a file
    void SaveList( string filename, OutputList*& outList );

    // @cmember Removes flushed Test Output from the list
    bool PurgeList( string rwFilename, OutputList*& outList );

    // @cmember Restores all Test Output from a file
    void OpenList( string filename, OutputList*& outList );

    void UseStInt(bool checkout);
};

#endif
