/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc Logging
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECLog_h
#define _CECLog_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <OSTAORB/OSTAORB.h>
#include <Logging/CyrLogFile.h>
#include <CyrWrapLib/CyrWrapLib.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CECLog | This class hides the CORBA functionality of the Cyrano
//          logging class.  It contains no additional functionality.  This
//          wrapper is for applciations that wish to implement logging
//          functionality.
class CECLog
{
    // @access Public Members
public:
    CECLog( CORBAServer* pServer, const char* name, long level = 0 );
    virtual ~CECLog();

    void AddTrace( const char* category, long level, const char* msg );
    void Connect( CECOrbProject* pProject );
    void Store( CECOrbFolder* pFolder );
    void SaveTraceLevels( CECOrbProject* pProject );
    void RestoreTraceLevels( CECOrbProject* pProject );
    void SetLevel( const char* category, long level);

    // @access Private Members
private:
    CORBAServer* m_server;  // @cmember CORBA helper functionality
    CyrLogFile* m_log;      // @cmember CORBA log object
    string   m_name;     // @cmember Registered name of log.
    bool        m_delete;   // @cmember Delete log file or not
}
;

#endif
