/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrStRepository
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrStFileIt_i_h
#define _CyrStFileIt_i_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Repository/CyrStRepository.h>
#include <fstream>
using std::fstream;
///////////////////////////////////////////////////////////////////////////////
// Constants
const long BYTESEQ_SIZE = 1024;

///////////////////////////////////////////////////////////////////////////////
// @class   CyrStFileItImpl | This class represents a simple CORBA mechanism
//          for iterating through the contents of a File.  It implements a
//          simple File open and reading system that can transfer the entire
//          contents of a File from a remote machine.
//
//          It is intended for use on both client and server sides of the
//          CORBA network and therefore contains no platform specific file
//          reading methods.  Instead the RogueWave file classes are used.
// @base    public | POA_CyrStByteIt
class CyrStFileItImpl : public virtual POA_CyrStByteIt,public PortableServer::RefCountServantBase
{
    // @access Public Members
public:
    static const char* CLASS_NAME;

    CyrStFileItImpl( const char* filename );  // @cmember Constructor
    virtual ~CyrStFileItImpl();               // @cmember Destructor

    // @cmember Retrieves the next sequence of bytes from the File
    ByteSeq* ReadBytes();
    void WriteBytes( const ByteSeq& bytes )
    {}
    ;
    long GetSize();  // @cmember Returns total size of File

    long ReadLong();
    void WriteLong( long data )
    {}
    ;
    short ReadShort();
    void WriteShort( short data )
    {}
    ;
    CORBA::Boolean ReadBool();
    void WriteBool( CORBA::Boolean data )
    {}
    ;
    CORBA::Octet ReadByte();
    void WriteByte( CORBA::Octet data )
    {}
    ;

    // @access Protected Members
protected:
    int  m_filedesc;   // @cmember Underlying pointer to file on disk
    long     m_size;   // @cmember Total length of File
}
;

#endif
