/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// BCGCyrTabWnd.cpp: implementation of the CBCGCyrTabWnd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "OSBCGTabWnd.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//**********************************************************************************************
COSBCGTabWnd::COSBCGTabWnd()
{
}

//**********************************************************************************************

COSBCGTabWnd::~COSBCGTabWnd()
{
}

//**********************************************************************************************

BOOL COSBCGTabWnd::Create (Style style, CWnd* pParentWnd, UINT nID,
                           Location location)
{
    return CBCGTabWnd::Create(style,CRect(0,0,0,0),pParentWnd,nID,location);
}


//**********************************************************************************************
void COSBCGTabWnd::AddTab(CRuntimeClass* pViewClass,
                          LPCTSTR lpszLabel,
                          CCreateContext* pContext,
                          UINT nID, UINT uiImageId)
{
    ASSERT_VALID(this);

    if (nID == -1)
        nID = GetTabsNum()+1;

    CWnd* pWnd = CreateView(pViewClass, pContext, nID);

    CBCGTabWnd::AddTab(pWnd, lpszLabel, uiImageId);
}


//**********************************************************************************************
CWnd* COSBCGTabWnd::CreateView(CRuntimeClass* pViewClass,
                               CCreateContext* pContext, UINT nID)
{
    ASSERT_VALID(this);
    ASSERT(pViewClass != NULL);
#ifdef WIN32
    ASSERT(pViewClass->IsDerivedFrom(RUNTIME_CLASS(CWnd)));
#endif

#if _MFC_VER < 0x042
    ASSERT(AfxIsValidAddress(pViewClass, sizeof(CRuntimeClass)));
#endif
    CWnd* pWnd = NULL;
    TRY
    {
        pWnd = (CWnd*)pViewClass->CreateObject();
        if (pWnd == NULL)
        AfxThrowMemoryException();
    }
    CATCH_ALL(e)
    {
        TRACE0("Out of memory creating a tab window view\n");
        return pWnd;
    }
    END_CATCH_ALL

    ASSERT(pWnd->IsKindOf(RUNTIME_CLASS(CWnd)));
    ASSERT(pWnd->m_hWnd == NULL);       // not yet created

    if (!pWnd->Create(NULL, NULL, WS_CHILD | WS_VISIBLE,
                      CRect(0,0,0,0), this, nID, pContext))
    {
        TRACE0("Warning: couldn't create the tab window view\n");
        return NULL;
    }


    return pWnd;
}
