/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// RepositoryAPI.cpp: implementation of the CRepositoryAPI class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "concertlib.h"
#include "ConstantesLib.h"
#include "RepositoryAPI.h"
#include "solutions.h"

#pragma warning( disable : 4786 )
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#pragma warning( disable : 4786 )

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------

CRepositoryAPI::CRepositoryAPI()
{
    m_pServer = new CYRServer;
}

//--------------------------------------------------------------------

CRepositoryAPI::~CRepositoryAPI()
{
    DisConnect() ;
    delete m_pServer;
    m_pServer = NULL;
}

//--------------------------------------------------------------------

void CRepositoryAPI::SetWindow(CWnd* _pWnd)
{
    m_pWnd = _pWnd;
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::Connect( const TCHAR* pczUser , const TCHAR* pczPwd )
{
    return true;
}

//--------------------------------------------------------------------

void CRepositoryAPI::DisConnect()
{
    if( m_pServer )
        m_pServer->CloseProject();
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::IsConnected()
{
    if( m_pServer )
        return TRUE ;
    else
        return FALSE ;
}

//--------------------------------------------------------------------

long CRepositoryAPI::GetProjectCaptureFiles(CStringArray& arstrFileList  )
{
    m_pServer->GetFileList(CYRServer::enSourceFolder,"",arstrFileList);

    CStringArray arstrCaptureFolders;
    m_pServer->GetFolderList(CYRServer::enCaptureFolder,"",arstrCaptureFolders);

    //Remove any files which are not SCL and do not have an
    //associated capture folder
    bool bFound = false;
    int nSclExtLen = strlen(STR_SCL_EXTENSION);
    for(int nSrcNdx=0;nSrcNdx<arstrFileList.GetSize();nSrcNdx++)
    {
        bFound = false;

        if(arstrFileList[nSrcNdx].Right(nSclExtLen) != STR_SCL_EXTENSION)
            arstrFileList.RemoveAt(nSrcNdx--);
        else
        {
            for(int nFolderNdx=0;nFolderNdx<arstrCaptureFolders.GetSize();nFolderNdx++)
            {
                if(arstrFileList[nSrcNdx] == arstrCaptureFolders[nFolderNdx])
                {
                    bFound = true;
                    break;
                }
            }
            if(!bFound)
                arstrFileList.RemoveAt(nSrcNdx--);
        }
    }

    return arstrFileList.GetSize();
}

//--------------------------------------------------------------------

void CRepositoryAPI::AddProjectCaptureFiles(const CString& strSCLFile )
{
    //SCL file
    m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",strSCLFile);

    //All file
    CString strAllFile = strSCLFile;
    strAllFile.Replace( STR_SCL_EXTENSION , STR_ALL_EXTENSION ) ;

    m_pServer->CheckOutFile(CYRServer::enCaptureFolder,strSCLFile,strAllFile);

    //FVR files
    AddFVR(strSCLFile);
}


//--------------------------------------------------------------------

BOOL CRepositoryAPI::RetrieveProjectCaptureFiles(const CString& strRepositoryFileName)
{
    //fichier SCL
    if(!m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",strRepositoryFileName))
    {
        //      AfxMessageBox(m_pServer->GetLastError());
        return FALSE;
    }

    CString strAllFile = strRepositoryFileName;
    strAllFile.Replace( STR_SCL_EXTENSION , STR_ALL_EXTENSION ) ;
    if(!m_pServer->CheckOutFile(CYRServer::enCaptureFolder,strRepositoryFileName,strAllFile))
    {
        AfxMessageBox(m_pServer->GetLastError());
    }

    //response_codes.inc
    if(!m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",STR_REPOSITORY_INCLUDE))
    {
        //$$$DisplayMessage( IDS_FILE_CHECKED , (CString) (ceIncludeFile.GetLocker()) ) ;
    }

    //GLOBAL_VARIABLES.INC

    if(!m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",STR_REPOSITORY_GLOBALVAR))
    {
        //$$$DisplayMessage( IDS_FILE_CHECKED , (CString) (ceIncludeFile.GetLocker()) ) ;
    }

    //FVR files

    CheckOutFVR(strRepositoryFileName);

    return TRUE;
}


//--------------------------------------------------------------------

BOOL CRepositoryAPI::SaveProjectCaptureFiles(const CString& strSCLFileName, bool bRelease )
{
    //SCL File
    m_pServer->CheckInFile(CYRServer::enSourceFolder,"",strSCLFileName);

    //All file
    CString strAllFile = strSCLFileName;
    strAllFile.Replace( STR_SCL_EXTENSION , STR_ALL_EXTENSION ) ;
    m_pServer->CheckInFile(CYRServer::enCaptureFolder,strSCLFileName,strAllFile);

    //4 - CheckIn Files and send them back to Repository
    m_pServer->CheckInFile(CYRServer::enSourceFolder,"",STR_REPOSITORY_INCLUDE);

    m_pServer->CheckInFile(CYRServer::enSourceFolder,"",STR_REPOSITORY_GLOBALVAR);

    AddFVR(strSCLFileName);
    CheckOutFVR(strSCLFileName);

    if (bRelease)
        CheckInFVR(strSCLFileName);

    return TRUE;
}

//--------------------------------------------------------------------

void CRepositoryAPI::DisplayMessage( UINT nID , const CString& strText )
{
    CString strError ;
    CString strMessage ;
    //---
    strError.LoadString( IDS_ERROR_LIB ) ;
    strMessage.Format( nID , strText ) ;
    m_pWnd->MessageBox(strMessage , strError , MB_OK ) ;
}

//--------------------------------------------------------------------

void CRepositoryAPI::DisplayMessage( UINT nID )
{
    CString strError ;
    CString strMessage ;
    //---
    strError.LoadString( IDS_ERROR_LIB ) ;
    strMessage.LoadString( nID ) ;
    m_pWnd->MessageBox( strMessage , strError , MB_OK ) ;
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::AddFVR(const CString& strSCLFile)
{
    CString strDataPath = m_pServer->GetFolderPath(CYRServer::enDataFolder);

    CString strFilename;

    CFileFind FileFinder;
    BOOL bWorking = FileFinder.FindFile(strDataPath + "*" + STR_FVR_EXTENSION);
    while(bWorking)
    {
        bWorking = FileFinder.FindNextFile();

        m_pServer->CheckInFile(CYRServer::enDataFolder,"",FileFinder.GetFileName() , "" );
    }

    return true;
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::CheckOutFVR(const CString& strSCLFile)
{
    CStringArray arstrFVR;
    m_pServer->GetFileList(CYRServer::enDataFolder,"",arstrFVR);

    int nFVRExtLen = strlen(STR_FVR_EXTENSION);
    for(int nFVR=0;nFVR<arstrFVR.GetSize();nFVR++)
    {
        if(arstrFVR[nFVR].Right(nFVRExtLen) == STR_FVR_EXTENSION)
        {
            m_pServer->CheckOutFile(CYRServer::enDataFolder,"",arstrFVR[nFVR]);
        }
    }
    return true;
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::CheckInFVR(const CString& strSCLFile)
{
    CStringArray arstrFVR;
    m_pServer->GetFileList(CYRServer::enDataFolder,"",arstrFVR);

    int nFVRExtLen = strlen(STR_FVR_EXTENSION);

    for(int nFVR=0;nFVR<arstrFVR.GetSize();nFVR++)
    {
        if(arstrFVR[nFVR].Right(nFVRExtLen) == STR_FVR_EXTENSION)
            m_pServer->CheckInFile(CYRServer::enDataFolder,"",arstrFVR[nFVR]);
    }
    return true;
}

//--------------------------------------------------------------------

void CRepositoryAPI::GetImpactProjectList(CStringArray& arstrProjectList)
{
    m_pServer->GetProjectList(arstrProjectList);
}

//--------------------------------------------------------------------

long CRepositoryAPI::GetProjectTemplateFiles(CStringArray& arstrFileList)
{
    m_pServer->GetFileList(CYRServer::enSourceFolder,"",arstrFileList);

    int nSclExtLen = strlen(STR_SCL_EXTENSION);
    int nIncExtLen = strlen(STR_INC_EXTENSION);

    for(int nSrcNdx=0;nSrcNdx<arstrFileList.GetSize();nSrcNdx++)
    {
        if( (arstrFileList[nSrcNdx].Right(nSclExtLen) != STR_SCL_EXTENSION) &&
                (arstrFileList[nSrcNdx].Right(nIncExtLen) != STR_INC_EXTENSION) )
        {
            arstrFileList.RemoveAt(nSrcNdx--);
        }
    }

    return arstrFileList.GetSize();
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::SaveProjectTemplateFiles(const CString& strSCLFileName, bool bRelease )
{
    return m_pServer->CheckInFile(CYRServer::enSourceFolder,"",strSCLFileName);
}

//--------------------------------------------------------------------

void CRepositoryAPI::AddProjectTemplateFiles(const CString& strSCLFile )
{
    m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",strSCLFile);
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::RetrieveProjectTemplateFiles(const CString& strRepositoryFileName)
{
    if(!m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",strRepositoryFileName))
    {
        AfxMessageBox(m_pServer->GetLastError());
        return FALSE;
    }

    return TRUE;
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::DeleteObjectFiles()
{
//	BOOL bResult = m_pServer->DeleteFolder(CYRServer::enObjectCodeFolder,"");
    
//	CString str = m_pServer->GetFolderPath(CYRServer::enObjectCodeFolder);
//	CreateDirectory(str,NULL);
//	return bResult;
	return TRUE;
}

//--------------------------------------------------------------------

BOOL CRepositoryAPI::OpenProject(const CString& strProject)
{
    return m_pServer->OpenProject(strProject,FALSE);
}
