
#ifndef _DATASTORE_HPP_
#define _DATASTORE_HPP_

#define DEFAULT_GROW_STRUCTBUFFER 16

//$$$JL#include "wm_user.h"

//---------------------------------------------------------------------------
// Return adress of [cSep or 0]-terminated string
LPCTSTR StringSepTokenize( LPCTSTR & pczCursor , const TCHAR cSep ) ;

//---------------------------------------------------------------------------
// 'CStructStore' is designed with the same pupose in mind as 'CStringStore' :
// storing structs at a fixed address (CArray reallocates its buffer, whitch
// renders it usuitable for storing itemdatas).
//---------------------------------------------------------------------------

template <class T> struct TStorage
{
    T           * m_pBuffer ;
    TStorage<T> * m_pNextBuffer ;
    int           m_nSize ;
    int           m_nRemains ;

    TStorage( int nSize ) ;
    ~TStorage() ;
} ;

//---------------------------------------------------------------------------

template <class T> class CStructStore
{
    TStorage<T> * m_pBuffer ;
    TStorage<T> * m_pCurrent ;
    int           m_nGrowBy ;

    BOOL NewBuffer() ;

public:
    CStructStore( int nGrowBy = -1 ) ;
    ~CStructStore() ;

    T * AddStruct( T * s ) ;

    int  FillArray( T * * ppDestBuffer , int nMax ) ;
    void Purge() ;
} ;

//---------------------------------------------------------------------------
//////////////////////////// TStorage-CStructStore //////////////////////////
//---------------------------------------------------------------------------

template <class T> TStorage<T>::TStorage( int nSize ) :
        m_pBuffer(NULL) ,
        m_pNextBuffer(NULL) ,
        m_nSize(nSize) ,
        m_nRemains(0)
{
    ASSERT( m_nSize > 0 ) ;

    TRY
    {
        m_pBuffer = new T [ m_nSize ] ;

        m_nSize = m_nRemains = m_nSize ;
    }
    CATCH(CMemoryException,e)
    {
        m_pBuffer = NULL ;
    }
    END_CATCH
}

//---------------------------------------------------------------------------

template <class T> TStorage<T>::~TStorage()
{
    delete [] m_pBuffer ;
}

//---------------------------------------------------------------------------

template <class T> CStructStore<T>::CStructStore( int nGrowBy /*=-1*/ ) :
        m_pBuffer(NULL) ,
        m_pCurrent(NULL) ,
        m_nGrowBy( nGrowBy > 0 ? nGrowBy : DEFAULT_GROW_STRUCTBUFFER )
{}

//---------------------------------------------------------------------------


template <class T> CStructStore<T>::~CStructStore()
{
#if 1
    Purge() ;
#else
    TStorage<T> * pBuffer = m_pBuffer ;
    // ----
    while ( pBuffer != NULL ) // Delete the linked list of buffers :
    {
        TStorage<T> * pCurrent = pBuffer ;
        // ----
        pBuffer = pCurrent->m_pNextBuffer ;

        delete pCurrent ;
    }
#endif
}

//---------------------------------------------------------------------------

template <class T> void CStructStore<T>::Purge()
{
    TStorage<T> * pBuffer = m_pBuffer ;
    // ----
    while ( pBuffer != NULL ) // Delete the linked list of buffers :
    {
        TStorage<T> * pCurrent = pBuffer ;
        // ----
        pBuffer = pCurrent->m_pNextBuffer ;

        delete pCurrent ;
    }

    m_pBuffer  = NULL ;
    m_pCurrent = NULL ;
}

//---------------------------------------------------------------------------

template <class T> int CStructStore<T>::FillArray( T * * ppDestBuffer , int nMax )
{
    int nCopyCount = 0 ;
    // ----
    if ( ppDestBuffer != NULL )
    {
        ASSERT( ::AfxIsValidAddress( ppDestBuffer , nMax * sizeof( T * ) ) ) ;

        TStorage<T> * pBuffer = m_pBuffer ;
        int           nCount  = 0 ;
        // ----
        while ( nCount < nMax && pBuffer != NULL )
        {
            TStorage<T> * pCurrent    = pBuffer ;
            int           nLocalCount = pCurrent->m_nSize - pCurrent->m_nRemains ;
            // ----
            for ( int i = 0 ; i < nLocalCount && nCount < nMax ; i++ , nCount++ )
            {
                ppDestBuffer[nCount] = &pCurrent->m_pBuffer[i] ;
            }

            pBuffer = pCurrent->m_pNextBuffer ;
        }
    }

    return nCopyCount ;
}

//---------------------------------------------------------------------------

template <class T> BOOL CStructStore<T>::NewBuffer()
{
    BOOL bRet = TRUE ;
    // ----
    TRY
    {
        TStorage<T> * pNewBuffer = new TStorage<T>( m_nGrowBy ) ;

        if ( m_pCurrent == NULL )
    {
        m_pBuffer = m_pCurrent = pNewBuffer ;
    }
    else
    {
        m_pCurrent->m_pNextBuffer = pNewBuffer ;
        m_pCurrent                    = pNewBuffer ;
    }
}
CATCH(CMemoryException,e)
    {
        bRet = FALSE ;
    }
    END_CATCH

    return bRet ;
}

//---------------------------------------------------------------------------

template <class T> T * CStructStore<T>::AddStruct( T * s )
{
    if ( s == NULL || ( ( m_pCurrent == NULL || m_pCurrent->m_nRemains == 0 ) && NewBuffer() == FALSE ) )
    {
        return NULL ;
    }

    T * pCopy = ( m_pCurrent->m_pBuffer + m_pCurrent->m_nSize - m_pCurrent->m_nRemains ) ;
    // ----
    m_pCurrent->m_nRemains-- ;

    *pCopy = *s ; // member-wise copy

    return pCopy ;
}

//---------------------------------------------------------------------------

template <class T_DataClass, class T_StorageClass>
void * _Store( T_DataClass * pData , T_StorageClass * * ppDs )
{
    if ( ppDs == NULL )
    {
        ASSERT( FALSE ) ;
        return NULL ;
    }

    if ( *ppDs == NULL )
    {
        TRY
        {
            *ppDs = new T_StorageClass () ;
        }
        CATCH(CMemoryException,e);
        {
            *ppDs = NULL ;
        }
        END_CATCH
    }

    return ( *ppDs != NULL ? static_cast<T_StorageClass *>(*ppDs)->AddData( pData ) : NULL ) ;
}

// --------------------------------------------------------------------------

template <class T> class CGenDataStore : public CDataStore
{
protected:
    // CArray< _EnumFormPopupMenu, _EnumFormPopupMenu & > m_DataArray ;
    CStructStore <T> m_DataStorage ;

public:
CGenDataStore() :
    CDataStore( 32 , 512 )
    {}


    virtual ~CGenDataStore()
    {}


    T * AddData( T * _pData , TCHAR cSep = (TCHAR)0 )
    {
        T * pData = NULL ;
        // ----
        if ( _pData != NULL )
        {
            pData = m_DataStorage.AddStruct( _pData ) ;

            if ( pData != NULL )
            {
                StringDup( pData , _pData , cSep ) ;
            }
        }

        return pData ;
    }

    BOOL StringDup( T * pDest , T * pSource , TCHAR cSep = (TCHAR)0 ) ; // User-defined copy of LPSTR's.

    void Purge()
    {
        CStringStore::Purge() ;
        m_DataStorage.Purge() ;
    }
} ;

//---------------------------------------------------------------------------

template <class T> void * Store( T * pData , CDataStore * * ppDs )
{
    return ::_Store( pData , reinterpret_cast< CGenDataStore<T> * * >(ppDs) ) ;
}

//---------------------------------------------------------------------------

template <class T> class CCurDataStore : public CGenDataStoreBase
{
    T    m_tData ;      // Copy of current data
    long m_lDataIndex ; // Not necessaryly an index but a unique identifier of the data.

public:
CCurDataStore( int nGrowBy = 32 ) :
    CGenDataStoreBase( nGrowBy ) ,
    m_lDataIndex(-1L)
    {
        memset( &m_tData , '\0' , sizeof( T ) ) ;
    }

    BOOL SetValue( T * pData , long lDataIndex )
    {
        if ( pData != NULL )
        {
            m_tData      = *pData ;
            m_lDataIndex = lDataIndex ;
            return StringDup( &m_tData , pData ) ;
        }

        return FALSE ;
    }

    long GetIndex() const
    {
        return m_lDataIndex ;
    }

    BOOL StringDup( T * pDest , T * pSource ) ; // User-defined copy of LPSTR's.

    const T * GetData() const
    {
        return &m_tData ;
    }

    operator const T & () const
    {
        return m_tData ;
    }
} ;

//---------------------------------------------------------------------------

template <class T> const T * SetCurData( T                  * pData ,
        long                 lDataIndex ,
        CCurDataStore<T> * * pp_t )
{
    const T * pRet = NULL ;
    // ----
    if ( pp_t != NULL && pData != NULL )
    {
        CCurDataStore<T> * p_t = *pp_t ;
        // ----
        if ( p_t == NULL || lDataIndex != p_t->GetIndex() ) // Update Needed
        {
            if ( p_t != NULL )
            {
                delete p_t ; // Nuke the old value.
                p_t = NULL ;
            }

            TRY
            {
                p_t = new CCurDataStore<T>() ;

                if ( p_t->SetValue( pData , lDataIndex ) == FALSE )
            {
                delete p_t ; // Better nothing than garbage !
            }
        }
        CATCH(CMemoryException,e)
            {
                p_t = NULL ;
            }
            END_CATCH
        }

        pRet = ( *pp_t = p_t )->GetData() ;
    }

    return pRet ;
}

//---------------------------------------------------------------------------

template <class T> class CCurDataDocViewInterface
{
    CCurDataStore<T> * m_pDataStore ;

    CWnd * m_pControl ; // Child of WM_USER_GETCURVAL target
    BOOL   m_bView ;

public:
CCurDataDocViewInterface() :
    m_pDataStore(NULL) ,
    m_pControl(NULL) ,
    m_bView(FALSE)
    {}


    ~CCurDataDocViewInterface()
    {
        delete m_pDataStore ;
    }

    CWnd * SetTargetChild( CWnd * pTargetChild )
    {
        CWnd * pOld = m_pControl ;
        // ----
        m_pControl = pTargetChild ;
        return m_pControl ;
    }

    void SetTargetView( BOOL bRet )
    {
        m_bView = bRet ;
    }



    const T * GetCurData()
    {
        CWnd * pCtrlParent ;
        if( !m_bView )
            pCtrlParent = ( m_pControl != NULL ? m_pControl->GetParent() : NULL ) ;
        else
            pCtrlParent = ( m_pControl != NULL ? m_pControl : NULL ) ;
        // ----
        ASSERT( pCtrlParent != NULL ) ; // Most probably not initialised (SetTargetChild) !

        if ( pCtrlParent != NULL )
        {
            long  lIndex = m_pDataStore != NULL ? m_pDataStore->GetIndex() : -1L ;
            // ----
            if ( pCtrlParent->SendMessage( WM_USER_GETCURVAL ,
                                           (WPARAM)m_pControl->GetDlgCtrlID() ,
                                           (LPARAM)lIndex ) != (LRESULT)FALSE )
            {
                // Must return TRUE if SetCurData<T>() was succesfully called.
                ASSERT( m_pDataStore != NULL ) ;

                return ( m_pDataStore != NULL ? m_pDataStore->GetData() : NULL ) ;
            }
        }

        return NULL ;
    }

    BOOL SetCurData( T * pData , long lDataIndex )
    {
        return ( ::SetCurData( pData , lDataIndex , reinterpret_cast< CCurDataStore<T> * * >(&m_pDataStore) ) != NULL ) ;
    }

    long GetIndex() const
    {
        return ( m_pDataStore != NULL ? m_pDataStore->GetIndex() : -1L ) ;
    }
} ;

//---------------------------------------------------------------------------

#endif // _DATASTORE_HPP_
