/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// EncodeDecode.cpp: implementation of the CEncodeDecode class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EncodeDecode.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//----------------------------------------------------------------------------

CEncodeDecode::CEncodeDecode()
{
}

//----------------------------------------------------------------------------


CEncodeDecode::~CEncodeDecode()
{
}

//----------------------------------------------------------------------------


BOOL  CEncodeDecode::Encode(const CString& _strInFileName, CString& _strEncode)
{
    CFileStatus rStatus;
    char        buf[80];
    char*       pOutBuf;
    int         i, n;
    int         nIndex;
    long        lAllocSize;


    CFile InFile;
    if ( !InFile.Open(_strInFileName,CFile::modeRead|CFile::typeBinary) )
        return FALSE;

    if ( !InFile.GetStatus( rStatus ) )
    {
        InFile.Close();
        return FALSE;
    }

    if ( rStatus.m_size<=0 )
        return FALSE;

    lAllocSize = long ( rStatus.m_size * 1.42 ); //depression du a l'encodage
    pOutBuf = new char[lAllocSize+1];

    nIndex = 0;
    for (;;)
    {
        /* 1 (up to) LINE_READSIZE character line */
        n = FRead(&InFile, buf, LINE_READSIZE);
        *( pOutBuf + nIndex ) = ENC(n);
        nIndex++;

        for (i=0; i<n; i += 3)
            OutEnc(&buf[i], pOutBuf, nIndex);

        if (n <= 0)
            break;
    }
    *( pOutBuf + nIndex ) = '\0';
    nIndex++;
    _strEncode = pOutBuf;
    delete pOutBuf;
    InFile.Close();

    return TRUE;

}

//----------------------------------------------------------------------------

BOOL CEncodeDecode::Decode(CString& _strEncode, const CString& _strOutFileName)
{
    char buf[80];
    char *bp;
    int n;
    int oldn;

    CFile OutFile;
    if ( !OutFile.Open(_strOutFileName,CFile::modeCreate | CFile::modeWrite |CFile::typeBinary) )
        return FALSE;

    BOOL bStop = FALSE;
    oldn = 0;
    for (;;)
    {
        strncpy(buf,(_strEncode.GetBuffer(0)) + oldn,sizeof buf);
        n = DEC(buf[0]);
        if (n <= 0)
            break;

        if ( n != LINE_READSIZE )
            bStop = TRUE;

        oldn += LINE_ENCSIZE;

        bp = &buf[1];
        while (n > 0)
        {
            OutDec(bp, &OutFile, n);
            bp += 4;
            n -= 3;
        }
        if (bStop)
            break;
    }

    OutFile.Close();
    return TRUE;
}

//----------------------------------------------------------------------------

int CEncodeDecode::FRead(CFile *_pInFile, char *_pBuf, int _cnt)
{
    int i;

    for (i=0; i<_cnt; i++)
    {
        if ( _pInFile->Read(_pBuf+i,1) == 0 )
            return(i);
    }
    return (_cnt);

}

//----------------------------------------------------------------------------

void CEncodeDecode::OutEnc(char *_pBuf, char *_pOutBuf, int& _nIndex)
{
    int c1, c2, c3, c4;

    c1 = *_pBuf >> 2;
    c2 = (*_pBuf << 4) & 060 | (_pBuf[1] >> 4) & 017;
    c3 = (_pBuf[1] << 2) & 074 | (_pBuf[2] >> 6) & 03;
    c4 = _pBuf[2] & 077;
    *(_pOutBuf + _nIndex) = ENC(c1);
    _nIndex++;
    *(_pOutBuf + _nIndex) = ENC(c2);
    _nIndex++;
    *(_pOutBuf + _nIndex) = ENC(c3);
    _nIndex++;
    *(_pOutBuf + _nIndex) = ENC(c4);
    _nIndex++;
}

//----------------------------------------------------------------------------

void CEncodeDecode::OutDec(char *_pBuf, CFile *_pOutFile, int _n)
{
    int c1, c2, c3;

    c1 = DEC(*_pBuf) << 2 | DEC(_pBuf[1]) >> 4;
    c2 = DEC(_pBuf[1]) << 4 | DEC(_pBuf[2]) >> 2;
    c3 = DEC(_pBuf[2]) << 6 | DEC(_pBuf[3]);
    if (_n >= 1)
        _pOutFile->Write(&c1,1);
    if (_n >= 2)
        _pOutFile->Write(&c2,1);
    if (_n >= 3)
        _pOutFile->Write(&c3,1);
}





