/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ColorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ExtendedListBox.h"

#include "resource.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/**********SYL 1.1.00*****************/

/////////////////////////////////////////////////////////////////////////////
// CExtendedListBox
// --------------------------------------------------------------------------
// constructor
// m_nExtent = entent to add to the ListBox
CExtendedListBox::CExtendedListBox( UINT nExtent) :CListBox() ,
        m_nExtent( nExtent ) ,
        m_pBrush( NULL )
{
}

// --------------------------------------------------------------------------


CExtendedListBox::~CExtendedListBox()
{
    if( m_pBrush )
    {
        delete m_pBrush ;
    }
}

// --------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CExtendedListBox, CListBox)
//{{AFX_MSG_MAP(CExtendedListBox)
ON_MESSAGE(LB_GETHORIZONTALEXTENT,OnMsgGetItemRect)
ON_WM_ERASEBKGND()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// --------------------------------------------------------------------------
//Get List Box Item Rect
LRESULT CExtendedListBox::OnMsgGetItemRect( WPARAM wIndex , LPARAM lRectPtr )
{
    CRect * pRect = reinterpret_cast<CRect *>(lRectPtr) ;
    return TRUE ;
}

// --------------------------------------------------------------------------
//Add a string to the list box
int CExtendedListBox::AddString(LPCTSTR lpszItem)
{
    // Get Extent of the text. if this extent is bigger than the ListBox one
    // the listbox extent is changed so that all the text is visible
    LONG lStringExtent = GetExtentForString(lpszItem);
    if(GetHorizontalExtent() < lStringExtent)
        SetHorizontalExtent(lStringExtent + m_nExtent );

    return CListBox::AddString(lpszItem);
}

// --------------------------------------------------------------------------
//Delete an item
int CExtendedListBox::DeleteString( UINT nIndex )
{
    int nCountLeft = CListBox::DeleteString(nIndex);
    if(nCountLeft == LB_ERR)
        return LB_ERR;

    //GetThe bigger extent of the different items in the control
    // and set it for the lisbox
    LONG lMaxExtent = GetExtentForEntireControl();
    SetHorizontalExtent(lMaxExtent + m_nExtent );

    return nCountLeft;
}

// --------------------------------------------------------------------------
// Insert lpszItem at the location nIndex (zero based index )
int CExtendedListBox::InsertString( int nIndex, LPCTSTR lpszItem )
{
    int nInsertPos = CListBox::InsertString(nIndex, lpszItem);
    if(nInsertPos == LB_ERR)
        return LB_ERR;

    // Get Extent of the text. if this extent is bigger than the ListBox one
    // the listbox extent is changed so that all the text is visible
    LONG lStringExtent = GetExtentForString(lpszItem);
    if(GetHorizontalExtent() < lStringExtent)
        SetHorizontalExtent(lStringExtent + m_nExtent );

    return nInsertPos;
}

// --------------------------------------------------------------------------
//Reset the listbox and reset the extent
void CExtendedListBox::ResetContent()
{
    SetHorizontalExtent(m_nExtent);

    CListBox::ResetContent();

    return;
}

// --------------------------------------------------------------------------
//The zero-based index of the last filename added to the list
int CExtendedListBox::Dir( UINT attr, LPCTSTR lpszWildCard )
{
    int nReturn = Dir( attr, lpszWildCard );

    if(nReturn == LB_ERR || nReturn == LB_ERRSPACE)
        return nReturn;

    // Get Extent of the text. if this extent is bigger than the ListBox one
    // the listbox extent is changed so that all the text is visible
    LONG lMaxExtent = GetExtentForEntireControl();
    SetHorizontalExtent(lMaxExtent + m_nExtent );

    return nReturn;
}

// --------------------------------------------------------------------------
// check if the item already exists
int CExtendedListBox::DataAlreadyExist( LPCSTR pczName )
{
    int nCount = GetCount() ;
    int nIndex = 0 ;
    CString strText ;
    //--
    while( nIndex < nCount )
    {
        GetText( nIndex , strText ) ;
        if( stricmp( strText , pczName ) == 0 )
        {
            return nIndex ;
        }
        nIndex ++ ;
    }
    return -1 ;
}

//////////////////////////////////////////////////////////////////////////////////
//
//helper functions
//////////////////////////////////////////////////////////////////////////////////

// --------------------------------------------------------------------------
//Get textmetrics
int WINAPI GetTextMetricsCorrectly( HDC hDC, LPTEXTMETRIC lpTM )
{
    int    nAveWidth ;
    char   rgchAlphabet[52] ;
    int  i ;
    SIZE size ;

    // Get the TM of the current font.  Note that GetTextMetrics
    // gets the incorrect AveCharWidth value for proportional fonts.
    // This is the whole poshort in this exercise.
    //
    if (lpTM)
        GetTextMetrics(hDC, lpTM);

    // If it's not a variable pitch font GetTextMetrics was correct
    // so just return.
    //
    if (lpTM && !(lpTM->tmPitchAndFamily & FIXED_PITCH))
        return lpTM->tmAveCharWidth ;
    else
    {
        for ( i = 0 ; i <= 25 ; i++)
            rgchAlphabet[i] = (char)(i+(int)'a') ;

        for ( i = 0 ; i<=25 ; i++)
            rgchAlphabet[i+25] = (char)(i+(int)'A') ;

        GetTextExtentPoint( hDC, (LPSTR)rgchAlphabet, 52, &size ) ;
        nAveWidth = size.cx / 26 ;
        nAveWidth = (nAveWidth + 1) / 2 ;
    }

    // Return the calculated average char width
    //
    return nAveWidth ;

} /* GetTextMetricsCorrectly()  */

// --------------------------------------------------------------------------
//Get the extent of lpszItem
LONG CExtendedListBox::GetExtentForString(LPCTSTR lpszItem)
{
    CDC* cdc = this->GetDC();
    TEXTMETRIC tm;
    GetTextMetricsCorrectly( cdc->m_hDC , &tm ) ;
    //cdc->GetTextMetrics(&tm);

    //calculate extent for current font
    CSize size = cdc->GetOutputTextExtent(lpszItem);
    size.cx += tm.tmAveCharWidth;

    this->ReleaseDC(cdc);

    return (LONG) size.cx;

}

// --------------------------------------------------------------------------

LONG CExtendedListBox::GetExtentForEntireControl()
{
    CDC* cdc = this->GetDC();
    TEXTMETRIC tm;
    //cdc->GetTextMetrics(&tm);
    GetTextMetricsCorrectly( cdc->m_hDC , &tm ) ;

    CString sText;
    LONG max_cx = 0;

    //do for everey list entry
    for(int n = 0; n<GetCount(); n++)
    {
        this->GetText(n, sText);
        if(sText != "")
        {
            //calcualte extent for current font
            CSize size = cdc->GetOutputTextExtent(sText);
            size.cx += tm.tmAveCharWidth;

            //set max extent
            if(max_cx <= size.cx)
                max_cx = size.cx;
        }
    }

    this->ReleaseDC(cdc);

    return max_cx;
}

// --------------------------------------------------------------------------

BOOL CExtendedListBox::OnEraseBkgnd( CDC* pDC )
{
    if ( m_pBrush == NULL )
    {
        m_pBrush = new CBrush( GetSysColor( COLOR_WINDOW ) ) ;
    }

    CRect  rc ;
    // ----
    GetClientRect( &rc ) ;

    pDC->FillRect( &rc , m_pBrush ) ;

    return TRUE ;
}

// --------------------------------------------------------------------------

BOOL CExtendedListBox::CreateEx(DWORD dwExStyle, DWORD dwStyle, const CRect & rect, CWnd * pParent, UINT
                                nId)
{
    CWnd* pWnd = this;

    return pWnd->CreateEx(dwExStyle, _T("LISTBOX"), NULL,
                          dwStyle, rect, pParent, nId);
}

