/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ExtendedRichEditCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "cyrres.h"
#include "ExtendedRichEditCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern DWORD CALLBACK _GXEditStreamInCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb);
extern DWORD CALLBACK _GXEditStreamOutCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb);

BEGIN_MESSAGE_MAP(CExtendedRichEditCtrl, CRichEditCtrl)
//{{AFX_MSG_MAP(CExtendedRichEditCtrl)
// NOTE - the ClassWizard will add and remove mapping macros here.
ON_WM_CONTEXTMENU()
ON_WM_RBUTTONDOWN()
ON_COMMAND(ID_CEXTENDEDRICHEDITCTRL_UNDO, OnUnDo)
ON_COMMAND(ID_CEXTENDEDRICHEDITCTRL_CUT, OnCut)
ON_COMMAND(ID_CEXTENDEDRICHEDITCTRL_COPY, OnCopy)
ON_COMMAND(ID_CEXTENDEDRICHEDITCTRL_PASTE, OnPaste)
ON_COMMAND(ID_CEXTENDEDRICHEDITCTRL_FORMAT, OnFormat)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()


//----------------------------------------------------------------------------------------

BOOL CCharFormat::operator==(CCharFormat& cf)
{
    return
        dwMask == cf.dwMask
        && dwEffects == cf.dwEffects
        && yHeight == cf.yHeight
        && yOffset == cf.yOffset
        && crTextColor == cf.crTextColor
        && bPitchAndFamily == cf.bPitchAndFamily
        && (lstrcmpA(szFaceName, cf.szFaceName) == 0);
}


//----------------------------------------------------------------------------------------
//m_bGray = TRUE if background color = gray false otherwise
CExtendedRichEditCtrl::CExtendedRichEditCtrl()
        :   m_bGray(FALSE),
        m_bAsAControl(TRUE)
{}

//----------------------------------------------------------------------------------------


CExtendedRichEditCtrl::~CExtendedRichEditCtrl()
{}

//----------------------------------------------------------------------------------------
// Set background gray

void CExtendedRichEditCtrl::SetGrayBackGround()
{
    m_bGray = TRUE ;
}

//----------------------------------------------------------------------------------------
void CExtendedRichEditCtrl::SetAsAControl(BOOL _bAsAControl)
{
    m_bAsAControl = _bAsAControl ;
}


//----------------------------------------------------------------------------------------
// called just before creation
void CExtendedRichEditCtrl::PreSubclassWindow()
{
    CRichEditCtrl::PreSubclassWindow();
    //Set the backGround to Gray or to System value
    //$$$JL ce bout de code honteux est du a un bug
    //on ne peut definir la couleur de fond que lorsque le controle
    //est utilise dans une dlg mais pas dans une vue
    if ( m_bAsAControl )
        SetBackgroundColor(  IsWindowEnabled() || ! m_bGray, ::GetSysColor( COLOR_BTNFACE ));
}

//----------------------------------------------------------------------------------------

void CExtendedRichEditCtrl::OnContextMenu( CWnd* pWnd, CPoint pos )
{}

//----------------------------------------------------------------------------------------
// Load Menu popup

void CExtendedRichEditCtrl::OnRButtonDown( UINT nFlags, CPoint point )
{
    CMenu   menu ;
    CMenu*  pSubMenu = NULL;

    // Load Menu
    if ( ! menu.LoadMenu(IDR_EXTEDIT_MENU) )
        return ;

    // Get PopupMenu
    pSubMenu = menu.GetSubMenu( 0 ) ;
    ASSERT( pSubMenu != NULL ) ;

    for ( int i=0; i<pSubMenu->GetMenuItemCount(); i++)
    {
        // Update item status (enable or not )
        UpdateState(pSubMenu, pSubMenu->GetMenuItemID(i));
    }

    ClientToScreen(&point);

    //Display the menu
    pSubMenu->TrackPopupMenu(
        TPM_RIGHTALIGN|
        TPM_RIGHTBUTTON|
        TPM_LEFTALIGN , point.x, point.y, this, NULL ) ;
}

//----------------------------------------------------------------------------------------

//Update the status of the item _nId of the menu _pMenu
void CExtendedRichEditCtrl::UpdateState(CMenu *_pMenu, UINT _nId)
{
    UINT iMenuState = MF_DISABLED|MF_GRAYED;
    BOOL bDoIt = TRUE;
    switch ( _nId )
    {
    case ID_CEXTENDEDRICHEDITCTRL_UNDO:
        iMenuState = ( CanUndo() ? MF_ENABLED : MF_DISABLED|MF_GRAYED);
        break;
    case ID_CEXTENDEDRICHEDITCTRL_CUT:
        iMenuState = MF_ENABLED;
        break;
    case ID_CEXTENDEDRICHEDITCTRL_COPY:
        iMenuState = MF_ENABLED;
        break;
    case ID_CEXTENDEDRICHEDITCTRL_PASTE:
        iMenuState = ( CanPaste() ? MF_ENABLED : MF_DISABLED|MF_GRAYED);
        break;
    case ID_CEXTENDEDRICHEDITCTRL_FORMAT:
        iMenuState = MF_ENABLED;
        break;
    default:
        bDoIt = FALSE;
        break;
    }

    if ( bDoIt )
        _pMenu->EnableMenuItem(_nId, iMenuState);

}

//----------------------------------------------------------------------------------------
// popup menu : undo
void CExtendedRichEditCtrl::OnUnDo()
{
    Undo();
}

//----------------------------------------------------------------------------------------
// popup menu : cut
void CExtendedRichEditCtrl::OnCut()
{
    Cut();
}

//----------------------------------------------------------------------------------------
// popup menu : copy
void CExtendedRichEditCtrl::OnCopy()
{
    Copy();
}

//----------------------------------------------------------------------------------------
// popup menu : paste
void CExtendedRichEditCtrl::OnPaste()
{
    Paste();
}

//----------------------------------------------------------------------------------------
// popup menu : font format
void CExtendedRichEditCtrl::OnFormat()
{
    CCharFormat mf;

    GetSelectionCharFormat(mf);

    CFontDialog dlg(mf, CF_BOTH|CF_NOOEMFONTS);
    if (dlg.DoModal() == IDOK)
    {
        dlg.GetCharFormat(mf);
        SetSelectionCharFormat(mf);
    }


}

//----------------------------------------------------------------------------------------
// set the text szRTF to format RTF
void CExtendedRichEditCtrl::SetRTFText(LPCSTR szRTF)
{
    EditStreamInStruct es = { szRTF, strlen(szRTF), 0 };
    EDITSTREAM esin = {(DWORD) &es, 0, _GXEditStreamInCallback};
    StreamIn(SF_RTF, esin);
}

//----------------------------------------------------------------------------------------
// Get the text to format RTF
CString CExtendedRichEditCtrl::GetRTFText( CString &sRTFResult )
{
    // Retrieve data from Rich Text Control
    EditStreamOutStruct es = { &sRTFResult, 0 };

    EDITSTREAM esResult = {(DWORD) (&es), 0, _GXEditStreamOutCallback};
    // NOTE: dwCookie is assigned a pointer to sResult !
    // EditStreamOutCallback will use this pointer and copy
    // the RTF data into the string
    StreamOut(SF_RTF, esResult);

    return sRTFResult;
}
