/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ExtendedWinApp.cpp: implementation of the CExtendedWinApp class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "cyrres.h"
#include "RootDocument.h"
#include "StubDocument.h"
#include "ExtendedMultiDocTemplate.h"
#include "ProgDlg.h"
#include "Splash.h"
#include "ExtendedWinApp.h"


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CExtendedWinApp::CExtendedWinApp(LPCTSTR lpszAppName)
        :   CWinApp(lpszAppName),
        m_pRootDocument(NULL)        ,
        m_pWaitingProcess(NULL)
{}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

CExtendedWinApp::~CExtendedWinApp()
{
    if ( m_pWaitingProcess != NULL )
        delete m_pWaitingProcess;
    m_pWaitingProcess = NULL;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//Progress bar Initialisation
void CExtendedWinApp::StartWaitingProcess(CString _strTitle, UINT _iTotal, BOOL _bEnableCancel)
{
    ASSERT ( m_pWaitingProcess == NULL );
    m_pWaitingProcess = new CProgressDlg( NULL , _strTitle, _iTotal, _bEnableCancel );
    m_pWaitingProcess->Create( NULL ) ;

}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//step of the progress bar
BOOL CExtendedWinApp::AdvanceWaitingProcess(LPCSTR _pczBuffer)
{
    ASSERT ( m_pWaitingProcess != NULL );
    if ( m_pWaitingProcess != NULL )
        m_pWaitingProcess->StepIt(_pczBuffer);

    return m_pWaitingProcess->m_bCancel;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// stop the progress bar
void CExtendedWinApp::EndWaitingProcess()
{
    ASSERT ( m_pWaitingProcess != NULL );

    if ( m_pWaitingProcess != NULL )
        delete m_pWaitingProcess;
    m_pWaitingProcess = NULL;

}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Init instances of the Application
BOOL CExtendedWinApp::InitInstance()
{

    CSplashWnd::EnableSplashScreen(TRUE);

    TRY
    {
    }
    CATCH(CMemoryException,e)
    {}

    END_CATCH

    if( !AfxOleInit() )
    {
        TRACE(_T("Unable to Initialize OLE\n"));
        return FALSE;
    }

    AfxInitRichEdit();

    return TRUE;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//Stop The Application
int CExtendedWinApp::ExitInstance()
{
    return TRUE;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//Init The main Document. This document is essential. All other documents
// communicate with this document
void CExtendedWinApp::SetRootDocument( CRootDocument * _pDoc )
{
    m_pRootDocument = _pDoc ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//Get the main document
CRootDocument * CExtendedWinApp::GetRootDocument() const
{
    return m_pRootDocument ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//Add one document to the associated document m_pRootDocument

CExtendedMultiDocTemplate *CExtendedWinApp::AddOneTemplate(   UINT       _nIDResource,
        CRuntimeClass       * _pDocClass  ,
        CRuntimeClass       * _pFrameClass,
        CRuntimeClass       * _pViewClass )
{
    CExtendedMultiDocTemplate *pTemplate;

    ASSERT( _pDocClass    != NULL ) ;
    ASSERT( _pFrameClass  != NULL ) ;
    ASSERT( _pViewClass   != NULL ) ;

    pTemplate = new CExtendedMultiDocTemplate(   _nIDResource   ,
                _pDocClass      ,
                _pFrameClass    ,
                _pViewClass );

    AddDocTemplate( pTemplate ) ;

    return pTemplate;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//Activate Frame
void  CExtendedWinApp::ActivateTemplate(  CExtendedMultiDocTemplate *_pTemplate )
{

    if ( !( GetRootDocument()->HasDocTemplate( _pTemplate ) ) )
    {
        _pTemplate->SetRootDocument( GetRootDocument() ) ;
        _pTemplate->OpenDocumentFile(NULL) ;
    }
    else
    {
        POSITION    pos  = _pTemplate->GetFirstDocPosition() ;
        ASSERT ( pos != NULL );

        CStubDocument* pDoc = static_cast<CStubDocument *>(_pTemplate->GetNextDoc( pos )) ;

        ActivateDocumentFrame(pDoc);
    }

}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//Activate Frame
CFrameWnd * CExtendedWinApp::ActivateDocumentFrame( CStubDocument * _pDoc ) const
{
    CFrameWnd * pFrame = NULL ;
    // ----
    if ( _pDoc == NULL )
        return NULL;

    POSITION pos = _pDoc->GetFirstViewPosition();
    if ( pos == NULL )
        return NULL;

    CView * pView  = _pDoc->GetNextView( pos ) ;
    ASSERT_VALID(pView);

    pFrame = pView->GetParentFrame() ;
    if ( pFrame == NULL )
        return NULL;

    pFrame->ActivateFrame() ;
    if ( pFrame->IsIconic() )
        pFrame->ShowWindow( SW_RESTORE ) ;
    else
        TRACE0("Error: Can not find a frame for document to activate.\n") ;

    return pFrame ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//Close all documents associated with m_pRootDocument
void CExtendedWinApp::CloseDependents( CRootDocument * _pDoc )
{
    POSITION posTplt = GetFirstDocTemplatePosition() ;

    //parcours des templates
    for ( ; posTplt != (POSITION)NULL ;  )
    {
        CDocTemplate * pDocTemplate = GetNextDocTemplate( posTplt ) ;
        ASSERT( pDocTemplate != NULL ) ;

        //parcours des documents
        POSITION posDoc = pDocTemplate->GetFirstDocPosition() ;
        for ( ; posDoc != (POSITION)NULL ;)
        {
            CDocument * pDoc = pDocTemplate->GetNextDoc( posDoc ) ;
            ASSERT( pDoc != NULL ) ;
            if ( ! pDoc->IsKindOf(RUNTIME_CLASS(CStubDocument) ) )
                continue;
            if ( static_cast<CStubDocument *>(pDoc)->GetRootDocument() != _pDoc )
                continue;

            //parcours des vues
            POSITION posView = pDoc->GetFirstViewPosition();
            for ( ; posView != (POSITION)NULL ; )
            {
                CView * pView = pDoc->GetNextView(posView);
                ASSERT( pView != NULL ) ;
                if ( pView != NULL )
                    pView->GetParentFrame()->DestroyWindow() ;
            }
        }
    }
}


// --------------------------------------------------------------------------

//Intercept all the messages and particularly the one with the splash

BOOL CExtendedWinApp::PreTranslateMessage(MSG* pMsg)
{

    if ( CSplashWnd::PreTranslateAppMessage(pMsg) )
        return TRUE;

    return CWinApp::PreTranslateMessage(pMsg);
}

