/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// FilterItem.cpp: implementation of the CFilterItem class.
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "FilterItem.h"

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CFilterItem::CFilterItem() :
        m_LoOp(opNone)   ,
        m_HiOp(opNone)   ,
        m_pLogItem(NULL) ,
        m_cLogOp(lopNone),
        m_dwUserData(0L)
{}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

CFilterItem::~CFilterItem()
{}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


CFilterItem & CFilterItem::operator = ( const CFilterItem & source )
{
    m_LoOp       = source.m_LoOp ;
    m_HiOp       = source.m_HiOp ;
    m_pLogItem   = source.m_pLogItem ;
    m_cLogOp     = source.m_cLogOp ;
    m_dwUserData = source.m_dwUserData ;
    //
    m_strLoVal   = source.m_strLoVal ;
    m_strHiVal   = source.m_strHiVal ;

    return *this ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
BOOL CFilterItem::SetRange( LPCTSTR pczVal , Op op , LPCTSTR pczHiVal /*=NULL*/ , Op op2 /*=opNone*/ )
{
    BOOL bRet = TRUE ;
    // ----
    TRY
    {
        m_strLoVal = pczVal ; // Floor
        m_LoOp     = op ;

        if ( pczHiVal != NULL )
    {
        m_strHiVal = pczHiVal ; // Ceiling
        m_HiOp     = op2 ;
    }
}
CATCH(CMemoryException,e)
    {
        bRet = FALSE ;
    }
    END_CATCH

    return bRet ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
BOOL CFilterItem::GetRange( LPCTSTR * p_pczVal , Op * p_op , LPCTSTR * p_pczHiVal /*=NULL*/ , Op * p_op2 /*=NULL*/ ) const
{
    if ( p_pczVal != NULL )
        *p_pczVal = m_strLoVal ;

    if ( p_op != NULL )
        *p_op = m_LoOp ;

    if ( p_pczHiVal != NULL )
        *p_pczHiVal = m_strHiVal ;

    if ( p_op2 != NULL )
        *p_op2 = m_HiOp ;

    return TRUE ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void CFilterItem::SetUserData( DWORD dwUserData )
{
    m_dwUserData = dwUserData ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
DWORD CFilterItem::GetUserData() const
{
    return m_dwUserData ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

void CFilterItem::SetLogItem( CFilterItem * pLogItem , LogOp lop )
{
    m_pLogItem = pLogItem ;

    if ( m_pLogItem == NULL )
    {
        m_cLogOp = lopNone ;
    }
    else
    {
        ASSERT( lop == lopAnd || lop == lopOr || lop == lopNot ) ;
        if ( ( lop & lopAnd ) == lopAnd )
            m_cLogOp &= ~lopOr ;
        else if ( ( lop & lopOr ) == lopOr )
            m_cLogOp &= ~lopAnd ;

        m_cLogOp |= lop ;
    }
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CFilterItem & CFilterItem::And( CFilterItem * pLogItem )
{
    SetLogItem( pLogItem , lopAnd ) ;
    return * this ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CFilterItem & CFilterItem::Or( CFilterItem * pLogItem )
{
    SetLogItem( pLogItem , lopOr ) ;
    return * this ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CFilterItem & CFilterItem::Not( CFilterItem * pLogItem )
{
    SetLogItem( pLogItem , lopNot ) ;
    return * this ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
BOOL CFilterItem::GetLogOp( LogOp lop /*=lopNone*/ ) const
{
    if ( lop == lopNone ) // Default means "Is there a logical operator ?"
        return ( m_cLogOp != lopNone ) ;

    if ( ( m_cLogOp & lop ) == lop )
        return TRUE ;

    return FALSE ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CFilterItem * CFilterItem::GetLogItem( LogOp * plop /*=NULL*/ ) const
{
    if ( plop != NULL )
    {
        *plop = static_cast<LogOp>(m_cLogOp) ;
    }
    return m_pLogItem ;
}

