/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"

#include "IDynMenu32.h"

#include "CyrRes.h"

// --------------------------------------------------------------------------

CDynMenu32DefaultImpl::CDynMenu32DefaultImpl() :
        m_pClient(NULL)
{}

// --------------------------------------------------------------------------


CDynMenu32DefaultImpl::~CDynMenu32DefaultImpl()
{}

// --------------------------------------------------------------------------

//create default menu popup :
//pClient = parent window
// nFullMenuID = the menu ID in the resource
// nSubMenuIndex = Sub menu Index
// dwMask = Mask

HRESULT CDynMenu32DefaultImpl::Create( IDynMenu32Client * pClient ,
                                       UINT               nFullMenuID ,
                                       int                nSubMenuIndex ,
                                       DWORD              dwMask )
{
    HRESULT hRes = E_INVALIDARG ;
    // ----
    if ( pClient != NULL && nFullMenuID != 0 )
    {
        CMenu  mnuFullChoice ;
        CWnd * pClientWnd     = NULL ;
        DWORD  dwDisableMask  = 0L ;
        DWORD  dwCheckMask    = 0L ;
        bool   bLastItemIsSep = true ; // Top of menu is assimilated to a separator.
        // ----
        m_pClient = pClient ;

        if ( SUCCEEDED( hRes = m_pClient->GetClientWindow( &pClientWnd ) ) )
        {
            CMenu * pSubMenu = NULL ;
            // ----
            ASSERT( pClientWnd != NULL) ;

            // LoadMenu temporary menu
            if ( mnuFullChoice.LoadMenu( nFullMenuID ) != FALSE &&
                    // Get temporary Sub Menu associated
                    ( pSubMenu = mnuFullChoice.GetSubMenu( nSubMenuIndex ) ) != NULL &&
                    //create the popup menu
                    m_menu.CreatePopupMenu() != FALSE )
            {
                const UINT nItemCount  = pSubMenu->GetMenuItemCount() ;
                UINT       nItemIndex  = 0 ;     // Does not take separators into account.
                bool       bSepPending = false ; // Separator needed before next item.
                // ----
                for ( int i = 0 ; i < static_cast<int>(nItemCount) && nItemIndex < 32 ; i++ )
                {
                    // Get each menuitem
                    int nItemID = pSubMenu->GetMenuItemID(i) ;
                    // ----
                    if ( nItemID == ID_SEPARATOR )
                    {
                        if ( bLastItemIsSep == false )
                        {
                            bLastItemIsSep = true ;
                            bSepPending    = true ;
                        }
                    }
                    else
                    {
                        DWORD dwItemMask = ( 0x00000001 << nItemIndex ) ;
                        // ----
                        ASSERT( nItemID > 0 ) ; // The specified item is a pop-up menu (as opposed to an item within the pop-up menu) !

                        if ( ( dwItemMask & dwMask ) != 0 )
                        {
                            CString strSubMenu ;
                            // ----
                            //Get the label associated with the temp menuitem
                            pSubMenu->GetMenuString( nItemID, strSubMenu , MF_BYCOMMAND ) ;

                            //Fill the menu m_menu with pSubMenu Items

                            //Separator
                            if ( bSepPending == true &&
                                    m_menu.AppendMenu( MF_SEPARATOR , 0 , strSubMenu ) == FALSE )
                            {
                                hRes = E_UNEXPECTED ;
                                goto Fail ; // -- !! --
                            }

                            //string item

                            if ( m_menu.AppendMenu( MF_STRING , nItemID , strSubMenu ) == FALSE )
                            {
                                hRes = E_UNEXPECTED ;
                                goto Fail ; // -- !! --
                            }

                            //init state

                            if ( ( dwItemMask & dwDisableMask ) != 0 )
                            {
                                m_menu.EnableMenuItem( nItemID , MF_DISABLED|MF_BYCOMMAND ) ;
                            }
                            else
                            {
                                m_menu.EnableMenuItem( nItemID , MF_ENABLED|MF_BYCOMMAND ) ;
                            }

                            if ( ( dwItemMask & dwCheckMask ) != 0 )
                            {
                                m_menu.CheckMenuItem( nItemID , MF_CHECKED|MF_BYCOMMAND ) ;
                            }
                            else
                            {
                                m_menu.CheckMenuItem( nItemID , MF_UNCHECKED|MF_BYCOMMAND ) ;
                            }
                        }

                        nItemIndex++ ;
                        ASSERT( nItemIndex < 32 ) ; // Limit reached ! Is it legible anyway ?
                    }

                }
            }
        }
    }

Fail: // -- !! --

    return hRes ;
}

// --------------------------------------------------------------------------

//Display the popup menu
HRESULT CDynMenu32DefaultImpl::DoPopup( const CPoint * pScreenPoint )
{
    HRESULT hRes = E_INVALIDARG ;
    // ----
    ASSERT( m_pClient != NULL ) ; // Looks like 'Create()' was never called !

    if ( m_pClient != NULL && pScreenPoint != NULL )
    {
        CWnd * pTargetWnd = NULL ;
        // ----
        //Get the window with wich the menu is associated
        if ( SUCCEEDED( hRes = m_pClient->GetCmdMsgTargetWindow( &pTargetWnd ) ) &&
                //Get the specific menu of this window
                SUCCEEDED( hRes = m_pClient->OnPreMenuPopup( &m_menu ) ) )
        {
            // Make sure no one has been fiddling with our Cmenu object :
            ASSERT( ::AfxIsValidAddress( &m_menu , sizeof( m_menu ) ) ) ;

            // display the menu at the desired position
            if ( m_menu.TrackPopupMenu( TPM_RIGHTALIGN|TPM_RIGHTBUTTON|TPM_LEFTALIGN ,
                                        pScreenPoint->x ,
                                        pScreenPoint->y ,
                                        pTargetWnd ,
                                        NULL ) == FALSE )
            {
                ASSERT(FALSE) ;
                hRes = E_UNEXPECTED ;
            }
        }
    }

    return hRes ;
}

// --------------------------------------------------------------------------

//create our own menu object

HRESULT CDynMenu32DefaultImpl::CreateObject(  CDynMenu32DefaultImpl * * ppDynMenu32 )
{
    HRESULT hRes = S_OK ;
    // ----
    if ( ppDynMenu32 != NULL )
    {
        ASSERT( *ppDynMenu32 == NULL ) ; // Are you sure this pointer is really free ?

        TRY
        {
            *ppDynMenu32 = new CDynMenu32DefaultImpl() ;
        }
        CATCH(CMemoryException,e)
        {
            ASSERT(FALSE) ;
            hRes         = E_OUTOFMEMORY ;
            *ppDynMenu32 = NULL ;
        }
        END_CATCH
    }
    else
    {
        hRes = E_INVALIDARG ;
    }

    return hRes ;
}

// --------------------------------------------------------------------------

//destroy our own menu object
HRESULT CDynMenu32DefaultImpl::DestroyObject( CDynMenu32DefaultImpl * * ppDynMenu32 )
{
    if ( ppDynMenu32 != NULL )
    {
        ASSERT( AfxIsValidAddress( *ppDynMenu32 , sizeof( CDynMenu32DefaultImpl ) ) ) ;

        delete *ppDynMenu32 ;
        *ppDynMenu32 = NULL ;
        return S_OK ;
    }
    return E_INVALIDARG ;
}

// --------------------------------------------------------------------------
// Global helpers :

//  create the pClient popup Menu
//  display it
//  destroy it
HRESULT CyrLibDoPopupMenu( IDynMenu32Client * pClient ,
                           UINT               nFullMenuID ,
                           int                nSubMenuIndex ,
                           const CPoint     * pScreenPoint ,
                           DWORD              dwMask )
{
    /*$$$OS   IDynMenu32 * pDynMenu32 = NULL ;
       HRESULT      hRes       = CyrLibGetClassFactory()->CreateObject( &pDynMenu32 ) ;
       // ----
       if ( SUCCEEDED( hRes ) )
          {
          ASSERT( pDynMenu32 != NULL ) ;
     
          if ( SUCCEEDED( hRes = pDynMenu32->Create( pClient , nFullMenuID , nSubMenuIndex , dwMask ) ) )
             {
             hRes = pDynMenu32->DoPopup( pScreenPoint ) ;
             }
          
          CyrLibGetClassFactory()->DestroyObject( pDynMenu32 , NULL ) ;
          }
     
       return hRes ;*/
    return 0;
}

// --------------------------------------------------------------------------

HRESULT CyrLibDoDefaultPopupMenu( IDynMenu32Client * pClient ,
                                  const CPoint     * pScreenPoint ,
                                  DWORD              dwMask /*=CDynMenu32DefaultImpl::mi_data_group*/ )
{
    return CyrLibDoPopupMenu( pClient , IDM_FULL_CHOICE , 0 , pScreenPoint , dwMask ) ;
}

// --------------------------------------------------------------------------

void CyrLibOnInitMenuPopup( CWnd * pThis , CMenu * pMenu, BOOL bSysMenu )
{
    ASSERT( pThis != NULL ) ;
    ASSERT( pMenu != NULL ) ;

    if (bSysMenu)
        return;     // don't support system menu

    ASSERT(pMenu != NULL);
    // check the enabled state of various menu items

    CCmdUI state;
    state.m_pMenu = pMenu;
    ASSERT(state.m_pOther == NULL);
    ASSERT(state.m_pParentMenu == NULL);

    // determine if menu is popup in top-level menu and set m_pOther to
    //  it if so (m_pParentMenu == NULL indicates that it is secondary popup)
    HMENU hParentMenu;
    if (AfxGetThreadState()->m_hTrackingMenu == pMenu->m_hMenu)
        state.m_pParentMenu = pMenu;    // parent == child for tracking popup
    else if ((hParentMenu = ::GetMenu(pThis->m_hWnd)) != NULL)
    {
        CWnd* pParent = pThis->GetTopLevelParent();
        // child windows don't have menus -- need to go to the top!
        if (pParent != NULL &&
                (hParentMenu = ::GetMenu(pParent->m_hWnd)) != NULL)
        {
            int nIndexMax = ::GetMenuItemCount(hParentMenu);
            for (int nIndex = 0; nIndex < nIndexMax; nIndex++)
            {
                if (::GetSubMenu(hParentMenu, nIndex) == pMenu->m_hMenu)
                {
                    // when popup is found, m_pParentMenu is containing menu
                    state.m_pParentMenu = CMenu::FromHandle(hParentMenu);
                    break;
                }
            }
        }
    }

    state.m_nIndexMax = pMenu->GetMenuItemCount();
    for (state.m_nIndex = 0; state.m_nIndex < state.m_nIndexMax;
            state.m_nIndex++)
    {
        state.m_nID = pMenu->GetMenuItemID(state.m_nIndex);
        if (state.m_nID == 0)
            continue; // menu separator or invalid cmd - ignore it

        ASSERT(state.m_pOther == NULL);
        ASSERT(state.m_pMenu != NULL);
        if (state.m_nID == (UINT)-1)
        {
            // possibly a popup menu, route to first item of that popup
            state.m_pSubMenu = pMenu->GetSubMenu(state.m_nIndex);
            if (state.m_pSubMenu == NULL ||
                    (state.m_nID = state.m_pSubMenu->GetMenuItemID(0)) == 0 ||
                    state.m_nID == (UINT)-1)
            {
                continue;       // first item of popup can't be routed to
            }

            state.DoUpdate( pThis , FALSE);    // popups are never auto disabled
        }

        else
        {
            // normal menu item
            // Auto enable/disable if frame window has 'm_bAutoMenuEnable'
            //    set and command is _not_ a system command.
            state.m_pSubMenu = NULL;
            state.DoUpdate(pThis, /* m_bAutoMenuEnable && */ state.m_nID < 0xF000);
        }                       // -PHL

        // adjust for menu deletions and additions
        UINT nCount = pMenu->GetMenuItemCount();
        if (nCount < state.m_nIndexMax)
        {
            state.m_nIndex -= (state.m_nIndexMax - nCount);
            while (state.m_nIndex < nCount &&
                    pMenu->GetMenuItemID(state.m_nIndex) == state.m_nID)
            {
                state.m_nIndex++;
            }
        }
        state.m_nIndexMax = nCount;
    }
}

// --------------------------------------------------------------------------
