#ifndef _CYR_ENTRY_GRID_H_
#define _CYR_ENTRY_GRID_H_
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "ExtendedGridWnd.h"

//--------------------------------------------------------------------
interface IEntryGrid ;

interface IEntryGridClient
{
    virtual HRESULT GetWnd( /*[out]*/ CWnd * * ppWnd ) const = 0 ;

    // The grid will notify its client of value editing. If not accepted, the modification
    // will be reversed.
    virtual HRESULT OnEntryGridEndEditing( /*[in]*/  IEntryGrid * pList ,
            /*[in]*/  DWORD        dwCookie ,
            /*[out]*/ BOOL       * pbAcceptVal ) const = 0 ;

    // Whenever current selection changes.
    virtual HRESULT OnEntryGridSelChange( /*[in]*/  IEntryGrid * pList ,
            /*[in]*/  DWORD        dwCookie ) const = 0 ;

    // User double-clicked or pressed 'enter'.
    virtual HRESULT OnEntryGridActivateItem( /*[in]*/  IEntryGrid * pList ,
            /*[in]*/  DWORD        dwCookie ,
            /*[in]*/  WORD         wCol ) const = 0 ;

    // On creation the list will ask its client about its area.
    // This information can be provided through either a RECT or
    // the ID of a control used as template (this control may be
    // detroyed by the list when it has completed creation).
    // The list may not implement either method (but at least one)
    // and 'Create()' may return E_NOTIMPL if provide with the wrong
    // type of information.
    virtual HRESULT GetEntryGridCreateInfo( /*[out]*/ LPRECT       pClientRect ,
            /*[out]*/ UINT       * pnAreaID ,
            /*[out]*/ UINT       * pnHeaderID ,
            /*[in]*/  IEntryGrid * pList ) const = 0 ;

    virtual HRESULT HandleEntryGridRow( /*[in]*/  IEntryGrid * pList ,
            /*[in]*/  DWORD        dwCookie ,
            /*[in]*/  DWORD        dwIndex ,
            /*[out]*/ BOOL       * pbContinue ) = 0 ;
} ;

//--------------------------------------------------------------------

interface IEntryGrid
{
    virtual HRESULT GetRowCount( /*[out]*/ DWORD * dwRowCount ) const = 0 ;
    virtual HRESULT GetColCount( /*[out]*/ DWORD * dwColCount ) const = 0 ;

    virtual HRESULT Create( /*[in]*/ IEntryGridClient * pClient ) = 0 ;

    // If sucessful 'AddRow()' returns a cookie that will provide acces to
    // the new row as long as it is in existence.
    virtual HRESULT AddRow( /*[out]*/ DWORD * pdwCookie ) = 0 ;

    // If sucessful 'AddRow()' returns a cookie that will provide acces to
    // the new row as long as it is in existence.
    virtual HRESULT RemoveRow( /*[out]*/ DWORD * pdwCookie ) = 0 ;

    virtual HRESULT GetValue( /*[in]*/ DWORD pdwCookie ,
                                       /*[in]*/ WORD  wCol ) = 0 ;

    virtual HRESULT SetValue( /*[in]*/ DWORD pdwCookie ,
                                       /*[in]*/ WORD  wCol ) = 0 ;

    // Enumerates contents by callig 'pClient->HandleEntryGridRow()' for
    // every row present.
    virtual HRESULT Enum( IEntryGridClient * pClient , DWORD * pdwEnumCount ) = 0 ;
} ;

//--------------------------------------------------------------------

#endif // _CYR_ENTRY_GRID_H_
