/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//  ProgDlg.cpp : implementation file
// CG: This file was added by the Progress Dialog component

#include "stdafx.h"
#include "CyrRes.h"
#include "ProgDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CProgressDlg dialog

//pWnd = Parent

CProgressDlg::CProgressDlg(CWnd *pWnd , CString strTitle , UINT nTotal, BOOL _bEnableCancel, UINT nStep )
{
    m_strCaption = strTitle ;

    m_bCancel=FALSE;
    m_nLower=0;
    m_nUpper=nTotal;
    m_bEnableCancel = _bEnableCancel;
    m_nStep=nStep;
    //{{AFX_DATA_INIT(CProgressDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_bParentDisabled = FALSE;
}

//-----------------------------------------------------------------------

CProgressDlg::~CProgressDlg()
{
    if(m_hWnd!=NULL)
        DestroyWindow();
}

//-----------------------------------------------------------------------

BOOL CProgressDlg::DestroyWindow()
{
    ReEnableParent();
    return CDialog::DestroyWindow();
}

//-----------------------------------------------------------------------

//Focus is put on the progress bar's parent

void CProgressDlg::ReEnableParent()
{
    if(m_bParentDisabled && (m_pParentWnd!=NULL))
        m_pParentWnd->EnableWindow(TRUE);
    m_bParentDisabled=FALSE;
}

//-----------------------------------------------------------------------

BOOL CProgressDlg::Create(CWnd *pParent)
{
    // Get the true parent of the dialog
    m_pParentWnd = CWnd::GetSafeOwner(pParent);

    // m_bParentDisabled is used to re-enable the parent window
    // when the dialog is destroyed. So we don't want to set
    // it to TRUE unless the parent was already enabled.

    if((m_pParentWnd!=NULL) && m_pParentWnd->IsWindowEnabled())
    {
        m_pParentWnd->EnableWindow(FALSE);
        m_bParentDisabled = TRUE;
    }

    if(!CDialog::Create(CProgressDlg::IDD,pParent))
    {
        ReEnableParent();
        return FALSE;
    }

    return TRUE;
}

//-----------------------------------------------------------------------

void CProgressDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CProgressDlg)
    DDX_Control(pDX, CG_IDC_PROGDLG_PROGRESS, m_Progress);
    //}}AFX_DATA_MAP
}

//-----------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CProgressDlg, CDialog)
//{{AFX_MSG_MAP(CProgressDlg)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()


//-----------------------------------------------------------------------

void CProgressDlg::OnCancel()
{
    m_bCancel=TRUE ;
}

//-----------------------------------------------------------------------

//Set the range of the progress bar

void CProgressDlg::SetRange(int nLower,int nUpper)
{
    m_nLower = nLower;
    m_nUpper = nUpper;
    m_Progress.SetRange(nLower,nUpper);
}

//-----------------------------------------------------------------------

//Set the position of the step and the string associated

int CProgressDlg::SetPos(int nPos , LPCTSTR pczBuffer )
{
    PumpMessages();
    int iResult = m_Progress.SetPos( nPos);
    UpdatePercent(nPos , pczBuffer);
    return iResult;
}

//-----------------------------------------------------------------------

//set the step

int CProgressDlg::SetStep(int nStep)
{
    m_nStep = nStep; // Store for later use in calculating percentage
    return m_Progress.SetStep(nStep);
}

//-----------------------------------------------------------------------

//Last position

int CProgressDlg::OffsetPos(int nPos , LPCTSTR pczBuffer )
{
    PumpMessages();
    int iResult = m_Progress.OffsetPos(nPos);
    UpdatePercent(iResult+nPos , pczBuffer );
    return iResult;
}

//-----------------------------------------------------------------------

//increment the process gradually

int CProgressDlg::StepIt( LPCTSTR pczBuffer )
{
    PumpMessages();
    int iResult = m_Progress.StepIt();
    UpdatePercent(iResult+m_nStep, pczBuffer  );
    return iResult;
}

//-----------------------------------------------------------------------

//catch all the messages to prevent doing anything else during the process

void CProgressDlg::PumpMessages()
{
    // Must call Create() before using the dialog
    ASSERT(m_hWnd!=NULL);

    MSG msg;
    // Handle dialog messages
    while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
    {
        if(!IsDialogMessage(&msg))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
}

//-----------------------------------------------------------------------

//display percentage during the process

void CProgressDlg::UpdatePercent(int nNewPos , LPCTSTR pczBuffer )
{
    CWnd *pWndPercent = GetDlgItem(CG_IDC_PROGDLG_PERCENT);
    CWnd *pWndText = GetDlgItem(CG_IDC_PROGDLG_TEXT);
    int nPercent;

    int nDivisor = m_nUpper - m_nLower;
    ASSERT(nDivisor>0);  // m_nLower should be smaller than m_nUpper

    int nDividend = (nNewPos - m_nLower);
    ASSERT(nDividend>=0);   // Current position should be greater than m_nLower

    nPercent = nDividend * 100 / nDivisor;

    // Since the Progress Control wraps, we will wrap the percentage
    // along with it. However, don't reset 100% back to 0%
    if(nPercent!=100)
        nPercent %= 100;

    // Display the percentage
    CString strBuf;
    strBuf.Format(_T("%d%c") , nPercent,_T('%'));

    CString strCur; // get current percentage
    pWndPercent->GetWindowText(strCur);

    if (strCur != strBuf)
        pWndPercent->SetWindowText(strBuf);

    if ( pczBuffer != NULL )
        pWndText->SetWindowText(pczBuffer);
}

/////////////////////////////////////////////////////////////////////////////
// CProgressDlg message handlers

BOOL CProgressDlg::OnInitDialog()
{
    CDialog::OnInitDialog();
    GetDlgItem( IDCANCEL )->ShowWindow( m_bEnableCancel ) ;
    m_Progress.SetRange(m_nLower,m_nUpper);
    m_Progress.SetStep(m_nStep);
    m_Progress.SetPos(m_nLower ) ;

    GetDlgItem(CG_IDC_PROGDLG_PERCENT)->SetWindowText(_T(""));
    SetWindowText(m_strCaption);

    return TRUE;
}

//-----------------------------------------------------------------------

