/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// RootDocument.cpp: implementation of the CRootDocument class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ExtendedWinApp.h"
#include "RootDocument.h"

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
IMPLEMENT_DYNCREATE(CRootDocument, CDocument)


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//create the main document of the application
CRootDocument::CRootDocument()
{
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

CRootDocument::~CRootDocument()
{
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//Add documents in the application (ex : one doc per view )

BOOL CRootDocument::AddDocTemplate( const CDocTemplate * _pDocTemplate )
{
    BOOL bRet = TRUE ;
    ASSERT( _pDocTemplate != NULL && _pDocTemplate->IsKindOf(RUNTIME_CLASS(CDocTemplate)) ) ;

    TRY
    {
        // All I want to know is that I've got one of this sort.
        m_mapDocTemplate.SetAt(
            const_cast<void *>(static_cast<const void *>(_pDocTemplate)) ,
            const_cast<void *>(static_cast<const void *>(_pDocTemplate)) ) ;
    }
    CATCH(CMemoryException,e)
    {
        ASSERT(FALSE) ;
        bRet = TRUE ;
    }
    END_CATCH

    return bRet  ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//remove one doc
BOOL CRootDocument::RemoveDocTemplate( const CDocTemplate * _pDocTemplate )
{
    ASSERT( _pDocTemplate != NULL && _pDocTemplate->IsKindOf(RUNTIME_CLASS(CDocTemplate)) ) ;
    return m_mapDocTemplate.RemoveKey(
               const_cast<void *>(static_cast<const void *>(_pDocTemplate)) );
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//verify if the doc already exist
BOOL CRootDocument::HasDocTemplate( const CDocTemplate * _pDocTemplate ) const
{
    void * pDummy ;

    ASSERT( _pDocTemplate == NULL || _pDocTemplate->IsKindOf(RUNTIME_CLASS(CDocTemplate)) ) ;
    return m_mapDocTemplate.Lookup(
               const_cast<void *>(static_cast<const void *>(_pDocTemplate)) , pDummy ) ;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//event when a new doc is created
BOOL CRootDocument::OnNewDocument()
{
    if (!CDocument::OnNewDocument())
        return FALSE;

    ((CExtendedWinApp *)AfxGetApp())->SetRootDocument(this);

    return TRUE;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//event when a doc is opened
BOOL CRootDocument::OnOpenDocument(LPCTSTR lpszPathName)
{
    if (!CDocument::OnOpenDocument(lpszPathName))
        return FALSE;

    ((CExtendedWinApp *)AfxGetApp())->SetRootDocument(this);

    return TRUE;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//event when a doc is closed
void CRootDocument::OnCloseDocument()
{
    ((CExtendedWinApp *)AfxGetApp())->CloseDependents( this ) ;
    ((CExtendedWinApp *)AfxGetApp())->SetRootDocument( NULL );

    CDocument::OnCloseDocument();
}
