/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECRegListDlg.cpp : implementation file
//

#include "stdafx.h"
#include "daemoncfg.h"
#include "CECRegListDlg.h"
#include "CECSimpleParse.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define WMU_REFRESH_TIMER                   (WM_USER+1200)

/////////////////////////////////////////////////////////////////////////////
// CECRegListDlg dialog


CECRegListDlg::CECRegListDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECRegListDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECRegListDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pCorba=new CORBAServer ( 0, NULL );

}


void CECRegListDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECRegListDlg)
    DDX_Control(pDX, IDC_REFRESH_STOP, m_RefreshStop);
    DDX_Control(pDX, IDC_REFRESH_START, m_RefreshStart);
    DDX_Control(pDX, IDC_TYPES, m_Types);
    DDX_Control(pDX, IDC_REGLIST, m_RegList);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECRegListDlg, CDialog)
//{{AFX_MSG_MAP(CECRegListDlg)
ON_CBN_SELCHANGE(IDC_TYPES, OnChangeTypes)
ON_WM_DESTROY()
ON_WM_TIMER()
ON_BN_CLICKED(IDC_REFRESH_START, OnRefreshStart)
ON_BN_CLICKED(IDC_REFRESH_STOP, OnRefreshStop)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECRegListDlg message handlers

BOOL CECRegListDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Initialize the column headings for the report view
    CRect r;
    m_RegList.GetClientRect(r);

    m_RegList.InsertColumn(0, "Type", LVCFMT_LEFT, r.Width()/2, 0);
    m_RegList.InsertColumn(1, "Name", LVCFMT_LEFT, r.Width()/2, 1);

    // Add the list of CORBA object types to the combo box
    m_Types.InsertString(ALL_ITEMS,"All");
    m_Types.InsertString(PROVIDERS_ITEM, "CyranoProvider");
    m_Types.InsertString(LOG_ITEM, "CyranoLog");
    m_Types.InsertString(FACTORY_ITEM, "CyranoFactory");
    m_Types.InsertString(TESTMANAGER_ITEM, "TestManager");
    m_Types.InsertString(TESTEXECUTER_ITEM, "TestExecuter");
    m_Types.InsertString(TESTRUN_ITEM, "CyrStTestRun");
    m_Types.InsertString(TMVARFACTORY_ITEM, "CyrTestManagerVariableFactory");
    m_Types.InsertString(GLOBALSTRING_ITEM, "CyrGlobalString");
    m_Types.InsertString(TIMESERVICE_ITEM, "TimeService");

    // Select the first item in the list ("All") and display registered items for this
    // selection
    m_Types.SetCurSel(0);
    OnChangeTypes();

    m_RefreshStop.EnableWindow(FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECRegListDlg::OnChangeTypes()
{
    // Find the selected type
    int nType = m_Types.GetCurSel();
    m_RegList.DeleteAllItems();


    // If "All" has been selected, make the "Type" column visible, otherwise hide it by
    // setting the column width to 0
    CRect r;
    m_RegList.GetClientRect(r);
    if (nType == ALL_ITEMS)
    {
        m_RegList.SetColumnWidth(0,r.Width()/2);
        m_RegList.SetColumnWidth(1,r.Width()/2);
    }
    else
    {
        m_RegList.SetColumnWidth(0,0);
        m_RegList.SetColumnWidth(1,r.Width());
    }

    // Fill the report view with items of the selected type.
    switch (nType)
    {
    case ALL_ITEMS:
        {
            AddItems("CyranoProvider");
            AddItems("CyranoLog");
            AddItems("CyranoFactory");
            AddItems("TestManager");
            AddItems("TestExecuter");
            AddItems("CyrStTestRun");
            AddItems("CyrTestManagerVariableFactory");
            AddItems("CyrGlobalString");
            AddItems("TimeService");
            break;
        }
    case PROVIDERS_ITEM:
        {
            AddItems("CyranoProvider");
            break;
        }
    case LOG_ITEM:
        {
            AddItems("CyranoLog");
            break;
        }
    case FACTORY_ITEM:
        {
            AddItems("CyranoFactory");
            break;
        }
    case TESTMANAGER_ITEM:
        {
            AddItems("TestManager");
            break;
        }
    case TESTEXECUTER_ITEM:
        {
            AddItems("TestExecuter");
            break;
        }
    case TESTRUN_ITEM:
        {
            AddItems("CyrStTestRun");
            break;
        }
    case TMVARFACTORY_ITEM:
        {
            AddItems("CyrTestManagerVariableFactory");
            break;
        }
    case GLOBALSTRING_ITEM:
        {
            AddItems("CyrGlobalString");
            break;
        }
    case TIMESERVICE_ITEM:
        {
            AddItems("TimeService");
            break;
        }
    }

    // Make sure the report view gets redrawn
    m_RegList.RedrawWindow();
}

void CECRegListDlg::AddItems(CString type)
{
    try
    {
        // Use the CORBA client to retieve a list of all registered items of the
        // given type
		CORBAServer::NameList_t ItemList=m_pCorba->GetListOfNames(type);

        // Parse the list, adding each one to the report view
        int i=0;
		for (std::vector<std::string>::size_type iter=0;iter<ItemList.size();iter++)
		{
            m_RegList.InsertItem(i,type, 0);
            m_RegList.SetItem(i, 1, LVIF_TEXT , ItemList[iter].Id.c_str(), 0, 0, LVIF_STATE, NULL);
            i++;
        }
    }
    catch(...)
    {}

}

void CECRegListDlg::OnDestroy()
{
    CDialog::OnDestroy();

    KillTimer(WMU_REFRESH_TIMER);

    if(m_pCorba != NULL)
    {
//        m_pCorbaClient->GetBOA()->destroy();
//        m_pCorbaClient->GetORB()->NP_destroy();
        delete m_pCorba;
        m_pCorba = NULL;
    }
}

void CECRegListDlg::OnTimer(UINT nIDEvent)
{
    OnChangeTypes();

    CDialog::OnTimer(nIDEvent);
}

void CECRegListDlg::OnRefreshStart()
{
    m_RefreshStart.EnableWindow(FALSE);
    m_RefreshStop.EnableWindow(TRUE);

    SetTimer(WMU_REFRESH_TIMER, 5000, NULL);

}

void CECRegListDlg::OnRefreshStop()
{
    KillTimer(WMU_REFRESH_TIMER);

    m_RefreshStop.EnableWindow(FALSE);
    m_RefreshStart.EnableWindow(TRUE);

}
