/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __PSTRING__
#define __PSTRING__

#include <string>
#include <ctype.h>

template <typename U>
inline std::string& MakeString(U strIn, unsigned long ulLen, std::string& strOut)
{
    U pc = strIn;
    for(unsigned long ul = 0; ul < ulLen; ul++)
    {
        strOut += *pc;
        ++pc;
    }
    return strOut;
}

template <typename U>
inline std::string MakeString(U strIn, unsigned long ulLen)
{
    std::string str;
    return MakeString(strIn, ulLen, str);
}

// Find substring in container string.
// pAfterSubString is an output value, pointing to the first character after
// the found substring in the container. If the substring was not found, then
// pAfterSubString is undefined.
// pContainerEnd points one character past the end of the container string.
template <typename U>
inline U FindString(U pContainer, U pContainerEnd, unsigned long ulContainerSize,
    const char* pcSubString, unsigned long ulSubStringSize, BOOL bCaseSensitive,
    U& pAfterSubString)
{
    if(ulSubStringSize <= ulContainerSize)
    {
        pContainerEnd -= ulSubStringSize;
        const char *pcSubStringEnd = pcSubString + ulSubStringSize;
        for(; pContainer != pContainerEnd; ++pContainer)
        {
            const char *pcSub = pcSubString;
            for(pAfterSubString = pContainer; pcSub != pcSubStringEnd; ++pcSub, ++pAfterSubString)
            {
                if(bCaseSensitive)
                {
                    if(*pAfterSubString != *pcSub)
                        break;
                }
                else if(toupper((char)*pAfterSubString) != toupper((char)*pcSub))
                    break;
            }
            if(pcSub == pcSubStringEnd)
                return pContainer;  // Match found.
        }
    }
    return NULL;
}

#endif  // __PSTRING__
