/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include  "context.hpp"
#include "gwhttp.hpp"

//------------------------------------------
// class CContext
//------------------------------------------
CContext::CContext()
        :uiConnectId_(0), serverName_(""), iPort_(0), requestUrl_(""), bThroughProxy_(FALSE), iHttpCookieId_(0), iScriptCookieId_(0), b100continue_(FALSE), bPrimary_(false), iTofRequestId_(0)
{}


CContext::~CContext()
{}



bool CContext::operator == (const CContext & context) const
{
    return uiConnectId_ == context.uiConnectId_;
}

bool CContext::operator < (const CContext & context) const
{
    return uiConnectId_ < context.uiConnectId_;
}

void CContext::setRequestUrl(string requestUrl)
{
    int iIndex = 0 ;
    requestUrl_ = requestUrl;
    while ((iIndex = CGWHttp::find_in_string(requestUrl_, CYRANO_SSL_URL,false, 0)) != string::npos)
    {
        requestUrl_.replace(iIndex, 8, "https://");
    }
}

//------------------------------------------
// class CContextList
//------------------------------------------
CContextList::CContextList()
{}


CContextList::~CContextList()
{
    reset();
}

void CContextList::add(CContext * pContext)
{
    LockGuard lock(monitor()) ;

    bool havetoinsert = true;
    CContextStlList::iterator i,istop;
    i     = list_.begin();
    istop = list_.end();


    while( i != istop)
    {
        if ((**i) == *pContext)
        {
            havetoinsert=false;
            break;
        }
        i++;
    }

    if (havetoinsert == true)
        list_.push_back(pContext);
}

void CContextList::remove(CContext * pContext)
{
    LockGuard lock(monitor()) ;
    list_.remove(pContext);
}

CContext * CContextList::find(CContext * pContext)
{
    LockGuard lock(monitor()) ;
    CContextStlList ::iterator i,istop;
    i     = list_.begin();
    istop = list_.end();
    while( i != istop)
    {
        if ((**i) == *pContext)
            break;

        i++;
    }
    CContext *pContextFound = *i;
    return pContextFound;
}

void CContextList::reset(void)
{
    LockGuard lock(monitor()) ;

    CContextStlList::iterator i,istop;
    i     = list_.begin();
    istop = list_.end();
    while( i != istop)
    {
        list_.erase(i);
        delete(*i);
        i++;
    }
}

