/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// PreferencesModellerGatewayPPage.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "GatewayOptionsDlg.h"
#include "htmlhelp.h"
#include "Help\GatewayOptionsDlg.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGatewayOptionsDlg property page
//--------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CGatewayOptionsDlg, CDialog)

//--------------------------------------------------------------------

CGatewayOptionsDlg::CGatewayOptionsDlg(_TPreferencesGatewayData *pData) :
        CDialog(CGatewayOptionsDlg::IDD),
        m_pData(pData)
{
}

//--------------------------------------------------------------------


CGatewayOptionsDlg::~CGatewayOptionsDlg()
{}

//--------------------------------------------------------------------


void CGatewayOptionsDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);

    //{{AFX_DATA_MAP(CGatewayOptionsDlg)
    DDX_Check(pDX, ID_CONSOLE_CHECK, m_bConsole);
    DDX_Check(pDX, ID_TRACE_CHECK, m_bTrace);
	DDX_Check(pDX, IDC_AUTO_COOKIE_CHECK, m_bAutoCookieCheck);
	//}}AFX_DATA_MAP

}

//--------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CGatewayOptionsDlg, CDialog)
//{{AFX_MSG_MAP(CGatewayOptionsDlg)
ON_BN_CLICKED(IDC_CHECK_BYPASS, OnCheckBypass)
ON_EN_CHANGE(IDC_EDIT_ADDRESS, OnChangeEditAddress)
ON_EN_CHANGE(IDC_EDIT_ADDRESS_NOPROXY, OnChangeEditAddressNoproxy)
ON_EN_CHANGE(IDC_EDIT_ADDRESS_PORT, OnChangeEditAddressPort)
ON_EN_CHANGE(IDC_EDIT_ADMIN_PORT, OnChangeEditAdminPort)
ON_EN_CHANGE(IDC_EDIT_SECURE, OnChangeEditSecure)
ON_EN_CHANGE(IDC_EDIT_SECURE_PORT, OnChangeEditSecurePort)
ON_BN_CLICKED(IDC_RADIO_LOCAL, OnRadioLocal)
ON_BN_CLICKED(IDC_RADIO_REMOTE, OnRadioRemote)
ON_EN_CHANGE(IDC_EDIT_PORT, OnChangeEditPort)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGatewayOptionsDlg message handlers
//--------------------------------------------------------------------

BOOL CGatewayOptionsDlg::OnInitDialog()
{
    CDialog::OnInitDialog();
    m_IPEditAddress.SubclassDlgItem(IDC_EDIT_ADDRESS, this );
    m_IPEditSecure.SubclassDlgItem(IDC_EDIT_SECURE, this );

    CButton *pButton = NULL;
    //---
    //Local or Remote
    if( m_pData->m_bLocal)
        pButton = static_cast< CButton*>( GetDlgItem( IDC_RADIO_LOCAL ) );
    else
        pButton = static_cast< CButton*>( GetDlgItem( IDC_RADIO_REMOTE ) );

    pButton->SetCheck( 1 ) ;

    //Admin Port
    CString strPort ;
    //---
    strPort.Format( "%ld" , m_pData->m_lAdministrationPort ) ;
    GetDlgItem( IDC_EDIT_ADMIN_PORT )->SetWindowText( strPort ) ;
    //Listen Port
    strPort.Format("%ld", m_pData->m_lListenPort ) ;
    GetDlgItem( IDC_EDIT_PORT )->SetWindowText( strPort ) ;

    //Proxy
    if( m_pData->m_bLocal )
        InitLocalProxy() ;
    else
        InitRemoteProxy() ;

    UpdateData(FALSE);
    return TRUE;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnCheckBypass()
{
    m_bModified = TRUE ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnChangeEditAddress()
{
    m_bModified = TRUE ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnChangeEditAddressNoproxy()
{
    m_bModified = TRUE ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnChangeEditAddressPort()
{
    m_bModified = TRUE ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnChangeEditAdminPort()
{
    m_bModified = TRUE ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnChangeEditSecure()
{
    m_bModified = TRUE ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnChangeEditSecurePort()
{
    m_bModified = TRUE ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnRadioLocal()
{
    m_bModified = TRUE ;
    InitLocalProxy() ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnRadioRemote()
{
    m_bModified = TRUE ;
    InitRemoteProxy() ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::OnChangeEditPort()
{
    m_bModified = TRUE ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::InitLocalProxy()
{
    //Proxy
    CString strPort ;
    //----
    //1 - Address
    m_IPEditAddress.SetWindowText( m_pData->m_ProxyInfo.m_LocalProxy.m_strProxyAddress ) ;
    strPort.Format( "%ld" , m_pData->m_ProxyInfo.m_LocalProxy.m_lAddressPort ) ;
    GetDlgItem( IDC_EDIT_ADDRESS_PORT )->SetWindowText( strPort ) ;
    //2 - SSL
    m_IPEditSecure.SetWindowText( m_pData->m_ProxyInfo.m_LocalProxy.m_strProxySecure ) ;
    strPort.Format( "%ld" , m_pData->m_ProxyInfo.m_LocalProxy.m_lSecurePort ) ;
    GetDlgItem( IDC_EDIT_SECURE_PORT )->SetWindowText( strPort ) ;

    //go through proxy for local machine
    static_cast<CButton*>( GetDlgItem( IDC_CHECK_BYPASS ) )->SetCheck( m_pData->m_ProxyInfo.m_LocalProxy.m_bByPassProxyLocal ) ;

    GetDlgItem( IDC_EDIT_ADDRESS_NOPROXY )->SetWindowText( m_pData->m_ProxyInfo.m_LocalProxy.m_strAddressesNoProxy ) ;

    m_IPEditAddress.EnableWindow( FALSE ) ;
    GetDlgItem( IDC_EDIT_ADDRESS_PORT )->EnableWindow( FALSE ) ;
    m_IPEditSecure.EnableWindow( FALSE ) ;
    GetDlgItem( IDC_EDIT_SECURE_PORT )->EnableWindow( FALSE ) ;
    GetDlgItem( IDC_CHECK_BYPASS )->EnableWindow( FALSE ) ;
    GetDlgItem( IDC_EDIT_ADDRESS_NOPROXY )->EnableWindow( FALSE ) ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::InitRemoteProxy()
{
    //Proxy
    CString strPort ;
    //----

    //1 - Address

    m_IPEditAddress.SetWindowText( m_pData->m_ProxyInfo.m_RemoteProxy.m_strProxyAddress ) ;
    strPort.Format( "%ld" , m_pData->m_ProxyInfo.m_RemoteProxy.m_lAddressPort ) ;
    GetDlgItem( IDC_EDIT_ADDRESS_PORT )->SetWindowText( strPort ) ;
    //2 - SSL
    m_IPEditSecure.SetWindowText( m_pData->m_ProxyInfo.m_RemoteProxy.m_strProxySecure ) ;
    strPort.Format( "%ld" , m_pData->m_ProxyInfo.m_RemoteProxy.m_lSecurePort ) ;
    GetDlgItem( IDC_EDIT_SECURE_PORT )->SetWindowText( strPort ) ;

    //go through proxy for Remote machine
    static_cast<CButton*>( GetDlgItem( IDC_CHECK_BYPASS ) )->SetCheck( m_pData->m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal ) ;

    GetDlgItem( IDC_EDIT_ADDRESS_NOPROXY )->SetWindowText( m_pData->m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy ) ;

    m_IPEditAddress.EnableWindow( TRUE ) ;
    GetDlgItem( IDC_EDIT_ADDRESS_PORT )->EnableWindow( TRUE ) ;
    m_IPEditSecure.EnableWindow( TRUE ) ;
    GetDlgItem( IDC_EDIT_SECURE_PORT )->EnableWindow( TRUE ) ;
    GetDlgItem( IDC_CHECK_BYPASS )->EnableWindow( TRUE ) ;
    GetDlgItem( IDC_EDIT_ADDRESS_NOPROXY )->EnableWindow( TRUE ) ;
}

//--------------------------------------------------------------------

void CGatewayOptionsDlg::SaveRemoteProxy()
{
    //Proxy
    CString strPort ;

    //----
    //1 - Address
    //---
    m_IPEditAddress.GetWindowText( m_pData->m_ProxyInfo.m_RemoteProxy.m_strProxyAddress ) ;

    GetDlgItem( IDC_EDIT_ADDRESS_PORT )->GetWindowText( strPort ) ;
    m_pData->m_ProxyInfo.m_RemoteProxy.m_lAddressPort = atol( strPort ) ;

    //2 - SSL
    m_IPEditSecure.GetWindowText( m_pData->m_ProxyInfo.m_RemoteProxy.m_strProxySecure ) ;

    GetDlgItem( IDC_EDIT_SECURE_PORT )->GetWindowText( strPort ) ;
    m_pData->m_ProxyInfo.m_RemoteProxy.m_lSecurePort = atol( strPort ) ;

    //go through proxy for local machine
    m_pData->m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal = static_cast<CButton*>( GetDlgItem( IDC_CHECK_BYPASS ) )->GetCheck() ;

    GetDlgItem( IDC_EDIT_ADDRESS_NOPROXY )->GetWindowText( m_pData->m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy ) ;
}

void CGatewayOptionsDlg::OnOK()
{
    UpdateData(TRUE);
    CButton *pButton = NULL;
    //---
    //Local or Remote
    pButton = static_cast< CButton*>( GetDlgItem( IDC_RADIO_LOCAL ) );
    if( pButton->GetCheck() )
        m_pData->m_bLocal = TRUE ;
    else
        m_pData->m_bLocal = FALSE ;

    //Admin Port
    CString strPort ;
    //---
    GetDlgItem( IDC_EDIT_ADMIN_PORT )->GetWindowText( strPort ) ;
    m_pData->m_lAdministrationPort  = atol( strPort) ;

    //Listen Port
    GetDlgItem( IDC_EDIT_PORT )->GetWindowText( strPort ) ;
    m_pData->m_lListenPort = atol( strPort) ;

    if( !m_pData->m_bLocal )
        SaveRemoteProxy() ;

    CDialog::OnOK();
}

BOOL CGatewayOptionsDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
