/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarValListPPage.cpp : implementation file
//
#include "stdafx.h"
#include "VarValListPPage.h"
#include "VarAutoGenDlg.h"
#include <ExtendedPropertySheet.h>
#include "htmlhelp.h"
#include "Help\VarValListPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVarValListPPage, CExtendedPropertyPage)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVarValListPPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CVarValListPPage)
ON_BN_CLICKED(IDC_AUTOGEN, OnAutoGenerate)
ON_WM_CTLCOLOR()
ON_WM_SHOWWINDOW()
ON_WM_CANCELMODE()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

CVarValListPPage::CVarValListPPage(CVarInfo * _pVarInfo)
        :   CExtendedPropertyPage(CVarValListPPage::IDD)
{
    m_pVarInfo = _pVarInfo ;
}

//---------------------------------------------------------------------------

CVarValListPPage::~CVarValListPPage()
{}

//---------------------------------------------------------------------------


BOOL CVarValListPPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog() ;

    m_ListValue.SubclassDlgItem( IDC_VALLIST_LIST, this);
    m_ListValue.SetStandardButtons();
    ResetGrid();

    return TRUE ;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarValListPPage::Values2Ctrl()
{
    long lNbValues;

    lNbValues = m_pVarInfo->m_VarValueList.m_Values.GetSize() ;

    ResetGrid();

    for(int i=0;i<lNbValues;i++)
    {
        m_ListValue.AddItem( m_pVarInfo->m_VarValueList.m_Values.GetAt(i) ) ;
    }
}

//---------------------------------------------------------------------------

void CVarValListPPage::Ctrl2Values()
{
    CButton*  pBut ;
    long lNbItems ;

    m_pVarInfo->m_VarValueList.m_Values.RemoveAll() ;
    lNbItems = m_ListValue.GetCount() ;

    m_pVarInfo->m_VarValueList.m_Values.SetSize(lNbItems) ;

    for(int i=0;i<lNbItems;i++)
    {

        CString strValue ;

        strValue = m_ListValue.GetItemText(i) ;
        m_pVarInfo->m_VarValueList.m_Values.SetAt(i, strValue) ;

    }

    pBut = static_cast<CButton*>(GetDlgItem(IDC_VAR_GENFILE)) ;
    m_pVarInfo->m_VarValueList.m_bGenerateFile = pBut->GetCheck() ;

}

//---------------------------------------------------------------------------

LRESULT CVarValListPPage::OnWizardBack()
{
    return CExtendedPropertyPage::OnWizardBack() ;
}

//---------------------------------------------------------------------------

LRESULT CVarValListPPage::OnWizardNext()
{
    Ctrl2Values() ;

    return CExtendedPropertyPage::OnWizardNext() ;
}

//---------------------------------------------------------------------------

BOOL CVarValListPPage::OnSetActive()
{
    Values2Ctrl() ;

    return CExtendedPropertyPage::OnSetActive();
}

//---------------------------------------------------------------------------

void CVarValListPPage::OnAutoGenerate()
{
    CVarAutoGen     VarAutoGen ;
    CVarAutoGenDlg  Dlg( &VarAutoGen ) ;
    CString         strValue ;
    CStringArray    arstrValues ;
    long            lNbValues ;

    if( Dlg.DoModal() == IDCANCEL )
        return ;

    for( long i = VarAutoGen.m_lLowerLimit; i <= VarAutoGen.m_lUpperLimit ; i += VarAutoGen.m_lStepValue )
    {
        strValue.Format( "%s%ld", VarAutoGen.m_strPrefix, i ) ;
        arstrValues.Add( strValue ) ;
    }

    lNbValues   = arstrValues.GetSize() ;

    if( lNbValues > 0 )
    {

        m_ListValue.SetRedraw( FALSE ) ;

        for( long i = 0; i < lNbValues; i++ )
        {
            m_ListValue.AddItem( arstrValues.GetAt( i ) ) ;
        }

        m_ListValue.SetRedraw() ;
    }
    m_ListValue.Invalidate();
}

//---------------------------------------------------------------------------

HBRUSH CVarValListPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

//---------------------------------------------------------------------------

void CVarValListPPage::ResetGrid()
{
    for( int i = m_ListValue.GetCount(); i> 0; i-- )
    {
        m_ListValue.RemoveItem( 0 );
    }
}

BOOL CVarValListPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
