/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef PSL__SCL_LIMITS_LOADED
#define PSL__SCL_LIMITS_LOADED 1

#define INTEGER_SIZE        sizeof (long)

/*
 *    List are preceded by 3 fields, the value list pointer and the seed and
 *    saved seed values (longs)
 */
#define LIST_SIZE       (INTEGER_SIZE + INTEGER_SIZE + INTEGER_SIZE)

/*
 *    Ranges are preceded by 2 values for the seed and saved seed
 */
#define RANGE_SIZE      (INTEGER_SIZE + INTEGER_SIZE)

/*
 *    Shareable variables (i.e. thread,global and testwide) differ from all
 *    other variable types in that there offset as encoded by SCL refers to
 *    an entry in a lookup table which is a table of pointers to the data.
 *    The size of a shareable variable is therefore that of a pointer.
 */
#define SHRVAR_SIZE     sizeof (long *)

/*
 *    Define the maximum number of open external data files at 10
 */
#define MAX_OPEN_FILES      10

/*
 *    Define the maximum number of files per script at 256
 */
#define MAX_FILES       256

/*
 *    Maximum number of global (shared) files is 256
 */
#define MAX_GBL_FILES       256

/*
 *    Define the base for passed file indexes as 512
 */
#define PASSED_FILE_BASE    512

/*
 *    Maximum number of array dimensions : 3
 */
#define MAX_ARRAY_DIM       3

/*
 *    Maximum number of pipes per script : 256
 */
#define MAX_PIPES       256

/*
 *    Define the base index for pipes passed as parameters.
 */
#define PASSED_PIPE_BASE    512

/*
 *    Maximum number of tests per script.
 */
#define MAX_TST         32767

/*
 *    Maximum number of test threads per script.
 */
#define MAX_TSTTHR      32767

/*
 *    Maximum number of threads per script.
 */
#define MAX_THR         32767
/*
 *    Maximum size of a variable or record.
 */
#define MAX_VARIABLE_SIZE   65535

/*
 *    Maximum size of a character literal or constant
 */
#define MAX_LITERAL_SIZE    65535

#endif  /* PSL__SCL_LIMITS_LOADED */
