/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//========================================================================
//
// CECGlobalCmdTarget.cpp
// ----------------------
//
//    This is the implementation file for the CECGlobalCmdTarget class
//
//=========================================================================

// This class is the base class for our CFrameWnd. It has been created for
// the sole purpose of providing a message map which any plug-in can send
// messgaes to. This class therefore is providing a method of communication
// so that the Plug-ins can activate functions or get data which reside in
// the Framework or other Plug-Ins

#include "stdafx.h"

#include "CECPlugInFrame.h"
#include "CECMainFrame.h"
#include "CECCommander.h"
#include "CECGlobalCmdTarget.h"
#include "../interfaces/CyrPlugins.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECGlobalCmdTarget

IMPLEMENT_DYNAMIC(CECGlobalCmdTarget, CECPluginFrame)

BEGIN_MESSAGE_MAP(CECGlobalCmdTarget, CECPluginFrame)
//{{AFX_MSG_MAP(CECGlobalCmdTarget)
ON_WM_CREATE()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP

//
//    Message handlers for the exposed functionality of the framework.
//
ON_MESSAGE (WMU_FIRE_GLOBAL_EVENT ,OnFireGlobalEvent)
ON_MESSAGE (WMU_FIRE_SPECIFIC_EVENT ,OnFireSpecificEvent)
ON_MESSAGE (STUDIO_CUSTOM_LINK_2 ,OnSetOutputWindow)
ON_MESSAGE (WMU_SET_PROJECT ,OnSetProject)
ON_MESSAGE (WMU_GET_KNOWN_EXTENSIONS ,OnGetKnownExtensions)
ON_MESSAGE (WMU_GET_USERKEY ,OnGetUserKey)
ON_MESSAGE (WMU_GET_INSTALLDIR ,OnGetInstallDir)
ON_MESSAGE (STUDIO_BEGIN_WAIT,  OnBeginWait)
ON_MESSAGE (STUDIO_END_WAIT,    OnEndWait)
ON_MESSAGE (STUDIO_STATUSBAR_TEXT, OnSetStatusBarText)
ON_MESSAGE (WMU_GET_SERVERPTR, OnGetServerPtr)
ON_MESSAGE (WMU_GET_PRODUCT_TYPE, OnGetProductType)
ON_MESSAGE (WMU_GET_PRODUCT_PREFIX, OnGetProductPrefix)
ON_MESSAGE (WMU_WIZARD_START, OnWizardStart)

END_MESSAGE_MAP()

CECGlobalCmdTarget::CECGlobalCmdTarget()
{}


CECGlobalCmdTarget::~CECGlobalCmdTarget()
{}

/////////////////////////////////////////////////////////////////////////////
// CECGlobalCmdTarget message handlers


int CECGlobalCmdTarget::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CECPluginFrame::OnCreate(lpCreateStruct) == -1)
        return -1;

    return 0;
}

void CECGlobalCmdTarget::ActivateFrame(int nCmdShow)
{
    CECPluginFrame::ActivateFrame(nCmdShow);
    RedrawWindow(NULL,NULL,RDW_FRAME|RDW_INVALIDATE|RDW_ALLCHILDREN|RDW_NOERASE|RDW_UPDATENOW);
}

//
//    OnSetOutputWindow
//    ==================
//
//    This function is called in response to the exposed framework function triggered by the
//    STUDIO_CUSTOM_LINK_2 message. Its purpose is to allow extension DLL's to put text into the
//    output window whose name matches that supplied.
//
LRESULT CECGlobalCmdTarget::OnSetOutputWindow   (WPARAM nMsg ,LPARAM lParam)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);


    //
    //    The first parameter is actually a pointer to teh name of the output window that we
    //    want to act on. the second parameter is a pointer to the text to output.
    //
    char* pWindowName = (char*) nMsg;
    char* pText = (char*) lParam;

    //
    //    ...then call the function in the APP to do the actual work
    //
    pApp->SetOutputWindow (pWindowName ,pText);

    return 0;
}


//
//    OnSetProject
//    ============
//
//    This function is called in response to the exposed framework function triggered by the
//    WMU_SET_PROJECT message. Its urpose is to allow extension DLL's to define the current active
//    project.
//
//    @parm nMsg unused
//    @parm lParam | Pointer to the CECComProject to be set
//
LRESULT CECGlobalCmdTarget::OnSetProject    (WPARAM nMsg ,LPARAM lParam)
{
    BeginWaitCursor();

    CString strProject((char*) lParam);

	CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
	
	if(strProject.IsEmpty())
		pApp->OnFileCloseproject();
	else
		pApp->OnSwitchProjects(strProject);

    EndWaitCursor();
    return 0;
}

//
//    OnFireGlobalEvent
//    =================
//
//    This function is called in response to the exposed framework function triggered by the
//    WMU_FIRE_GLOBAL_EVENT message. Its purpose is to allow extension DLL's to initiate a global
//    event to be fired.
//
//    @parm nMsg Event to be fired
//
LRESULT CECGlobalCmdTarget::OnFireGlobalEvent   (WPARAM nMsg ,LPARAM lParam)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);
    pApp->GlobalFireEvent(nMsg, lParam);
    return 0;
}

//
//    OnFireSpecificEvent
//    =================
//
//    This function is called in response to the exposed framework function triggered by the
//    WMU_FIRE_SPECIFIC_EVENT message. Its purpose is to allow extension DLL's to initiate a event
//    being triggered in a specific DLL.
//
//    @parm nMsg Event to be fired
//    @rdesc As returned by the handler of the message
//
LRESULT CECGlobalCmdTarget::OnFireSpecificEvent (WPARAM nMsg ,LPARAM lParam)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);
    return pApp->SpecificFireEvent(nMsg, lParam);
}

//
//    OnGetKnownExtensions
//    ====================
//
//    This function is called in response to the exposed framework function triggered by the
//    WMU_GET_KNOWN_EXTENSIONS message. Its purpose is to allow extension DLL's to recover an array
//    of the file extensions declared to the application.
//
//    @rdesc Return a pointer to the current active project cast to a LRESULT
//
LRESULT CECGlobalCmdTarget::OnGetKnownExtensions    (WPARAM nMsg ,LPARAM)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);
    return (LRESULT) pApp->GetKnownFileExtensions();
}

//
// OnGetUserKey
// ============
//
// This function is used to obtain the user key which a product is using
// for any required registry access
//
LRESULT CECGlobalCmdTarget::OnGetUserKey    (WPARAM,LPARAM)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    CString szUserKey = "Software\\" + pApp->ConstructUserKey() + "\\Commander\\";
    return (LRESULT)_strdup(szUserKey);
}

//
// OnGetInstallDir
// ===============
//
// This function is used to obtain the installation directory from any plug-in
//
LRESULT CECGlobalCmdTarget::OnGetInstallDir (WPARAM,LPARAM)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    // Simply return the installation directory...
    return (LRESULT) pApp->GetInstallationDirectory();
}

//
// OnSetStatusBarText
// ==================
//
// This function is called in response to the exposed framework function triggered by the
// STUDIO_STATUSBAR_TEXT message. Its purpose is to allow extension DLL's to put text into the
// statusbar window for information purposes.
//
LRESULT CECGlobalCmdTarget::OnSetStatusBarText  (WPARAM nMsg ,LPARAM)
{

    char* pText = (char*) nMsg;

    SetText (pText);

    //::Sleep (1000);
    return 0;
}

//
// OnGetServerPtr
// ==============
//
// This function is used to obtain the Server Pointer
//
LRESULT CECGlobalCmdTarget::OnGetServerPtr  (WPARAM,LPARAM)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();

    return (LRESULT) pApp->GetServer();
}

LRESULT CECGlobalCmdTarget::OnGetProductType    (WPARAM,LPARAM)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    CString szType = pApp->GetCoreProductTitle();
    return (LRESULT)_strdup(szType);
}

LRESULT CECGlobalCmdTarget::OnGetProductPrefix(WPARAM,LPARAM)
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    CString szType = pApp->GetCoreProductProjectPrefix();
    return (LRESULT)_strdup(szType);
}

LRESULT CECGlobalCmdTarget::OnBeginWait(WPARAM,LPARAM)
{
    BeginWaitCursor();
    return 0;
}

LRESULT CECGlobalCmdTarget::OnEndWait(WPARAM,LPARAM)
{
    EndWaitCursor();
    return 0;
}

LRESULT CECGlobalCmdTarget::OnWizardStart(WPARAM pStrDLL,LPARAM pStrInfo)
{
    CString strDLL = (LPCTSTR)pStrDLL;
    CString strInfo = (LPCTSTR)pStrInfo;

    // Get a pointer to the App ..
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();

    // And from that we can obtain the DLLLoader details ...
    CECDLLFunctions *pDllLoader = pApp->GetDLLLoader();
    ASSERT (pDllLoader != NULL);

    LPEXECUTEWIZARD lpfnExecuteWizard = (LPEXECUTEWIZARD)pDllLoader->GetFunctionPtr("ExecuteWizard",strDLL);
    if(lpfnExecuteWizard)
    {
        int index = strInfo.Find('|',0);
        if (index == -1)
        {
            lpfnExecuteWizard(strInfo, "");
        }
        else
        {

            int nLen = strInfo.GetLength();
            lpfnExecuteWizard(strInfo.Left(index),strInfo.Right(nLen-index-1));
        }
    }
    return 0;
}




void CECGlobalCmdTarget::OnSetFocus(CWnd* pOldWnd) 
{
	CECPluginFrame::OnSetFocus(pOldWnd);

//	if (GetStyle() & WS_VISIBLE)
//	    OnFireGlobalEvent(WMU_REFRESH_TREE_VIEWS,0);	

}
