/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//========================================================================
//
// CECWorkspace.cpp
// ----------------
//
//    This is the implementation file for the CECWorkspace class
//
//=========================================================================

#include "stdafx.h"
#include "CECCommander.h"
#include "CECWorkspace.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECWorkspace

CECWorkspace::CECWorkspace()
{
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();

    //
    //    Set up the headings for the initialization file strings.
    //
    m_szWindowsSectionTxt                   = "Window Size/Position";
    m_szEnvironmentSectionTxt               = "Environment";
    m_szToolbarSectionTxt                   = "ToolBars_"+pApp->GetCoreProductName();

    //
    //  The following fields appear in the Window Size/Position section
    //
    m_szMainFrameWindowPositionTxt          = "Mainframe";
    m_szMainWindowZoomStateTxt              = "Mainframe maximised";
    m_szFullScreenTxt                       = "Full Screen";
    m_szFullScreenDlgTxt                    = "Full Screen Dialog";
    m_szStatusBarVisible                    = "Status Bar Visible";

    //
    //  The following fields appear in the Environment Section
    //
    m_szMDIMaximizedTxt                     = "MDI Window Maximized";
    m_szCurrentProjectTxt                   = "Current Project";

    m_szToolBarInfoTxt                          = "Toolbar Info";

    //
    // And initialize the member variables
    //
    SetDefaults();
}

CECWorkspace::~CECWorkspace()
{}

//
// SetDefaults
// ===========
//
// Set up some Defaults for the member variables
//

void CECWorkspace::SetDefaults()
{
    m_bFullScreen               = FALSE;
    m_bMainWindowZoomState      = FALSE;
    m_bMDIMaximized             = TRUE;
    m_szMainFrameWindowPosition = _T("");
    m_szFullScreenDlgPosition   = _T("20,20,0,0");
    m_bStatusBarVisible         = TRUE;
}
//
//    RestoreWorkspace
//    ================
//
//    This routine is called at the start of Commander after the control bars and all windows have been
//    created to restore the Commander Workspace to the state which was in when it was last executed.
//
//    This involves reading the registry to recover the name of the project which was being worked on
//    last as well as environmental settings.
//
BOOL    CECWorkspace::RestoreWorkspace ()
{

    // Recover the data from the registry
    CWinApp* pApp = AfxGetApp();
    ASSERT(pApp != NULL);

    //  Restore the Dock bar state
    RestoreDockState();

    //  And the Current Project
    m_szCurrentProject          = pApp->GetProfileString(m_szEnvironmentSectionTxt, m_szCurrentProjectTxt,          _T(""));

    //  And the MainFrame Window Position
    m_szMainFrameWindowPosition = pApp->GetProfileString(m_szWindowsSectionTxt, m_szMainFrameWindowPositionTxt,     _T("0, 0, 800, 600"));

    // Restore full screen status
    m_bFullScreen               = pApp->GetProfileInt(m_szWindowsSectionTxt,    m_szFullScreenTxt,                  0);

    // Restore Zoom State - Is Mainframe Maximized
    m_bMainWindowZoomState      = pApp->GetProfileInt(m_szWindowsSectionTxt,    m_szMainWindowZoomStateTxt,         1);

    // Restore 'Is MDI Maximized' status
    m_bMDIMaximized             = pApp->GetProfileInt(m_szEnvironmentSectionTxt,m_szMDIMaximizedTxt,                0);

    m_szFullScreenDlgPosition   = pApp->GetProfileString(m_szWindowsSectionTxt,m_szFullScreenDlgTxt,                _T("20,20,0,0"));

    m_bStatusBarVisible         = pApp->GetProfileInt(m_szWindowsSectionTxt,    m_szStatusBarVisible,                   1);

    return TRUE;
}

//
// RectToString
// ============
//
// Utility function to convert a rect into it's string format
//
CString CECWorkspace::RectToString(const CRect& r)
{
    CString szString;
    szString.Format("%d,%d,%d,%d", r.left, r.top, r.right, r.bottom);

    return szString;
}
//
//
//
// SaveProjectSettings
// ===================
//
// This routine is called when we wish to save the project settings (on exit from commander)
//
void CECWorkspace::SaveSettings ()
{
    CWinApp* pApp = AfxGetApp();
    ASSERT (pApp != NULL);

    // Save the state of any and all control bars.
    SaveDockState ();

    // Save the Current Project
    pApp->WriteProfileString(m_szEnvironmentSectionTxt, m_szCurrentProjectTxt,          m_szCurrentProject);

    // Save the window size/pos
    pApp->WriteProfileString(m_szWindowsSectionTxt,     m_szMainFrameWindowPositionTxt, m_szMainFrameWindowPosition);

    // Save the Zoom state
    pApp->WriteProfileInt   (m_szWindowsSectionTxt,     m_szMainWindowZoomStateTxt,     m_bMainWindowZoomState);

    // Save full screen status
    pApp->WriteProfileInt   (m_szWindowsSectionTxt,     m_szFullScreenTxt,              m_bFullScreen);

    // Save state of the 'MDI Maximized' flag. This goes in the ENVIRONMENT section
    pApp->WriteProfileInt   (m_szEnvironmentSectionTxt, m_szMDIMaximizedTxt,            m_bMDIMaximized);

    pApp->WriteProfileString    (m_szWindowsSectionTxt,     m_szFullScreenDlgTxt,               m_szFullScreenDlgPosition);

    pApp->WriteProfileInt(m_szWindowsSectionTxt,    m_szStatusBarVisible,               m_bStatusBarVisible);
}

//
//    SaveDockState
//    =============
//
//    Save the state of all toolbars.
//
void CECWorkspace::SaveDockState    ()
{
    /*
        SECDockState    DockState;
     
        SECWorkbook* pMainFrame = (SECWorkbook*) AfxGetMainWnd();
        ASSERT (pMainFrame != NULL);
     
        int nToolBarInfo=1;
        SECToolBarManager* pToolBarMgr = (SECToolBarManager*)pMainFrame->GetControlBarManager();
        if(pToolBarMgr->CoolLookEnabled())
            nToolBarInfo |= 2;
     
        if(pToolBarMgr->LargeBtnsEnabled())
            nToolBarInfo |= 4;
     
        if(pToolBarMgr->ToolTipsEnabled())
            nToolBarInfo |= 8;
     
        AfxGetApp()->WriteProfileInt(m_szToolbarSectionTxt, m_szToolBarInfoTxt, nToolBarInfo);
     
        pMainFrame->SaveBarState (m_szToolbarSectionTxt);
     
        pMainFrame->GetDockState (DockState);
        DockState.SaveState (m_szToolbarSectionTxt);
    */
}


//
//    RestoreDockState
//    ================
//
//    Restore the state of all toolbars.
//
void CECWorkspace::RestoreDockState ()
{
    /*
        SECDockState    DockState;
     
        SECWorkbook* pMainFrame = (SECWorkbook*) AfxGetMainWnd();
        if(!pMainFrame)
            return;
        
        pMainFrame->LoadBarState (m_szToolbarSectionTxt);
            
        DockState.LoadState (m_szToolbarSectionTxt);
        pMainFrame->SetDockState(DockState);
     
        int nToolBarInfo = AfxGetApp()->GetProfileInt(m_szToolbarSectionTxt,    m_szToolBarInfoTxt,                 0);
        if(nToolBarInfo)
        {
            SECToolBarManager* pToolBarMgr = (SECToolBarManager*)pMainFrame->GetControlBarManager();
            pToolBarMgr->EnableCoolLook(nToolBarInfo & 2);
            pToolBarMgr->EnableLargeBtns(nToolBarInfo & 4);
            pToolBarMgr->EnableToolTips(nToolBarInfo & 8);
        }
    */
}

//
// StringToRect
// ============
//
// Parses the co-ordinates of a window into a Rect. Co-ordinates delimited by
// comma.
CRect CECWorkspace::StringToRect(const CString& strSizePos)
{
    CRect rect;
    CRect defaultRect(0, 0, 800, 600);

    CString strLocalCopy = strSizePos;
    int nPos;

    //
    // Initialise r
    rect.left = rect.right = rect.top = rect.bottom = 0;

    //
    // Retrieve left
    rect.left = atoi( strLocalCopy);

    //
    // Retrieve top
    nPos = strLocalCopy.Find(",");
    if (nPos == -1) return defaultRect;
    strLocalCopy = strLocalCopy.Mid(nPos + 1);
    rect.top = atoi( strLocalCopy );

    //
    // Retrive the right
    nPos = strLocalCopy.Find(",");
    if (nPos == -1) return defaultRect;
    strLocalCopy = strLocalCopy.Mid(nPos + 1);
    rect.right = atoi( strLocalCopy );

    //
    // Retrive the bottom
    nPos = strLocalCopy.Find(",");
    if (nPos == -1) return defaultRect;
    strLocalCopy = strLocalCopy.Mid(nPos + 1);
    rect.bottom = atoi( strLocalCopy );

    return rect;
}

