/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CECWorkspace.h
// --------------
//
//    This is the include file for the CECWorkspace class
//
//=========================================================================
#if !defined(AFX_CECWORKSPACE_H__C9D662D3_0857_11D3_B683_00600856D1F5__INCLUDED_)
#define AFX_CECWORKSPACE_H__C9D662D3_0857_11D3_B683_00600856D1F5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// CECWorkspace.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CECWorkspace window

class CECWorkspace : public CObject
{
    // Construction
public:
    CECWorkspace();   // standard constructor

    virtual ~CECWorkspace   ();

    // Implementation

    // Recover the previous workspace
    BOOL RestoreWorkspace();

    // Save a Window position
    CString RectToString(const CRect& r);

    // Determine the rect associated with this window
    CRect StringToRect(const CString& strSizePos);

    // Save the Workspace settings on request
    void SaveSettings();

    // Save the Dock State
    void SaveDockState();

    // And Restore the Dock State
    void RestoreDockState();

    // Set up some default values
    void SetDefaults();

public:

    // Readers and writers
    void SetFullScreenMode(BOOL bState)
    {
        m_bFullScreen                  = bState;
    }
    void SetWindowZoomState(BOOL bState)
    {
        m_bMainWindowZoomState         = bState;
    }
    void SetMainWindowRect(const CRect& r)
    {
        m_szMainFrameWindowPosition    = RectToString(r);
    }
    void SetCurrentProject(CString szProject)
    {
        m_szCurrentProject             = szProject;
    }
    void SetMDIMaximized(BOOL bState)
    {
        m_bMDIMaximized                = bState;
    }
    void SetFullScreenDlgPos(const CRect& r)
    {
        m_szFullScreenDlgPosition      = RectToString(r);
    }
    void SetStatusBarVisible(BOOL bVisible)
    {
        m_bStatusBarVisible = bVisible;
    }

    BOOL IsFullScreen()
    {
        return m_bFullScreen;
    }
    BOOL IsZoomed()
    {
        return m_bMainWindowZoomState;
    }
    BOOL IsMDIMaximized()
    {
        return m_bMDIMaximized;
    }
    BOOL IsStatusBarVisible()
    {
        return m_bStatusBarVisible;
    }

    CString GetMainWindowRect()
    {
        return m_szMainFrameWindowPosition;
    }
    CString GetCurrentProject()
    {
        return m_szCurrentProject;
    }
    CString GetFullScreenDlgPos()
    {
        return m_szFullScreenDlgPosition;
    }

private:
    // Section Headers
    CString     m_szWindowsSectionTxt;
    CString     m_szEnvironmentSectionTxt;
    CString     m_szToolbarSectionTxt;

    // Items
    CString     m_szMainFrameWindowPositionTxt;
    CString     m_szMainWindowZoomStateTxt;
    CString     m_szFullScreenTxt;
    CString     m_szMDIMaximizedTxt;
    CString     m_szCurrentProjectTxt;
    CString     m_szFullScreenDlgTxt;
    CString     m_szStatusBarVisible;
    CString     m_szToolBarInfoTxt;

    // And their values
    BOOL        m_bFullScreen;
    BOOL        m_bMDIMaximized;
    BOOL        m_bMainWindowZoomState;
    CString     m_szMainFrameWindowPosition;
    CString     m_szCurrentProject;
    CString     m_szFullScreenDlgPosition;

    BOOL        m_bStatusBarVisible;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CECWORKSPACE_H__C9D662D3_0857_11D3_B683_00600856D1F5__INCLUDED_)
