/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRExternalFile.cpp
// -------------------
//
//    This is the implementation file for the CYRExternalFile class
//
//=========================================================================

#include "stdafx.h"
#include "resource.h"
//
//    Project includes
//
#include "CYRExternalFile.h"
#include "CECSourceFileUtils.h"
#include "CECTargetEnvironment.h"
#include "..\interfaces\CyrPlugins.h"
#include "server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRExternalFile

CYRExternalFile::CYRExternalFile()
{
    CECTargetEnvironment te;
    te.GetCurrentContext(m_szContext);
    m_pServer = (CYRServer *)AfxGetMainWnd()->SendMessage (WMU_GET_SERVERPTR);
}

CYRExternalFile::~CYRExternalFile()
{}

/////////////////////////////////////////////////////////////////////////////
// CYRExternalFile message handlers

// #define STRICT
//    #include <windows.h>
//    #include <shlobj.h>


int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM /*pData*/)
{
    TCHAR szDir[MAX_PATH];

    switch(uMsg)
    {
    case BFFM_INITIALIZED:
        {
            if (GetCurrentDirectory(sizeof(szDir)/sizeof(TCHAR), szDir))
            {
                // WParam is TRUE since you are passing a path.
                // It would be FALSE if you were passing a pidl.
                SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)szDir);
            }
            break;

        }
    case BFFM_SELCHANGED:
        {
            // Set the status window to the currently selected path.
            if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szDir))
            {
                SendMessage(hwnd,BFFM_SETSTATUSTEXT,0,(LPARAM)szDir);
            }
            break;
        }
    default:
        {
            break;
        }
    }
    return 0;
}

//
// Import
// ======
//
// Import a file into the current project
//
void CYRExternalFile::Import()
{
    char    FileBuffer[16635];

    FileBuffer[0] = '\0';

    //
    // Set up file filter and initialise the FILE open dialog.
    // Allow mutiple file selection.
    //
    char szFileFilter[] = "Scripts (*.scl)|*.scl|Include files(*.inc)|*.inc|Data Files (*.dat)|*.dat|TOF Files (*.tof)|*.tof|All Files (*.*)|*.*||";
    CFileDialog dlgFileOpen(TRUE,                       // file open dlg
                            NULL,                       // default extension
                            NULL,                       // initial file name
                            OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_ALLOWMULTISELECT,  // flags
                            szFileFilter);              // file filter
    dlgFileOpen.m_ofn.lpstrFile = FileBuffer;           // Use our own buffer to hold the files
    dlgFileOpen.m_ofn.nMaxFile = sizeof (FileBuffer);   // ...and setup its size
    dlgFileOpen.m_ofn.lpstrTitle = "Select files to import into current project";
    // Set up title to use.

    //
    // Platform depenedent issues..
    // On NT3.51 the function GetNextPathName returns a garbaged filename, it
    // somehow replaces space characters with the slash (\) characters.
    // The function GetPath seems to work fine,     THEREFORE
    // if we are working with NT3.51 we do not allow multiple selections
    // and use the GetPath function to retrieve the selected filename.
    //
    dlgFileOpen.m_ofn.Flags |= OFN_ALLOWMULTISELECT;    // allows multiple selectionss
    if (dlgFileOpen.DoModal() != IDOK)
        return;


    // Build up list of files selected by user
    CString         szFileName;

    // And provide a count of the success rate
    int nImports = 0;


    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage(STUDIO_BEGIN_WAIT);
    POSITION pos;
    for (pos = dlgFileOpen.GetStartPosition(); pos != NULL;)
    {
        szFileName = dlgFileOpen.GetNextPathName(pos);
        szFileName.MakeUpper();

        // And add to our source control system
        if (CheckInFile(szFileName))
        {
            // Increment the success counter
            nImports++;
        }
    }
    pMainWnd->SendMessage(STUDIO_END_WAIT);

    // Display a message box showing how many files were imported
    CString msg = "1 file was imported into the project.";
    if (nImports!=1)
    {
        msg.Format("%d files were imported into the project.",nImports);
    }
    AfxMessageBox(msg);
}

//
// Export
// ======
//
// Export a file from the current project
//
void CYRExternalFile::Export(CString szFileName)
{
    // As this is simply a wrapper around the actual export function
    // all we need to do is build up an array containing only this
    // file and call the normal export function
    CStringArray arFiles;

    arFiles.Add(szFileName);

    // And call the general Export function
    Export(&arFiles);
}

//
// Export
// ======
//
// Export an array of files from the current project
//
void CYRExternalFile::Export(CStringArray* arFiles)
{
    CString szFileName;

    // And get the local export directory we wish to export to ...
    CString szLocalDir;

    if (!GetExportDir(szLocalDir))
        return;

    // And provide a count of the success rate
    int nExports = 0;

    for (int i = 0; i < arFiles->GetSize(); i++)
    {
        // And get the full path to the file
        szFileName = szLocalDir + "\\" + arFiles->GetAt(i);

        //      // Create it ...
        if (CreateEmptyFile(szFileName))
        {
            // And fetch the source from our source control mechanism
            if (CheckOutFile(szFileName))
            {
                // Increment the success counter
                nExports++;
            }
        }
        else
        {
            break;
        }
    }
    // Display a message box showing how many files were imported
    CString msg = "1 file was exported from the project.";
    if (nExports != 1)
    {
        msg.Format("%d files were exported from the project.",nExports);
    }
    AfxMessageBox(msg);
}

//
// GetExportDir
// ============
//
// Browser to obtain a local directory with which to export to
//
BOOL CYRExternalFile::GetExportDir(CString& szLocalDir)
{
    szLocalDir.Empty();

    // Declare the BROWSEINFO which does the trick
    BROWSEINFO bi;
    TCHAR szDir[MAX_PATH];
    LPITEMIDLIST pidl;
    LPMALLOC pMalloc;

    if (SUCCEEDED(SHGetMalloc(&pMalloc)))
    {
        ZeroMemory(&bi,sizeof(bi));
        bi.hwndOwner = NULL;
        bi.pszDisplayName = 0;
        bi.pidlRoot = 0;
        bi.lpszTitle = "Please choose a directory you wish to export into:";
        bi.lpfn = BrowseCallbackProc;
        bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT;

        // And browse for folders only
        pidl = SHBrowseForFolder(&bi);

        if (pidl)
        {
            if (SHGetPathFromIDList(pidl, szDir))
            {
                // Signal a successful directory capture by filling in szLocalDir
                szLocalDir = szDir;
            }

            // Free up the allocated memory
            pMalloc->Free(pidl);
            pMalloc->Release();
        }
    }
    return (szLocalDir.IsEmpty()) ? FALSE : TRUE;
}

//
// CreateEmptyFile
// ===============
//
// Create an empty file ready for the export function to dump the contents
// of a file into.
//
BOOL CYRExternalFile::CreateEmptyFile(CString& szEmptyFile)
{
    // First make sure it does not already exist
    CFileStatus fs;
    if (CFile::GetStatus(szEmptyFile,fs))
    {
        CString szMessage;

        szMessage.Format("Do you wish to overwrite the file\n\n%s", szEmptyFile);

        if (AfxMessageBox(szMessage, MB_YESNO) == IDYES)
        {
            // OK we can blitz it
            try
            {
                CFile::Remove(szEmptyFile);
            }
            catch(CFileException e)
            {
                CString szMessage;
                szMessage.Format("Failed to delete the file\n\n%s", szEmptyFile);

                AfxMessageBox(szMessage);
                return FALSE;
            }
        }
        else
        {
            // Better keep it so quit the export operation completely
            // to allow the user to choose a different directory
            return FALSE;
        }
    }

    // Ok Now we know that there is no such file identified by the
    // path contained in szEmptyFile we can create an empty file
    try
    {
        // Construct a CFile object
        CFile file(szEmptyFile.GetBuffer(0), CFile::modeCreate);
        file.Close();
    }
    catch (CFileException* e)
    {
        // And report any errors we may of encountered
        e->ReportError();
        e->Delete();

        return FALSE;
    }
    return TRUE;
}

BOOL CYRExternalFile::CheckInFile(CString strFilename)
{
    BOOL bNeedsCompile = FALSE;

    //Get the filename without the path and make sure the file is not read only.
    CFileStatus status;
    CFile file(strFilename, CFile::modeRead | CFile::shareDenyNone);
    file.GetStatus (strFilename, status);
    status.m_mtime = CTime::GetCurrentTime();
    status.m_attribute &= (~CFile::readOnly);
    file.SetStatus (strFilename, status);

    CString strFile = file.GetFileName();
    file.Close();


    //
    // Get the target folder type into which the file will be checked in. This is based on the
    // file extension (default to source folder). Only really "tof" files that go somewhere
    // else. Also flag that "scl" and "tss" file need compiling when they are imported.
    //
    CYRServer::ProjectFolder nFolder;
    CString TargetFolder;
    if (strFile.Right(3).CompareNoCase("TOF") == 0)
    {
        nFolder = CYRServer::enObjectCodeFolder;
    }
    else if (strFile.Right(3).CompareNoCase("SCL") == 0)
    {
        nFolder = CYRServer::enSourceFolder;
        bNeedsCompile = TRUE;

        if (m_szContext == "WEB")
        {
            ImportWebCapture(strFile);
        }
    }
    else if (strFile.Right(3).CompareNoCase("TSS") == 0)
    {
        nFolder = CYRServer::enSourceFolder;
        bNeedsCompile = TRUE;
    }
    else
    {
        nFolder = CYRServer::enDataFolder;
    }

    //
    // Create the full target path for the file.
    //
    CString strFullPath = m_pServer->GetFolderPath(nFolder) + strFile;

    //Is this file already in scc ...
    m_pServer->CheckOutFile(nFolder,"",strFile);

    //check that we can create the file
    if(CreateEmptyFile(strFullPath))
    {
        //Copy the file to the relevant project folder
        if(!CopyFile(strFilename,strFullPath,FALSE))
        {
            return false;
        }
    }
    else
        return false;

    //
    //Check the file into SCC
    //
    if (bNeedsCompile)
    {
        CECSourceFileUtils utils(m_pServer,strFullPath);
        utils.Compile();
    }

    return m_pServer->CheckInFile(nFolder,"",strFile);
}


BOOL CYRExternalFile::CheckOutFile(CString strFilename)
{
    //Get the filename without the path
    CFile f(strFilename,0);
    CString strFile = f.GetFileName();
    f.Close();

    CString strWorkingDir = m_pServer->GetFolderPath(CYRServer::enSourceFolder);

    //Copy the file to the relevant project folder
    return CopyFile(strWorkingDir+"\\"+strFile,strFilename,FALSE);
}

BOOL CYRExternalFile::ImportWebCapture(CString szScriptFile)
{
    CString captureFile = szScriptFile;
    captureFile.Replace(".SCL", ".ALL");

    if(m_pServer->DoesFileExist(CYRServer::enCaptureFolder,szScriptFile,captureFile))
    {
        return TRUE;
    }

    CString szEmptyFile = m_pServer->GetFolderPath(CYRServer::enCaptureFolder) + szScriptFile + "\\" + captureFile;

    // Construct a CFile object
    CFile file(szEmptyFile.GetBuffer(0), CFile::modeCreate);
    file.Close();

    return m_pServer->CheckInFile(CYRServer::enCaptureFolder,szScriptFile,captureFile,captureFile);
}

