/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// cecsourcefileutils.cpp: implementation of the CECSourceFileUtils class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "cecsourcefileutils.h"
#include "CYRTestProperties.h"
#include "..\Interfaces\CyrPlugins.h"
#include "..\Common\CECRegistryTools.h"
#include "solutions.h"
#include "..\Common\CECXServerError.h"
#include "server.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#define BUFSIZE 4096

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECSourceFileUtils::CECSourceFileUtils(CYRServer *pServer,CString strLocalPath)
{
    m_strLocalPath = strLocalPath;
    m_hChildStdoutWr = NULL;
    m_hChildStdoutRd = NULL;
    m_pServer = pServer;
}

CECSourceFileUtils::~CECSourceFileUtils()
{
}


BOOL CECSourceFileUtils::Compile(bool bClearOutputWnd /*= true*/, bool bCompileDependencies /*= false*/)
{
    CWaitCursor wc;

    BOOL retVal = FALSE;
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);
    pMainWnd->SendMessage(STUDIO_BEGIN_WAIT);

    char* pCompileWnd = "Compile";
    if(bClearOutputWnd)
    {
        pMainWnd->SendMessage (STUDIO_CUSTOM_LINK_2, (WPARAM)pCompileWnd, NULL);
    }

    // Check to see if the file needs compiling
    BOOL bNeedsCompile = NeedsCompile();
    if (!bNeedsCompile && bCompileDependencies)
    {
        pMainWnd->SendMessage(STUDIO_END_WAIT);
        return CompileDependencies();
    }
    else if (!bNeedsCompile)
    {
        pMainWnd->SendMessage(STUDIO_END_WAIT);
        return true;
    }

    //Get the name of the file we're compiling
    int nLen = m_strLocalPath.GetLength();
    int nPos = m_strLocalPath.ReverseFind('\\');
    CString filename = m_strLocalPath.Right(nLen-nPos-1);
    filename = filename.SpanExcluding(".");

    //Tell the punter what we're compiling
    CString strMsg("---------------------------------------------------------------");
    pMainWnd->SendMessage (STUDIO_CUSTOM_LINK_2, (WPARAM)pCompileWnd, (LPARAM)(LPCTSTR)strMsg);
    strMsg.Format("Compiling %s...",filename);
    pMainWnd->SendMessage (STUDIO_CUSTOM_LINK_2, (WPARAM)pCompileWnd, (LPARAM)(LPCTSTR)strMsg);

    // And set up the status bar text
    pMainWnd->SendMessage(STUDIO_STATUSBAR_TEXT, (WPARAM) strMsg.GetBuffer(0));

    //
    // Create the command line
    //
    CString cmd = BuildCommandLine();

    //  if (!CheckOutTOF())
    //  {
    //      strMsg = ("Compilation failed...");
    //      pMainWnd->SendMessage(STUDIO_STATUSBAR_TEXT, (WPARAM) strMsg.GetBuffer(0));
    //      pMainWnd->SendMessage(STUDIO_END_WAIT);
    //      return FALSE;
    //  }

    //  CheckOutDepFile();
    //
    // Execute the compilation
    //
    CString sourcePath = m_pServer->GetFolderPath(CYRServer::enSourceFolder);

    CString oldWorkDir = "";

    DWORD MAX_STR = 256;
    GetCurrentDirectory(MAX_STR, oldWorkDir.GetBuffer(MAX_STR));


    SetCurrentDirectory(sourcePath);

    BOOL bExec = ExecuteCommand(cmd);
    if (!bExec)
    {
        strMsg = ("Compilation failed...");
        pMainWnd->SendMessage(STUDIO_STATUSBAR_TEXT, (WPARAM) strMsg.GetBuffer(0));

        AfxMessageBox("Unable to start compiler.\nPlease check your installation.");
        pMainWnd->SendMessage(STUDIO_END_WAIT);
        return FALSE;
    }

    SetCurrentDirectory(oldWorkDir);

    //
    // If the compilation fails, send the output text to the compile window.
    //

    if (!IsCompiled())
    {
        strMsg = ("Compilation failed...");
        pMainWnd->SendMessage(STUDIO_STATUSBAR_TEXT, (WPARAM) strMsg.GetBuffer(0));

        LPCTSTR msg = (LPCTSTR)m_strOutput;
        pMainWnd->SendMessage (STUDIO_CUSTOM_LINK_2, (WPARAM)pCompileWnd, (LPARAM)msg);
    }
    //
    // If it compiles the send a message back to the compile window then check in
    // the tof file
    //
    else
    {
        strMsg = "Compilation successful.";
        pMainWnd->SendMessage (STUDIO_CUSTOM_LINK_2, (WPARAM)pCompileWnd, (LPARAM)strMsg.GetBuffer(0));
        pMainWnd->SendMessage(STUDIO_STATUSBAR_TEXT, (WPARAM) strMsg.GetBuffer(0));

        CheckInTOF();
        CheckInDepFile();
        retVal = TRUE;

        if (bCompileDependencies)
        {
            retVal = CompileDependencies();
        }
    }

    pMainWnd->SendMessage(STUDIO_END_WAIT);
    return retVal;
}

CString CECSourceFileUtils::BuildCommandLine()
{
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    CString cmd;
    CString argList = "";
    CString argDebug = "";
    CString argSteps = "";
    CString argType = "";

    //
    // Make sure the extension of the source file is lower case
    //
    int nLen = m_strLocalPath.GetLength();
    CString strExt = m_strLocalPath.Right(3);
    strExt.MakeLower();

    m_strLocalPath = m_strLocalPath.Left(nLen - 3) + strExt;

    //
    // Find the path to the scl executable
    //
    CString execPath = GetCompilerPath();

    //
    // Work out the output file name from the given script name.
    //
    BuildTOFName();

    //
    // Build the command line based on the build settings
    //
    if (strExt.CompareNoCase("scl") == 0)
    {
        argType = "-t script";
    }
    else if (strExt.CompareNoCase("tss") == 0)
    {
        argType = "-t test";
    }
    else if(strExt.CompareNoCase("ths") == 0)
    {
        argType = "-t thread";
    }

    //  cmd.Format("%sscl.exe -o \"%s\" %s %s %s %s \"%s\"", execPath,m_strTOFPath+"\\"+m_strTOFName, argList,
    //              argDebug, argSteps, argType, m_strLocalPath);
    cmd.Format("%sscl.exe -o \"%s\" %s \"%s\"", execPath,m_strTOFPath+m_strTOFName,
               argType, m_strLocalPath);
    return cmd;
}

CString CECSourceFileUtils::GetCompilerPath()
{
    // Get path to scl.exe
    CString exePath;

    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);
    LPCSTR frameworkDir = (char*) pMainWnd->SendMessage (WMU_GET_INSTALLDIR);
    CString szFrameworkDir(frameworkDir);

    int  nPos = szFrameworkDir.ReverseFind('\\');
    exePath = szFrameworkDir.Left(nPos+1)+"Engines\\";

    return exePath;
}

void CECSourceFileUtils::BuildTOFName()
{
    // Extract the file name
    int nLen = m_strLocalPath.GetLength();
    int nPos = m_strLocalPath.ReverseFind('\\');

    CString filename = m_strLocalPath.Right(nLen-nPos-1);
    m_strLocalPath.MakeUpper();

    // Create the object code path
    m_strTOFPath = m_strLocalPath.Left(nPos+1);
    CString search = "\\" + m_pServer->GetFolderName(CYRServer::enSourceFolder) + "\\";
    search.MakeUpper();
    CString replace = "\\" + m_pServer->GetFolderName(CYRServer::enObjectCodeFolder) + "\\";
    replace.MakeUpper();
    m_strTOFPath.Replace(search, replace);

    //
    //    To avoid confusion and potential duplicate file names, we set the output file
    //    to have an extension which is related to the input file as follows :-
    //
    //      .scl    => .tof & .scd
    //      .tss    => .tso & .tsd
    //      .ths    => .tho & .thd
    //
    CString Extension = filename.Right(3);
    CString BaseFile = filename.Left (filename.GetLength() - 3);

    m_strTOFName = BaseFile;
    m_strDepName = BaseFile;

    if (Extension.CompareNoCase("scl") == 0)
    {
        m_strTOFName += "tof";
        m_strDepName += "scd";
    }

    else if (Extension.CompareNoCase("tss") == 0)
    {
        m_strTOFName += "tso";
        m_strDepName += "tsd";
    }

    else if (Extension.CompareNoCase("ths") == 0)
    {
        m_strTOFName += "tho";
        m_strDepName += "thd";
    }

    else
    {
        AfxMessageBox ("A file of this type cannot be compiled");
        return;
    }

}

BOOL CECSourceFileUtils::ExecuteCommand(CString cmd)
{
    //
    //    The command is being executed locally.
    //
    SECURITY_ATTRIBUTES saAttr;

    //
    // Set the bInheritHandle flag so pipe handles are inherited.
    //
    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
    saAttr.bInheritHandle = TRUE;
    saAttr.lpSecurityDescriptor = NULL;

    //
    // The steps for redirecting child's STDOUT:
    //     2.  Create anonymous pipe to be STDOUT for child.
    //     3.  Set STDOUT of parent to be write handle of pipe, so
    //         it is inherited by child.
    //

    //
    // Create a pipe for the child's STDOUT.
    //
    if (!CreatePipe (&m_hChildStdoutRd, &m_hChildStdoutWr, &saAttr, 0))
        AfxMessageBox("Stdout pipe creation failed\n");

    //
    // Now create the child process to execute the command
    //
    BOOL bExec = InvokeImage (cmd);

    //
    //  Read from pipe that is the standard output for child process.
    //
    ReadFromChild();

    return bExec;
}

BOOL CECSourceFileUtils::InvokeImage    (CString command)
{
    PROCESS_INFORMATION ProcInfo;
    CString     ErrorBuffer;
    STARTUPINFO StartInfo = {sizeof(StartInfo)};
    BOOL        InheritHandles = FALSE;
    int         CreateFlags = 0;


    //
    //    If we have requested that the window should be hidden then set up the STARTUPINFO
    //
    StartInfo.dwFlags = STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
    StartInfo.hStdOutput = m_hChildStdoutWr;
    StartInfo.hStdError = m_hChildStdoutWr;
    StartInfo.wShowWindow = SW_HIDE;
    InheritHandles = TRUE;
    CreateFlags = CREATE_NEW_CONSOLE;

    //
    //    Create a process to execute the specified image.
    //
    if (!CreateProcess (NULL
                        ,command.GetBuffer(0)            // Command Line
                        ,NULL                            // Process Security attributes
                        ,NULL                            // Primary thread security attributes
                        ,InheritHandles                  // Handles are {NOT} inherited
                        ,CreateFlags                     // Creation flags
                        ,NULL                            // Use Parents environment
                        ,NULL                            // Use Parents current directory
                        ,&StartInfo                      // STARTUPINFO pointer
                        ,&ProcInfo))                     // Receives Process Information
    {
        LPTSTR lpMsgBuf;

        FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                       NULL,
                       GetLastError(),
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                       (LPTSTR) &lpMsgBuf,
                       0,
                       NULL);

        //
        //    Output the error message via a message box if we have requested the child process output
        //    be directed to a window, If we have requested that child process output be sent to a string
        //    then the error should be logged into that string
        //
        m_strOutput = lpMsgBuf;


        //
        //    Free the buffer.
        //
        LocalFree( lpMsgBuf );
        return FALSE;

    }
    return TRUE;
}

//
//    ReadFromChild
//    =============
//
//    This routine is called from a multi-threaded operation to read from the pipe
//    created to a child process which is executing a DOS command for us. We send
//    the output recovered to the output window
//
void    CECSourceFileUtils::ReadFromChild   ()
{
    CString TextBuffer;
    CString s;

    DWORD dwRead;
    CHAR chBuf[BUFSIZE];

    //
    // Close the write end of the pipe before reading from the
    // read end of the pipe.
    //
    if (!CloseHandle(m_hChildStdoutWr))
        AfxMessageBox("Closing handle failed");

    //
    // Read output from child, and write it to the output window edit box.
    //
    TextBuffer="";
    for (;;)
    {
        if ((!ReadFile(m_hChildStdoutRd, chBuf, BUFSIZE, &dwRead, NULL)) || (dwRead == 0)) break;
        chBuf[dwRead] = '\0';
        TextBuffer += chBuf;
    }

    m_strOutput = TextBuffer;

}



BOOL CECSourceFileUtils::IsCompiled()
{
    return m_strOutput.IsEmpty();
}

int CECSourceFileUtils::ReadDependencies()
{
    m_Dependencies.RemoveAll();

    CString dependPath = m_strLocalPath;
    dependPath.MakeUpper();

    // Uppercase all directory names when doing the replace

    // First the Source directory

    CString szSource(m_pServer->GetFolderName(CYRServer::enSourceFolder));
    szSource.MakeUpper();

    // And the Dependency folder
    CString szDepends(m_pServer->GetFolderName(CYRServer::enDependenciesFolder));
    szDepends.MakeUpper();

    // And do the replace so we can locate the dependency

    // This does need to change at some point as it will replace all instances of
    // the word source in the path !!!
    dependPath.Replace(szSource, szDepends);

    int nLen = dependPath.GetLength();
    dependPath = dependPath.Left(nLen-1)+"D";

    // Make sure the file exists!
    CFileStatus fs;
    if(!CFile::GetStatus(dependPath,fs))
    {
        return 0;
    }

    // Read the file parsing each line into a DEPENENCY structure
    try
    {
        CStdioFile depend(dependPath, CFile::modeRead | CFile::typeText);

        CString strDepend;
        depend.ReadString(strDepend);

        while (!strDepend.IsEmpty())
        {
            CString file;
            GetNextField('|',strDepend,file);

            CString type;
            GetNextField('|',strDepend,type);

            DEPENDENCY d;
            d.type = atoi(type);
            d.file = file;
            d.SrcType = 0;
            d.subType = 0;

            if ((d.type == EXE_SCL) || (d.type == SCL))
            {
                CString thrType;
                GetNextField('|', strDepend, thrType);
                if (thrType == "-") thrType = '0';
                d.subType = atoi(thrType);

                CString SrcType;
                GetNextField('|', strDepend, SrcType);
                d.SrcType = atoi(SrcType);
            }

            AddDependancyExtension(d);
            m_Dependencies.Add(d);
            depend.ReadString(strDepend);
        }
    }
    catch (CFileException e)
    {}


    return m_Dependencies.GetSize();
}

DEPENDENCY CECSourceFileUtils::GetDependency(int index)
{
    return m_Dependencies[index];
}


CString CECSourceFileUtils::GetWorkDir()
{
    // Get the work directory
    CString KeyName = SOLUTIONS_SERVER_KEY;
    CString ValueName = "ProjectDir";

    CECRegistryTools reg;
    return reg.GetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName);

}

CString CECSourceFileUtils::GetProjectName()
{
    // Get the project name
    return m_pServer->GetOpenProjectName();
}

int CECSourceFileUtils::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr=="")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep,0);

    if (nPos == -1)
    {
        field = inStr;
        inStr="";
        return 0;
    }
    else if (nPos == 0)
    {
        return -1;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr=inStr.Right(len-nPos-1);
        return 0;
    }
}

BOOL CECSourceFileUtils::CheckOutTOF()
{
    return m_pServer->CheckOutFile(CYRServer::enObjectCodeFolder,"",m_strTOFName);
}

BOOL CECSourceFileUtils::CheckInTOF()
{
    return m_pServer->CheckInFile(CYRServer::enObjectCodeFolder,"",m_strTOFName);
}

BOOL CECSourceFileUtils::CheckOutDepFile()
{
    return m_pServer->CheckOutFile(CYRServer::enDependenciesFolder,"",m_strDepName);
}

BOOL CECSourceFileUtils::CheckInDepFile()
{
    // Move generated dependancy file from the object code dir to the
    // dependancy dir
    CString sourcePath = m_strTOFPath +  m_strDepName;

    CString destPath = m_pServer->GetFolderPath(CYRServer::enDependenciesFolder) + m_strDepName;

    CFileStatus fs;
    if (CFile::GetStatus(sourcePath,fs))
    {
        if (CopyFile(sourcePath, destPath,FALSE))
        {
            DeleteFile(sourcePath);
        }
    }

    // CheckIn / Add the dependancy file
    return m_pServer->CheckInFile(CYRServer::enDependenciesFolder,"",m_strDepName);
}

void CECSourceFileUtils::AddDependancyExtension(DEPENDENCY& dep)
{
    switch (dep.type)
    {
    case TSS:
        {
            dep.file = dep.file+".tss";
            break;
        }
        //      case THS:
        //      {
        //          dep.file = dep.file+".ths";
        //          break;
        //      }
    case SCL:
        {
            dep.file = dep.file+".scl";
            break;
        }
    case FILE:
        {
            break;
        }
    case INCL:
        {
            break;
        }
    case IMG:
        {
            break;
        }
    case EXE_SCL:
        {
            if (dep.SrcType == 0)
            {
                dep.file = dep.file+".scl";
            }
            else
            {
                dep.file = dep.file+".tof";
            }
            break;
        }
    }
}

BOOL CECSourceFileUtils::CompileDependencies()
{
    CString sourcePath = m_pServer->GetFolderPath(CYRServer::enSourceFolder);
    BOOL bRetVal = TRUE;

    int nDep =  ReadDependencies();

    for (int i = 0; i<nDep; i++)
    {
        DEPENDENCY D = GetDependency(i);

        if (D.type == TSS || D.type == SCL || D.type == EXE_SCL && D.SrcType == 0)
        {
            CECSourceFileUtils SU(m_pServer,sourcePath + D.file);
            BOOL bRet = SU.Compile(FALSE, TRUE);
            if (!bRet)
            {
                bRetVal = FALSE;
            }
        }
    }
    return bRetVal;
}

BOOL CECSourceFileUtils::NeedsCompile()
{
    // Get the modified time of the source file

    CTime SourceModifyTime;
    CFileStatus fsSource;
    CFile::GetStatus(m_strLocalPath, fsSource);
    SourceModifyTime = fsSource.m_mtime;


    // Get the modified time of the object file
    CString strExt = m_strLocalPath.Right(3);
    CString objPath = m_strLocalPath;
    objPath.MakeUpper();
    CString oldFolder = m_pServer->GetFolderPath(CYRServer::enSourceFolder);
    CString newFolder = m_pServer->GetFolderPath(CYRServer::enObjectCodeFolder);
    oldFolder.MakeUpper();
    newFolder.MakeUpper();

    objPath.Replace(oldFolder, newFolder);
    int nLen = objPath.GetLength();
    if (strExt.CompareNoCase("scl") == 0)
    {
        objPath = objPath.Left(nLen -3) + "TOF";
    }
    else if (strExt.CompareNoCase("tss") == 0)
    {
        objPath = objPath.Left(nLen -3) + "TSO";
    }


    CTime ObjectModifyTime;
    CFileStatus fsObj;
    if(!CFile::GetStatus(objPath, fsObj))
        return true;

    ObjectModifyTime = fsObj.m_mtime;

    if (SourceModifyTime>ObjectModifyTime)
    {
        return true;
    }

    return false;

}
