/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECMonitorView.cpp : implementation file
//

#include "stdafx.h"
#include "CECMonitorView.h"
#include "..\..\Interfaces\CyrPlugins.h"
#include "Solutions.h"
#include "OSTAORB/OSTAORB.h"

#include "resource.h"
#include "CECMonitorView.h"
#include "winsock2.h"
#include "FilterDlg.h"
#include "htmlhelp.h"
#include "ctime.hpp"
#include "CECSimpleParse.h"

#include "CECAuditLogger.h"
extern CECAuditLogger* pAuditLogger;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define WMU_REFRESH_XTIMER                  (WM_USER+9600)
#define WMU_REFRESH_COMPLETE                (WM_USER+9601)

#include "..\..\HTMLHelp\XManagerMon\Test_Manager_Monitoring_screen.inl"

/////////////////////////////////////////////////////////////////////////////
// CECMonitorView

IMPLEMENT_DYNCREATE(CECMonitorView, CFormView)

CECMonitorView::CECMonitorView()
        : CFormView(CECMonitorView::IDD)
{
    //{{AFX_DATA_INIT(CECMonitorView)
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_pCorba=new CORBAServer ( 0, NULL );
    m_bUpdating = false;
    m_strUserFilter = FILTER_ALL;
    m_strInjectorFilter = FILTER_ALL;
    m_strProjectFilter = FILTER_ALL;
}


CECMonitorView::~CECMonitorView()
{
    if (m_pCorba != NULL)
    {
        delete m_pCorba;
        m_pCorba = NULL;
    }
    m_mapIPToName.RemoveAll();
}


void CECMonitorView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECMonitorView)
    DDX_Control(pDX, IDC_MANAGER_LIST, m_ManagerList);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECMonitorView, CFormView)
//{{AFX_MSG_MAP(CECMonitorView)
ON_WM_SIZE()
ON_WM_ERASEBKGND()
//}}AFX_MSG_MAP

ON_WM_TIMER()
ON_WM_CLOSE()
ON_NOTIFY(NM_DBLCLK, IDC_MANAGER_LIST, OnDblclkManagerList)
ON_WM_HELPINFO()
ON_WM_PAINT()
ON_NOTIFY(LVN_ITEMCHANGED, IDC_MANAGER_LIST, OnItemchangedManagerList)
ON_WM_CONTEXTMENU()
ON_COMMAND(ID_FILTER, OnFilter)
ON_WM_DRAWITEM()
ON_COMMAND(IDC_FULL_HELP, OnFullHelp)
//}}AFX_MSG_MAP
ON_COMMAND(IDC_INJECTOR_VIEW, OnInjectorView)
ON_COMMAND(IDC_KILL_TEST, OnKillTest)
ON_COMMAND(IDC_STOP_TEST, OnStopTest)
ON_COMMAND(IDC_DETAILS, OnShowDetails)
ON_MESSAGE(WMU_REFRESH_COMPLETE,OnRefreshComplete)
ON_COMMAND(IDC_LOGAUDIT, OnLogAudit)
ON_COMMAND(IDC_LOGAUDIT_STOP, OnLogAuditStop)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECMonitorView diagnostics

#ifdef _DEBUG
void CECMonitorView::AssertValid() const
{
    CFormView::AssertValid();
}

void CECMonitorView::Dump(CDumpContext& dc) const
{
    CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CECMonitorView message handlers

void CECMonitorView::OnInitialUpdate()
{
    CFormView::OnInitialUpdate();

    // Make the window the size of the main dialog

    GetDocument()->SetTitle("Active Tests");
    GetParentFrame()->RecalcLayout();

    //  ResizeParentToFit(FALSE);

    SIZE sz= {0};
    SetScrollSizes(MM_TEXT,sz);

    CRect r;
    GetClientRect(r);
    m_ManagerList.SetExtendedStyle(LVS_EX_FULLROWSELECT);
	m_ManagerList.SetWindowPos(NULL,0,0,r.Width(),r.Height(),SWP_NOZORDER);

    SetupColumns();
    SetTimer(WMU_REFRESH_XTIMER, 5000, NULL);
    DoUpdate();

    // Update the buttons ...
    UpdateDialogControls(GetParent(),FALSE);
}

BOOL CECMonitorView::PreCreateWindow(CREATESTRUCT& cs)
{
    // Make sure that we don't create anything too small
    if (cs.cx < FRAMEMINWIDTH) cs.cx = FRAMEMINWIDTH;
    if (cs.cy < FRAMEMINDEPTH) cs.cy = FRAMEMINDEPTH;

    return CFormView::PreCreateWindow(cs);
}

BOOL CECMonitorView::OnEraseBkgnd(CDC *pDC)
{
    return true;
}

void CECMonitorView::OnSize(UINT nType, int cx, int cy)
{
    CFormView::OnSize(nType, cx, cy);

    if (m_ManagerList.m_hWnd != NULL)
    {
        m_ManagerList.SetWindowPos(NULL,0,0,cx,cy,SWP_NOZORDER);
    }
}

void CECMonitorView::SetupColumns()
{
    CRect r;
    m_ManagerList.GetWindowRect(r);

    m_nColumnCount = m_ManagerList.InsertColumn(0,"User", LVCFMT_LEFT, r.Width()/6 , 0);
    m_nColumnCount = m_ManagerList.InsertColumn(1,"Started by", LVCFMT_LEFT, r.Width()/6 , 0);
    m_nColumnCount = m_ManagerList.InsertColumn(2,"Project", LVCFMT_LEFT, r.Width()/6 , 0);
    m_nColumnCount = m_ManagerList.InsertColumn(3,"Test", LVCFMT_LEFT, r.Width()/6 , 0);
    m_nColumnCount = m_ManagerList.InsertColumn(4,"Start", LVCFMT_LEFT, r.Width()/6 , 0);
    m_nColumnCount = m_ManagerList.InsertColumn(5,"PID", LVCFMT_LEFT, r.Width()/6 , 0);
}

void CECMonitorView::OnTimer(UINT nIDEvent)
{
    // And perform a refresh if we do not have anything selected

    // Its a bit of a toughy this one. If we decide to update then
    // there is absolutely no way of knowing what we had selected
    // previously as Test Managers may come and go so screwing up
    // any indexing method we have - but if a user does have a set
    // of selections selected, he probably wants to take action on
    // them and does not require them to be updated anyway.
    //  if (!m_Selections.GetSize())
    {
        DoUpdate();
    }

    CFormView::OnTimer(nIDEvent);
}

void CECMonitorView::UpdateManagerArray()
{
    static BOOL bCorrupt = FALSE;

    try
    {
	    CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		string RegList = "";
	    for(std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)
	    {
			RegList += TMList[iter].Id + "|";
	    }

        CECSimpleParse parser(RegList);
        RemoveFinishedItems(RegList.c_str());

        CString ActiveManager = parser.getNext('|').c_str();
        while (!ActiveManager.IsEmpty())
        {
            MANAGER_INFO info;
            CString szManager = ActiveManager;
            int nIndex = GetManagerIndex(szManager);
            if (nIndex == -1)
            {
                info.szRegName = szManager;

                // Strip off the pid from the manager name(get it from the status object!)
                int nPos = szManager.ReverseFind('_');
                szManager = szManager.Left(nPos);

                //Find the test name, now the last field of the string.
                int nLen = szManager.GetLength();
                nPos = szManager.ReverseFind('_');
                info.szTest = szManager.Right(nLen - nPos - 1);
                szManager =  szManager.Left(nPos);

                // The manager string is now parsed down to NODE_PROJECT_Tests
                nPos = szManager.ReverseFind('_');
                szManager = szManager.Left(nPos);
                nPos = szManager.ReverseFind('_');
				nLen = szManager.GetLength();
				info.szProject = szManager.Right(nLen - nPos - 1);

                info.bAuditLogging = FALSE;

                if(GetStatusInfo(info))
                {
                    // add the info about this manager to the array
                    m_ManagerArray.Add(info);
                }

            }
            else
            {
                info = m_ManagerArray[nIndex];
                if(GetStatusInfo(info))
                {
                    // add the info about this manager to the array
                    m_ManagerArray[nIndex] = info;
                }

            }
            ActiveManager = parser.getNext('|').c_str();
        }

    }
    catch(...)
    {
        if(GetParent() != NULL)
            GetParent()->PostMessage(WM_CLOSE);
    }
}

BOOL CECMonitorView::GetStatusInfo(MANAGER_INFO &info)
{
    BOOL bStatus=TRUE;
    try
    {
        //Get the Test manager status object

        CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, info.szRegName.GetBuffer(0));
        TestManager_var TM = TestManager::_narrow(obj);
        //Get the PID
        info.nPID = TM->GetPID();
        //Get the username
        info.szUser = TM->GetUser();

        info.szNode = TM->GetNodeName();


        //Get the start time
        CTTime StartTime(TM->StartTime());
        string TimeString = StartTime.Format("%H:%M:%S");// %d/%m/%Y" );
        info.szTime = (CString)TimeString.c_str();
        return bStatus;

    }
    catch(...)
    {
        bStatus = FALSE;
    }

    return bStatus;
}


void CECMonitorView::ApplyFilters()
{
    m_ManagerList.SetRedraw(FALSE);

    //save the selection state
    CDWordArray arSelection;
    POSITION pos = m_ManagerList.GetFirstSelectedItemPosition();
    while(pos != NULL)
    {
        arSelection.Add(m_ManagerList.GetNextSelectedItem(pos));
    }
    m_ManagerList.DeleteAllItems();

    int nCount = m_ManagerArray.GetSize();
    if (nCount == 0)
    {
        m_ManagerList.SetRedraw(TRUE);
        return;
    }

    for(int i =0; i<nCount; i++)
    {
        MANAGER_INFO info = m_ManagerArray[i];
        BOOL bAddManager = TRUE;
        if ((info.szNode != m_strInjectorFilter) && (m_strInjectorFilter != FILTER_ALL))
        {
            bAddManager = FALSE;
        }
        if ((info.szUser != m_strUserFilter) && (m_strUserFilter != FILTER_ALL))
        {
            bAddManager = FALSE;
        }
        if ((info.szProject != m_strProjectFilter) && (m_strProjectFilter != FILTER_ALL))
        {
            bAddManager = FALSE;
        }

        if (bAddManager)
        {
            int nItem = m_ManagerList.InsertItem(i,info.szUser, 0);
            m_ManagerList.SetItem(nItem, 1, LVIF_TEXT , info.szNode, 0, 0, LVIF_STATE, NULL);
            m_ManagerList.SetItem(nItem, 2, LVIF_TEXT , info.szProject, 0, 0, LVIF_STATE, NULL);
            m_ManagerList.SetItem(nItem, 3, LVIF_TEXT , info.szTest, 0, 0, LVIF_STATE, NULL);
            m_ManagerList.SetItem(nItem, 4, LVIF_TEXT , info.szTime, 0, 0, LVIF_STATE, NULL);

            CString szPid;
            szPid.Format("%d", info.nPID);
            m_ManagerList.SetItem(nItem, 5, LVIF_TEXT , szPid, 0, 0, LVIF_STATE, NULL);
        }
    }

    //restore the selection
    for(int nNdx=0;nNdx<arSelection.GetSize();nNdx++)
    {
        m_ManagerList.SetItemState(arSelection[nNdx], LVIS_SELECTED, LVIS_SELECTED);
    }

    m_ManagerList.SetRedraw();
}


void CECMonitorView::OnStopTest()
{
    while(m_bUpdating)
        Sleep(10);

    int nIndex = GetCurrentSelection();
    if (nIndex == -1)
        return;

    MANAGER_INFO inf = m_ManagerArray[nIndex];

    CString TMName(inf.szRegName.GetBuffer(0));

    try
    {
        CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMName.GetBuffer(0));
        TestManager_var TM=TestManager::_narrow(obj);
        TM->Stop();
    }
    catch (...)
    {}

}
void CECMonitorView::OnKillTest()
{
    while(m_bUpdating)
        Sleep(10);

    int nIndex = GetCurrentSelection();
    if (nIndex == -1)
        return;

    MANAGER_INFO inf = m_ManagerArray[nIndex];

    CString TMName(inf.szRegName.GetBuffer(0));

    try
    {
        CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMName.GetBuffer(0));
        TestManager_var TM=TestManager::_narrow(obj);
        TM->Kill();
    }
    catch (...)
    {}

}
void CECMonitorView::OnShowDetails()
{
    while(m_bUpdating)
        Sleep(10);

    int nIndex = GetCurrentSelection();
    if(nIndex==-1)
        return;

    MANAGER_INFO inf = m_ManagerArray[nIndex];

    CWnd* pMainWnd = AfxGetMainWnd();

    CString monitorInfo;
    monitorInfo.Format("%s|%s" ,SOLUTIONS_GENERIC_DLL ,inf.szRegName);

    pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                           ,(WPARAM )WMU_LEVEL_MONITOR
                           ,(LPARAM) &monitorInfo);
}


void CECMonitorView::OnClose()
{
    KillTimer(WMU_REFRESH_XTIMER);
    if (m_pCorba != NULL)
    {
        delete m_pCorba;
        m_pCorba = NULL;
    }
    CFormView::OnClose();
}

void CECMonitorView::OnDblclkManagerList(NMHDR* pNMHDR, LRESULT* pResult)
{
    int nIndex = GetCurrentSelection();
    if(nIndex!=-1)
    {
        //
        // And display the details if we did double click on an item
        OnShowDetails();
    }

    *pResult = 0;
}

void CECMonitorView::DoUpdate()
{
    if(!m_bUpdating)
    {
        m_bUpdating = true;
        AfxBeginThread(&UpdateThread,this);
    }
}

LRESULT CECMonitorView::OnRefreshComplete(WPARAM,LPARAM)
{
    ApplyFilters();

    UpdateDialogControls(GetParent(),FALSE);
    return 0;
}

UINT CECMonitorView::UpdateThread(LPVOID pParam)
{
    CECMonitorView *pParent = (CECMonitorView*)pParam;

    CoInitialize(NULL);

    pParent->UpdateManagerArray();
    pParent->m_bUpdating = false;
    if(IsWindow(pParent->GetSafeHwnd()))
        pParent->SendMessage(WMU_REFRESH_COMPLETE);

    CoUninitialize();
    return 0;
}

BOOL CECMonitorView::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);
    return true;
}

void CECMonitorView::OnPaint()
{
    CPaintDC dc(this); // device context for painting

    CRect rc;
    m_ManagerList.GetWindowRect(&rc);
    ScreenToClient(&rc);
    if(rc.IntersectRect(rc,&dc.m_ps.rcPaint))
    {
        m_ManagerList.Invalidate();
    }
    // Do not call CFormView::OnPaint() for painting messages
}


int CECMonitorView::GetCurrentSelection()
{
    POSITION pos = m_ManagerList.GetFirstSelectedItemPosition();
    if(pos == NULL)
        return -1;

    return m_ManagerList.GetNextSelectedItem(pos);
}

void CECMonitorView::OnItemchangedManagerList(NMHDR* pNMHDR, LRESULT* pResult)
{
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

    if(pNMListView->uChanged & LVIF_STATE)
    {
        UpdateDialogControls(GetParent(),FALSE);
    }

    *pResult = 0;
}

void CECMonitorView::OnInjectorView()
{
    while(m_bUpdating)
        Sleep(10);

    int nIndex = GetCurrentSelection();
    if(nIndex==-1)
        return;

    CWnd* pMainWnd = AfxGetMainWnd();

    MANAGER_INFO inf = m_ManagerArray[nIndex];
    
	CString monitorInfo;
    monitorInfo.Format("%s|%s" ,SOLUTIONS_GENERIC_DLL ,inf.szRegName);

    pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                           ,(WPARAM )WMU_INJECTOR_VIEW
                           ,(LPARAM) &monitorInfo);
}

void CECMonitorView::OnContextMenu(CWnd* pWnd, CPoint point)
{
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    CMenu* pPopup = NULL;

    if(pWnd->GetDlgCtrlID() == IDC_MANAGER_LIST)
    {
        VERIFY(menu.LoadMenu(IDR_TESTLIST_POPUP));
        pPopup = menu.GetSubMenu(0);

        if(!m_ManagerList.GetFirstSelectedItemPosition())
        {
            pPopup->RemoveMenu(IDC_LOGAUDIT ,MF_BYCOMMAND);
            pPopup->RemoveMenu(IDC_LOGAUDIT_STOP ,MF_BYCOMMAND);
            pPopup->RemoveMenu(IDC_INJECTOR_VIEW ,MF_BYCOMMAND);
            pPopup->RemoveMenu(IDC_DETAILS ,MF_BYCOMMAND);
            pPopup->RemoveMenu(IDC_KILL_TEST ,MF_BYCOMMAND);
        }
        else
        {

            int nIndex = GetCurrentSelection();
            if(nIndex==-1)
                return;

            BOOL bAudit = m_ManagerArray[nIndex].bAuditLogging;

            /*      Temporarily take out Audit Logging      */
            //      pPopup->EnableMenuItem(IDC_LOGAUDIT, bAudit);
            //      pPopup->EnableMenuItem(IDC_LOGAUDIT_STOP, !bAudit);
            pPopup->RemoveMenu(IDC_LOGAUDIT ,MF_BYCOMMAND);
            pPopup->RemoveMenu(IDC_LOGAUDIT_STOP ,MF_BYCOMMAND);
        }
    }
    else
        return;

    ASSERT(pPopup != NULL);

    // Show the popup menu
    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);
}

void CECMonitorView::OnLogAudit()
{
    int nIndex = GetCurrentSelection();
    if(nIndex==-1)
        return;

    MANAGER_INFO inf = m_ManagerArray[nIndex];
    m_ManagerArray[nIndex].bAuditLogging = TRUE;

	pAuditLogger->StartLogging(inf.szRegName);
}

void CECMonitorView::OnLogAuditStop()
{
    int nIndex = GetCurrentSelection();
    if(nIndex==-1)
        return;

    MANAGER_INFO inf = m_ManagerArray[nIndex];
    m_ManagerArray[nIndex].bAuditLogging = FALSE;

	pAuditLogger->StopLogging(inf.szRegName);
}

void CECMonitorView::RemoveFinishedItems(CString RegList)
{

    for (int i =0; i<m_ManagerArray.GetSize(); i++)
    {
        if (RegList.Find(m_ManagerArray[i].szRegName) == -1)
        {
            m_ManagerArray.RemoveAt(i);
            i--;
        }
    }
}

int CECMonitorView::GetManagerIndex(CString szRegName)
{
    for (int i =0; i<m_ManagerArray.GetSize(); i++)
    {
        if (m_ManagerArray[i].szRegName == szRegName)
        {
            return i;
        }
    }

    return -1;
}

void CECMonitorView::OnFilter()
{
    CStringArray arstrInjectors,arstrUsers,arstrProjects;

    arstrInjectors.Add(FILTER_ALL);
    arstrUsers.Add(FILTER_ALL);
    arstrProjects.Add(FILTER_ALL);

    int nCount = m_ManagerArray.GetSize();

    for (int nNdx=0;nNdx<nCount;nNdx++)
    {
        MANAGER_INFO inf = m_ManagerArray[nNdx];

        arstrInjectors.Add(inf.szNode);
        arstrUsers.Add(inf.szUser);
        arstrProjects.Add(inf.szProject);
    }

    CFilterDlg dlg;
    dlg.SetInjectors(arstrInjectors);
    dlg.SetUsers(arstrUsers);
    dlg.SetProjects(arstrProjects);
    dlg.SetSelectedInjector(m_strInjectorFilter);
    dlg.SetSelectedUser(m_strUserFilter);
    dlg.SetSelectedProject(m_strProjectFilter);
    if(dlg.DoModal() == IDOK)
    {
        m_strInjectorFilter = dlg.GetSelectedInjector();
        m_strUserFilter = dlg.GetSelectedUser();
        m_strProjectFilter = dlg.GetSelectedProject();
        ApplyFilters();
    }
}

void CECMonitorView::OnFullHelp()
{
    HtmlHelp(m_hWnd, m_szHtmlHelpFile, HH_DISPLAY_TOPIC, (DWORD)"xmanager.htm");
}
