/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECResultViewer.cpp : implementation file
//

#include "stdafx.h"
#include "CECResultViewer.h"
#include "..\..\Common\CECClipboard.h"
#include "..\..\Interfaces\CyrPlugins.h"
#include "server.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECResultViewer

IMPLEMENT_DYNCREATE(CECResultViewer, CFormView)

CECResultViewer::CECResultViewer()
        : CFormView(CECResultViewer::IDD)
{
    //{{AFX_DATA_INIT(CECResultViewer)
    //}}AFX_DATA_INIT
}


CECResultViewer::~CECResultViewer()
{}


void CECResultViewer::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECResultViewer)
    DDX_Control(pDX, IDC_XRESULTSCTRL, m_XResults);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECResultViewer, CFormView)
//{{AFX_MSG_MAP(CECResultViewer)
ON_WM_SIZE()
//}}AFX_MSG_MAP
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECResultViewer diagnostics

#ifdef _DEBUG
void CECResultViewer::AssertValid() const
{
    CFormView::AssertValid();
}

void CECResultViewer::Dump(CDumpContext& dc) const
{
    CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CECResultViewer message handlers

void CECResultViewer::OnSize(UINT nType, int cx, int cy)
{
    //  CFormView::OnSize(nType, cx, cy);

    if (m_XResults.m_hWnd != NULL)
    {
        m_XResults.SetWindowPos(&wndTop, 0,0,cx,cy,SWP_NOZORDER);
        m_XResults.Invalidate();
    }
}

void CECResultViewer::OnInitialUpdate()
{
    CFormView::OnInitialUpdate();

    // Get a handle to the Mainframe
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    // Now get the server login information from the framework
    CYRServer *pServer = (CYRServer*)pMainWnd->SendMessage (WMU_GET_SERVERPTR);
    CString szUser = pServer->GetUserName();
    CString szPass = pServer->GetPassword();

    m_XResults.XSetServerInfo(szUser, szPass);

    // Get the active project name
    CString szProj = pServer->GetOpenProjectName();
    // Get the information about the test run to view
    CString test;
    CString testRun;
    int nType = 0;

    CECClipboard cb;
    CString strInfo = cb.GetClipboardText();
    if (!strInfo.IsEmpty())
    {

        GetNextField('|', strInfo, test);
        GetNextField('|', strInfo, testRun);
        CString szType;
        GetNextField('|', strInfo, szType);

        nType = atoi(szType);
    }

    m_XResults.XInitialize(szProj, test, testRun, nType);
    CRect r;
    GetWindowRect(r);

    m_XResults.SetWindowPos(&wndTop, 0,0, r.Width(), r.Height(),SWP_NOZORDER);

    m_XResults.Invalidate();
}

int CECResultViewer::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr=="")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep,0);

    if (nPos == -1)
    {
        field = inStr;
        inStr="";
        return 0;
    }
    else if (nPos == 0)
    {
        return -1;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr=inStr.Right(len-nPos-1);
        return 0;
    }
}

BEGIN_EVENTSINK_MAP(CECResultViewer, CFormView)
//{{AFX_EVENTSINK_MAP(CECResultViewer)
ON_EVENT(CECResultViewer, IDC_XRESULTSCTRL, 1 /* XUpdateTitle */, OnXUpdateTitle, VTS_BSTR VTS_BSTR VTS_I2)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CECResultViewer::OnXUpdateTitle(LPCTSTR strTestName, LPCTSTR strRunName, short nType)
{
    CString title;
    switch (nType)
    {
    case RESULT_AUDIT:
        {
            title.Format("Result Viewer - Audit: %s %s", strTestName, strRunName);
            break;
        }
    case RESULT_REPORT:
        {
            title.Format("Result Viewer - Report: %s %s", strTestName, strRunName);
            break;
        }
    case RESULT_HISTORY:
        {
            title.Format("Result Viewer - History: %s %s", strTestName, strRunName);
            break;
        }
    }
    GetDocument()->SetTitle(title);

}

BOOL CECResultViewer::OnEraseBkgnd(CDC *pDC)
{
    return true;
}
