/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#if !defined(AFX_CECTESTLEVELMONITOR_H__33E367A1_B169_11D3_9A95_00500447FC27__INCLUDED_)
#define AFX_CECTESTLEVELMONITOR_H__33E367A1_B169_11D3_9A95_00500447FC27__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// CECTestLevelMonitor.h : header file
//
#include <vector>
using std::vector;
#include "OSTAORB/OSTAORB.h"
#include "CORBAObjects\TestManager.h"
#include "CORBAObjects\TestLevel.h"
#include "CORBAObjects\TestExecuter.h"
#include "ctime.hpp"
#include "VUserDetailsDlg.h"

#include "resource.h"

/////////////////////////////////////////////////////////////////////////////
// CECTestLevelMonitor form view

#ifndef __AFXEXT_H__
#include <afxext.h>
#endif

class CECTestLevelMonitor : public CFormView
{
protected:
    void ShowTestStatus(CString strManagerName);
    CECTestLevelMonitor();           // protected constructor used by dynamic creation
    DECLARE_DYNCREATE(CECTestLevelMonitor)

    //this enum must match the order of the bitmaps in IDB_THREADLIST_BMP
    typedef enum{enThread} ImagelistNdx;

    // Form Data
public:
    //{{AFX_DATA(CECTestLevelMonitor)
    enum { IDD = IDD_LEVEL_MONITOR };
    CButton m_ctlShowGraphs;
    CButton m_ctlSubTests;
    CButton m_BackButton;
    CStatic m_ThreadLabel;
    CStatic m_TotalThreads;
    CListCtrl   m_ThreadList;
    CListCtrl   m_LevelList;
    CButton m_ExpandButton;
    CButton m_AllThreads;
    BOOL    m_bShowSubTests;
    //}}AFX_DATA

    // Attributes
public:

    // Operations
public:

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CECTestLevelMonitor)
public:
    virtual void OnInitialUpdate();
protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    //}}AFX_VIRTUAL

    // Implementation
protected:
    virtual ~CECTestLevelMonitor();
#ifdef _DEBUG
    virtual void AssertValid() const;
    virtual void Dump(CDumpContext& dc) const;
#endif

    // Generated message map functions
    //{{AFX_MSG(CECTestLevelMonitor)
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg void OnClickLevelList(NMHDR* pNMHDR, LRESULT* pResult);
    afx_msg void OnExpand();
    afx_msg void OnBack();
    afx_msg void OnAllThreads();
    afx_msg void OnClose();
    afx_msg void OnDblclkLevelList(NMHDR* pNMHDR, LRESULT* pResult);
    afx_msg void OnShowSubtests();
    afx_msg void OnDestroy();
    afx_msg void OnPaint();
    afx_msg void OnLevelSelectColumns();
    afx_msg void OnThreadSelectColumns();
    afx_msg void OnThreadVuDetails();
    afx_msg void OnDblclkThreadList(NMHDR* pNMHDR, LRESULT* pResult);
    afx_msg void OnShowGraph();
    afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
    afx_msg void OnFullHelp();
   //}}AFX_MSG
    afx_msg void OnContextMenu(CWnd*, CPoint point);
    DECLARE_MESSAGE_MAP()
private:
    CString m_szHtmlHelpFile;

    BOOL m_bAllThreads;
    int m_nExpandLevel;
    int m_nSelTest;
    void UpdateSubTestList();
    void UpdateThreadList();
    CString m_strRegManager;
    void SetupColumns();
    CORBAServer* m_pCorba;
    BOOL InitManagerStatus();
    TestManager_var m_TM;
    TestLevel_var m_ActiveLevel;
    char* GetNextField(char sep, char*& inStr);
    vector<TestLevel_var> m_TestLevelList;
    void AddChildThreads(TestLevel_var level,CMap<UINT,UINT&,CThreadInfo,CThreadInfo&> &newThreads);

    CWinThread *m_pUpdateThread;
    CImageList m_ImageList;
    volatile bool m_bUpdateNow;
    volatile bool m_bQuit;
    volatile bool m_bAborted;

    // HashKey is taken from CMapStringToString
    //  it converts a String into a DWORD
    static UINT CECTestLevelMonitor::HashKey(LPCTSTR key)
    {
        UINT nHash = 0;
        while (*key)
            nHash = (nHash<<5) + nHash + *key++;
        return nHash;
    }

    CMap <UINT,UINT&,CThreadInfo,CThreadInfo&> m_mapThreads;
    CMap <UINT,UINT&,int,int&> m_mapThreadIterations;
    CMapStringToString m_mapIPToName;

    void UpdateThreadListView(CMap<UINT,UINT&,CThreadInfo,CThreadInfo&> &newThreads);
    void SaveSettings();
    void RestoreSettings();

    void FilterColumns(CListCtrl *pListCtrl);

    void LoadThreadInfo(const ThreadInfo& info,CThreadInfo& threadInfo);
    CString GetComputerNameFromIP(const CString& strIP);
    CVUserDetailsDlg m_dlgVUDetails;

    static UINT UpdateThread(LPVOID);
    int GetFirstInView(CListCtrl* pList);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CECTESTLEVELMONITOR_H__33E367A1_B169_11D3_9A95_00500447FC27__INCLUDED_)
