/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CYRGenericCtrls.cpp : Defines the initialization routines for the DLL.
//

#include "stdafx.h"
#include <afxdllx.h>
#include "..\..\Interfaces\CyrPlugins.h"
#include "..\..\Interfaces\CyrCtrls.h"
#include "CECAuditLogger.h"

#include "CYRGenericCtrlDoc.h"
#include "CYRGenericCtrlChildFrm.h"
#include "CECMonitorView.h"
#include "CECResultViewer.h"
#include "CECTestLevelMonitor.h"
#include "CYREventSink.h"
#include "CECDynTestGraphs.h"
#include "CECInjectorView.h"
#include "CECHttpStatsView.h"
#include "CECLoggingView.h"

static AFX_EXTENSION_MODULE CYRGenericCtrlsDLL = { NULL, NULL };
CMultiDocTemplate*      pStatusScreenDocTemplate    = NULL;
CMultiDocTemplate*      pResViewDocTemplate     = NULL;
CMultiDocTemplate*      pLevelMonDocTemplate    = NULL;
CMultiDocTemplate*      pDynGraphsTemplate      = NULL;
CMultiDocTemplate*      pInjectorViewTemplate   = NULL;
CMultiDocTemplate*      pHttpStatsTemplate      = NULL;
CMultiDocTemplate*      pLoggingDocTemplate = NULL;
CYREventSink*           pEventSink              = NULL;
CECAuditLogger*         pAuditLogger            = NULL;

static CString GetHelpFilename();

#include "CECExecuteTestDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("CYRGENERICCTRLS.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CYRGenericCtrlsDLL, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.

        new CDynLinkLibrary(CYRGenericCtrlsDLL);


        pAuditLogger = new CECAuditLogger();
    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CYRGENERICCTRLS.DLL Terminating!\n");
        // Terminate the library before destructors are called
        AfxTermExtensionModule(CYRGenericCtrlsDLL);

    }
    return 1;   // ok
}


extern "C" AFX_EXT_API
    BOOL GetCtrlsInfo(CTRLSINFO* pCtrlsInfo)
{
    new CDynLinkLibrary(CYRGenericCtrlsDLL);

    (void) strcpy (pCtrlsInfo->strName, "Generic Ctrls");
    (void) strcpy (pCtrlsInfo->strType, "Generic Ctrls");
    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL Generic_ExecuteTest(CString ProjName, CString TestName, CString & ManagerName)
{
    CECExecuteTestDlg dlg;
    dlg.SetExecuteInfo(ProjName, TestName);
    dlg.DoModal();
    return dlg.IsExecuted();
}

extern "C" AFX_EXT_API
    void Generic_MonitorTest(CString ManagerName)
{
    pStatusScreenDocTemplate->OpenDocumentFile(NULL);
}

extern "C" AFX_EXT_API
    BOOL GetCtrlDocumentInfo(CTRLDOCUMENTINFO* pCtrlDocInfo[])
{
    int ResourceIdx = -1;

    pCtrlDocInfo[++ResourceIdx] = new CTRLDOCUMENTINFO;
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strName,              "Monitor Screen");
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strFileExt,           "sts");

    pStatusScreenDocTemplate =  new CMultiDocTemplate(
                                    IDI_TEST_STATUS,
                                    RUNTIME_CLASS(CYRGenericCtrlDoc),
                                    RUNTIME_CLASS(CYRGenericCtrlChildFrm),
                                    RUNTIME_CLASS(CECMonitorView));
    pCtrlDocInfo[ResourceIdx]->dwStyles = 0;
    pCtrlDocInfo[ResourceIdx]->pDocTemplate = pStatusScreenDocTemplate;


    pCtrlDocInfo[++ResourceIdx] = new CTRLDOCUMENTINFO;
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strName,              "Result Viewer");
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strFileExt,           "rvw");
    pResViewDocTemplate =  new CMultiDocTemplate(
                               IDI_TEST_STATUS,
                               RUNTIME_CLASS(CYRGenericCtrlDoc),
                               RUNTIME_CLASS(CYRGenericCtrlChildFrm),
                               RUNTIME_CLASS(CECResultViewer));
    pCtrlDocInfo[ResourceIdx]->dwStyles = 0;
    pCtrlDocInfo[ResourceIdx]->pDocTemplate = pResViewDocTemplate;


    pCtrlDocInfo[++ResourceIdx] = new CTRLDOCUMENTINFO;
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strName,              "Detailed Test Monitor");
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strFileExt,           "tlm");

    pLevelMonDocTemplate =  new CMultiDocTemplate(
                                IDI_LEVEL_MONITOR,
                                RUNTIME_CLASS(CYRGenericCtrlDoc),
                                RUNTIME_CLASS(CYRGenericCtrlChildFrm),
                                RUNTIME_CLASS(CECTestLevelMonitor));
    pCtrlDocInfo[ResourceIdx]->dwStyles = 0;
    pCtrlDocInfo[ResourceIdx]->pDocTemplate = pLevelMonDocTemplate;


    pCtrlDocInfo[++ResourceIdx] = new CTRLDOCUMENTINFO;
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strName,              "Dynamic Test Graphs");
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strFileExt,           "tgr");
    pDynGraphsTemplate =  new CMultiDocTemplate(
                              IDI_DYN_GRAPHS,
                              RUNTIME_CLASS(CYRGenericCtrlDoc),
                              RUNTIME_CLASS(CYRGenericCtrlChildFrm),
                              RUNTIME_CLASS(CECDynTestGraphs));
    pCtrlDocInfo[ResourceIdx]->dwStyles = 0;
    pCtrlDocInfo[ResourceIdx]->pDocTemplate = pDynGraphsTemplate;


    pCtrlDocInfo[++ResourceIdx] = new CTRLDOCUMENTINFO;
    pInjectorViewTemplate =  new CMultiDocTemplate(
                                 IDR_INJECTOR_VIEW,
                                 RUNTIME_CLASS(CYRGenericCtrlDoc),
                                 RUNTIME_CLASS(CYRGenericCtrlChildFrm),
                                 RUNTIME_CLASS(CECInjectorView));
    pCtrlDocInfo[ResourceIdx]->dwStyles = 0;
    pCtrlDocInfo[ResourceIdx]->pDocTemplate = pInjectorViewTemplate;


    pCtrlDocInfo[++ResourceIdx] = new CTRLDOCUMENTINFO;
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strName,              "Http Statistics Viewer");
    pHttpStatsTemplate =  new CMultiDocTemplate(
                              IDI_VIEW_HTTP_STATS,
                              RUNTIME_CLASS(CYRGenericCtrlDoc),
                              RUNTIME_CLASS(CYRGenericCtrlChildFrm),
                              RUNTIME_CLASS(CECHttpStatsView));
    pCtrlDocInfo[ResourceIdx]->dwStyles = 0;
    pCtrlDocInfo[ResourceIdx]->pDocTemplate = pHttpStatsTemplate;


    pCtrlDocInfo[++ResourceIdx] = new CTRLDOCUMENTINFO;
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strName,              "Log File");
    (void) strcpy (pCtrlDocInfo[ResourceIdx]->strFileExt,           "log");
    pLoggingDocTemplate =  new CMultiDocTemplate(
                               IDI_LOGGING_SCREEN,
                               RUNTIME_CLASS(CYRGenericCtrlDoc),
                               RUNTIME_CLASS(CYRGenericCtrlChildFrm),
                               RUNTIME_CLASS(CECLoggingView));
    pCtrlDocInfo[ResourceIdx]->dwStyles = 0;
    pCtrlDocInfo[ResourceIdx]->pDocTemplate = pLoggingDocTemplate;

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetCtrlResInfo(CTRLRESOURCEINFO  *pResourceInfo[])
{
    int nID = -1;
    pResourceInfo[++nID] = new CTRLRESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Monitor Test...");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Run");
    pResourceInfo[nID]->dwStyles =  CYR_NEEDS_PROJECT|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = IDB_MONITOR_TEST;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Run");

    pResourceInfo[++nID] = new CTRLRESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&System Tracing");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Tools");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_PROJECT;
    pResourceInfo[nID]->nCommandID = IDB_LOGGING;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "");

    pResourceInfo[++nID] = new CTRLRESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Injector View Refresh Rate");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Injectors");
    pResourceInfo[nID]->dwStyles = CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_INJECTOR_REFRESH_RATE;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "");

    return TRUE;
}

extern "C" AFX_EXT_API
    LRESULT OnEvent(UINT *pID, UINT* pMsg)
{
    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }
        return ::SendMessage(pEventSink->GetSafeHwnd(), *pID, *pMsg, 0);
    }
    return 0;
}

extern "C" AFX_EXT_API
    BOOL OnUpdateCommandUI(UINT *pID)
{
    BOOL bState = FALSE;

    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }

        bState = pEventSink->IsEnabled(*pID);
    }

    return bState;
}

static CString GetHelpFilename()
{
    CString strHelpPath = AfxGetApp()->m_pszHelpFilePath;
    int nPos = strHelpPath.ReverseFind('\\');
    if(nPos != -1)
    {
        strHelpPath = strHelpPath.Left(nPos);
        return strHelpPath + "\\Commander.chm";
    }
    return "";
}

extern "C" AFX_EXT_API
    BOOL OnDestroy()
{
    if (pEventSink != NULL)
    {
        delete pEventSink;
        pEventSink = NULL;
    }
    if (pAuditLogger != NULL)
    {
        delete pAuditLogger;
        pAuditLogger = NULL;
    }

    return TRUE;
}
