/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECProjectViewerDlg.cpp : implementation file
//

#include "stdafx.h"
#include "AnalyzerDefs.h"
#include "..\..\interfaces\cyrplugins.h"
#include "CECProjectViewerDlg.h"
#include "CYREventSink.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CYREventSink* pEventSink;
/////////////////////////////////////////////////////////////////////////////
// CECProjectViewerDlg dialog


CECProjectViewerDlg::CECProjectViewerDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECProjectViewerDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECProjectViewerDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}



void CECProjectViewerDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECProjectViewerDlg)
    DDX_Control(pDX, IDC_XPROJECTVIEWERCTRL, m_ProjectViewer);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECProjectViewerDlg, CDialog)
//{{AFX_MSG_MAP(CECProjectViewerDlg)
ON_WM_SIZE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECProjectViewerDlg message handlers
void CECProjectViewerDlg::OnOK()
{
}


void CECProjectViewerDlg::OnCancel()
{
}


BOOL CECProjectViewerDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Get a handle to the Mainframe
    CYRServer *pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);

    // Now get the server login information from the framework
    CString szUser = pServer->GetUserName();
    CString szPass = pServer->GetPassword();

    char *pszPrefix = (char*) AfxGetMainWnd()->SendMessage (WMU_GET_PRODUCT_PREFIX);
    CString szPrefix(pszPrefix);
    free(pszPrefix);

    m_ProjectViewer.SetServer((long)pServer);
    m_ProjectViewer.XInitialize(szPrefix, "OUTPUT_TREE");
    m_ProjectViewer.XEnableMenus("OUTPUT_TREE", FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECProjectViewerDlg::OnSize(UINT nType, int cx, int cy)
{

    if( m_ProjectViewer.GetSafeHwnd() != NULL )
    {
        m_ProjectViewer.SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER | SWP_SHOWWINDOW );
    }

    CDialog::OnSize(nType, cx, cy);

}

BEGIN_EVENTSINK_MAP(CECProjectViewerDlg, CDialog)
//{{AFX_EVENTSINK_MAP(CECProjectViewerDlg)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 1 /* XChangeActiveProject */, OnXChangeActiveProject, VTS_BSTR)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 3 /* XOpenTestRun */, OnXOpenTestRun, VTS_BSTR VTS_BSTR)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 5 /* XViewResults */, OnXViewResults, VTS_BSTR VTS_BSTR VTS_I2)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Event Handler from ProjectViewer control when the active project
//          has been changed.
// @parm    LPCTSTR | ProjectName | New active project name
void CECProjectViewerDlg::OnXChangeActiveProject(LPCTSTR ProjectName)
{
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    //    Set the project in the framework
    pMainWnd->SendMessage (WMU_SET_PROJECT ,0 ,(LPARAM)ProjectName);
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Event Handler from ProjectViewer control when the user opens a
//          test run.
// @parm    LPCTSTR | TestName | The name of the test name in which the run belongs
// @parm    LPCTSTR | TestRunName | The test run name
void CECProjectViewerDlg::OnXOpenTestRun(LPCTSTR TestName, LPCTSTR TestRunName)
{
    //
    // Allow the main frame to perform the opening of the data.
    //
    pEventSink->OpenTestRun(TestName, TestRunName);
}

void CECProjectViewerDlg::SetActiveProject(CString szProject)
{
    m_ProjectViewer.SetActiveProject(szProject);
}

void CECProjectViewerDlg::OnXViewResults(LPCTSTR TestName, LPCTSTR TestRunName, short nType)
{
    //
    // Allow the main frame to perform the opening of the data.
    //
    pEventSink->OpenTestRun(TestName, TestRunName);
}
