/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc     Document
// @module  CECWorkspace | Workspace to load/save window sizes and positions
//
////////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_CECWORKSPACE_H__7CA19D91_A6E4_11D2_BCE7_00608CB45114__INCLUDED_)
#define AFX_CECWORKSPACE_H__7CA19D91_A6E4_11D2_BCE7_00608CB45114__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// CECWorkspace.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CECWorkspace window

// @class Workspace - window size/position
// @base public | CObject
class CECWorkspace : public CObject
{
    // Construction

    // @access Public Members
public:

    // @cmember TODO_DESCRIPTION
    CECWorkspace();

    // Attributes
public:

    // Operations
public:

    // Implementation

    // @access Public Members
public:

    // @cmember Runlist window position
    CString m_RunList;

    // @cmember Cumulative Frequency graph window position
    CString m_CumFreqGraph;

    // @cmember Standard Frequency graph window position
    CString m_StdFreqGraph;

    // @cmember Percentage Frequency graph window position
    CString m_PercentFreqGraph;

    // @cmember Summary Screen window position
    CString m_Summary;

    // @cmember Sortred List window position
    CString m_SortedList;

    // @cmember RTAT graph window position
    CString m_RTATGraph;

    // @cmember OTA graph window position
    CString m_OTAGraph;

    // @cmember Page Swap graph window position
    CString m_PageSwapGraph;

    // @cmember net use graph window position
    CString m_NetUseGraph;

    // @cmember Percent CPU graph window position
    CString m_PercentCPUGraph;

    // @cmember Load the window positions from the registry
    BOOL RestoreWorkspace();

    // @cmember Destructor
    virtual ~CECWorkspace();

    // @cmember Main window position
    CString m_MainWindow;

    // @cmember Holds the Zoom state of the Main Window
    BOOL m_bMainWindowZoomState;

    // @cmember Parse the window position string into a CRect object
    void ParseWindowSizePos(const CString& strSizePos, CRect& r);

    // @cmember Store the window position for a given window
    void SaveWindowPos(CString& strSizePos, const CRect& r);

    // @cmember Save the current window positions to the registry
    void SaveSettings();


    // @access Private Members
private:

    // @cmember Registry String for Summary screen position
    CString m_szSummary;

    // @cmember Registry String for Timer List Window position
    CString m_szSortedList;

    // @cmember Registry String for Cumulative frequency graph position
    CString m_szCumFreqGraph;

    // @cmember Registry String for Standard frequency graph position
    CString m_szStdFreqGraph;

    // @cmember Registry String for Percentage frequency graph position
    CString m_szPercentFreqGraph;

    // @cmember Registry String for RTAT graph position
    CString m_szRTATGraph;

    // @cmember Registry String for OTA graph position
    CString m_szOTAGraph;

    // @cmember Registry String for RunList Window position
    CString m_szRunList;

    // @cmember Registry String for Main Window zoom state
    CString m_szMainWindowZoomState;

    // @cmember Registry String for Main Window position
    CString m_szMainWindow;

    // @cmember Registrty Key for holding window positions
    CString m_szWindowsPositions;

    // @cmember Registry String for Page Swap graph position
    CString m_szPageSwapGraph;

    // @cmember Registry String for Net Use graph position
    CString m_szNetUseGraph;

    // @cmember Registry String for Percent CPU graph position
    CString m_szPercentCPUGraph;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CECWORKSPACE_H__7CA19D91_A6E4_11D2_BCE7_00608CB45114__INCLUDED_)
