/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRAnalyzer.cpp
// --------------
//
//    This is the Plug In Controller file for the CYRAnalyzer DLL.
//
//=========================================================================

#include "stdafx.h"
#include <afxdllx.h>
#include "resource.h"

static AFX_EXTENSION_MODULE CYRAnalyzerDLL      = { NULL, NULL };

static CString GetHelpFilename();
CString GetProductName();


#include "CYREventSink.h"
#include "..\..\interfaces\CYRPlugins.h"
#include "CECProjectViewerDlg.h"
#include "CECGlobalSettings.h"
#include "AnalyzerDoc.h"
#include "CECVirtGridView.h"
#include "CECDocManager.h"
#include "CECSummaryView.h"
#include "CECSortedListView.h"
#include "AnalyzerSheet.h"
#include "TabbedChildFrame.h"
#include "GraphView.h"
#include "TabbedDoc.h"
#include "CECHttpListView.h"
#include "ChildFrm.h"
#include "CompareRunGraphDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CECProjectViewerDlg*    pProjectViewerDlg       = NULL;
CECGlobalSettings*      g_pGlobalSettings       = NULL;
CYREventSink*           pEventSink              = NULL;
CECDocManager*          g_pDocManager           = NULL;

extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("CYRAnalyzer.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CYRAnalyzerDLL, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.
        g_pGlobalSettings = new CECGlobalSettings;

    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CYRAnalyzer.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CYRAnalyzerDLL);
    }
    return 1;   // ok
}

// @mfunc   Over-ridden to add document templates to the CECDocManager
// @parm    CDocTemplate* | pTemplate | Pointer to the Document template to be added

void AddDocTemplate(CDocTemplate* pTemplate)
{
    if (g_pDocManager == NULL)
        g_pDocManager = new CECDocManager;
    g_pDocManager->AddDocTemplate(pTemplate);
}

extern "C" AFX_EXT_API
    BOOL GetProductInfo(PRODUCTINFO *pProductInfo)
{
    new CDynLinkLibrary(CYRAnalyzerDLL);

    (void) strcpy (pProductInfo->strDisplayName,        "CYRANO Analyzer");
    (void) strcpy (pProductInfo->strInternalName,       "Analyzer");
    (void) strcpy (pProductInfo->strProjectPrefix,      "OpenSTA");
    (void) strcpy (pProductInfo->strTitle,      "CYRANO Analyzer PlugIn Version");
    pProductInfo->nIcon = IDI_ANALYZER_ICON;
    (void) strcpy (pProductInfo->strVersion,    "3.7.4.5");
    (void) strcpy (pProductInfo->strAboutText,  "This is the new version of CYRANO Analyzer");

    pProductInfo->dwStyles = CYR_HAS_SERVER|CYR_HAS_PROJECT|CYR_IS_SUBPRODUCT;
    pProductInfo->nBackground = 0;

#ifdef BETA
pProductInfo->nSplashScreen = 0://IDB_BETA_SPLASH;
#else
    pProductInfo->nSplashScreen = 0;//IDB_RELEASE_SPLASH;
#endif


                                  return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetResourceInfo(RESOURCEINFO *pResourceInfo[])
{
    int nID = -1;

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "Selected &Runs");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Analyzer");
    pResourceInfo[nID]->dwStyles = CYR_EXTENDED_UI_UPDATE|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_VIEW_RUNLIST;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Analyzer Toolkit");

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Modify Selection");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Analyzer");
    pResourceInfo[nID]->dwStyles = CYR_EXTENDED_UI_UPDATE|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_EDIT_SELECTION;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Analyzer Toolkit");

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "Timer &List");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Analyzer");
    pResourceInfo[nID]->dwStyles = CYR_EXTENDED_UI_UPDATE|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_SORTED_LIST;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Analyzer Toolkit");

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Statistical Summary");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Analyzer");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_SEPARATOR|CYR_EXTENDED_UI_UPDATE|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_SUMMARY;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Analyzer Toolkit");

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Http Statistics List");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Analyzer");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_SEPARATOR|CYR_EXTENDED_UI_UPDATE|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_HTTP_LIST;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Analyzer Toolkit");

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Compare Test Runs");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Analyzer");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_SEPARATOR|CYR_EXTENDED_UI_UPDATE|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_COMPARE_RUNS;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Analyzer Toolkit");

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetWindowInfo(WINDOWINFO *pWindowInfo[])
{
    int nID = -1;

    pWindowInfo[++nID] = new WINDOWINFO;

    (void) strcpy (pWindowInfo[nID]->strName,   "Product View");
    pWindowInfo[nID]->nType     = PROJECT_WINDOW;
    pWindowInfo[nID]->dwStyles  = 0;
    pWindowInfo[nID]->hWnd      = 0;

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetOutputInfo(OUTPUTINFO *pOutputInfo[])
{
    /*  int nID = -1;
            
        pOutputInfo[++nID] = new OUTPUTINFO;
     
        (void) strcpy (pOutputInfo[nID]->strName,               "Output");
        pOutputInfo[nID]->hWnd = 0;
     
        pOutputInfo[++nID] = new OUTPUTINFO;
     
        (void) strcpy (pOutputInfo[nID]->strName,               "Find");
        pOutputInfo[nID]->hWnd = 0;
     
     
        return TRUE; */
    return false;
}

extern "C" AFX_EXT_API
    BOOL GetDocumentInfo(DOCUMENTINFO *pDocumentInfo[])
{
    //
    // Register the application's document templates.  Document templates
    //  serve as the connection between documents, frame windows and views.
    //
    CMultiDocTemplate *pDocTemplate;
    pDocTemplate = new CMultiDocTemplate(
                       IDR_GRAPHTYPE,
                       RUNTIME_CLASS(CTabbedDoc),
                       RUNTIME_CLASS(CTabbedChildFrame), // custom MDI child frame
                       RUNTIME_CLASS(CView));
    AddDocTemplate(pDocTemplate);

    pDocTemplate = new CMultiDocTemplate(
                       IDR_SUMMARYTYPE,
                       RUNTIME_CLASS(CTabbedDoc),
                       RUNTIME_CLASS(CTabbedChildFrame), // custom MDI child frame
                       RUNTIME_CLASS(CView));
    AddDocTemplate(pDocTemplate);

    pDocTemplate = new CMultiDocTemplate(
                       IDR_SORTEDTYPE,
                       RUNTIME_CLASS(CTabbedDoc),
                       RUNTIME_CLASS(CTabbedChildFrame), // custom MDI child frame
                       RUNTIME_CLASS(CECSortedListView));
    AddDocTemplate(pDocTemplate);

    pDocTemplate = new CMultiDocTemplate(
                       IDR_HTTP_LIST_TYPE,
                       RUNTIME_CLASS(CTabbedDoc),
                       RUNTIME_CLASS(CTabbedChildFrame), // custom MDI child frame
                       RUNTIME_CLASS(CECHttpListView));
    AddDocTemplate(pDocTemplate);

    pDocTemplate = new CMultiDocTemplate(
                       IDR_COMPARERUNTYPE,
                       RUNTIME_CLASS(CCompareRunGraphDoc),
                       RUNTIME_CLASS(CChildFrame), // custom MDI child frame
                       RUNTIME_CLASS(CGraphView));
    AddDocTemplate(pDocTemplate);

    return false;
}

extern "C" AFX_EXT_API
    BOOL OnUpdateCommandUI(UINT *pID)
{
    BOOL bState = FALSE;

    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }

        bState = pEventSink->IsEnabled(*pID);
    }

    return bState;
}

extern "C" AFX_EXT_API
    BOOL OnUpdateCommandUIEx(UINT *pID,CCmdUI *pCmdUI)
{
    BOOL bState = FALSE;

    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }

        bState = pEventSink->OnCommandUIUpdate(*pID,pCmdUI);
    }

    return bState;
}

extern "C" AFX_EXT_API
    BOOL OnEvent(UINT *pID, UINT* pMsg)
{
    BOOL bState = FALSE;
    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }

        if (pMsg == NULL)
        {
            return ::SendMessage(pEventSink->GetSafeHwnd(), *pID, 0, NULL);
        }
        else
        {
            return ::SendMessage(pEventSink->GetSafeHwnd(), *pID, *pMsg, 0);
        }
    }
    return bState;
}

extern "C" AFX_EXT_API
    CWnd* CreateDockWnd(CWnd* pParent)
{
    if (pProjectViewerDlg == NULL)
    {
        pProjectViewerDlg = new CECProjectViewerDlg();
    }

    pProjectViewerDlg->Create(IDD_PROJECT_VIEWER, pParent);
    pProjectViewerDlg->ShowWindow(SW_SHOW);

    return (CWnd*)pProjectViewerDlg;
}

extern "C" AFX_EXT_API
    BOOL OnDestroy()
{
    if (pEventSink != NULL)
    {
        delete pEventSink;
        pEventSink = NULL;
    }

    delete g_pGlobalSettings;
    g_pGlobalSettings = NULL;
    delete g_pDocManager;
    g_pDocManager = NULL;
    return TRUE;
}

static CString GetHelpFilename()
{
    CString strHelpPath = AfxGetApp()->m_pszHelpFilePath;
    int nPos = strHelpPath.ReverseFind('\\');
    if(nPos != -1)
    {
        strHelpPath = strHelpPath.Left(nPos);
        return strHelpPath + "\\Commander.chm";
    }
    return "";
}
