/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// GraphView.cpp : implementation file
//

#include "stdafx.h"
#include "GraphView.h"
#include "resource.h"
#include "CECGraphCustomize.h"
#include "GraphTypes.h"
#include "CECExcelExport.h"
#include "TabbedDoc.h"
#include "TabbedChildFrame.h"
#include "ChildFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGraphView

IMPLEMENT_DYNCREATE(CGraphView, CFormView)

CGraphView::CGraphView():
        CFormView(CGraphView::IDD)
{
    m_pDocument = NULL;
    m_pGraph = NULL;
}

CGraphView::~CGraphView()
{
    delete m_pGraph;
}

void CGraphView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CGraphView)
    DDX_Control(pDX,IDC_XCHARTCTRL,m_ctlChart);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CGraphView, CFormView)
//{{AFX_MSG_MAP(CGraphView)
ON_COMMAND(ID_GRAPH_CUSTOMIZE, OnGraphCustomize)
ON_WM_CONTEXTMENU()
ON_COMMAND(ID_EXPORT, OnExport)
ON_WM_DESTROY()
ON_WM_SIZE()
ON_WM_ERASEBKGND()
//}}AFX_MSG_MAP
ON_MESSAGE(WMU_DATA_CHANGED,OnGraphDataChanged)
ON_COMMAND_RANGE(GRAPH_MENU_ITEM_ID_START,GRAPH_MENU_ITEM_ID_END,OnGraphMenuItemCommand)
ON_UPDATE_COMMAND_UI(ID_FILE_PRINT,OnUpdateFilePrint)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphView drawing

void CGraphView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
    CFormView::OnDraw(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// CGraphView diagnostics

#ifdef _DEBUG
void CGraphView::AssertValid() const
{
    CFormView::AssertValid();
}

void CGraphView::Dump(CDumpContext& dc) const
{
    CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGraphView message handlers
void CGraphView::OnInitialUpdate()
{
    CFormView::OnInitialUpdate();

    GetParentFrame()->RecalcLayout();

    CRect r;
    GetClientRect(r);
    m_ctlChart.SetWindowPos(NULL, 0,0,r.Width(),r.Height(),SWP_NOZORDER);

    SIZE sz= {0};
    SetScrollSizes(MM_TEXT,sz);
}

void CGraphView::OnGraphCustomize()
{
    CECGraphCustomize dlg("Customize Graph");
    dlg.SetChartStyle(m_pGraph->GetGraphType());
    if(dlg.DoModal()==IDOK)
    {
        CString str;
        CMDIFrameWnd *pParent = (CMDIFrameWnd*)AfxGetMainWnd();
        CWnd *pMDIClient = CWnd::FromHandle(pParent->m_hWndMDIClient);
		
		CWnd *pChild = pMDIClient->GetWindow(GW_CHILD);
        while(pChild)
        {
            CTabbedChildFrame *pChildFrame = dynamic_cast<CTabbedChildFrame *>(pChild);
            if(pChildFrame)
            {
                int nNumTabs = pChildFrame->GetTabCount();
                for(int nTab=0;nTab<nNumTabs;nTab++)
                {
				   	CWnd *pWnd;
					pChildFrame->GetTabInfo(nTab,str,pWnd);

					CGraphView *pView =dynamic_cast<CGraphView *>(pWnd);
					if(pView)
					{
						pView->m_pGraph->SetTitleComponent();
						pView->m_pGraph->SetDisplayComponent();
						pView->m_pGraph->SetDataColours();
					}
                }
            }
			else
			if(pChild->IsKindOf(RUNTIME_CLASS(CChildFrame)))
			{
				CGraphView *pView = dynamic_cast<CGraphView*>(((CChildFrame*)pChild)->GetActiveView());
				if(pView)
				{
					pView->m_pGraph->SetTitleComponent();
					pView->m_pGraph->SetDisplayComponent();
					pView->m_pGraph->SetDataColours();
				}
			}

            pChild = pChild->GetWindow(GW_HWNDNEXT);
        }

        m_pGraph->SetGraphType(dlg.GetChartStyle());
    }
}

void CGraphView::OnContextMenu(CWnd* pWnd, CPoint point)
{
    //This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    VERIFY(menu.LoadMenu(CG_IDR_POPUP_ANALYZER_VIEW));

    //hide the export button if Excel is not available
    if(!CECExcelExport::IsExcelRegistered())
        menu.RemoveMenu(ID_EXPORT,MF_BYCOMMAND);

    //Add any graph specific menu items
    CGraphTypeBase::MenuItemArray arMenuItems;

    m_pGraph->GetMenuItems(arMenuItems);

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    for(int nNdx=0;nNdx<arMenuItems.GetSize();nNdx++)
    {
        pPopup->InsertMenu(-1,MF_BYPOSITION,arMenuItems[nNdx].uiCommandID,arMenuItems[nNdx].strItem);
        pPopup->EnableMenuItem(arMenuItems[nNdx].uiCommandID,MF_ENABLED|MF_BYCOMMAND);
    }

    CWnd* pWndPopupOwner = this;

    while (pWndPopupOwner->GetStyle() & WS_CHILD)
        pWndPopupOwner = pWndPopupOwner->GetParent();

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,
                           pWndPopupOwner);
}

void CGraphView::OnExport()
{
    m_pGraph->ExportToExcel();
}

void CGraphView::OnGraphMenuItemCommand(UINT nID)
{
    m_pGraph->OnCommand(nID,this);
}

LRESULT CGraphView::OnGraphDataChanged(WPARAM,LPARAM lParam)
{
    int nRun = (int)lParam;
    CTabbedDoc*pDoc = dynamic_cast<CTabbedDoc*>(m_pDocument);
    if(pDoc)
    {
        pDoc->UpdateCombinedRun(nRun);
    }
    Invalidate(FALSE);
    return 0;
}

void CGraphView::OnDestroy()
{
    CFormView::OnDestroy();
}

void CGraphView::OnSize(UINT nType, int cx, int cy)
{
    CFormView::OnSize(nType, cx, cy);

    if(m_ctlChart.GetSafeHwnd())
        m_ctlChart.MoveWindow(0,0,cx,cy);
}

BOOL CGraphView::OnEraseBkgnd(CDC *pDC)
{

    return TRUE;//CFormView::OnEraseBkgnd(pDC);
}


void CGraphView::OnUpdateFilePrint(CCmdUI *pCmdUI)
{
    pCmdUI->Enable(TRUE);
}
