/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TabbedChildFrame.cpp : implementation file
//

#include "stdafx.h"
#include "TabbedChildFrame.h"
#include "CYREventSink.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CYREventSink* pEventSink;

#define ID_TAB_CTRL 0x2345

/////////////////////////////////////////////////////////////////////////////
// CMyTabCtrl message handlers

BEGIN_MESSAGE_MAP(CMyTabCtrl, CTabCtrl)
	//{{AFX_MSG_MAP(CMyTabCtrl)
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CMyTabCtrl::OnEraseBkgnd(CDC* pDC) 
{
	CRect rcClient;
	GetClientRect(&rcClient);

	CRect rcTabs(rcClient);
	AdjustRect(FALSE,rcTabs);

	pDC->ExcludeClipRect(rcTabs);

	return CTabCtrl::OnEraseBkgnd(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// CTabbedChildFrame

IMPLEMENT_DYNCREATE(CTabbedChildFrame, CAnalyzerSheet)

CTabbedChildFrame::CTabbedChildFrame()
{
    m_bInitialised = false;
    m_pCurrentView = NULL;
}

CTabbedChildFrame::~CTabbedChildFrame()
{}


BEGIN_MESSAGE_MAP(CTabbedChildFrame, CAnalyzerSheet)
//{{AFX_MSG_MAP(CTabbedChildFrame)
ON_WM_DESTROY()
ON_WM_SIZE()
ON_WM_ERASEBKGND()
//}}AFX_MSG_MAP
ON_NOTIFY(TCN_SELCHANGE,ID_TAB_CTRL,OnSelchangeTab)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTabbedChildFrame message handlers

BOOL CTabbedChildFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
    return m_TabCtrl.Create(WS_VISIBLE|WS_CHILD|TCS_SINGLELINE|TCS_FOCUSONBUTTONDOWN|TCS_BOTTOM,
                            CRect(0,0,lpcs->cx,lpcs->cy),this,ID_TAB_CTRL);
}

BOOL CTabbedChildFrame::AddTab(CRuntimeClass *pViewRTC,const CString& strLabel)
{
    int nNumItems = m_TabCtrl.GetItemCount();
    int nNewTab = m_TabCtrl.InsertItem(nNumItems,strLabel);
    if(nNewTab != -1)
    {
        CView *pView = (CView*)pViewRTC->CreateObject();
        if(!pView)
            return false;

        if(pView->Create(NULL,strLabel,WS_CHILD,CRect(0,0,0,0),&m_TabCtrl,0))
        {
            pView->OnInitialUpdate();
            m_arViews.Add(pView);

            m_TabCtrl.SetCurSel(nNewTab);
            UpdateView();
            return true;
        }
    }
    return false;
}

BOOL CTabbedChildFrame::AddTab(CView *pView,const CString& strLabel)
{
    if(!pView)
        return false;

    int nNumItems = m_TabCtrl.GetItemCount();
    int nNewTab = m_TabCtrl.InsertItem(nNumItems,strLabel);
    if(nNewTab != -1)
    {
        pView->SetParent(&m_TabCtrl);
        pView->OnInitialUpdate();
        m_arViews.Add(pView);
        m_TabCtrl.SetCurSel(nNewTab);
        UpdateView();
        return true;
    }

    return false;
}


BOOL CTabbedChildFrame::RemoveTab(int nTabNdx)
{
    if(m_TabCtrl.DeleteItem(nTabNdx))
    {
        CView *pView = m_arViews[nTabNdx];
        m_arViews.RemoveAt(nTabNdx);
        //      pView->DestroyWindow();
        //      delete pView;

        int nNumTabs = m_TabCtrl.GetItemCount();
        if(!nNumTabs)
            m_pCurrentView = NULL;
        else
        {
            if(nTabNdx>=nNumTabs)
                nTabNdx = nNumTabs-1;

            m_TabCtrl.SetCurFocus(nTabNdx);
            m_TabCtrl.SetCurSel(nTabNdx);
        }
        UpdateView();

        return true;
    }
    return false;
}

int CTabbedChildFrame::GetTabCount()
{
    return m_TabCtrl.GetItemCount();
}

BOOL CTabbedChildFrame::GetTabInfo(int nNdx,CString& strLabel,CWnd*& pWnd)
{
    if(m_arViews.GetSize()<=nNdx)
        return false;

    pWnd = m_arViews[nNdx];

    const int nTextBufSize = 200;
    char szText[nTextBufSize];
    TCITEM item;
    item.mask = TCIF_TEXT;
    item.pszText = szText;
    item.cchTextMax = nTextBufSize;
    if(!m_TabCtrl.GetItem(nNdx,&item))
        return false;

    strLabel = item.pszText;

    return true;
}

int CTabbedChildFrame::GetTabIndex(const CString& strTabName)
{
    int nTabCount = GetTabCount();

    const int nTextBufSize = 200;
    char szText[nTextBufSize];
    TCITEM item;
    item.mask = TCIF_TEXT;
    item.pszText = szText;
    item.cchTextMax = nTextBufSize;
    for(int nNdx=0;nNdx<nTabCount;nNdx++)
    {
        if(!m_TabCtrl.GetItem(nNdx,&item))
            continue;
        if(strTabName.Compare(item.pszText)==0)
            return nNdx;
    }

    return -1;
}

void CTabbedChildFrame::OnDestroy()
{
    for(int nView=0;nView<m_arViews.GetSize();nView++)
    {
        m_arViews[nView]->DestroyWindow();
        //      delete m_arViews[nView];
    }


    pEventSink->SendMessage(WM_PARENTNOTIFY,MAKEWPARAM(WM_DESTROY,0),(LPARAM)GetSafeHwnd());

    CAnalyzerSheet::OnDestroy();
}


void CTabbedChildFrame::OnSize(UINT nType, int cx, int cy)
{
    CAnalyzerSheet::OnSize(nType, cx, cy);

    if(m_TabCtrl.GetSafeHwnd())
    {
        m_TabCtrl.MoveWindow(0,0,cx,cy);

        CRect rc(0,0,cx,cy);
        m_TabCtrl.AdjustRect(FALSE,rc);

        if(m_pCurrentView)
            m_pCurrentView->MoveWindow(rc,FALSE);
    }
}

void CTabbedChildFrame::OnSelchangeTab(NMHDR* pNMHDR, LRESULT* pResult)
{
    UpdateView();

    *pResult = 0;
}

void CTabbedChildFrame::UpdateView()
{
    int nNumViews = m_arViews.GetSize();
    int nItem = m_TabCtrl.GetCurSel();
    if((nItem != -1) && (nNumViews>nItem) && m_arViews[nItem])
    {
        if(m_pCurrentView && m_pCurrentView->GetSafeHwnd())
            m_pCurrentView->ShowWindow(SW_HIDE);

        m_pCurrentView= m_arViews[nItem];
        m_pCurrentView->ShowWindow(SW_SHOW);

        CRect rc;
        GetClientRect(&rc);
        m_TabCtrl.AdjustRect(FALSE,rc);

        if(m_pCurrentView)
            m_pCurrentView->MoveWindow(rc,FALSE);
    }
}

BOOL CTabbedChildFrame::OnEraseBkgnd(CDC* pDC)
{
    return true;
}


