/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TabbedDoc.cpp : implementation file
//

#include "stdafx.h"
#include "TabbedDoc.h"
#include "CECGlobalSettings.h"
#include "CYREventSink.h"
#include "TabbedChildFrame.h"
#include "GraphView.h"
#include "GraphTypes.h"
#include "CECSummaryView.h"
#include "CECSortedListView.h"
#include "CECHttpListView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CECGlobalSettings* g_pGlobalSettings;
extern CYREventSink *pEventSink;
/////////////////////////////////////////////////////////////////////////////
// CTabbedDoc

IMPLEMENT_DYNCREATE(CTabbedDoc, CAnalyzerDoc)

CTabbedDoc::CTabbedDoc()
{
    m_pTabbedFrame = NULL;
    m_nGraphType = enInvalidGraphType;
    m_bNeedsTimers = false;
	m_bNeedsFrequencyData=false;
    m_bNeedsOTA = false;
    m_bNeedsHttp = false;
    m_bHasCombined = false;
	m_bNeedsVUData = false;
    m_prtcViewClass = NULL;
    m_pCombinedView = NULL;
}

BOOL CTabbedDoc::OnNewDocument()
{
    if (!CAnalyzerDoc::OnNewDocument())
        return FALSE;
    return TRUE;
}

CTabbedDoc::~CTabbedDoc()
{
    //  if(m_pCombinedView)
    //      delete m_pCombinedView;

}



BEGIN_MESSAGE_MAP(CTabbedDoc, CAnalyzerDoc)
//{{AFX_MSG_MAP(CTabbedDoc)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTabbedDoc diagnostics

#ifdef _DEBUG
void CTabbedDoc::AssertValid() const
{
    CAnalyzerDoc::AssertValid();
}

void CTabbedDoc::Dump(CDumpContext& dc) const
{
    CAnalyzerDoc::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTabbedDoc serialization

void CTabbedDoc::Serialize(CArchive& ar)
{
    if (ar.IsStoring())
    {
        // TODO: add storing code here
    }

    else
    {
        // TODO: add loading code here
    }

}

/////////////////////////////////////////////////////////////////////////////
// CTabbedDoc commands
void CTabbedDoc::SetupGraph(GraphTypesEnum nGraphType,CTabbedChildFrame *pFrame)
{
    m_nGraphType = nGraphType;
    m_pTabbedFrame = pFrame;
    ASSERT(m_pTabbedFrame != NULL);

    m_bNeedsTimers = false;
    m_bNeedsOTA = false;
    m_bNeedsHttp = false;
    m_bHasCombined = false;
	m_bNeedsFrequencyData=false;
	m_bNeedsVUData = false;
    m_prtcViewClass = RUNTIME_CLASS(CGraphView);
    CString strWindowText;

    switch(m_nGraphType)
    {
    case enCumFreqGraph:
        m_bNeedsFrequencyData = true;
        m_bHasCombined = true;
        strWindowText = _T("Timer Values Cumulative Frequency");
        break;
    case enStdFreqGraph:
        m_bNeedsFrequencyData = true;
        m_bHasCombined = true;
        strWindowText = _T("Timer Values Standard Frequency");
        break;
    case enPercentFreqGraph:
        m_bNeedsFrequencyData = true;
        m_bHasCombined = true;
        strWindowText = _T("Timer Values Percentage Frequency");
        break;
    case enRTATGraph:
        m_bNeedsTimers = true;
        m_bHasCombined = true;
        strWindowText = _T("Timer Values v Active Users");
        break;
    case enRTTimeGraph:
        m_bNeedsTimers = true;
        m_bHasCombined = true;
        strWindowText = _T("Timer Values v Elapsed Time");
        break;
    case enUsersTimeGraph:
        m_bNeedsTimers = true;
        m_bHasCombined = true;
        strWindowText = _T("Active Users v Elapsed Time");
        break;
    case enSimpleOTAGraph:
        m_bNeedsOTA = true;
        strWindowText = _T("Custom Data v Elapsed Time");
        break;
    case enPercentCPUGraph:
        m_bNeedsOTA = true;
        m_bHasCombined = true;
        strWindowText = _T("Percentage CPU usage v Elapsed Time");
        break;
    case enPageSwapGraph:
        m_bNeedsOTA = true;
        m_bHasCombined = true;
        strWindowText = _T("Page Swapping v Elapsed Time");
        break;
    case enPercentNetUseGraph:
        m_bNeedsOTA = true;
        m_bHasCombined = true;
        strWindowText = _T("Percentage Network Usage v Elapsed Time");
        break;
    case enSummaryScreen:
        m_bNeedsTimers = true;
        m_prtcViewClass = RUNTIME_CLASS(CECSummaryView);
        strWindowText = _T("Statistical Summary");
        break;
    case enSortedList:
        m_bNeedsTimers = true;
        m_prtcViewClass = RUNTIME_CLASS(CECSortedListView);
        strWindowText = _T("Timer List");
        break;
    case enKbPerSecondGraph:
        m_bNeedsHttp = true;
        m_bHasCombined = true;
        strWindowText = _T("HTTP Monitored Bytes / Sec v Elapsed Time");
        break;
    case enAVPRTPerUserGraph:
        m_bNeedsHttp = true;
        m_bHasCombined = true;
        strWindowText = _T("HTTP Response Time (Average / Sec) v Number Of Responses / Sec");
        break;
    case enErrorsPerUserGraph:
        m_bNeedsHttp = true;
        m_bHasCombined = true;
        strWindowText = _T("HTTP Errors v Active Users");
        break;
    case enErrorsTimeGraph:
        m_bNeedsHttp = true;
        m_bHasCombined = true;
        strWindowText = _T("HTTP Errors v Elapsed Time");
        break;
    case enHitsPerSecondGraph:
        m_bNeedsHttp = true;
        m_bHasCombined = true;
        strWindowText = _T("HTTP Responses / Sec v Elapsed Time");
        break;
    case enPagesPerSecondGraph:
        m_bNeedsHttp = true;
        m_bHasCombined = true;
        strWindowText = _T("HTTP Response Time (Average / Sec) v Elapsed Time");
        break;
    case enHttpList:
        m_bNeedsHttp = true;
        strWindowText = _T("Http Statistics List");
        m_prtcViewClass = RUNTIME_CLASS(CECHttpListView);
        break;
	case enVUsersTimeGraph:
		m_bNeedsVUData = true;
        strWindowText = _T("HTTP Active Users v Elapsed Time");
        m_bHasCombined = true;
        break;
    default:
        ASSERT(FALSE);
        break;
    }

    m_pTabbedFrame->SetWindowText(strWindowText);
    m_pTabbedFrame->SetTitle(strWindowText);

    if(m_bHasCombined)
    {
        ASSERT(m_pCombinedView==NULL);

        m_pCombinedView = (CView*)m_prtcViewClass->CreateObject();
        m_pCombinedView->Create(NULL, "Combined", WS_CHILD,CRect(0,0,0,0),m_pTabbedFrame, 0);
        m_pCombinedView->OnInitialUpdate();

        if(m_pCombinedView->IsKindOf( RUNTIME_CLASS(CGraphView) ) )
        {
            ((CGraphView*)m_pCombinedView)->m_pGraph = CreateNewGraph((CGraphView*)m_pCombinedView);
            ((CGraphView*)m_pCombinedView)->SetDocument(this);
        }
    }

    UpdateTabs();
}

void CTabbedDoc::UpdateTabs()
{
    ASSERT(m_prtcViewClass!=NULL);

    //Get a list of the indices of the selected test runs
    CString configStr = "|" + pEventSink->GetSelectedRuns() + "|";

    //Remove combined tab so we can add it again
    //at the end, so it appears as the right most tab
    if(m_bHasCombined)
    {
        int nCombinedNdx = m_pTabbedFrame->GetTabIndex("Combined");
        if(nCombinedNdx != -1)
            m_pTabbedFrame->RemoveTab(nCombinedNdx);
    }

    CString strNdx;

    //Check whether each test run needs a tab or not
    int nNumRuns = g_pGlobalSettings->GetLoadedCount();
    for(int nRun=0;nRun<nNumRuns;nRun++)
    {
        CString strTestRun;
        g_pGlobalSettings->GetRunInfo(nRun,strTestRun);

        //Find if this run should be displayed
        strNdx.Format("|%d|",nRun);
        BOOL bIsRequired = (configStr.Find(strNdx,0) != -1);

        //If there are no timers, we do not need to display this test run
        if(m_bNeedsTimers && !pEventSink->GetHasTimers())
            bIsRequired = false;

		if(m_bNeedsFrequencyData && !pEventSink->GetHasFrequencyData())
			bIsRequired = false;

        //If there is no OTA data, we do not need to display this test run
        if(m_bNeedsOTA && !pEventSink->GetHasOTA())
            bIsRequired = false;

        //If there is no HTTP data, we do not need to display this test run
        if(m_bNeedsHttp && !pEventSink->GetHasHttp())
            bIsRequired = false;

		if(m_bNeedsVUData && !pEventSink->GetHasVUData())
			bIsRequired = false;

        //If a tab exists for this run get its index
        CString strTabName = FormatRunName(strTestRun);
        int nTabNdx = m_pTabbedFrame->GetTabIndex(strTabName);
        if(!bIsRequired && (nTabNdx!=-1))
        {
            RemoveTab(nTabNdx);

            if(m_bHasCombined)
                RemoveCombinedData(nRun);
        }
        else
        if(bIsRequired && (nTabNdx==-1))
        {
            if(!AddTab(strTabName,nRun))
                continue;

            if(m_bHasCombined)
                AddCombinedData(nRun);
        }
    }


    int nNumTabs = m_pTabbedFrame->GetTabCount();
    //Show the combined tab if there is more than one tab
    if(m_bHasCombined && nNumTabs > 1)
    {
        m_pTabbedFrame->AddTab(m_pCombinedView,"Combined");
        nNumTabs++;

        if(m_pCombinedView->IsKindOf( RUNTIME_CLASS(CGraphView) ) )
            ((CGraphView*)m_pCombinedView)->m_pGraph->SetTitleComponent();
    }

    //Hide the window if there are no tabs
    m_pTabbedFrame->ShowWindow(nNumTabs ? SW_SHOW : SW_HIDE);

    m_pTabbedFrame->Invalidate();
}

BOOL CTabbedDoc::RemoveTab(int nTabNdx)
{
    CString strTabName;
    CWnd *pWnd=NULL;
    m_pTabbedFrame->GetTabInfo(nTabNdx,strTabName,pWnd);

    m_pTabbedFrame->RemoveTab(nTabNdx);
    if(pWnd != m_pCombinedView)
        pWnd->DestroyWindow();

    return true;
}

BOOL CTabbedDoc::AddTab(const CString& strTabName,int nTestRun)
{
    m_pTabbedFrame->AddTab(m_prtcViewClass,strTabName);

    CString strTab;
    CWnd *pWnd=NULL;
    int nNdx = m_pTabbedFrame->GetTabIndex(strTabName);
    m_pTabbedFrame->GetTabInfo(nNdx,strTab,pWnd);

    if(!pWnd)
        return false;

    if(pWnd->IsKindOf( RUNTIME_CLASS(CGraphView) ) )
    {
        CGraphView *pView = (CGraphView*)pWnd;
        pView->SetDocument(this);

        CGraphTypeBase *pGraph = CreateNewGraph(pView);
        if(!pGraph || !pGraph->AddTestRun(nTestRun))
        {
            RemoveTab(nNdx);
            return false;
        }
    }
    else
        if(pWnd->IsKindOf( RUNTIME_CLASS(CECSummaryView)))
        {
            CECSummaryView* pView = (CECSummaryView*)pWnd;
            pView->Initialize(nTestRun);
        }
        else
            if(pWnd->IsKindOf(RUNTIME_CLASS(CECSortedListView)))
            {
                CECSortedListView *pView = (CECSortedListView*)pWnd;
                pView->Initialize(nTestRun);
            }
            else
                if(pWnd->IsKindOf(RUNTIME_CLASS(CECHttpListView)))
                {
                    CECHttpListView *pView = (CECHttpListView*)pWnd;
                    pView->Initialize(nTestRun);
                }
                else
                {
                    ASSERT(FALSE);
                }

    return true;
}

BOOL CTabbedDoc::AddCombinedData(int nTestRun)
{
    ASSERT(m_pCombinedView!=NULL);

    if(m_pCombinedView->IsKindOf( RUNTIME_CLASS(CGraphView) ) )
    {
        CGraphView *pCombinedView = (CGraphView*)m_pCombinedView;

        CString strTestRun;
        g_pGlobalSettings->GetRunInfo(nTestRun,strTestRun);

        //If a tab exists for this run get its index
        CString strTabName = FormatRunName(strTestRun);
        int nTabNdx = m_pTabbedFrame->GetTabIndex(strTabName);

        CWnd *pWnd=NULL;
        m_pTabbedFrame->GetTabInfo(nTabNdx,strTabName,pWnd);

        if(pWnd->IsKindOf( RUNTIME_CLASS(CGraphView) ) )
        {
            ((CGraphTypeBase*)pCombinedView->m_pGraph)->AddDataFromGraph((CGraphView*)pWnd);
        }
    }
    return true;
}

BOOL CTabbedDoc::RemoveCombinedData(int nTestRun)
{
    ASSERT(m_pCombinedView!=NULL);

    if(m_pCombinedView->IsKindOf( RUNTIME_CLASS(CGraphView) ) )
    {
        CGraphView *pView = (CGraphView*)m_pCombinedView;
        ((CGraphTypeBase*)pView->m_pGraph)->RemoveTestRun(nTestRun);
    }
    else
        return false;

    return true;
}

CString CTabbedDoc::FormatRunName(const CString& strRunName)
{
    int nPos = strRunName.ReverseFind('.');
    ASSERT(nPos != -1);
    return strRunName.Left(nPos);
}

CGraphTypeBase *CTabbedDoc::CreateNewGraph(CGraphView *pView)
{
    switch(m_nGraphType)
    {
    case enCumFreqGraph:
        return new CCumFreqGraph(pView);
    case enStdFreqGraph:
        return new CStdFreqGraph(pView);
    case enPercentFreqGraph:
        return new CPercentFreqGraph(pView);
    case enRTATGraph:
        return new CRTATGraph(pView);
    case enRTTimeGraph:
        return new CRTTimeGraph(pView);
    case enSimpleOTAGraph:
        return new CCustomOTAGraph(pView);
    case enPercentCPUGraph:
        return new CPercentCPUGraph(pView);
    case enPageSwapGraph:
        return new CPageSwapGraph(pView);
    case enPercentNetUseGraph:
        return new CNetUseGraph(pView);
    case enKbPerSecondGraph:
        return new CThroughputGraph(pView);
    case enAVPRTPerUserGraph:
        return new CAvPageRTPerUserGraph(pView);
    case enErrorsPerUserGraph:
        return new CErrorsPerUserGraph(pView);
    case enErrorsTimeGraph:
        return new CErrorsTimeGraph(pView);
    case enHitsPerSecondGraph:
        return new CHitsPerSecondGraph(pView);
    case enPagesPerSecondGraph:
        return new CPagesPerSecondGraph(pView);
	case enVUsersTimeGraph:
		return new CVUsersTimeGraph(pView);
    default:
        ASSERT(FALSE);
        break;
    }

    return NULL;
}

void CTabbedDoc::RemoveAllTabs()
{
    int nNumTabs=m_pTabbedFrame->GetTabCount();
    for(int nTab=nNumTabs-1;nTab>=0;nTab--)
    {
        RemoveTab(nTab);
    }
}

void CTabbedDoc::UpdateCombinedRun(int nRunIndex)
{
    if(m_bHasCombined)
    {
        RemoveCombinedData(nRunIndex);
        AddCombinedData(nRunIndex);
    }
}
