// CompareRunGraphDoc.cpp : implementation file
//

#include "stdafx.h"
#include "CECGlobalSettings.h"
#include "CYREventSink.h"
#include "CECVirtGridView.h"
#include "CompareRunGraphDoc.h"
#include "GraphView.h"
#include "ComDef.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CECGlobalSettings* g_pGlobalSettings;
extern CYREventSink* pEventSink;
/////////////////////////////////////////////////////////////////////////////
// CCompareRunGraphDoc

IMPLEMENT_DYNCREATE(CCompareRunGraphDoc, CDocument)

CCompareRunGraphDoc::CCompareRunGraphDoc():
	m_pGraphView(NULL),
	m_pCompareRunGraph(NULL)
{
}

CCompareRunGraphDoc::~CCompareRunGraphDoc()
{
}

BOOL CCompareRunGraphDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	POSITION pos = GetFirstViewPosition();
	while(pos)
	{
		CView *pView = GetNextView(pos);
		if(pView->IsKindOf(RUNTIME_CLASS(CGraphView)))
		{
			m_pGraphView = (CGraphView*)pView;
		}
	}
	return TRUE;
}

BEGIN_MESSAGE_MAP(CCompareRunGraphDoc, CDocument)
	//{{AFX_MSG_MAP(CCompareRunGraphDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCompareRunGraphDoc diagnostics

#ifdef _DEBUG
void CCompareRunGraphDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CCompareRunGraphDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCompareRunGraphDoc serialization

void CCompareRunGraphDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CCompareRunGraphDoc commands
void CCompareRunGraphDoc::AddResponseTimer(const CString& strTimer)
{
	CreateGraph();
	m_pCompareRunGraph->AddResponseTimer(strTimer);
}

void CCompareRunGraphDoc::AddQuery(const CString& strQuery,const CString& strInjector)
{
	CreateGraph();
	m_pCompareRunGraph->AddQuery(strQuery,strInjector);
}

void CCompareRunGraphDoc::CreateGraph()
{
	if(!m_pCompareRunGraph)
		m_pCompareRunGraph = new CCompareRunGraph(m_pGraphView,"Test run comparison");
}

/////////////////////////////////////////////////////////////////////////////
// CGraphSummaryDlg dialog

CGraphSummaryDlg::CGraphSummaryDlg(CGraphTypeBase *pGraph, CWnd* pParent /*=NULL*/)
	: CDialog(CGraphSummaryDlg::IDD, pParent)
{
	m_pGraph = pGraph;
	//{{AFX_DATA_INIT(CGraphSummaryDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CGraphSummaryDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CGraphSummaryDlg)
	DDX_Control(pDX, IDOK, m_ctlOkBtn);
	DDX_Control(pDX, IDC_DATA_LIST, m_ctlDataList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CGraphSummaryDlg, CDialog)
	//{{AFX_MSG_MAP(CGraphSummaryDlg)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphSummaryDlg message handlers

BOOL CGraphSummaryDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	CString strUsers("Users/Test");
	int nWidth = 3*m_ctlDataList.GetStringWidth(strUsers)/2;
	m_ctlDataList.InsertColumn(0,strUsers,LVCFMT_LEFT,nWidth,0);

	CXChartCtrl &chart = m_pGraph->GetView()->m_ctlChart;
	
    COleSafeArray saData( chart.GetData() );
	
	long lStartRow,lStartCol;
	saData.GetLBound(1,&lStartRow);
	saData.GetLBound(2,&lStartCol);

	long lNumRows,lNumCols;
	saData.GetUBound(1,&lNumRows);
	saData.GetUBound(2,&lNumCols);

	long lIndex[2];
	for(long lRow = lStartRow;lRow<=lNumRows;lRow++)
	{
		for(long lCol=lStartCol;lCol<=lNumCols;lCol++)
		{
			lIndex[0] = lRow;
			lIndex[1] = lCol;
			
			VARIANT var;
			saData.GetElement(lIndex,&var);
			
			if(lRow==lStartRow)
			{
				int nColumn = lCol-lStartCol+1;
				CString strSeries = chart.GetSeriesAnnotation(nColumn);
				int nWidth = 3*m_ctlDataList.GetStringWidth(strSeries)/2;
				m_ctlDataList.InsertColumn(nColumn,strSeries,LVCFMT_LEFT,nWidth,nColumn);
			}

			if(var.vt == VT_R8)
			{
				CString str;
				str.Format("%f",var.dblVal);
				m_ctlDataList.SetItemText(lRow-lStartRow,lCol-lStartCol,str);
			}
			else
			{
				char *szUsers = _com_util::ConvertBSTRToString(var.bstrVal);
				CString strUsers(szUsers);
				free(szUsers);
				m_ctlDataList.InsertItem(lRow-lStartRow,strUsers);
			}
		}
	}
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CGraphSummaryDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	if(m_ctlDataList.m_hWnd)
	{
		CRect rcList;
		m_ctlDataList.GetWindowRect(&rcList);
		ScreenToClient(&rcList);

		CRect rcBtn;
		m_ctlOkBtn.GetWindowRect(&rcBtn);
		ScreenToClient(&rcBtn);

		m_ctlOkBtn.SetWindowPos(NULL,
								(cx/2)-rcBtn.Width()/2,
								cy-rcBtn.Height()-10,
								0,
								0,
								SWP_NOSIZE|SWP_NOZORDER);
		
		m_ctlDataList.SetWindowPos(NULL,
									0,
									0,
									cx-(rcList.left*2),
									cy-rcList.top-rcBtn.Height()-20,
									SWP_NOZORDER|SWP_NOMOVE);
	}
}
