/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECOTAEditorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"

#include "..\..\Interfaces\CyrPlugIns.h"
#include "Server.h"

#include "CECOTAEditorDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECOTAEditorDlg dialog


CECOTAEditorDlg::CECOTAEditorDlg(const CString &strTof, CWnd* pParent /*=NULL*/)
        : CDialog(CECOTAEditorDlg::IDD, pParent)

{
    //{{AFX_DATA_INIT(CECOTAEditorDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    CWnd* pMainWnd = AfxGetMainWnd();
    if(pMainWnd == NULL)
    {
        ASSERT(FALSE);
        return;
    }

    //Get the current project
    m_pServer = (CYRServer *)pMainWnd->SendMessage (WMU_GET_SERVERPTR);
    if(!m_pServer->IsThereAnOpenProject())
    {
        //There is no current project, so we shouldn't be here
        ASSERT(FALSE);
        return;
    }

    m_strTofName = _T("ota.tof");
    if(!strTof.IsEmpty())
        m_strTofName = strTof;

    //Check out tof file
    m_pServer->CheckOutFile(CYRServer::enObjectCodeFolder,"",m_strTofName);

    //Generate the full path to the configuration file
    m_strTOFFilePath = m_pServer->GetFolderPath(CYRServer::enObjectCodeFolder);
    m_strTOFFilePath += m_strTofName;
}


void CECOTAEditorDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECOTAEditorDlg)
    DDX_Control(pDX, IDC_XOTAEDITORCTRL, m_ctlXOTAEditor);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECOTAEditorDlg, CDialog)
//{{AFX_MSG_MAP(CECOTAEditorDlg)
ON_WM_SIZE()
ON_WM_DESTROY()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECOTAEditorDlg message handlers

void CECOTAEditorDlg::OnOK()
{
    m_ctlXOTAEditor.SaveTOF(COleVariant(m_strTOFFilePath));

    //Check out the configuration from source control
    m_pServer->CheckInFile(CYRServer::enObjectCodeFolder,"",m_strTofName);

    CDialog::OnOK();
}

BOOL CECOTAEditorDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    int nWidth = AfxGetApp()->GetProfileInt("OTA EDITOR DLG","SIZE X",-1);
    int nHeight = AfxGetApp()->GetProfileInt("OTA EDITOR DLG","SIZE Y",-1);

    if(nWidth!=-1 && nHeight!=-1)
    {
        SetWindowPos(NULL,0,0,nWidth,nHeight,SWP_NOZORDER|SWP_NOMOVE);
    }

    //If the TOF file exists load it
    CFileStatus fs;
    if(CFile::GetStatus(m_strTOFFilePath,fs))
    {
        COleVariant varPath(m_strTOFFilePath);
        m_ctlXOTAEditor.LoadTOF(varPath);
    }

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}



void CECOTAEditorDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    CButton *pOkBtn = (CButton *)GetDlgItem(IDOK);
    CButton *pCancelBtn = (CButton *)GetDlgItem(IDCANCEL);

    //If the dialog controls have been created
    //resize them to fit
    if(pOkBtn && pCancelBtn)
    {
        //Get the current sizes of the dialog controls
        CRect rcEditor;
        m_ctlXOTAEditor.GetWindowRect(&rcEditor);
        ScreenToClient(&rcEditor);

        CRect rcOk,rcCancel;
        pOkBtn->GetWindowRect(&rcOk);
        ScreenToClient(&rcOk);

        pCancelBtn->GetWindowRect(&rcCancel);
        ScreenToClient(&rcCancel);

        //Place editor OCX to fill gap above buttons
        const int nSeparator = 10;
        int nWidth = cx-2*nSeparator;
        int nHeight = cy - 3*nSeparator-rcOk.Height();
        m_ctlXOTAEditor.SetWindowPos(NULL,0,0,nWidth,nHeight,SWP_NOMOVE|SWP_NOZORDER);

        //move buttons into the center under the OCX
        int nXPos = cx / 2 - nSeparator - rcOk.Width();
        int nYPos = cy - rcOk.Height()-nSeparator;
        pOkBtn->SetWindowPos(NULL,nXPos,nYPos,0,0,SWP_NOSIZE|SWP_NOZORDER);
        pOkBtn->Invalidate();

        nXPos = cx / 2 + nSeparator;
        pCancelBtn->SetWindowPos(NULL,nXPos,nYPos,0,0,SWP_NOSIZE|SWP_NOZORDER);
        pCancelBtn->Invalidate();
    }
}

void CECOTAEditorDlg::OnDestroy()
{
    CDialog::OnDestroy();

    CRect rc;
    GetWindowRect(&rc);
    AfxGetApp()->WriteProfileInt("OTA EDITOR DLG","SIZE X",rc.Width());
    AfxGetApp()->WriteProfileInt("OTA EDITOR DLG","SIZE Y",rc.Height());
}
