/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRGeneric.cpp
// --------------
//
//    Defines the initialization routines for the DLL.
//
//=========================================================================

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static AFX_EXTENSION_MODULE CYRGenericDLL = { NULL, NULL };
static CString GetHelpFilename();
CString GetProductName();


#include <afxdllx.h>
#include "resource.h"
#include "CECProjectViewerDlg.h"

#include "CYREventSink.h"
#include "..\..\Interfaces\CyrPlugIns.h"
#include "..\..\Interfaces\CyrCtrls.h"
#include "..\..\Common\CECDLLFunctions.h"

#include "..\..\Common\CECRegistryTools.h"
#include "Solutions.h"
#include "CYRGroupTestWizard.h"
#include "..\..\Common\TestWizardDefs.h"

CYREventSink*           pEventSink              = NULL;
CECDLLFunctions*        pDllFunctions           = NULL;
CECProjectViewerDlg*    pProjectViewerDlg       = NULL;
CMultiDocTemplate*      pScriptDebugDocTemplate = NULL;
CMultiDocTemplate*      pStatusScreenDocTemplate = NULL;
CString                 m_szManagerName;


extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("CYRGeneric.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CYRGenericDLL, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.
        pDllFunctions = new CECDLLFunctions;
        CString strProduct = GetProductName();
        ASSERT(!strProduct.IsEmpty());
        pDllFunctions->Initialize(strProduct ,"CYRGeneric");

    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CYRGeneric.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CYRGenericDLL);

        delete pDllFunctions;
        pDllFunctions=NULL;
    }

    return 1;   // ok
}


extern "C" AFX_EXT_API
    BOOL GetProductInfo(PRODUCTINFO* /*pProductInfo*/)
{
    new CDynLinkLibrary(CYRGenericDLL);

    return FALSE;
}

extern "C" AFX_EXT_API
    BOOL GetResourceInfo(RESOURCEINFO *pResourceInfo[])
{
    int nID = -1;

    // Test Control Command ID's
    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Execute Test...");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Run");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_PROJECT|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = IDB_EXECUTE_TEST;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Run");

    // Import/Export Functions
    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Import...");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&File");
    pResourceInfo[nID]->dwStyles =  CYR_NEEDS_PROJECT|CYR_NEEDS_SEPARATOR;
    pResourceInfo[nID]->nCommandID = IDB_IMPORT_FILE;
    pResourceInfo[nID]->strPreferredToolbar[0] = '\0';

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&Project Path...");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Options");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_PROJECT|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = IDB_SET_WORKFOLDER;
    pResourceInfo[nID]->strPreferredToolbar[0] = '\0';

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "&New Injector");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Injectors");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_PROJECT|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = IDB_NODE_DEFINITIONS;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "");

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "New &Group Test");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Test");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_PROJECT|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_NEW_GROUP_TEST;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Test");

    // Add the resources supplied by the control dlls

    int CtrlResourceIdx = 0;//-1;
    CTRLRESOURCEINFO*       pCtrlResInfo[MAX_DOCUMENTS] = {0};
    LPCTRLRESFUNC GetCtrlResInfo = (LPCTRLRESFUNC)pDllFunctions->GetFunctionPtr("GetCtrlResInfo", "Generic Ctrls", FALSE);
    GetCtrlResInfo(pCtrlResInfo);

    while (pCtrlResInfo[CtrlResourceIdx] != NULL)
    {
        pResourceInfo[++nID] = new RESOURCEINFO;

        (void) strcpy (pResourceInfo[nID]->strLabel, pCtrlResInfo[CtrlResourceIdx]->strLabel);
        (void) strcpy (pResourceInfo[nID]->strPreferredMenu, pCtrlResInfo[CtrlResourceIdx]->strPreferredMenu);
        pResourceInfo[nID]->dwStyles = pCtrlResInfo[CtrlResourceIdx]->dwStyles;
        pResourceInfo[nID]->nCommandID = pCtrlResInfo[CtrlResourceIdx]->nCommandID;

        (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, pCtrlResInfo[CtrlResourceIdx]->strPreferredToolbar);
        delete pCtrlResInfo[CtrlResourceIdx];
        pCtrlResInfo[CtrlResourceIdx] = NULL;
        CtrlResourceIdx++;

        //This has been added so that the loop does not
        //drop out on the first loop in release,(suspect
        //a compiler optimisation is to blame)
        Sleep(0);
    }
    return TRUE;

}

extern "C" AFX_EXT_API
    BOOL GetWindowInfo(WINDOWINFO *pWindowInfo[])
{
    int nID = -1;

    pWindowInfo[++nID] = new WINDOWINFO;

    (void) strcpy (pWindowInfo[nID]->strName,   "Project View");
    pWindowInfo[nID]->nType     = PROJECT_WINDOW;
    pWindowInfo[nID]->dwStyles  = 0;
    pWindowInfo[nID]->hWnd      = 0;

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetOutputInfo(OUTPUTINFO *pOutputInfo[])
{
    int nID = -1;

    pOutputInfo[++nID] = new OUTPUTINFO;

    (void) strcpy (pOutputInfo[nID]->strName,               "Compile");
    pOutputInfo[nID]->hWnd = 0;

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetDocumentInfo(DOCUMENTINFO *pDocumentInfo[])
{
    int ResourceIdx = -1;

    // Add the Document templates supplied by the control dlls

    int CtrlResourceIdx = -1;

    CTRLDOCUMENTINFO*   pCtrlDocumentInfo[MAX_DOCUMENTS] = {0};

    LPCTRLDOCFUNC GetCtrlDocumentInfo = (LPCTRLDOCFUNC)pDllFunctions->GetFunctionPtr("GetCtrlDocumentInfo", "Generic Ctrls", FALSE);
    GetCtrlDocumentInfo(pCtrlDocumentInfo);

    while (pCtrlDocumentInfo[++CtrlResourceIdx] != NULL)
    {
        pDocumentInfo[++ResourceIdx] = new DOCUMENTINFO;

        // Simply register the Template by adding it to the App
        pStatusScreenDocTemplate = pCtrlDocumentInfo[CtrlResourceIdx]->pDocTemplate;

        (void) strcpy (pDocumentInfo[ResourceIdx]->strName, pCtrlDocumentInfo[CtrlResourceIdx]->strName);
        (void) strcpy (pDocumentInfo[ResourceIdx]->strFileExt,  pCtrlDocumentInfo[CtrlResourceIdx]->strFileExt);

        pDocumentInfo[ResourceIdx]->pDocTemplate = pStatusScreenDocTemplate;
        pDocumentInfo[ResourceIdx]->dwStyles = pCtrlDocumentInfo[CtrlResourceIdx]->dwStyles;

        delete pCtrlDocumentInfo[CtrlResourceIdx];
        pCtrlDocumentInfo[CtrlResourceIdx] = NULL;
        CtrlResourceIdx++;
    }

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetWizardInfo(WIZARDINFO *pWizardInfo[])
{
    int nID = -1;

    pWizardInfo[++nID] = new WIZARDINFO;

    (void) strcpy (pWizardInfo[nID]->strName,               GROUPTEST);

    pWizardInfo[nID]->dwStyles = 0;

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL ExecuteWizard(const CString& strWizName,const CString& strTestName)
{
    CWnd *pMainWnd = AfxGetMainWnd();

    if(strWizName == GROUPTEST)
    {
        CYRGroupTestWizardSheet dlg("Group Test Editor",pMainWnd,0,strTestName);
        if(dlg.DoModal()==IDOK)
        {
            CString szDll(SOLUTIONS_GENERIC_DLL);
            pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_REFRESH_TREE_VIEWS,(LPARAM)&szDll);
            return true;
        }
    }

    return false;
}

extern "C" AFX_EXT_API
    BOOL OnUpdateCommandUI(UINT *pID)
{
    LPCTRLONUPDATE onUpdate = (LPCTRLONUPDATE)pDllFunctions->GetFunctionPtr("OnUpdateCommandUI", "Generic Ctrls", FALSE);
    onUpdate(pID);

    BOOL bState = FALSE;

    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }

        bState = pEventSink->IsEnabled(*pID);
    }

    return bState;
}

extern "C" AFX_EXT_API
    LRESULT OnEvent(UINT *pID, UINT* pMsg)
{
    LPCTRLONEVENT onEvent = (LPCTRLONEVENT)pDllFunctions->GetFunctionPtr("OnEvent", "Generic Ctrls", FALSE);
    onEvent(pID, pMsg);

    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }
        return ::SendMessage(pEventSink->GetSafeHwnd(), *pID, *pMsg, 0);
    }
    return 0;
}

extern "C" AFX_EXT_API
    BOOL OnDestroy()
{
    LPONDESTROYFUNC lpfnOnDestroy = (LPONDESTROYFUNC)pDllFunctions->GetFunctionPtr("OnDestroy", "Generic Ctrls", FALSE);
    if(lpfnOnDestroy)
    {
        lpfnOnDestroy();
    }

    if (pEventSink != NULL)
    {
        delete pEventSink;
        pEventSink = NULL;
    }

    return TRUE;
}

extern "C" AFX_EXT_API
    CWnd* CreateDockWnd(CWnd* pParent)
{
    pProjectViewerDlg = new CECProjectViewerDlg();
    pProjectViewerDlg->Create(IDD_PROJ_VIEWER, pParent);
    pProjectViewerDlg->ShowWindow(SW_SHOW);

    return (CWnd*)pProjectViewerDlg;
}

static CString GetHelpFilename()
{
    CString strHelpPath = AfxGetApp()->m_pszHelpFilePath;
    return strHelpPath;
    //  int nPos = strHelpPath.ReverseFind('\\');
    //  if(nPos != -1)
    //  {
    //      strHelpPath = strHelpPath.Left(nPos);
    //      return strHelpPath + "\\Commander.chm";
    //  }
    //  return "";
}

//
//  GetProductName()
//  ================
//
//  Obtain the current product from the command line or registry
//

CString GetProductName()
{
    // First we get the product name from the command line
    int i = 1;
    while (__argc > i)
    {
        CString szField = __argv[i++];

        if (szField.CompareNoCase("-i") == 0)
        {
            if (__argc > i)
                return __argv[i];
        }
    }

    //No product specified, so use first in registry
    CString KeyName = SOLUTIONS_FRAMEWORK_KEY;

    CECRegistryTools reg;
    if(reg.Open(HKEY_LOCAL_MACHINE, KeyName + "\\Products",KEY_READ) != ERROR_SUCCESS)
        return "";

    TCHAR szBuffer[256];
    DWORD dwSize = 256;
    DWORD Index = 0;
    HKEY Key = reg.GetHKey();

    if(RegEnumKeyEx(Key, Index, szBuffer, &dwSize, NULL, NULL, NULL, NULL) == ERROR_SUCCESS)
        return szBuffer;

    return "";
}


