/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRDelayPropertiesDlg.cpp
// --------------------------
//
//    This is the implementation file for the CYRDelayPropertiesDlg class.
//
//=========================================================================

#include "stdafx.h"
#include "resource.h"

#include "CYRDelayPropertiesDlg.h"
#include "..\..\Common\CECClipboard.h"
#include "htmlhelp.h"

/*
#include "..\..\..\HTMLHelp\CYRImpact\Delay_Properties_dialog_box.inl"
*/
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRDelayPropertiesDlg dialog


CYRDelayPropertiesDlg::CYRDelayPropertiesDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CYRDelayPropertiesDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CYRDelayPropertiesDlg)
    m_nDelay = 10;
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_pSpinDelay = NULL;
    m_pDelayProperties = NULL;

    m_szOriginalCBText = _T("");
}


void CYRDelayPropertiesDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRDelayPropertiesDlg)
    DDX_Text(pDX, IDC_DELAY_EDIT, m_nDelay);
    DDV_MinMaxUInt(pDX, m_nDelay, 1, 9999);
    DDX_Control(pDX, IDC_DELAY_SPIN, m_SpinDelay);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRDelayPropertiesDlg, CDialog)
//{{AFX_MSG_MAP(CYRDelayPropertiesDlg)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRDelayPropertiesDlg message handlers

BOOL CYRDelayPropertiesDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    //Setup html help handler

    // TODO: Add extra initialization here
    // Link in the Spin Control with the window control
    m_pSpinDelay = (CSpinButtonCtrl*) GetDlgItem(IDC_DELAY_SPIN);
    ASSERT (m_pSpinDelay != NULL);

    // Set up the spin control range
    m_pSpinDelay->SetRange(1,9999);

    // Now tell it to control a certain edit box
    m_pSpinDelay->SetBuddy(GetDlgItem(IDC_DELAY_EDIT));

    // And finally set up their initial state
    m_pSpinDelay->SetPos(m_nDelay);

    // Collect the thread we are to edit from the clipboard
    CECClipboard cb;
    m_szOriginalCBText = cb.GetClipboardText();

    // And create a new CECThread Object to hold the data in
    if (m_pDelayProperties == NULL)
    {
        m_pDelayProperties = new CECDelay();

    }
    // Import the clipboard text into the CECTest format
    m_pDelayProperties->ImportCFText(m_szOriginalCBText);

    // And extract the fields from the object
    m_nDelay = m_pDelayProperties->GetDelayPeriod();

    // Send them to the dialog
    UpdateData(FALSE);

    // And update the dialog controls incase this is the second time in here
    UpdateDialogControls(GetParent(),FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CYRDelayPropertiesDlg::OnOK()
{
    if(!UpdateData(TRUE))
        return;

    // Set up the Thread Properties Object
    m_pDelayProperties->SetDelayPeriod(m_nDelay);

    // Put the Thread Name on the Clipboard
    CECClipboard cb;
    cb.SetClipboardText(m_pDelayProperties->ExportCFText());

    if (m_pDelayProperties != NULL)
    {
        delete m_pDelayProperties;
        m_pDelayProperties = NULL;
    }

    CDialog::OnOK();
}

void CYRDelayPropertiesDlg::OnCancel()
{
    CECClipboard cb;
    cb.SetClipboardText(m_szOriginalCBText);

    if (m_pDelayProperties != NULL)
    {
        delete m_pDelayProperties;
        m_pDelayProperties = NULL;
    }

    CDialog::OnCancel();
}

BOOL CYRDelayPropertiesDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    /*
        CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";
     
        if (pHelpInfo->iCtrlId == IDC_STATIC)
            return true;
     
        CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);
     
		if(pWnd)
	        HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);
    */
    return true;
}
