/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRImpact.cpp
// -------------
//
//    This is the Plug In Controller file for the CYRImpact DLL.
//
//=========================================================================


#include "stdafx.h"
#include <afxdllx.h>
#include "resource.h"
#include "CYREventSink.h"
#include "..\..\Interfaces\CyrPlugIns.h"
#include "..\..\Common\CECDLLFunctions.h"

#include "CECProjectCreator.h"
#include "Solutions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CYREventSink* pEventSink = NULL;
CECDLLFunctions*    pDllFunctions = NULL;

static AFX_EXTENSION_MODULE CYRImpactDLL = { NULL, NULL };
static CString GetHelpFilename();


#include "CYRSimpleTestWizard.h"

extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("CYRImpact.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CYRImpactDLL, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.

        CECGridCtrl::GridCtrlInit(hInstance);

    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CYRImpact.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CYRImpactDLL);

    }

    return 1;   // ok
}


extern "C" AFX_EXT_API
    BOOL GetProductInfo(PRODUCTINFO *pProductInfo)
{
    new CDynLinkLibrary(CYRImpactDLL);

    (void) strcpy (pProductInfo->strDisplayName,        "OpenSTA");
    (void) strcpy (pProductInfo->strTitle,      "OpenSTA PlugIn Version");
    (void) strcpy (pProductInfo->strInternalName,       "OpenSTA");
    (void) strcpy (pProductInfo->strProjectPrefix,      "OpenSTA");
    pProductInfo->nIcon = IDI_PRODUCT_ICON;
    (void) strcpy (pProductInfo->strVersion,    "0.9");
    (void) strcpy (pProductInfo->strAboutText,  "OpenSTA");

    pProductInfo->dwStyles = CYR_HAS_PROJECT | CYR_HAS_SERVER;// | CYR_HAS_CONTEXT;
    pProductInfo->nBackground = 0;//IDB_IMPACT_BACKGROUND;

#ifdef BETA
    pProductInfo->nSplashScreen = IDB_BETA_SPLASH;
#else
    pProductInfo->nSplashScreen = IDB_RELEASE_SPLASH;
#endif

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetResourceInfo(RESOURCEINFO *pResourceInfo[])
{
    int nID = -1;

    //  pResourceInfo[++nID] = new RESOURCEINFO;

    //  (void) strcpy (pResourceInfo[nID]->strLabel,                "Test &Recorder");
    //  (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "Too&ls");
    //  pResourceInfo[nID]->dwStyles = CYR_NEEDS_SEPARATOR;
    //  pResourceInfo[nID]->nCommandID = IDB_RECORDER;
    //  (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Tools");

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "New &Test");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Test");
    pResourceInfo[nID]->dwStyles = CYR_NEEDS_PROJECT|CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = ID_NEW_SIMPLE_TEST;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Test");

    pResourceInfo[++nID] = new RESOURCEINFO;

    (void) strcpy (pResourceInfo[nID]->strLabel,                "Script &Modeler");
    (void) strcpy (pResourceInfo[nID]->strPreferredMenu,        "&Tools");
    pResourceInfo[nID]->dwStyles = CYR_BASIC_CMD;
    pResourceInfo[nID]->nCommandID = IDB_DESIGNER;
    (void) strcpy (pResourceInfo[nID]->strPreferredToolbar, "Test");

    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetWindowInfo(WINDOWINFO *pWindowInfo[])
{
    return FALSE;
}

extern "C" AFX_EXT_API
    BOOL GetOutputInfo(OUTPUTINFO *pOutputInfo[])
{
    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL GetDocumentInfo(DOCUMENTINFO *pDocumentInfo[])
{
    return FALSE;
}

extern "C" AFX_EXT_API
    BOOL GetWizardInfo(WIZARDINFO *pWizardInfo[])
{
    int nID = -1;

    pWizardInfo[++nID] = new WIZARDINFO;

    (void) strcpy (pWizardInfo[nID]->strName,               "Test");

    pWizardInfo[nID]->dwStyles = 0;
    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL ExecuteWizard(const CString& strWizName,const CString& strTestName)
{
    CWnd *pMainWnd = AfxGetMainWnd();

    if(strWizName == "Test")
    {
        CYRSimpleTestWizardSheet dlg("Test Editor",pMainWnd,0,strTestName);
        if(dlg.DoModal()==IDOK)
        {
            CString szDll(SOLUTIONS_GENERIC_DLL);
            pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_REFRESH_TREE_VIEWS,(LPARAM)&szDll);
            return true;
        }
    }

    return false;
}

extern "C" AFX_EXT_API
    BOOL OnUpdateCommandUI(UINT *pID)
{
    // First we set our enable flag to false
    BOOL bState = FALSE;

    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }

        // And post this message out to the Event Sink to process
        bState = pEventSink->IsEnabled(*pID);
    }

    // And return the Enabled Flag which will be set to represent the state
    // of the control we have just quizzed
    return bState;
}

extern "C" AFX_EXT_API
    LRESULT OnEvent(UINT *pID, LONG* pMsg)
{
    if (pID)
    {
        if (pEventSink->GetSafeHwnd() == NULL)
        {
            pEventSink = new CYREventSink();
        }

        if(pMsg != NULL)
        {
            return ::SendMessage(pEventSink->GetSafeHwnd(), *pID, *pMsg, 0);
        }
        else
        {
            return ::SendMessage(pEventSink->GetSafeHwnd(), *pID, 0, 0);
        }
    }
    return 0;
}

extern "C" AFX_EXT_API
    BOOL OnDestroy()
{
    if (pEventSink != NULL)
    {
        delete pEventSink;
        pEventSink = NULL;
    }
    return TRUE;
}

extern "C" AFX_EXT_API
    BOOL CreateProject(CString strName, CString strDesc)
{
    CECProjectCreator creator;
    return  creator.Create(strName, strDesc);
}

static CString GetHelpFilename()
{
    CString strHelpPath = AfxGetApp()->m_pszHelpFilePath;
    int nPos = strHelpPath.ReverseFind('\\');
    if(nPos != -1)
    {
        strHelpPath = strHelpPath.Left(nPos);
        return strHelpPath + "\\Commander.chm";
    }
    return "";
}
