/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRTestPropertiesDlg.cpp
// ------------------------
//
//    This is the implementation file for the CYRTestPropertiesDlg class.
//
//=========================================================================

#include "stdafx.h"
#include "resource.h"

#include "CYRTestPropertiesDlg.h"
#include "..\..\Common\CECClipboard.h"
#include "..\..\Interfaces\CyrPlugIns.h"
#include <rw\cToken.h>
#include "CyrStapi\CECComImpactFolders.h"
#include "htmlhelp.h"

/*
#include "..\..\..\HTMLHelp\CYRImpact\Test_Properties_dialog_box.inl"
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRTestPropertiesDlg dialog


CYRTestPropertiesDlg::CYRTestPropertiesDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CYRTestPropertiesDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CYRTestPropertiesDlg)
    m_szTestName = _T("");
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_pTestProperties = NULL;

    m_szOriginalCBText = _T("");
}


void CYRTestPropertiesDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRTestPropertiesDlg)
    DDX_Control(pDX, IDC_NODE_COMBO, m_LogicalNodeList);
    DDX_Text(pDX, IDC_TESTNAME_EDIT, m_szTestName);
    DDV_MaxChars(pDX, m_szTestName, 16);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRTestPropertiesDlg, CDialog)
//{{AFX_MSG_MAP(CYRTestPropertiesDlg)
ON_BN_CLICKED(IDC_CREATE_NODE, OnCreateNode)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRTestPropertiesDlg message handlers

void CYRTestPropertiesDlg::OnCreateNode()
{
    CECClipboard cb;
    UpdateData(TRUE);

    int nSel = m_LogicalNodeList.GetCurSel();
    if (nSel != CB_ERR)
    {
        CString szText;
        m_LogicalNodeList.GetLBText(nSel, szText);
        cb.SetClipboardText(szText);
    }

    // Invoke the Node Creation Dialog (from Generic)
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT ,(LPARAM)STUDIO_CUSTOM_LINK_3);

    // Set the Node name on the Clipboard
    CString szText = cb.GetClipboardText();

    PrimeComboBox();

    int nCurSel = m_LogicalNodeList.FindString(-1, szText);
    if (nCurSel == CB_ERR) nCurSel = 0;

    m_LogicalNodeList.SetCurSel(nCurSel);
}

void CYRTestPropertiesDlg::OnOK()
{
    UpdateData(TRUE);

    CString szText("");

    int nSel = m_LogicalNodeList.GetCurSel();
    if (nSel != CB_ERR)
    {
        m_LogicalNodeList.GetLBText(nSel, szText);
    }

    // Get the VTEST data name
    m_DataNameEdit.GetData(m_szTestName, TRUE);

    if (m_szTestName.IsEmpty())
    {
        AfxMessageBox(IDS_TEST_PROPS_NAME);
        return;
    }

    // Set up the Test Properties Object
    m_pTestProperties->SetName(m_szTestName);
    m_pTestProperties->SetLogicalNode(szText);

    // And export the object as a CF_TEXT description
    CECClipboard cb;
    cb.SetClipboardText(m_pTestProperties->ExportCFText());

    if (m_pTestProperties != NULL)
    {
        delete m_pTestProperties;
        m_pTestProperties = NULL;
    }

    CDialog::OnOK();
}

BOOL CYRTestPropertiesDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Collect the test we are to edit from the clipboard
    CECClipboard cb;
    m_szOriginalCBText = cb.GetClipboardText();

    // And create a new CECTest Object to hold the data in
    if (m_pTestProperties == NULL)
    {
        m_pTestProperties = new CECTest();
    }

    // Import the clipboard text into the CECTest format
    m_pTestProperties->ImportCFText(m_szOriginalCBText);

    // And extract the fields from the object
    m_szTestName = m_pTestProperties->GetName();

    // Add the V-Test Dataname - yuch
    m_DataNameEdit.SubclassEdit (IDC_TESTNAME_EDIT, this, 0);
    m_DataNameEdit.SetLength(16);
    m_DataNameEdit.SetData(m_szTestName);

    // Add in all available Nodes
    PrimeComboBox();

    int nCurSel = m_LogicalNodeList.FindString(-1, m_pTestProperties->GetLogicalNode());
    if (nCurSel == CB_ERR) nCurSel = 0;

    m_LogicalNodeList.SetCurSel(nCurSel);

    // Send them to the dialog
    UpdateData(FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//
// PrimeComboBox
// =============
//
// Populate the node combo box
//

void CYRTestPropertiesDlg::PrimeComboBox()
{
    //
    // Firstly delete all old entries ...
    //
    m_LogicalNodeList.ResetContent();

    //
    // And Always add in a Local Node
    //
    m_LogicalNodeList.AddString("<Local>");
    //
    //  Get the current project, then the root folder, and then the node folder
    //
    CWnd *pMainWnd = AfxGetMainWnd();
    CECComProject *pProject = (CECComProject*)pMainWnd->SendMessage(WMU_GET_PROJECT);

    CECComFolder RootFolder(pProject);
    CECComNodeFolder NodeFolder(&RootFolder);

    //
    //  Get a list of the nodes in the folder
    //
    long NodeCount;
    RWCString NodeList = NodeFolder.GetFiles(NodeCount);

    RWCTokenizer token( NodeList );
    RWCString node = token( "|" );
    while (!node.isNull())
    {
        CString szNodeName = (CString)node.data();

        //Add the nodes to the combo box
        m_LogicalNodeList.AddString(szNodeName);

        node = token( "|" );
    }
}

//
// GetWorkDir
// ==========
//
// Get hold of the path to where all the files are to be written
//
CString CYRTestPropertiesDlg::GetWorkDir()
{
    // Get a handle to the mainframe
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    //Ask for the working directory
    CString szDll("CYRGeneric");
    CString szWorkDir = *(CString*)pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_GET_PROJECT_WORKDIR,(LPARAM)&szDll);

    return szWorkDir;
}

//
// StripOffTrailingSlash
// =====================
//
// Ensure we do not have a trailing slash at the end of our path
//
void CYRTestPropertiesDlg::StripOffTrailingSlash(CString& szPath)
{
    CString szLastChar = szPath.Right(1);

    if (szLastChar.CompareNoCase("\\") == 0)
    {
        // OK so strip it off
        szPath = szPath.Left(szPath.GetLength() - 1);
    }
}

void CYRTestPropertiesDlg::OnCancel()
{
    // And place the original text back on the clipboard
    CECClipboard cb;
    cb.SetClipboardText(m_szOriginalCBText);

    if (m_pTestProperties != NULL)
    {
        delete m_pTestProperties;
        m_pTestProperties = NULL;
    }

    CDialog::OnCancel();
}

BOOL CYRTestPropertiesDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    /*
        CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";
     
        if (pHelpInfo->iCtrlId == IDC_STATIC)
            return true;
     
        CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);
     
		if(pWnd)
	        HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);
    */
    return true;
}
