/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CECResultsFile
// @module  CECResultsFile | File Access class
///////////////////////////////////////////////////////////////////////////////

#ifndef CYRANO_TP_STANDALONE
#include "CORBARepository.h"
#include "CECResultsFile.h"
#include "CECInetInfo.h"
#include "CyrHelper.inl"

///////////////////////////////////////////////////////////////////////////////
//@mfunc    constructor
//@parm	    CECOrbFolder TestRun
//@parm	    string filename
//@parm	    string full filename including path
CECResultsFile::CECResultsFile(CECOrbFolder* m_TestRun, const char* FileName,
                               const char* FullFileName)
{
    // We need to get the name of the current file with and without path
    // The full path version is used to identify the file to transfer
    CECInetInfo info;

    m_FileName = FileName;
    m_FullFileName = FullFileName;

    // the CECOrbFile constructor will throw a CyrRepositoryException
    // which will be caught in the CORBARepository (calling object)
    m_OrbFile = new CECOrbFile(m_TestRun, m_FileName.c_str(),"Results File");
}

///////////////////////////////////////////////////////////////////////////////
//@mfunc Destructor
CECResultsFile::~CECResultsFile()
{
    delete m_OrbFile;
}


void CECResultsFile::Store(bool del)
{

    // when we check in we need to pass in the full path to the local file
    try
    {
        m_OrbFile->Lock();
        m_OrbFile->CheckIn(m_FullFileName.c_str(),true);

        // now remove the local file
        if (del)
            CyrDeleteFile(m_FullFileName.c_str());
    }
    catch(...)
    {
    }

}

#ifndef TH_POOL_VER

void CECResultsFile::Concatenate()
{
    FILE * fdConcat = NULL ;

#ifdef WIN32
    if ((fdConcat = fopen(m_FullFileName.c_str(), "wb+")) == NULL)
#else
    if ((fdConcat = fopen(m_FullFileName.c_str(), "w+")) == NULL)
#endif
    {
        return;
    }

    FILE *fdStat = NULL ;
    char buffer[1024] ;
    int iThreadId = 0;
    while (1)
    {
        char szBuf[256];

        sprintf(szBuf, "%s-%d.stat", m_FullFileName.substr(0, m_FullFileName.find_last_of(".")).c_str(), iThreadId);

#ifdef WIN32
        if ((fdStat = fopen(szBuf, "rb")) == NULL)
#else
        if ((fdStat = fopen(szBuf, "r")) == NULL)
#endif
            break ;

        while (!feof(fdStat))
        {
            int iRead = 0 ;
            if ((iRead = fread(buffer, 1, sizeof(buffer), fdStat)) > 0)
                fwrite(buffer, iRead, 1, fdConcat) ;
        }
        fclose(fdStat) ;
        remove(szBuf);
        iThreadId ++ ;
    }
    fclose(fdConcat) ;


    // --------------------------------------
    // Do the same for URLs files
    // --------------------------------------
    fdConcat = NULL ;
    string urlFileName = m_FullFileName.substr(0, m_FullFileName.find_last_of(".")) + ".urls";

#ifdef WIN32
    if ((fdConcat = fopen(urlFileName.c_str(), "wb+")) == NULL)
#else
    if ((fdConcat = fopen(urlFileName.c_str(), "w+")) == NULL)
#endif
    {
        return;
    }

    FILE *fdUrls = NULL ;
    iThreadId = 0;
    while (1)
    {
        char szBuf[256];

        sprintf(szBuf, "%s-%d.urls", m_FullFileName.substr(0, m_FullFileName.find_last_of(".")).c_str(), iThreadId);

#ifdef WIN32
        if ((fdUrls = fopen(szBuf, "rb")) == NULL)
#else
        if ((fdUrls = fopen(szBuf, "r")) == NULL)
#endif
            break ;

        while (!feof(fdUrls))
        {
            int iRead = 0 ;
            if ((iRead = fread(buffer, 1, sizeof(buffer), fdUrls)) > 0)
                fwrite(buffer, iRead, 1, fdConcat) ;
        }
        fclose(fdUrls) ;
        remove(szBuf);
        iThreadId ++ ;
    }
    fclose(fdConcat) ;
}


#endif // TH_POOL_VER

#endif
