/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ScriptCache.h
#ifndef _CYR_SCRIPTCACHE_H
#define _CYR_SCRIPTCACHE_H
/**
 * Class Definititon for script caching class.
 * Produces tasks from scripts that may be loaded or already cached
 * Must be thread safe.
 **/
#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include <string>
#include <map>
#ifdef HPUX
#include <strstream.h>
#else
#include <strstream>
#endif
#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif
#if defined (WIN32)
//++
class ScriptCache;
ostream& operator<<(ostream&, const ScriptCache&);
ostrstream& operator<<(ostrstream&, const ScriptCache&);
//--
#endif

// local includes
#ifndef TH_POOL_VER
#   include "ThreadTask.h"
#else
#   include "TaskScript.h"
#endif

#include "TScriptRule.h"

// map to hold the scripts that have already been read in
typedef map<string, TaskScript*, less<string> > script_map_t;

// class definition

// Note: need synchronisation from calling methods.
class ScriptCache
{
    script_map_t _scripts;
    TScriptRule& _rule;
    int _numScripts; // start cleaning up when have this num scripts in cache
    int _cacheSize;  // start cleaning up when cache is bigger than this
    int _sizeWghtng; // Weighting of which scripts get purged first on size

protected:
    virtual bool spaceNeeded() const;

public:
    //Constructors & Destructor
    ScriptCache(TScriptRule* rule);
    virtual ~ScriptCache();

    //Accessors

    //Mutators
    void cleanup(bool lazy = true);
#ifndef TH_POOL_VER
    virtual ThreadTask* makeTask(const string& scriptname);
#else
    virtual TaskScript* makeTask(const string& scriptname);
#endif
    virtual bool doesTaskExist(const string& scriptname)const;
    virtual void setParams(int numScripts, int cacheSize, int sizeWghtng);

    //Friends
    friend ostream& operator<<(ostream& os, const ScriptCache& cache);
    friend ostrstream& operator<<(ostrstream& os, const ScriptCache& cache);
}
; // ScriptCache

ostream& operator<<(ostream& os, const ScriptCache& cache);
ostrstream& operator<<(ostrstream& os, const ScriptCache& cache) ;

#endif // _CYR_SCRIPTCACHE_H
