/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// mdbgetopt.h


#ifndef MDBGETOPT_H
#define MDBGETOPT_H


#include <string>
#include <map>
#include <vector>
#include "mdbtypes2.h"

#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif

#if defined(WIN32)
#pragma warning(disable:4786)
#include <iostream>
typedef map<string, string, less<string>, allocator<string> > CLARGLIST;
typedef map<string, int, less<string>, allocator<int> > VOLIST;
typedef vector<string, allocator<string> > ARGVEC;
#else
#include <iostream.h>
typedef map<string, string, less<string> > CLARGLIST;
typedef map<string, int, less<string> > VOLIST;
typedef vector<string > ARGVEC;
#endif



// Command line option value types
#define CLO_NOVAL       1       // No value allowed with option
#define CLO_INTVAL      2       // Integer value required with option
#define CLO_UNSVAL      3       // Unsigned integer value required with option
#define CLO_STRINGVAL   4       // String value required with option

class Cmdbgetopt
{

public:

    // Constructors/Destructor
    Cmdbgetopt(int argc, char * argv[], string facil);
    Cmdbgetopt(string cmdline, string facil);
    virtual	~Cmdbgetopt();

    void    init(int argc, char * argv[], string facil);
    bool    fail(void)
    {
        return error;
    }
    string  error_text(void)
    {
        return errtxt;
    }
    void    define_option(string option, int valtype);
    void    validate(void);
    bool    option_supplied(string opt);
    string  get_value_for(string opt);
    string  get_value_for(string opt, string dflt);
    int     get_int_value_for(string opt);
    int     get_int_value_for(string opt, int dflt);
    void    print(void);

    // Public member functions
public:

protected:
    // Variable list (map)
    CLARGLIST   args;
    // Valid options list (map)
    VOLIST      valid_opts;
    // Error flag
    bool        error;
    // Error description
    string      errtxt;
    // Error message prefix
    string      pfx;

};


#endif
