/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Ctimlist implementation

#include "timlist.h"
#include "cxxutils.h"

Ctimlist::Ctimlist()
{}

// Destruction

Ctimlist::~Ctimlist()
{}

// Operator to write timer declarations text to
// an output stream

ostream& operator<<(ostream& os, Ctimlist& tl)
{
    for(TIMLIST::iterator it = tl.tims.begin() ; it != tl.tims.end() ; it++)
        os << setiosflags(ios::left) << setw(ISQ_NAME_INDENT) << VDECTIM
        << setw(0) << (*it) << "\n";
    return os;
}

// Define a new timer.
// We rely on the behaviour of STL "set" to prevent
// duplicate timer name insertion.
int Ctimlist::define_timer(string name, unsigned)
{
    // Type returned by set insert
    typedef pair<TIMLIST::iterator, bool> ir;

    string_toupper(name);
    // Add timer to list
    ir s = tims.insert(name);
    // If pair member "second" is true insert was ok
    if(s.second)
        return 0;
    return -1;
}

// Get count of defined timers
int Ctimlist::get_timer_count(void)
{
    // Return count of entries list
    return (int)tims.size();
}

// Return non zero if timer "name" is defined
int Ctimlist::is_timer_defined(string name)
{
    string_toupper(name);
    TIMLIST::iterator it = tims.find(name);
    if(it == tims.end())
        return 0;
    return 1;
}

// Delete timer with "name"
int Ctimlist::delete_timer(string name)
{
    string_toupper(name);
    TIMLIST::iterator it = tims.find(name);
    if(it == tims.end())
        return -1;
    tims.erase(it);
    return 0;
}

// Delete all timers in list
int Ctimlist::delete_all_timers(void)
{
    tims.erase(tims.begin(), tims.end());
    return 0;
}
