/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TExecuterRcrd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CYR_TEXECUTERRECORDS
#define CYR_TEXECUTERRECORDS

#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include "TRecord.h"

#ifdef TH_POOL_VER
#   include "FastLock.hpp"
#endif

/**
 * Definition of Call Script Record class
 **/
class TRcrdCscr : public TRecord
{

public:
    static const AtvfKey KEY;

    // Constructors & Destructor
TRcrdCscr(const TParamRule* headRuleP) : TRecord(headRuleP),_labelIndex(-1),_prmListP(NULL)
    {}

    virtual ~TRcrdCscr()
    {
        if(_prmListP)delete _prmListP;
    };

    // Control methods
    virtual int execute(TContext& ctxt);
    virtual int prepare(TScript* script, int objIndex, bool verbose=false);

    virtual TRecord* duplicate() const
    {
        return new TRcrdCscr(*this);
    };

protected:
    // @cmember Pointer to parameter list object.
    TtValList* _prmListP;
    int	_labelIndex;// Destination label index in script
    // @cmember Mutex used to allow safe writes during run-phase.
#ifndef TH_POOL_VER
    static CTMutex m_mutex;
#else
    static CFastLock _recAccessFl;
#endif

private:
    static const bool _registeredCreator;
    static const bool _RegisterCreator();
}
; // TRcrdCscr

/**
 * Definition of Entry Record class
 **/
class TRcrdEntry : public TRecord
{

public:
    static const AtvfKey KEY;

    // Constructors & Destructor
TRcrdEntry(const TParamRule* headRuleP) : TRecord(headRuleP),_prmListP(NULL)
    {}

    virtual ~TRcrdEntry()
    {}
    ;

    // Control methods
    virtual int execute(TContext& ctxt);
    virtual int prepare(TScript* script, int objIndex, bool verbose=false);

    virtual TRecord* duplicate() const
    {
        return new TRcrdEntry(*this);
    };

protected:
    // @cmember Pointer to parameter list object.
    TtValList* _prmListP;
    // @cmember Mutex used to allow safe writes during run-phase.
#ifndef TH_POOL_VER
    static CTMutex m_mutex;
#else
    static CFastLock _recAccessFl;
#endif

private:
    static const bool _registeredCreator;
    static const bool _RegisterCreator();
}
; // TRcrdEntry

#endif
