/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc
///////////////////////////////////////////////////////////////////////////////

#ifndef _OtaDetail_h
#define _OtaDetail_h


///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <string>

#ifndef VMS
using std::string;
#endif

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

///////////////////////////////////////////////////////////////////////////////
#include <vector>
using std::vector;

typedef vector<string> ResultStrings;

///////////////////////////////////////////////////////////////////////////////
// @class   OtaDetailQuery | This class contains the results from an OTA
//          detailed query.  It also maintains other properties such as
//          description and header information.  When the script has been run
//          the information is written to a file and the file is copied into
//          the reposatory.
class OtaDetailQuery
{
    // @access Public Members
public:
    // @cmember Constructor
    OtaDetailQuery( string query, string desc );
    virtual ~OtaDetailQuery();   // @cmember Destructor

    string GetQuery();           // @cmember Returns query string
    string GetDescription();     // @cmember Returns query description
    string GetHeaders();         // @cmember Returns query headers
    void SetHeaders( string headers );     // @cmember Sets query header information
    void AddResult( string result );       // @cmember Adds result to list
    void WriteToFile( const char* filename );   // @cmember Output results to file

    // @access Private Members
private:
    string m_query;          // @cmember Query string executed
    string m_desc;           // @cmember Contains description of query
    string m_headers;        // @cmember Contains column headers
    ResultStrings m_results;    // @cmember Contains results of query
}
;

#endif
