/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyranoOTA
// @module  OtaTarget | This module contains the implementation of the
//          OtaTarget class.  This class is used to store details about an
//          OTA target.
//
///////////////////////////////////////////////////////////////////////////////


#include "OtaTarget.h"
#include "OtaDefs.h"
#include <library.hpp>
#include <TPLog.h>

OtaTarget::OtaTarget(string name, int type, string server, string user, string password, int maxrows) :
        m_name(name), m_type(type), m_server(server),
        m_user(user), m_password(password), m_maxrows(maxrows), m_error(false)
{}
;
OtaTarget::~OtaTarget()
{}
;

void OtaTarget::Log()
{
    char msg[256];
    const char *type = TargetTypes[m_type];
    sprintf(msg, "%s: Type = %s, Server = %s, Username = %s, Maxrows = %d",
            m_name.c_str(), type, m_server.c_str(), m_user.c_str(), m_maxrows);
    OtaAddTrace(CYR_OTA,TRACE_CONFIG,msg);
}

void *OtaTarget::GetFunction(CTLibrary lib, string procname )
{
    CTProcedure* proc = NULL;
    void *procaddr = NULL;
    try
    {
        proc = new CTProcedure( lib, procname.c_str() );
        procaddr = proc->getAddress();
    }
    catch( ... )
    {}

    return procaddr;
}
