/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"

#if 0
#include "TEMainStructure.hpp"

int main(int argc, char* argv[])
{
    int iRes = 0;

    HMODULE ole32Lib = LoadLibrary("ole32.dll");    // To avoid troubles on W2K.

    CMainExecuterStructure TExecuter;
    if(TExecuter.Init(argc, argv) == FALSE)
        return 1;

    iRes = TExecuter.Run();

    TExecuter.Complete();

    FreeLibrary(ole32Lib);

    return iRes;
}

#else

#ifdef USE_PERF_DATA
#   include "PerfCounters.hpp"
    using namespace OpenSTA_Performance;
#   include "PerfCounterMap.hpp"
#endif

//
//Include Headers that may use Roguewave before headers that may set 'using namespace std'
//
#ifndef CYRANO_TP_STANDALONE
#	include "OSTAORB/OSTAORB.h"
#	include <GlobalVariable.h>//for CyrGlobalVariableException
#else
#	include <CECSACORBA.h>
#endif

#ifdef WIN32
#	pragma warning(disable:4786)
#	pragma warning(disable:4503)
#endif

#ifdef CYRANO_TEST_MANAGER
#	include "TestManager_i.h"
#else
#	include "TestExecuter_i.h"
#endif

#include <string>
#include <new.h>
#include "mdbexcept.h"
#include "mdbgetopt.h"
#include "cxxutils.h"
#include <CyrHelper.inl>
//#include "CECInetInfo.h"
#include "mutex.hpp"

#ifdef CYRANO_TEST_MANAGER
#	include "TMthreadcontrol.h"
#endif

#include "mdbtypes2.h"
#include "FastLock.hpp"
#include "Common.hpp"

extern "C"
{
#	include <psl_macros.h>
#	include <psl_rtndef.h>
}

using std::string;

#if defined(CYRANO_TEST_EXECUTER)
#	define		    VERSION "3.0.0.0"
#	define		    ODLFILE "TestExecuter.odl" 
    const string	TPTEXT  = "TestExecuter_cmd";
#elif defined(CYRANO_HTTP_EXECUTER)
#	define		    VERSION "3.0.0.0"		
#	define		    ODLFILE "TEHttp.odl"
    const string    TPTEXT	= "TestExecuter_web";
#else
#	error "Undefined TestManager/TestProcess"
#endif

static TestExecuterImpl *TExecuter = NULL;

extern CTSemaphore blocking_semaphore;

#ifndef CYRANO_TP_STANDALONE
    //But just in case we do need to access it elsewhere here is a _ptr ptr
    TestExecuter_ptr	TExecuter_ptr = TestExecuter::_nil();
#endif

string exedir;
static CFastLock *g_pCleanup_mutex;

#ifdef CYRANO_TP_STANDALONE
    string g_tofFileDir;
#endif 


#include "AsyncVirtualUserController.h"
#include "TagParsing.hpp"

#include "TPLog.h"

CAsyncVirtualUserController* _threadPoolController  = NULL;

#ifdef USE_PERF_DATA
OpenSTA_Performance::CPerfSource* g_pPerformance    = NULL;
#endif

// Get pid for this TestProcess
const DWORD g_dwProcessId = GetCurrentProcessId();


#ifdef CYRANO_TP_STANDALONE

struct stUsersParams
{
    string zOdl;
    string zTof;
    DWORD dwNbUsers;
    DWORD dwCreationStep;
    DWORD dwCreationDelay;
};

#ifdef CYRANO_TP_STANDALONE
    stUsersParams g_usersStartParams;
#endif

// Thread function that creates the users.
DWORD WINAPI CreateUsers(void *pVoidParams)
{
    // Note: should wait for CThreadPool::OnStart to be called.
    stUsersParams *pParams = (stUsersParams*)pVoidParams;
    for(DWORD dw=0 ; dw<pParams->dwNbUsers ; dw+=pParams->dwCreationStep)
    {
        Sleep(pParams->dwCreationDelay);
        
        cout << "Starting " << __min(pParams->dwCreationStep, pParams->dwNbUsers-dw) << " users." << endl;
        if(_threadPoolController->StartNewUsersGroup(__min(pParams->dwCreationStep, pParams->dwNbUsers-dw), pParams->zTof) == -1)
        {
            cout << "Could not create users." << endl;
            break;
        }
    }
    cout << "All users started." << endl;

    return 0;
}

void StartUsersCreationThread(stUsersParams *pParams)
{
    BeginThreadEx(NULL, 0, CreateUsers, (PVOID)pParams, 0, NULL);
}

#endif

int     Run(CORBAServer* pServer, string zExeName, string zHostIpAddress, Cmdbgetopt &cl);
void    DeleteStatisticsFiles(const string &zHostIpAddress, int maxthreads = 2000);
void    sqlr_cleanup();

int main(int argc, char* argv[])
{
    HMODULE ole32Lib = LoadLibrary("ole32.dll");    // To avoid troubles on W2K.

    CFastLock::Init();

#ifdef TH_POOL_VER
    if(TRecord::InitClass() == FALSE)
    {
        cout << "ERROR initialising the TRecord class." << endl;
        return 1;
    }
    if(CORBARepository::InitClass() == FALSE)
    {
        cout << "ERROR initialising the CORBARepository class." << endl;
        return 1;
    }
    if(CAsyncVirtualUser::InitClass() == FALSE)
    {
        cout << "ERROR initialising the CAsyncVirtualUser class." << endl;
        return 1;
    }
    if(CVirtualUsersGroup::InitClass() == FALSE)
    {
        cout << "ERROR initialising the CVirtualUsersGroup class." << endl;
        return 1;
    }
#endif

    g_pCleanup_mutex = new CFastLock();

    int iRes = 0;

    TCHAR szDirAndName[255];
    GetModuleFileName(NULL, szDirAndName, 255);

    string zExeName = filename_base(szDirAndName);
    exedir = filename_dir(szDirAndName);

#ifdef CYRANO_TP_STANDALONE
    g_tofFileDir = exedir;
#endif

    Cmdbgetopt cl(argc, argv, "TestManager: ");

    cl.define_option("-u", CLO_NOVAL);      // uninstall
    cl.define_option("-i", CLO_STRINGVAL);  // install DLL full path
    cl.define_option("-ps", CLO_UNSVAL);    // pool size
    cl.define_option("-ct", CLO_UNSVAL);    // concurrent threads
    cl.define_option("-v", CLO_INTVAL);     // trace level
    cl.define_option("-d", CLO_NOVAL);      // debug mode

#ifdef CYRANO_TP_STANDALONE
    cl.define_option("-o", CLO_STRINGVAL);  // ODL file name
    cl.define_option("-t", CLO_STRINGVAL);  // TOF file name
    cl.define_option("-c", CLO_UNSVAL);     // users count
    cl.define_option("-cs", CLO_UNSVAL);    // creation step
    cl.define_option("-cd", CLO_UNSVAL);    // creation delay
#endif

    try
    {
	    cl.validate();
    }
    catch(string &err)
    {
        cout << "Parameter error: " << err << endl;
        FreeLibrary(ole32Lib);
        return 1;
    }

#ifdef USE_PERF_DATA

    g_pPerformance = new OpenSTA_Performance::CPerfSource(L"OpenSTA_TE", L"OpenSTA_Open", L"OpenSTA_Collect", L"OpenSTA_Close");
    if(g_pPerformance == NULL)
    {
        cout << "ERROR: could not access performance counters." << endl;
        FreeLibrary(ole32Lib);
        return 1;
    }

    if(g_pPerformance->ParseMap(OpenSTAPerf, MAP_SIZE(OpenSTAPerf)) != ERROR_SUCCESS)
    {
        cout << "ERROR: failed to parse performance map" << endl;
        delete g_pPerformance;
        FreeLibrary(ole32Lib);
        return 1;
    }

#endif  // USE_PERF_DATA

    if(cl.option_supplied("-u"))
    {
#ifdef USE_PERF_DATA
        // Uninstall the counters.
        g_pPerformance->Uninstall();
        cout << "Performance counters removed." << endl;
#endif
        
        FreeLibrary(ole32Lib);
#ifdef USE_PERF_DATA
        delete g_pPerformance;
#endif
        return 0;
    }
    else
    {
        if(cl.option_supplied("-i"))
        {
#ifdef USE_PERF_DATA
            // Install the counters.
            WCHAR lpMultiByteStr [256];
            MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, cl.get_value_for("-i").c_str(), -1, lpMultiByteStr, 256);
            if(g_pPerformance->Install(lpMultiByteStr) != ERROR_SUCCESS)
            {
                cout << "ERROR: failed to install the performance counters" << endl;
            }
            else
            {
                cout << "Performance counters installed." << endl;
            }
#endif
            
            FreeLibrary(ole32Lib);
#ifdef USE_PERF_DATA
            delete g_pPerformance;
#endif

            return 0;
        }
#ifdef CYRANO_TP_STANDALONE
        else
        {
            if((cl.option_supplied("-o") && cl.option_supplied("-t") && cl.option_supplied("-c") && cl.option_supplied("-cd") && cl.option_supplied("-ps") && cl.option_supplied("-cs")) == FALSE)
            {
                cout << "Usage: ThreadPool.exe <-i DLL Path> | <-u> | <<-o ODL File> <-t TOF File> <-c Nb Users> <-cs Creation steps> <-cd Creation delay> <-ps Thread pool size> <-ct Concurrent threads>>" << endl;
                FreeLibrary(ole32Lib);
                delete g_pPerformance;

                return 1;
            }
            // else will enter the Run(...) function, launch the user starting thread...
        }
#endif
    }

    cout << "CYRANO Concert starting" << endl;

#ifdef USE_PERF_DATA

    if(g_pPerformance->AdjustTitleIndexes() == ERROR_SUCCESS)
    {
        cout << "ERROR: failed to adjust performance counter numbers" << endl;
        delete g_pPerformance;
        return 1;
    }

    DWORD dwRes = g_pPerformance->Initialize();
    if(dwRes == ERROR_SUCCESS || dwRes == ERROR_ALREADY_EXISTS)
    {
        cout << "ERROR: failed to initialise performance source" << endl;
        delete g_pPerformance;
        return 1;
    }

#endif

    logr = new TPLog();

    // Initialize the parser.
    InitTagParsing();

    string zHostIpAddress;
    try
    {
        zHostIpAddress = GetHostIPAddressAsUnderscoreString();
    }
    catch(...)
    {
        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, "Caught Exception on startup. Could not resolve host name." );
        sqlr_cleanup();
        FreeLibrary(ole32Lib);
#ifdef USE_PERF_DATA
        delete g_pPerformance;
#endif
        return 1;
    }

    string FullName = CyrGetStatFileName(zHostIpAddress.c_str(), TRUE).data();  // TODO: CyrGetStatFileName called twice (look in Run).

    if(CAsyncTCPSocket::Init() != 0 
        || CAsyncSecureSocket::Init() != NO_ERROR 
        || CGlobalDataBuffHandler::Init(0, 0) == FALSE  // Use default values.
        || CStatistics::Init(FullName, FullName.substr(0, FullName.find_last_of('.')) + ".urls", 0) == FALSE
        || CAsyncHTTPRequest::InitClass(2000*4) == FALSE)   // Max number of users times max requests per user.
    {
        cout << "Error initialising structures." << endl;
        assert(0);
#ifdef USE_PERF_DATA
        delete g_pPerformance;
#endif
        return 1;
    }

    // Initialise CORBA for logging purposes
    CORBAServer* pServer;
    try
    {
        pServer = new CORBAServer(argc, argv);
    }
    catch(...)
    {
        // TODO: pas encore fait le Setup sur logr. Est-ce un pb ?
        cout << "Error connecting to the CORBA server." << endl;

        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, "Caught Exception on startup, is Nameservice Running?");
        sqlr_cleanup();
        FreeLibrary(ole32Lib);
#ifdef USE_PERF_DATA
        delete g_pPerformance;
#endif
        return 1;
    }

#ifndef CYRANO_TP_STANDALONE
    try
    {

        // Initialise Global Logging Object.
        int loglevel = cl.get_int_value_for("-v");  // Returns 0 if not set.
        string zRegisteredLogName = string(TPTEXT) + '_' + zHostIpAddress + '_' + int_to_string((unsigned)g_dwProcessId);

        if(logr) logr->Setup(pServer, zRegisteredLogName.c_str(), loglevel);
    }
    catch(const CORBA::Exception &e)
    {
        cout << ("Caught CORBA::" + CORBAExString(e)).c_str() << endl;

        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, ("Caught CORBA::" + CORBAExString(e)).c_str());
        sqlr_cleanup();
        FreeLibrary(ole32Lib);
        return 1;
    }
    catch(...)
    {
        cout << "Caught Exception on startup, is Nameservice Running?" << endl;

        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, "Caught Exception on startup, is Nameservice Running?" );
        sqlr_cleanup();
        FreeLibrary(ole32Lib);
        return 1;
    }

#endif

    try
    {
        //
        //	  ...and then run
        //

        iRes = Run(pServer, zExeName, zHostIpAddress, cl);
    }
    catch(string &es)
    {
        // Detected error
        cout << es << endl;

        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, es.c_str());
        sqlr_cleanup();
        iRes = 1;
    }
    catch(mdbIOException& e)
    {
        cout << (e.what() + ": " + e.ioname()).c_str() << endl;

        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, (e.what() + ": " + e.ioname()).c_str());
        sqlr_cleanup();
        iRes = 1;
    } 
    catch(mdbParseException& e) 
    {
        cout << e.what().c_str() << endl;

        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, e.what().c_str());
        sqlr_cleanup();
        iRes = 1;
    }
    catch(CyrRepositoryException& e) 
    {
        cout << e.what().data() << endl;

        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, e.what().data());
        sqlr_cleanup();
        iRes = 1;
    }
    catch(exception& e)
    {
        cout << e.what();

        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, e.what());
        sqlr_cleanup();
        iRes = 1;
    }
#ifndef CYRANO_TP_STANDALONE
    catch(CORBA::Exception& e)
    {
        cout << ("Caught CORBA::" + CORBAExString(e)).c_str() << endl;

        // Some other unexpected exception caught
        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, ("Caught CORBA::" + CORBAExString(e)).c_str());
        sqlr_cleanup();
        iRes = 1;
    }
#endif
    catch(...)
    {
        cout << "Caught exception" << endl;

        // Some other unexpected exception caught
        if(logr) logr->AddTrace(STARTUP, ERRORLEVEL, "Caught exception");
        sqlr_cleanup();
        iRes = 1;
    }

    CAsyncSecureSocket::Uninit();
    CAsyncTCPSocket::Uninit();
    CGlobalDataBuffHandler::UnInit();

    FreeLibrary(ole32Lib);

    if(logr)
        delete logr;

    if(pServer)
        delete pServer; // End of CORBA communication.

    if(g_pCleanup_mutex)
        delete g_pCleanup_mutex;
    CAsyncVirtualUser::UninitClass();
    CVirtualUsersGroup::UninitClass();
    TRecord::UninitClass();
    CORBARepository::UninitClass();

    return iRes;
}



//
//	  Run 
//
//	  This is the main processing section of sqlr.
//
int Run(CORBAServer* pServer, string zExeName, string zHostIpAddress, Cmdbgetopt &cl)
{
    logr->AddTrace(STARTUP, TRACELEVEL, "Start Execution");
    logr->AddTrace(DBGSTD, DEBUGLEVEL, "Before TScript");

    // Read in atvf(tof) file to create the Script Rules, and commence execution.
    // Create rule for this apps file type

    // TODO: grer exceptions si fichier pas trouv ... notamment, est-ce que le fichier est dj dans ce rpertoire ?
#ifndef CYRANO_TP_STANDALONE
    TScriptRule *pRule = new TScriptRule(exedir + FN_PATHSEP + ODLFILE);
#else
    TScriptRule *pRule = new TScriptRule(cl.get_value_for("-o"));
#endif

    logr->AddTrace(DBGSTD, DEBUGLEVEL, "TScriptRule created.");

    //
    //	  Initialize the CORBA Initialise - pends Test Manager
    //
    // Create thread control object
    
    // Get the test name passed in on the command line - if any at 
    // all. This shall determine in External mode whether this object
    // shall act as a Test Manager(TestName) or Test Executer(no TestName)

    // Get the node name

#ifndef CYRANO_TP_STANDALONE
    // Set the name to be registered, For test managers this is<nodename.project.test.testname.pid>
    char strProcess[64];
    sprintf(strProcess, "%s_%8.8lX", zHostIpAddress.c_str(), g_dwProcessId);

    //	  Create the instance of the test manager CORBA object
    TExecuter = new TestExecuterImpl(pServer, string(strProcess), zHostIpAddress, g_dwProcessId, zExeName, pRule);
    assert(TExecuter);
    TExecuter_ptr = TExecuter->_this();

    if(cl.option_supplied("-d"))  //if run in debug mode, register as such in order for the test manager to use the current debugging process instead of starting a new one.
    {
        // Test Manager registration in order to force our debug version to be used.
        string ObjectClass  = TestExecuter::CLASS_NAME;
        //TODO   string debug_reg    = "DEBUG." + zHostIpAddress + "." + strip_extension(zExeName);
#ifdef TH_POOL_VER
        string debug_reg    = "DEBUG." + zHostIpAddress + '.' + strip_extension("TExecuter_web.exe");
#else
        string debug_reg    = "DEBUG." + zHostIpAddress + '.' + strip_extension("TExecuterTD_web.exe");
#endif

        string trcmsg       = "Debug requested, register as " + debug_reg;

        logr->AddTrace(STARTUP, DEBUGLEVEL, trcmsg.c_str());

        CORBA::String_var dbg_reg = CORBA::string_dup(debug_reg.c_str());
        CORBA::String_var obj_reg = CORBA::string_dup(ObjectClass.c_str());

        pServer->NameObject(TExecuter_ptr, obj_reg.inout(), dbg_reg.inout());
    }

#endif


    // Start the thread pool controller.
    string zTraceFile = exedir + "\\Trace.txt";
    switch(cl.get_int_value_for("-v"))
    {
    case 4:
    	_threadPoolController = new CAsyncVirtualUserController(pRule, "connection request errors full sync Startup Shutdown ScriptCommand Debug-Generic Note Audit Repository SecureSocket", zTraceFile.c_str());
        break;
    case 3:
    	_threadPoolController = new CAsyncVirtualUserController(pRule, "connection request errors full sync SecureSocket", zTraceFile.c_str());
        break;
    case 2:
    	_threadPoolController = new CAsyncVirtualUserController(pRule, "connection request errors sync SecureSocket", zTraceFile.c_str());
        break;
    case 1:
    	_threadPoolController = new CAsyncVirtualUserController(pRule, "errors", zTraceFile.c_str());
        break;
    case 0:
    default:
        _threadPoolController = new CAsyncVirtualUserController(pRule);
        break;
    }

    if(_threadPoolController == NULL)
    {
        cout << "ERROR: could not create the thread pool controller." << endl;
        return 0;
    }

    DWORD dwThreadPoolSize = cl.get_int_value_for("-ps");
    DWORD dwNbConcurrentThreads = cl.get_int_value_for("-ct");

    if(dwNbConcurrentThreads == 0)
    {
        // Automatically compute the thread pool parameters.
        SYSTEM_INFO sI;
        GetSystemInfo(&sI);
        dwNbConcurrentThreads = sI.dwNumberOfProcessors;
    }

    if(dwThreadPoolSize < dwNbConcurrentThreads)
    {
        dwThreadPoolSize = dwNbConcurrentThreads * 25;
    }

    if(_threadPoolController->Init(dwThreadPoolSize, dwNbConcurrentThreads) != 0)
    {
        cout << "ERROR: could not initialize the thread pool controller." << endl;
        return 0;
    }

#ifdef CYRANO_TP_STANDALONE
    g_usersStartParams.zOdl           = cl.get_value_for("-o");
    g_usersStartParams.zTof           = cl.get_value_for("-t");
    g_usersStartParams.dwNbUsers      = cl.get_int_value_for("-c");
    g_usersStartParams.dwCreationStep = cl.get_int_value_for("-cs");
    g_usersStartParams.dwCreationDelay= cl.get_int_value_for("-cd");

    StartUsersCreationThread(&g_usersStartParams);
#else
    // If a Test Executer then wait for this semaphore which should be set
    // on the call of Stop which will be called when a test manager tries to close
    // down all it's child processes

    //TODO??????
    blocking_semaphore.lock();
#endif

	// Set the time offset for the results creation.
	CStatistics::SetTimeOffset(Repository->GetTimeOffset());

    try
    {
        if(_threadPoolController->Run() != 0)
        {
            cout << "ERROR: could not execute the threadpool." << endl;
        }
    }
    catch(mdbRuntimeError &err)
    {
        cout << err.what() << endl;
    }

#ifndef CYRANO_TP_STANDALONE
    //------------------------------------------
    // To put the stat file back to the server
    //------------------------------------------
    string FullName = CyrGetStatFileName(zHostIpAddress.c_str(), TRUE).data();
    string Name     = CyrGetStatFileName(zHostIpAddress.c_str(), FALSE).data();

    CStatistics::Release();

    Repository->SetLocalResultsFile(FullName.c_str(), Name.c_str(), FALSE);
    //------------------------------------------
    
#endif


    //
    //	  All done - perform any cleanup required
    //

    sqlr_cleanup();
    _threadPoolController->DropUsers();

    delete _threadPoolController;

    DeleteStatisticsFiles(zHostIpAddress);

    delete pRule;

    return 0;
}


void DeleteStatisticsFiles(const string &zHostIpAddress, int maxthreads)
{
    char szPrefixStatFileName[40];
    sprintf(szPrefixStatFileName, "TEW_%s-%d", zHostIpAddress.c_str(), g_dwProcessId);

    int iThreadId;
    int iMaxThread = maxthreads/10 + maxthreads%10;

    // Try to remove an eventual concatenated stat file with same Pid
    char szBuf[40];
    sprintf(szBuf, "%s.stat", szPrefixStatFileName);
    remove(szBuf);
    sprintf(szBuf, "%s.url", szPrefixStatFileName);
    remove(szBuf);
    
    // Try to remove eventual stat files with same Pid
    for (iThreadId = 0; iThreadId < iMaxThread; iThreadId++) 
    {
        sprintf(szBuf, "%s-%d.stat", szPrefixStatFileName, iThreadId);
        remove(szBuf);
        sprintf(szBuf, "%s-%d.url", szPrefixStatFileName, iThreadId);
        remove(szBuf);
    }
}

// sqlr_cleanup
// ============
//
//
void sqlr_cleanup(void)
{
    //no-one else here for a bit!
//    RWLockGuard<RWMutexLock> guard(cleanup_mutex);//use guard to deal with exceptions
    CLock guard (*g_pCleanup_mutex);
    if(logr) logr->AddTrace(SHUTDOWN, DEBUGLEVEL, "Clean up on Exiting "); 
    
    // Catch any exceptions during cleanup. This could happen if
    // a thread was hard killed, leaving some object in an undefined 
    // state.
    
    try
    {
        if (TExecuter)
        {
            TExecuter->SetToCloseDown();    // Tell monitoring that closedown is imminent and clear up TestProces Object member data.

#ifndef CYRANO_TP_STANDALONE
            CORBA::release(TExecuter_ptr);
            TExecuter->_remove_ref();
            TExecuter=NULL;
#endif
        }
    }
#ifndef CYRANO_TP_STANDALONE
    catch(CyrGlobalVariableException& e) 
    {
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,(const char*)e.what);
        exit(0);
    }
#endif
    catch(CyrRepositoryException& e) 
    {
        string logstr;
        logstr = e.what() + ',' + e.where() + " Status: " + int_to_string(e.status()).c_str();
        if(logr)logr->AddTrace(SHUTDOWN, ERRORLEVEL, logstr.c_str());
        exit(0);
    }
#ifndef CYRANO_TP_STANDALONE
    catch(CORBA::Exception& e)
    {
        string msg = "Caught CORBA::" + CORBAExString(e) + " in clean-up";
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,msg.c_str());
        exit(0);
    }
#endif
    catch(exception &e)
    {
        string msg = string("Caught exception in clean-up :") + string(e.what());
        if(logr)logr->AddTrace(SHUTDOWN, ERRORLEVEL, msg.c_str());
        exit(0);
    }
    catch( ... )
    {
        if(logr)logr->AddTrace(SHUTDOWN, ERRORLEVEL, "Caught exception in clean-up ");
        exit(0);
    }
} // End of sqlr cleanup.

#endif
