/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"

#if 0

#include "TEMainStructure.hpp"

CMainExecuterStructure g_TExecuter;

extern "C"
{

    __declspec(dllexport) int TELibMain(HWND hwind, char *TofFile, char *OdlDir)
    {
        if(g_TExecuter.Init(hwind, TofFile, OdlDir) == FALSE)
            return 1;

        // Inform modeller the replay is beginning
        logr->AddTrace(STARTUP, TRACELEVEL, "!Begin replay" );

        BOOL bSucceeded = TRUE;
        string msg;

	    // Read in atvf(tof) file to create the Script Rules, and commence execution
        try
        {
            g_TExecuter.Run();
        }
        catch(string es)
        {
            bSucceeded = FALSE;
            msg = es;
        }
        catch(mdbIOException& e)
        {
            msg = e.what() + ": " + e.ioname();
            bSucceeded = FALSE;
        }
        catch(mdbParseException& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch(CyrRepositoryException& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch(exception& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch( ... )
        {
            bSucceeded = FALSE;
            msg = "Unknwon exception caught";
        }

        if(bSucceeded == FALSE)
        {
            logr->AddTrace(STARTUP, ERRORLEVEL, msg.c_str());
            g_TExecuter.Complete();
            return 1;
        }
        else
            return 0;
    }



    __declspec( dllexport ) void TELibStop(int i)
    {
        if (i != 0)
        {
			g_TExecuter.Stop();
            g_TExecuter.Complete();
        }
        else
        {
            // Not used in the TP model.
        }
    }

} // extern "C"


#else

// Former implementation

#include <CECSACORBA.h>

#ifdef WIN32
#	pragma warning(disable:4786)
#	pragma warning(disable:4503)
#endif

#ifdef CYRANO_TEST_MANAGER
#	include "TestManager_i.h"
#else
#	include "TestExecuter_i.h"
#endif

#include <string>
#include <new.h>
#include "mdbexcept.h"
#include "mdbgetopt.h"
#include "cxxutils.h"
#include <CyrHelper.inl>
#include "CECInetInfo.h"
#include "mutex.hpp"

#ifdef CYRANO_TEST_MANAGER
#	include "TMthreadcontrol.h"
#endif

#include "mdbtypes2.h"
#include "FastLock.hpp"
#include "Common.hpp"

extern "C"
{
#	include <psl_macros.h>
#	include <psl_rtndef.h>
}

using std::string;

#if defined(CYRANO_TEST_EXECUTER)
#	define		    VERSION "3.0.0.0"
#	define		    ODLFILE "TestExecuter.odl" 
    const string	TPTEXT  = "TestExecuter_cmd";
#elif defined(CYRANO_HTTP_EXECUTER)
#	define		    VERSION "3.0.0.0"		
#	define		    ODLFILE "TEHttp.odl"
    const string    TPTEXT	= "TestExecuter_web";
#else
#	error "Undefined TestManager/TestProcess"
#endif

#include "CORBARepository.h"

static TestExecuterImpl *TExecuter = NULL;

extern CTSemaphore blocking_semaphore;


string exedir="";
string filevardir="";
string g_tofFileDir="";

// Global declaration for stopping execution
TScriptRule* pRule = NULL;


static CFastLock *g_pCleanup_mutex;


#include "AsyncVirtualUserController.h"
#include "TagParsing.hpp"

#include "TPLog.h"

CAsyncVirtualUserController* _threadPoolController = NULL;

// Get pid for this TestProcess
const DWORD g_dwProcessId = GetCurrentProcessId();


struct stUsersParams
{
    string zOdl;
    string zTof;
    DWORD dwNbUsers;
    DWORD dwCreationStep;
    DWORD dwCreationDelay;
};

stUsersParams	g_usersStartParams;

const char *g_successMsg = "Script Completed Successfully";
const char *g_failureMsg = "Script Execution Failed.";

// Thread function that creates the users.
DWORD WINAPI CreateUsers(void *pVoidParams)
{
    try
    {
        stUsersParams *pParams = (stUsersParams*)pVoidParams;
        for(DWORD dw=0 ; dw<pParams->dwNbUsers ; dw+=pParams->dwCreationStep)
        {
            Sleep(pParams->dwCreationDelay);
        
	        logr->AddTrace(STARTUP, TRACELEVEL, "Starting.");
            if(_threadPoolController->StartNewUsersGroup(__min(pParams->dwCreationStep, pParams->dwNbUsers-dw), pParams->zTof) == -1)
            {
		        logr->AddTrace(STARTUP, TRACELEVEL, "Could not create users.");
                break;
            }
        }
	    logr->AddTrace(STARTUP, TRACELEVEL, "All users started.");
    }
    catch(...)
    {
		logr->AddTrace(DBGSTD, TRACELEVEL, "Unknown error creating users.");
    }

    return 0;
}


void StartUsersCreationThread(stUsersParams *pParams)
{
    BeginThreadEx(NULL, 0, CreateUsers, (PVOID)pParams, 0, NULL);
}


DWORD WINAPI StartReplay(void *pVoidParams)
{
	try
	{
		if(_threadPoolController->Run() != 0)
		{
			logr->AddTrace(DBGSTD, TRACELEVEL, "ERROR: could not execute the threadpool.");
		}
	}
    catch(CyrException &err)
    {
        logr->AddTrace(DBGSTD, TRACELEVEL, err.what().c_str());
    }
	catch(mdbRuntimeError &err)
	{
        logr->AddTrace(DBGSTD, TRACELEVEL, err.what().c_str());
	}
    catch(...)
    {
		logr->AddTrace(DBGSTD, TRACELEVEL, "Unknown error.");
    }

    BOOL bExecutionRes = FALSE;

	if (_threadPoolController)
	{
        bExecutionRes = (_threadPoolController->GetFailedUsersCount() == 0);

	    _threadPoolController->DropUsers();
		delete _threadPoolController;
		_threadPoolController = NULL;
	}

	if (pRule)
    {
        delete pRule;
        pRule = NULL;
    }

    CStatistics::Release();
    TRecord::UninitClass();
    CAsyncVirtualUser::UninitClass();
    CVirtualUsersGroup::UninitClass();

    CAsyncSecureSocket::Uninit();
    CAsyncTCPSocket::Uninit();
    CGlobalDataBuffHandler::UnInit();

    if(g_pCleanup_mutex)
        delete g_pCleanup_mutex;

    if(logr)
    {
        delete logr;
        logr = NULL;
    }

    if(gHwnd != NULL)
    {
        if(bExecutionRes)
        {
            // Execution succeeded.
            PostMessage(gHwnd, 1027, 0, (LPARAM)((LPSTR)g_successMsg));
        }
        else
        {
            // Execution failed.
            PostMessage(gHwnd, 1027, 0, (LPARAM)((LPSTR)g_failureMsg));
        }
    }

    return 0;
}






int     Run (CORBAServer* pServer,string exename);
void    DeleteStatisticsFiles(const string &zHostIpAddress, int maxthreads = 2000);
void    sqlr_cleanup();





extern "C"
{

__declspec( dllexport ) int TELibMain(HWND hwind, char *TofFile, char *OdlDir)
{
    CFastLock::Init();

    if(TRecord::InitClass() == FALSE)
    {
        cout << "ERROR initialising the TRecord class." << endl;
        return 1;
    }
    if(CORBARepository::InitClass() == FALSE)
    {
        cout << "ERROR initialising the CORBARepository class." << endl;
        return 1;
    }
    if(CAsyncVirtualUser::InitClass() == FALSE)
    {
        cout << "ERROR initialising the CAsyncVirtualUser class." << endl;
        return 1;
    }
    if(CVirtualUsersGroup::InitClass() == FALSE)
    {
        cout << "ERROR initialising the CVirtualUsersGroup class." << endl;
        return 1;
    }

    g_pCleanup_mutex = new CFastLock();

    exedir = "";
	filevardir = "";
	g_tofFileDir = "";

    if (!logr)
		logr = new TPLog();

    if (hwind == NULL)
    {
        // Detected error while checking args
        logr->AddTrace(STARTUP,ERRORLEVEL,"Handle for window is NULL !");

        sqlr_cleanup();
        return 1;
    }

    gHwnd = hwind;

    // Inform modeller the replay is beginning
    logr->AddTrace(STARTUP, TRACELEVEL, "!Begin replay" );

	// Some initializations
    if(CAsyncTCPSocket::Init() != 0 
        || CAsyncSecureSocket::Init() != NO_ERROR 
        || CStatistics::Init("", "", 0) == FALSE
        || CGlobalDataBuffHandler::Init() == FALSE
        || CAsyncHTTPRequest::InitClass(2000*4) == FALSE)   // Max number of users times max requests per user.
    {
        // TODO
        assert(0);
        return 1;
    }



	// Read in atvf(tof) file to create the Script Rules, and commence execution
    try
    {
		// Initialize the parser.
		InitTagParsing();

	
		exedir = OdlDir;
        filevardir = TofFile;
        // remove TOF file name
        string::size_type lastSlash = filevardir.find_last_of("\\");
        if (lastSlash != string::npos)
        {
            filevardir.erase(lastSlash);
            g_tofFileDir = filevardir;
        }
        else
        {
            filevardir = "";
        }


        Run(NULL,TofFile);

        //
        //    All done - perform any cleanup required
        //
    }
    catch(string es)
    {
        // Detected error
        logr->AddTrace(STARTUP,ERRORLEVEL,es.c_str());
        sqlr_cleanup();
        return 1;
    }
    catch(mdbIOException& e)
    {
        string msg=e.what()+": "+e.ioname();
        logr->AddTrace(STARTUP,ERRORLEVEL,msg.c_str());
        sqlr_cleanup();
        return 1;
    }
    catch(mdbParseException& e)
    {
        logr->AddTrace(STARTUP,ERRORLEVEL,e.what().c_str());
        sqlr_cleanup();
        return 1;
    }
    catch(CyrRepositoryException& e)
    {
        logr->AddTrace(STARTUP,ERRORLEVEL,e.what().c_str());
        sqlr_cleanup();
        return 1;
    }
    catch(exception& e)
    {
        logr->AddTrace(STARTUP,ERRORLEVEL,e.what());
        sqlr_cleanup();
        return 1;
    }
    catch( ... )
    {
        // Some other unexpected exception caught
        logr->AddTrace(STARTUP,ERRORLEVEL,"Caught exception");
        sqlr_cleanup();
        return 1;
    }

    return 0;
}



__declspec( dllexport ) void TELibStop(int i)
{
    if(i != 0)
    {
        if(logr)
            logr->AddTrace(SHUTDOWN,ERRORLEVEL,"Stop execution.");

		if (_threadPoolController)
		{
			_threadPoolController->Stop();
		}
    }
    else
    {
        // Not used in the TP model.
    }
}

} // extern "C"





//
//	  Run 
//
//	  This is the main processing section of sqlr.
//
int Run(CORBAServer* pServer, string TofFile)
{
    try
    {
	    logr->AddTrace(DBGSTD, DEBUGLEVEL, "Before TScript");

	    // Read in atvf(tof) file to create the Script Rules, and commence execution.
		// Create rule for this apps file type
		// Create rule for this apps file type

		string odlfile;
		// Build odl file name
		odlfile = exedir + ODLFILE;

		string msg="Using odl file: " + odlfile;
		logr->AddTrace(STARTUP,DEBUGLEVEL,msg.c_str());

		// TODO: grer exceptions si fichier pas trouv ... notamment, est-ce que le fichier est dj dans ce rpertoire ?
		pRule = new TScriptRule(odlfile);

		logr->AddTrace(DBGSTD, DEBUGLEVEL, "TScriptRule created.");


		// Start the thread pool controller.
        string zTraceFileName = exedir + '\\' + "Trace.txt";
        _threadPoolController = new CAsyncVirtualUserController(pRule, "connection request errors full sync Startup Shutdown ScriptCommand Debug-Generic Note Audit Repository SecureSocket", zTraceFileName.c_str());
		// "connection request errors full sync Startup Shutdown ScriptCommand Debug-Generic Note Audit Repository SecureSocket"

		if(_threadPoolController == NULL)
		{
		    logr->AddTrace(DBGSTD, TRACELEVEL, "ERROR: could not create the thread pool controller.");
			return 0;
		}


		if(_threadPoolController->Init(1, 1) != 0)
		{
		    logr->AddTrace(DBGSTD, TRACELEVEL, "ERROR: could not initialize the thread pool controller.");
			return 0;
		}

		g_usersStartParams.zOdl           = odlfile;
		g_usersStartParams.zTof           = TofFile;
		g_usersStartParams.dwNbUsers      = 1;
		g_usersStartParams.dwCreationStep = 1;
		g_usersStartParams.dwCreationDelay= 0;

        int ProcessID = getpid();

        string HostName="localhost";
        // Set the name to be registered, For test managers this is<nodename.project.test.testname.pid>
        char    strProcess[64];

        sprintf(strProcess, "%s.%8.8lX", HostName.c_str(), ProcessID);

        //    Create the instance of the test manager CORBA object
        TExecuter=new TestExecuterImpl(pServer, string(strProcess), HostName, ProcessID, TofFile, pRule);

        TExecuter->Initialise("","","","");

		StartUsersCreationThread(&g_usersStartParams);

        BeginThreadEx(NULL, 0, StartReplay, (PVOID)NULL, 0, NULL);
    }
    catch(...)  // TODO: remove.
    {
	    logr->AddTrace(DBGSTD, TRACELEVEL, "ERROR in Run");
    }
    return 0;
}


void DeleteStatisticsFiles(const string &zHostIpAddress, int maxthreads)
{
    char szPrefixStatFileName[40];
    sprintf(szPrefixStatFileName, "TEW_%s-%d", zHostIpAddress.c_str(), g_dwProcessId);

    int iThreadId;
    int iMaxThread = maxthreads/10 + maxthreads%10;

    // Try to remove an eventual concatenated stat file with same Pid
    char szBuf[40];
    sprintf(szBuf, "%s.stat", szPrefixStatFileName);
    remove(szBuf);
    sprintf(szBuf, "%s.url", szPrefixStatFileName);
    remove(szBuf);
    
    // Try to remove eventual stat files with same Pid
    for (iThreadId = 0; iThreadId < iMaxThread; iThreadId++) 
    {
        sprintf(szBuf, "%s-%d.stat", szPrefixStatFileName, iThreadId);
        remove(szBuf);
        sprintf(szBuf, "%s-%d.url", szPrefixStatFileName, iThreadId);
        remove(szBuf);
    }
}

// sqlr_cleanup
// ============
//
//
void sqlr_cleanup(void)
{
    //no-one else here for a bit!
    CLock guard (g_pCleanup_mutex);
    if(logr) logr->AddTrace(SHUTDOWN, DEBUGLEVEL, "Clean up on Exiting "); 
    
    // Catch any exceptions during cleanup. This could happen if
    // a thread was hard killed, leaving some object in an undefined 
    // state.
    
    try
    {
        if (TExecuter)
        {
            TExecuter->SetToCloseDown();    // Tell monitoring that closedown is imminent and clear up TestProces Object member data.
        }
    }
    catch(CyrRepositoryException& e) 
    {
        string logstr;
        logstr = e.what() + ',' + e.where() + " Status: " + int_to_string(e.status()).c_str();
        if(logr)logr->AddTrace(SHUTDOWN, ERRORLEVEL, logstr.c_str());
        exit(0);
    }
    catch(exception &e)
    {
        string msg = string("Caught exception in clean-up :") + string(e.what());
        if(logr)logr->AddTrace(SHUTDOWN, ERRORLEVEL, msg.c_str());
        exit(0);
    }
    catch( ... )
    {
        if(logr)logr->AddTrace(SHUTDOWN, ERRORLEVEL, "Caught exception in clean-up ");
        exit(0);
    }
} // End of sqlr cleanup.

#endif



