/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "AsyncFileIOHandler.hpp"
#include "FileManager.hpp"
#include "TofPrmDef.h"
#include "mdbExcept.h"  // TODO: remove.

CAsyncFileIOHandler::CAsyncFileIOHandler(CFileManager *pFileMgr, CIOCP &iocp, CTrace *pTrace)
    : m_pTrace(pTrace), m_file(iocp, pTrace), m_pFileMgr(pFileMgr), m_dwLastError(0)
{
}

// Methods.
void CAsyncFileIOHandler::OnEvent(DWORD dwNumberOfBytesTransferred, BOOL bIsIo, DWORD /*dwEventType*/)
{
    // TODO: tester qu'on a bien lu/crit le bon nombre de bytes?

    if(bIsIo == FALSE)
    {
        // TODO: unexpected event?
        assert(0);
    }

    // Adjust file offset for next read/write operation.
    m_dwOffset += dwNumberOfBytesTransferred;

    // TODO: is that OK if we expect that all the m_dwBytesToTransfer bytes are read at once? (if not, should have entered OnEventFailed?).
    assert(m_dwBytesToTransfer == dwNumberOfBytesTransferred);

    if(m_dwDesiredAccess & TOF_FILACC_INPUT)
    {
        // We're reading from a file.
        if(ProcessReadData(dwNumberOfBytesTransferred) == TRUE)
        {
            // Signal the File Manager that we are over with the IO it started.
            m_pFileMgr->SignalIoIsOver();
        }
    }
    else
    {
        // We're writting to a file.
        // Signal the File Manager that we are over with the IO it started.
        m_pFileMgr->SignalIoIsOver();
    }
}

void CAsyncFileIOHandler::OnEventFailed(DWORD dwNumberOfBytesTransferred, BOOL bIsIo, DWORD dwEventType, DWORD dwError)
{
    if(bIsIo == FALSE)
    {
        // TODO: unexpected event?
        assert(0);
    }

    // Adjust file offset for next read/write operation.
    m_dwOffset += dwNumberOfBytesTransferred;
    m_dwBytesToTransfer -= dwNumberOfBytesTransferred;

    m_dwLastError = dwError;

    switch(dwError)
    {
    case ERROR_HANDLE_EOF:
        // Reached the end of the file during the async. op.
        // TODO: the data is OK?
        if(m_dwBytesToTransfer == 0)
        {
            // Reached the end of the file but all the data was transfered. Signal the VU the read was OK.
            m_pFileMgr->SignalIoIsOver();
        }
        else
        {
            // Signal the VU the read op. failed.
            m_pFileMgr->SignalIoFailed();
        }
        break;

    default:
        // Signal the VU the read op. failed.
        m_pFileMgr->SignalIoFailed();
        break;
    };
}

// Return TRUE if we're over processing the data.
BOOL CAsyncFileIOHandler::ProcessReadData(DWORD dwNumberOfBytesRead)
{
    BOOL bReadOver = FALSE;

    if((m_dwOptions & TOF_FILOPT_BINFIL) == FALSE)
    {
        // Not binary file reading. Must make sure we leave the file clean.
        // Two cases now: either dwBufferLength is long enough to store the whole data
        // of that record, or read too much data and should move back the file cursor
        // to the begining of the next record.

        // No newline found. Means the buffer was not large enough. Can't do anything to help, 
        // but move the file cursor to the end of the current record in case the next record 
        // would be read later.
        switch(m_eReadPhase)
        {
        case READING_DATA:
            {
                CHAR *cNlPos = (CHAR*) memchr (m_sReadBuffer, '\n', dwNumberOfBytesRead);

                if(cNlPos == NULL)
                {
                    m_eReadPhase = SKIPPING_DATA;
                    DWORD dwRes = m_file.Read(m_acSkipData, SKIP_DATA_BUFF_SIZE, m_dwOffset);
                    //TODO: grer dwRes
                }
                else
                {
                    // Found the end of the record in the data we read. In case we moved too far, move the file
                    // cursor back to the next file record.
                    DWORD dwMoveBack = dwNumberOfBytesRead - (cNlPos - m_sReadBuffer);
                    m_dwOffset -= dwMoveBack;

                    bReadOver = TRUE;
                }

                break;
            }

        case SKIPPING_DATA:
            {
                CHAR *cNlPos = (CHAR*) memchr (m_acSkipData, '\n', dwNumberOfBytesRead);

                if(cNlPos == NULL)
                {
                    DWORD dwRes = m_file.Read(m_acSkipData, SKIP_DATA_BUFF_SIZE, m_dwOffset);
                    //TODO: grer dwRes
                }
                else
                {
                    // Found the end of the record in the data we read. In case we moved too far, move the file
                    // cursor back to the next file record.
                    DWORD dwMoveBack = dwNumberOfBytesRead - (cNlPos - m_acSkipData);
                    m_dwOffset -= dwMoveBack;

                    bReadOver = TRUE;
                }

                break;
            }

        default:
            {
                // TODO error.
                assert(0);
            }
        };

    }
    else
    {
        // We read what we were asked to. Signal the VU we are over.
        bReadOver = TRUE;
    }

    return bReadOver;
}

// Return value: 0 = success, other = CreateFile() failure code.
DWORD CAsyncFileIOHandler::Open(LPCSTR zFilePath, DWORD dwDesiredAccess, DWORD dwOptions)
{
    m_dwOffset = 0L;
    m_dwDesiredAccess = dwDesiredAccess;
    m_dwOptions = dwOptions;
//TODO: grer dwOptions.
    return m_file.Open(zFilePath, 
                        (dwDesiredAccess & TOF_FILACC_INPUT) ? GENERIC_READ : GENERIC_WRITE, 
                        (dwDesiredAccess & TOF_FILACC_APPEND) ? CREATE_ALWAYS : OPEN_ALWAYS, 
                        this);
}

// Return value: 0 = success, other = CloseHandle() failure code
DWORD CAsyncFileIOHandler::Close()
{
    return m_file.Close();
}

// Return value: 0 = success, other = WSAGetLastError() failure code.
DWORD CAsyncFileIOHandler::Read(LPSTR zBuffer, DWORD dwBufferLength)
{
    // TODO: return a value like ERROR_READ_FAULT since opened for output. Or is that done by the ReadFile WIN32 API?
    if(!(m_dwDesiredAccess & TOF_FILACC_INPUT))
        throw mdbParseException("Attempt to read file opened for output");

    m_eReadPhase = READING_DATA;
    m_dwBytesToTransfer = dwBufferLength;
    DWORD dwRes = m_file.Read(zBuffer, dwBufferLength, m_dwOffset);

    return dwRes;
}

// Return value: 0 = success, other = GetLastError() failure code.
DWORD CAsyncFileIOHandler::Write(const string &zBuffer)
{
    m_zSendBuffer = zBuffer;

    if((m_dwOptions & TOF_FILOPT_BINFIL) == FALSE)
	    m_zSendBuffer += '\n';

    if(m_dwDesiredAccess & TOF_FILACC_INPUT)
	    throw mdbParseException("Attempt to write to file opened for input");

    return m_file.Write(m_zSendBuffer.c_str(), m_zSendBuffer.length(), m_dwOffset);
}

void CAsyncFileIOHandler::Rewind()
{
    m_dwOffset = 0L;
}

BOOL CAsyncFileIOHandler::IsOpen()
{
    return m_file.IsOpen();
}

// Return the error code resulting from the last R/W op. as well as the buffer
// sent as a parameter in the Read or Write method. In the case of a W op., that
// buffer did not change.
DWORD CAsyncFileIOHandler::GetResults(CHAR *&ppBuffer) const
{
    ppBuffer = m_sReadBuffer;

    return m_dwLastError;
}

// Inspired from TContext::FileControl.
DWORD CAsyncFileIOHandler::FileControl(int operation)
{
    switch(operation)
    {
    case TOF_FCNOP_CLOSE:
        {
            Close();
            break;
        }

    case TOF_FCNOP_REWIND:
        {
            Rewind();
            break;
        }

    default:
        {
            // TODO: error
            assert(0);
        }
    };

    return 0;
}// end of TContext::FileControl
