/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"

#include <assert.h>
#include "FileManager.hpp"
#include "AsyncFileIOHandler.hpp"
#include "AsyncVirtualUser.h"
#include "Fastlock.hpp"

using namespace std;

CFileManager::~CFileManager()
{
    for(tFilesMap::iterator it=m_filesMap.begin() ; it!=m_filesMap.end() ; it++)
    {
        delete (*it).second;
    }
}

// --------------------------------------------------------------------------------
// Create a new file slot for file with ID szFileID. Return NULL on error.
// --------------------------------------------------------------------------------
CAsyncFileIOHandler *CFileManager::AllocateFileStruct(const string &szFileID)
{
    CAsyncFileIOHandler *pNewFile = new CAsyncFileIOHandler(this, *m_pIocp, m_pTrace);

    CLock lock (m_sL);

#ifdef _DEBUG
    // TODO: should test this even in release mode?
    tFilesMap::iterator it = m_filesMap.find(szFileID);
    assert(it == m_filesMap.end());
#endif

    if(pNewFile)
    {
        m_filesMap[szFileID] = pNewFile;
        return pNewFile;
    }
    else
        return NULL;
}

// --------------------------------------------------------------------------------
// Return a pointer to the CAsyncFileIOHandlerobject matching szFileID or NULL if not found.
// --------------------------------------------------------------------------------
CAsyncFileIOHandler *CFileManager::RetrieveFileStruct(const string &szFileID)
{
    CLock lock (m_sL);

    tFilesMap::iterator it = m_filesMap.find(szFileID);

    if(it != m_filesMap.end())
        return (*it).second;
    else
        return NULL;
}

// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
void CFileManager::ReleaseFileStruct(const string &szFileID)
{
    CLock lock (m_sL);

    // TODO: tester la valeur de retour.
    m_filesMap.erase(szFileID);
}

void CFileManager::SignalIoIsOver() const
{
    m_pVU->OnFileIoOver();
}

void CFileManager::SignalIoFailed() const
{
}
