/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __FILE_MANAGER_HPP__
#define __FILE_MANAGER_HPP__

#include "AsyncRequest.hpp"
#include "AsyncFile.hpp"
#include "Fastlock.hpp"

#include <map>

// This class implements a container for file handlers.

class CAsyncVirtualUser;
class CAsyncFileIOHandler;

class CFileManager
{
public:
    //// --------------------------------------------------------------------------------
    //// Construction / Destruction.
    //// --------------------------------------------------------------------------------
    CFileManager() : m_pIocp(NULL), m_pTrace(NULL), m_pVU(NULL) {};
    virtual ~CFileManager();

    //// --------------------------------------------------------------------------------
    //// User methods.
    //// --------------------------------------------------------------------------------

    // Initialization. Creates the requests.
    // The derived overrides must call this base implementation first.
    virtual DWORD Init(CTrace* pTrace, CAsyncVirtualUser *pVU, CIOCP *pIocp)
    {
        m_pTrace= pTrace;
        m_pVU   = pVU;
        m_pIocp = pIocp;

        return 0;
    }

    CAsyncFileIOHandler *AllocateFileStruct(const std::string &szFileID);
    CAsyncFileIOHandler *RetrieveFileStruct(const std::string &szFileID);
    void ReleaseFileStruct(const std::string &szFileID);

    void SignalIoIsOver() const;
    void SignalIoFailed() const;

protected:

    //// --------------------------------------------------------------------------------
    //// Data.
    //// --------------------------------------------------------------------------------

    typedef std::map<std::string, CAsyncFileIOHandler*, std::less<std::string> > tFilesMap; // Map of string (file ID) to CAsyncFile pointers.
    tFilesMap m_filesMap;  // Map of files handled by that structure.
    CTrace *m_pTrace;
    CAsyncVirtualUser *m_pVU;   // The VU that we're gonna signal when the current CAsyncFileIOHandler is over with its IO.
                                // This assumes that their's only one file IO pending at a time for each VU.

    CFastLock   m_sL;   // Fastlock serialising accesses to the files map.

    CIOCP * m_pIocp;

private:
    //// Forbidden operators.
    void operator=(const CFileManager&);
};

#endif  // __ASYNC_FILE_MANAGER_HPP__
