/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __STATISTICS_HPP__
#define __STATISTICS_HPP__

#include <time.h>
#include <string>
#include <assert.h>

#include "FastLock.hpp"

///////////////////////////////////////////////////////////////////////////////
//
// 
//
///////////////////////////////////////////////////////////////////////////////

class CStatistics
{
public:
//    CStatistics() {};

    static BOOL Init(const std::string &zStatsFileName, const std::string &zUrlsFileName, DWORD dwViewSize);
    static void Release();

    void    SetUserId(const IN_ADDR &IpAdd, DWORD dwProcessId, DWORD dwThreadId);

    void                SetUrl(const std::string &szURL)            { m_szURL = szURL;  }
    const std::string & GetURL()                            const   { return m_szURL;   }

    void    OnRequestStart();   // To be called upon request start.
    void    OnRequestOver();    // To be called upon request completion.

    void    SetReplyCode(WORD wReplyCode)   { m_statsData.m_wHttpReplyCode = wReplyCode;}
    void    IncDataFlow(DWORD dwDataFlow)   { m_statsData.m_dwDataFlow += dwDataFlow;   }

    clock_t GetElapsedTime()    const   { return m_statsData.m_ctElapsedTime; }
    short   GetHttpReplyCode()  const   { return m_statsData.m_wHttpReplyCode;}

    void    DumpData();
	static void SetTimeOffset(long t_offset);

protected:

    //// Data structures.
    // Structure that stores each file structure information.
    typedef struct SFileInfo_
    {
        HANDLE  m_hFile;
        LPSTR   m_pView;
        DWORD   m_dwCurrViewOffset;
        DWORD   m_dwFileSize;
        DWORD   m_dwViewSize;
    } SFileInfo;

// Force alignment to two bytes to allow memcopies to VM.
#pragma pack(2)

    typedef struct SUserId_
    {
        IN_ADDR m_IpAdd;
        DWORD   m_dwPId;
        WORD    m_wUserId;
    } SUserId;

    // Structure that will be mapped into the statistics data file when request is over.
    typedef struct SOutputStats_
    {
	    time_t	m_ttTimeStamp;      // 4 bytes.
        SUserId m_userId;           // 10 bytes.
	    clock_t m_ctElapsedTime;    // 4 bytes. Time elapsed between request execution and results reception.
        WORD    m_wHttpReplyCode;   // 2 bytes.
        DWORD   m_dwDataFlow;       // 4 bytes.
    } SOutputStats;

#pragma pack()

    //// Helpers.
    static void ReleaseFile(SFileInfo &fileInfo);
    static BOOL CreateMemMappedFile(SFileInfo &fileInfo, const std::string &zFileName, DWORD dwViewSize);
    BOOL EnlargeView(SFileInfo &fileInfo, CONST VOID *pData, DWORD dwDataSize);

protected:
    // Statistics info.
    FILETIME                m_ftRequestSystemTime;      // To compute the request duration.
    LARGE_INTEGER           m_liRequestStartCounter;    // Will use either that one or previous.

    static LARGE_INTEGER    sm_liPerfCtrFrequency;      // Set to zero if perf. counters not supported.

    std::string  m_szURL;
    SOutputStats m_statsData;

    // Output files info.
    static SFileInfo    sm_statsFile;
    static SFileInfo    sm_urlsFile;
    static BOOL         sm_bErrorDontWrite;

    static CFastLock *  sm_pWriteSl;
    static long  sm_tOffset;
};


// -------------------------------------------------------------------------------
// Inlined methods.
// -------------------------------------------------------------------------------

inline void CStatistics::SetUserId(const IN_ADDR &IpAdd, DWORD dwProcessId, DWORD dwThreadId)
{
    m_statsData.m_userId.m_IpAdd    = IpAdd;
    m_statsData.m_userId.m_dwPId    = dwProcessId;
    m_statsData.m_userId.m_wUserId  = (WORD)dwThreadId;
}


#endif  // __STATISTICS_HPP__
