/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** V - T E S T ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: VTEST_RTNDEF                */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  This module defines function prototypes for all the general      */
/*  V-TEST routines.                             */
/*                                       */
/*===========================================================================*/

#ifndef PSL__VTEST_RTNDEF_LOADED
#define PSL__VTEST_RTNDEF_LOADED 1


/*    Include STDDEF.H to ensure that SIZE_T is defined  */

#include    <stddef.h>

/*   Define Incomplete Structures for all referenced types */

#include <psl_structures.h>     /* Common Structures */

#include <vtest_structures.h>       /* V-TEST Structures */

/*   Define function prototypes */

unsigned long   CHK_LGN_CMND_CHR    (        char   *command_chr);

#ifdef VMS
void   SETUP_VTEST_MSG     (void);
#endif

void   SLB_SETUP_LDB       (struct  ldb    *ldb_ptr);

unsigned long   VTEST_ANSWERBACK_CHK    (         char *value_ptr
        ,         char *translated_ptr
        ,unsigned long max_length);

void   VTEST_ANSWERBACK_FMT    (char   *inp_string_ptr
                                ,char   *out_string_ptr);

unsigned long   VTEST_CHK_PROXY (struct diagstk *diag_ptr,
                                 const char *username_ptr,
                                 const char *nodename_ptr);

#ifdef VMS
unsigned long   VTEST_CHK_SRVACC (struct diagstk *diag_ptr,
                                  unsigned long scope,
                                  unsigned long groupid,
                                  unsigned long userid);
#else
unsigned long   VTEST_CHK_SRVACC (unsigned long scope,
                                  unsigned long groupid,
                                  unsigned long userid);
#endif

unsigned long   VTEST_CHK_VP_VERSION (void);

void   VTEST_CNV_NODHDR (struct ldb *ldb_ptr,
                         struct ndh *ndh_ptr);

void   VTEST_CNV_NODE (struct ldb *ldb_ptr,
                       char *node_ptr);

void   VTEST_CNV_SCPHDR (struct ldb *ldb_ptr,
                         struct scph *scph_ptr);

void   VTEST_CNV_SCRIPT (struct ldb *ldb_ptr,
                         char *script_ptr);

void   VTEST_CNV_TSTHDR (struct ldb *ldb_ptr,
                         struct tlh *tlh_ptr);

void   VTEST_CNV_TEST (struct ldb *ldb_ptr,
                       char *test_ptr);


/*
 *    Define function prototypes for routines used by the DTC routines.
 */
unsigned long   VTEST_CONVERT_DTC   (struct   diagstk *diagstk_ptr
                                     ,struct   dtc     *dtc_buf_ptr
                                     ,     char    original_version
                                     ,     char    convert_version);

unsigned long   VTEST_CREATE_REMOTE_EXECUTER
(struct   diagstk *diagstk_ptr
 ,struct   ipc_hed *ipc_hed_ptr
 ,     char    *username_ptr
 ,struct   ndr     *ndr_ptr
 ,         char    *remote_port_ptr
 ,unsigned long    (*INTERRUPT_FN)(void*)
 ,         void    *interrupt_ptr);

unsigned long   VTEST_CREATE_PROCESS    (struct   diagstk *diagstk_ptr
        ,         char    *image_ptr
        ,         char    *username
        ,unsigned long    priority
        ,         Boolean spawn_process
        ,unsigned long    *pid_ptr);

unsigned long   VTEST_ESTABLISH_COMMS   (struct   ipc_hed *ipc_hed_ptr
        ,struct   diagstk *diagstk_ptr
        ,         char    transport
        ,         char    *node_ptr
        ,         char    *port_ptr
        ,         char    max_dtc_version
        ,         char    *ret_version_ptr
        ,struct   ipc_ctx **ret_lnk_pptr
        ,unsigned long    (*INTERRUPT_FN)(void*)
        ,         void    *interrupt_ptr);

void    VTEST_GET_DEFSLB (char *filename_ptr,
                          unsigned long filename_size);


unsigned long   VTEST_EXPAND_LEVEL  (struct   ipc_hed *ipc_hed_ptr
                                     ,struct   diagstk *diagstk_ptr
                                     ,struct   ipc_ctx *lnk_ctx_ptr
                                     ,         char    lnk_dtc_version
                                     ,         char    *routing_ptr
                                     ,struct   dtc     *dtc_rcv_ptr
                                     ,     long    context
                                     ,     char    mode
                                     ,unsigned long    flags
                                     ,unsigned long    (*INTERRUPT_FN)(void*)
                                     ,         void    *interrupt_ptr);

unsigned long   VTEST_GET_FULL_TRM  (struct   ipc_hed *ipc_hed_ptr
                                     ,struct   diagstk *diagstk_ptr
                                     ,struct   ipc_ctx *lnk_ctx_ptr
                                     ,         char    lnk_dtc_version
                                     ,         char    *routing_ptr
                                     ,struct   dtc     *dtc_rcv_ptr
                                     ,     long    context
                                     ,     char    mode
                                     ,unsigned long    (*INTERRUPT_FN)(void*)
                                     ,         void    *interrupt_ptr);

#ifdef VMS
unsigned long   VTEST_GET_RLOCK     (struct diagstk *diag_ptr,
                                     unsigned long *lockid_ptr,
                                     char *resrc_ptr);
#endif /* VMS */

unsigned long   VTEST_GET_STATUS    (struct   ipc_hed *ipc_hed_ptr
                                     ,struct   diagstk *diagstk_ptr
                                     ,struct   ipc_ctx *lnk_ctx_ptr
                                     ,         char    lnk_dtc_version
                                     ,         char    *routing_ptr
                                     ,struct   srv_sts *srv_sts_ptr
                                     ,unsigned long    (*INTERRUPT_FN)(void*)
                                     ,         void    *interrupt_ptr);

unsigned long   VTEST_GET_USERDATA  (struct   diagstk *diag_ptr
                                     ,         char    *username_ptr
                                     ,         long    *gid_ptr
                                     ,         long    *uid_ptr);

unsigned long   VTEST_GET_PORTNAME  (struct   ipc_hed *ipc_hed_ptr
                                     ,struct   diagstk *diagstk_ptr
                                     ,         char    transport
                                     ,         char    *nodename_ptr
                                     ,unsigned long    prcpid
                                     ,         char    *ret_port_ptr
                                     ,unsigned long    (*INTERRUPT_FN)(void*)
                                     ,         void    *interrupt_ptr);

void   VTEST_IPC_DIAGNOSTICS   (struct ipc_hed *ipc_hed_ptr
                                ,struct   diagstk *diagstk_ptr
                                ,struct dtc     *dtc_msg_ptr);

unsigned long   VTEST_READ_LINK     (struct   diagstk *diagstk_ptr
                                     ,struct   ipc_hed *ipc_hed_ptr
                                     ,struct   ipc_ctx *lnk_ctx_ptr
                                     ,     char    lnk_dtc_version
                                     ,struct   dtc     *rcv_buf_ptr
                                     ,         void    **msg_handle_pptr
                                     ,unsigned long    (*INTERRUPT_FN)(void*)
                                     ,         void    *interrupt_ptr);

unsigned long   VTEST_REPLY_TO_LINK (struct   ipc_hed  *ipc_hed_ptr
                                     ,struct   diagstk  *diagstk_ptr
                                     ,struct   ipc_ctx  *lnk_ctx_ptr
                                     ,struct   dtc      *snd_buf_ptr
                                     ,unsigned long     snd_msg_size
                                     ,     char     lnk_dtc_version
                                     ,         long     flags
                                     ,     void     *user_handle_ptr);

void   VTEST_TRACE_DTC     (struct   dtc     *dtc_msg_ptr
                            ,         Boolean send_flag);

unsigned long   VTEST_UPD_ACRCNT (void);

unsigned long   VTEST_UPD_DEV_TRMCNT (char band);

unsigned long   VTEST_WRITE_TO_LINK (struct   ipc_hed  *ipc_hed_ptr
                                     ,struct   diagstk *diagstk_ptr
                                     ,struct   ipc_ctx  *lnk_ctx_ptr
                                     ,struct   dtc      *snd_buf_ptr
                                     ,unsigned long     snd_msg_len
                                     ,     char     lnk_dtc_version
                                     ,         long     flags
                                     ,         long     rsp_timeout
                                     ,     void     *user_handle_ptr);

unsigned long   VTEST_REMOTE_SERVERS    (struct   diagstk *diagstk_ptr
        ,struct   ipc_hed *ipc_hed_ptr
        ,         char    transport
        ,         char    *node_ptr
        ,struct   psl_vm_llb *srv_ll_ptr
        ,unsigned long  (*INTERRUPT_FN)(void*)
        ,     void  *interrupt_ptr);

unsigned long   VTEST_INPRATE_CHK   (char *value_ptr);


unsigned long   VTEST_TRANSLATE_RATE    (         char   *character_rate
        ,         long   *numeric_rate_ptr);

unsigned long   VTEST_GET_CHILD_COUNT   (struct   ipc_hed *ipc_hed_ptr
        ,struct   diagstk *diagstk_ptr
        ,struct   ipc_ctx *lnk_ctx_ptr
        ,         char    lnk_dtc_version
        ,         char    *routing_ptr
        ,         long    *pItemCount
        ,unsigned long    flags
        ,unsigned long    (*INTERRUPT_FN)(void*)
        ,         void    *interrupt_ptr);

/*
 *     Currency checking routines.
 *
 *    Define the LINE_INFO Structure which will contain ALL the information 
 *    concerning the line we wish to scan
 */
typedef struct LINE_INFO
{
    int m_bStatus;
    char m_szCurrency[3];
    double m_dbValue;
    int m_nPosition;
    int m_nDecimalPlaces;
    char m_szFormat[132];
}
LINE_INFO;


/*
 *    And declare the function prototypes
 *
 *    CYR_LINE_SCAN - External function which is called to do the scan 
 */
LINE_INFO   CYR_LINE_SCAN (char *szLine
                           ,int  nPos
                           ,char DecimalSymbol
                           ,char GroupingSymbol);

/*
 *   CYR_MATCH_VALUE - External function which is called to do the match 
 */
int CYR_MATCH_VALUE     (char   *szLine
                         ,int    nPos
                         ,double dbValue
                         ,char*  MatchValue1
                         ,char*  MatchValue2);

/*
 *   FindLowerCellBoundary - Internal function to find the lower cell limit 
 */
void    FindLowerCellBoundary   (char *szLine, int nPos);

/*
 *   FindUpperCellBoundary - Internal function to find the upper cell limit 
 */
void    FindUpperCellBoundary   (char *szLine, int nPos);

/*
 *  CalculateValue - Internal function to find the value in the cell 
 */
double CalculateValue   (char *szLine ,char* pMatchValue);

/*
 *    CalculateFormat - Internal function used to find the values format 
 */
void    CalculateFormat (char *szLine, char* szFormat);



/*
 *    Platform dependant routines
 */
#ifdef  VMS
unsigned long   VTEST_LOCAL_SERVERS  (struct   diagstk *diagstk_ptr
                                      ,unsigned long    vtest_lockid
                                      ,struct   psl_vm_llb *srv_ll_pptr);
#else   /* VMS */
unsigned long   VTEST_LOCAL_SERVERS  (struct   diagstk *diagstk_ptr
,struct   psl_vm_llb *srv_ll_pptr);

#endif  /* VMS */

#endif  /* PSL__VTEST_RTNDEF_LOADED */
